//
//  BoardView.swift
//  nusa
//
//  Created by Kevin Maulana on 01/11/19.
//

import Foundation
import PalioLite
class BoardView: NSObject, FlutterPlatformView {
    var plugin: SwiftPalioLitePlugin?
    var boardView: WhiteboardCanvas?
    var id: Int64?
    var channel: FlutterMethodChannel?
    
    init(plugin: SwiftPalioLitePlugin,boardView: WhiteboardCanvas?, id: Int64) {
        self.plugin = plugin
        self.boardView = boardView
        self.id = id
        super.init()
        
        channel = FlutterMethodChannel(name: "palio_lite/board_\(String(id))", binaryMessenger: plugin.registrar!.messenger())
        channel!.setMethodCallHandler { (call, result) in
            switch (call.method){
            case "setBackground":
                self.handleSetBackground(call: call, result: result); break
            case "setPenColor":
                self.handlePenColor(call: call, result: result); break
            case "setPenSize":
                self.handlePenSize(call: call, result: result); break
            default:
                result(nil)
            }
        }
    }
    
    private func handleSetBackground(call: FlutterMethodCall, result: FlutterResult) {
        let data = call.arguments as! [String: Any?]
        let color = data["color"] as! Int
        let hexString = String(format: "%08X", color)
        boardView?.setBackground(color: UIColor(hexString: hexString))
        result(nil)
    }
    
    private func handlePenColor(call: FlutterMethodCall, result: FlutterResult) {
        let data = call.arguments as! [String: Any?]
        let color = data["color"] as! Int
        let hexString = String(format: "%08X", color)
        boardView?.setLineColor(color: UIColor(hexString: hexString))
        result(nil)
    }
    
    private func handlePenSize(call: FlutterMethodCall, result: FlutterResult) {
        let data = call.arguments as! [String: Any?]
        let size = data["size"] as! Double
        boardView?.setStrokeSize(size: size)
        result(nil)
    }
    
    func view() -> UIView {
        return boardView!
    }
    
}
