import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
// import 'package:crypto/crypto.dart';

class API {
  static const MethodChannel _channel = const MethodChannel('palio_lite');

  static const EventChannel _eventChannel =
      const EventChannel('palio_lite_event');

  static StreamSubscription<dynamic>? _sink;

  static void init() {
    _sink = _eventChannel
        .receiveBroadcastStream()
        .listen(_eventListener, onError: onError);
  }

  static Future<void> destroy() async {
    await _sink?.cancel();
  }

  static void Function(dynamic err)? onError;

  static void Function()? onSuccess;

  static void Function(dynamic err)? onFailed;

  static void Function(dynamic map)? onReceiveMessage;

  static void Function(dynamic map)? onMessage;

  static void Function(dynamic map)? onTyping;

  static void Function(int state, String status)? onStatus;

  static void Function(String code, String fPin, String groupId)? onGroup;

  static void Function(String code, String fPin, String topicId)? onTopic;

  static void Function(String code, String fPin, String groupId, String member)?
      onMemberGroup;

  static void Function(int state, String message)? incomingCall;

  static void Function(int state, String message)? statusCall;

  static void Function(int state, String message)? startSS;

  static void Function(int state, String message)? joinSS;

  static void Function(int state, String message)? onStartLS;

  static void Function(int state, String message)? onJoinLS;

  static void Function(String message, String status)? onLoginProcess;

  static void Function(String status, String message)? onPostUpdate;

  static Future<String> getPin() async {
    return await _channel.invokeMethod("getPin");
  }

  static Future<String> connect(String apiKey) async {
    return await _channel.invokeMethod("connect", {'api_key': apiKey});
  }

  static Future<String> signUp(
      String name, String msisdn, String password, String thumb_id) async {
    return await _channel.invokeMethod('signUp', {
      'name': name,
      'msisdn': msisdn,
      'password': password,
      'thumb_id': thumb_id
    });
  }

  static Future<String> signUpOTP(
      String name, String msisdn, String password, String thumb_id) async {
    return await _channel.invokeMethod('signUpOTP', {
      'name': name,
      'msisdn': msisdn,
      'password': password,
      'thumb_id': thumb_id
    });
  }

  static Future<String> signIn(String name, String password) async {
//    String md5Pass = md5.convert(utf8.encode(password)).toString();
    return await _channel
        .invokeMethod('signIn', {'name': name, 'password': password});
  }

  static Future<String> createGroup(String nameGroup) async {
    return await _channel
        .invokeMethod('createGroup', {'group_name': nameGroup});
  }

  static Future<String> updateGroup(
    String nameGroup,
    String idGroup,
    String open,
    String thumbId,
    String quote,
  ) async {
    return await _channel.invokeMethod('updateGroup', {
      'group_id': idGroup,
      'group_name': nameGroup,
      'open': open,
      'thumb_id': thumbId,
      'quote': quote,
    });
  }

  static Future<String> addGroupMember(String idGroup, String memberId) async {
    return await _channel.invokeMethod(
        'addGroupMember', {'group_id': idGroup, 'member_pin': memberId});
  }

  static Future<String> changePosition(
      String idGroup, String memberId, String position) async {
    return await _channel.invokeMethod('changePosition',
        {'group_id': idGroup, 'member_pin': memberId, 'position': position});
  }

  static Future<String> exitGroup(String idGroup, String memberId) async {
    return await _channel.invokeMethod(
        'exitGroup', {'group_id': idGroup, 'member_pin': memberId});
  }

  static Future<String> addChat(String idGroup, String title) async {
    return await _channel
        .invokeMethod('addChat', {'group_id': idGroup, 'title': title});
  }

  static Future<String> updateChat(
      String idGroup, String title, String chatId) async {
    return await _channel.invokeMethod(
        'updateChat', {'group_id': idGroup, 'chat_id': chatId, 'title': title});
  }

  static Future<String> deleteChat(String idGroup, String chatId) async {
    return await _channel
        .invokeMethod('deleteChat', {'group_id': idGroup, 'chat_id': chatId});
  }

  static Future<dynamic> download(String file) async {
    return await _channel.invokeMethod('download', {'file': file});
  }

  static Future<dynamic> initiateAudioCall(String deviceId) async {
    return await _channel
        .invokeMethod('initiateAudioCall', {'device_id': deviceId});
  }

  static Future<dynamic> addAudioCall(String deviceId) async {
    return await _channel.invokeMethod('addAudioCall', {'device_id': deviceId});
  }

  static Future<dynamic> receiveAudioCall(String deviceId) async {
    return await _channel
        .invokeMethod('receiveAudioCall', {'device_id': deviceId});
  }

  static Future<dynamic> initiateVideoCall(
      String deviceId, String cameraId) async {
    return await _channel.invokeMethod(
        'initiateVideoCall', {'device_id': deviceId, 'camera_id': cameraId});
  }

  static Future<dynamic> addVideoCall(String deviceId, String cameraId) async {
    return await _channel.invokeMethod(
        'addVideoCall', {'device_id': deviceId, 'camera_id': cameraId});
  }

  static Future<dynamic> receiveVideoCall(
      String deviceId, String cameraId) async {
    return await _channel.invokeMethod(
        'receiveVideoCall', {'device_id': deviceId, 'camera_id': cameraId});
  }

  static Future<dynamic> terminateCall(String deviceId) async {
    return await _channel
        .invokeMethod('terminateCall', {'device_id': deviceId});
  }

  static Future<dynamic> terminateConfCall(String deviceId) async {
    return await _channel
        .invokeMethod('terminateConfCall', {'device_id': deviceId});
  }

  static Future<dynamic> postRegistration() async {
    return await _channel.invokeMethod('postRegistration');
  }

  static Future<dynamic> addFriend(String fPin) async {
    return await _channel.invokeMethod('addFriend', {'f_pin': fPin});
  }

  static Future<dynamic> removeFriend(String lPin) async {
    return await _channel.invokeMethod('removeFriend', {'l_pin': lPin});
  }

  static Future<dynamic> suggestionPerson(String pLastSeq) async {
    return await _channel
        .invokeMethod('suggestionPerson', {'p_last_seq': pLastSeq});
  }

  static Future<dynamic> searchFriend(String search, String limit) async {
    return await _channel
        .invokeMethod('searchFriend', {'search': search, "limit": limit});
  }

  static Future<dynamic> setSpeaker(bool speaker) async {
    return await _channel.invokeMethod('setSpeaker', {'speaker': speaker});
  }

  static Future<dynamic> muteMicrophone(bool mute) async {
    return await _channel.invokeMethod('muteMicrophone', {'mute': mute});
  }

  static Future<dynamic> setCCallAudio(String device_id, bool speaker) async {
    return await _channel.invokeMethod(
        'turnAudioCCall', {'device_id': device_id, 'speaker': speaker});
  }

  static Future<dynamic> retrieveTimeline(
      String score,
      String lastLastUpdate,
      String filterAccount,
      String filterCategory,
      String search,
      String shr) async {
    return await _channel.invokeMethod('retrieveTimeline', {
      'score': score,
      'last_last_update': lastLastUpdate,
      'filter_account': filterAccount,
      'filter_category': filterCategory,
      'search': search,
      'shr': shr
    });
  }

  static Future<dynamic> retrievePostProfile(
      String f_pin, String last_created_date,
      {String type = "1", String merchant_id = "", String storyId = ""}) async {
    if (f_pin.isEmpty) {
      return;
    }
    return await _channel.invokeMethod('retrievePostProfile', {
      'f_pin': f_pin,
      'last_created_date': last_created_date,
      'type': type,
      'merchant_id': merchant_id,
      'storyId': storyId
    });
  }

  static Future<dynamic> retrieveChannel(String offset, String filterAccount,
      String filterCategory, String search, String shr) async {
    return await _channel.invokeMethod('retrieveChannel', {
      'offset': offset,
      'filter_account': filterAccount,
      'filter_category': filterCategory,
      'search': search,
      'shr': shr
    });
  }

  static Future<dynamic> retrieveNearby(
      String latitude, String longitude) async {
    return await _channel.invokeMethod('retrieveNearby', {
      'latitude': latitude,
      'longitude': longitude,
    });
  }

  static Future<dynamic> searchPerson(String name) async {
    return await _channel.invokeMethod('searchPerson', {
      'name': name,
    });
  }

  static Future<dynamic> trackPerson(String fPin) async {
    return await _channel.invokeMethod('trackPerson', {
      'f_pin': fPin,
    });
  }

  static Future<dynamic> statusCovid() async {
    return await _channel.invokeMethod('statusCovid');
  }

  static Future<dynamic> pullMainContent() async {
    return await _channel.invokeMethod('pullMainContent');
  }

  static Future<dynamic> pullStoryList(String filter, String offset) async {
    return await _channel
        .invokeMethod('pullStoryList', {'filter': filter, 'offset': offset});
  }

  static Future<dynamic> pullStoryListPersonal(
      String filter, String offset, String l_pin) async {
    return await _channel.invokeMethod('pullStoryListPersonal',
        {'filter': filter, 'offset': offset, 'l_pin': l_pin});
  }

  static Future<dynamic> updateCounter(String lPin) async {
    return await _channel.invokeMethod('updateCounter', {
      'l_pin': lPin,
    });
  }

  static Future<dynamic> startLS(String title, int cameraId) async {
    return await _channel
        .invokeMethod('startLS', {'title': title, 'camera_id': cameraId});
  }

  static Future<dynamic> leftLS(String deviceId) async {
    return await _channel.invokeMethod('leftLS', {'device_id': deviceId});
  }

  static Future<dynamic> endLS() async {
    return await _channel.invokeMethod('endLS');
  }

  static Future<dynamic> joinLS(String deviceId) async {
    return await _channel.invokeMethod('joinLS', {'device_id': deviceId});
  }

  static Future<dynamic> sendLSBroadcast(
      String title, String type, String typeValue, String category) async {
    return await _channel.invokeMethod('sendLSBroadcast', {
      'title': title,
      'type': type,
      'typeValue': typeValue,
      'category': category
    });
  }

  static Future<dynamic> submitKuisioner(
      String data, String latitude, String longitude) async {
    return await _channel.invokeMethod('submitKuisioner',
        {'data': data, 'latitude': latitude, 'longitude': longitude});
  }

  static Future<dynamic> submitKuisionerAdditional(
      String data, String surveyId) async {
    return await _channel.invokeMethod(
        'submitKuisionerAdditional', {'data': data, 'survey_id': surveyId});
  }

  static Future<dynamic> joinLiveVideo(
      String broadcasterId, String requestId) async {
    return await _channel.invokeMethod('joinLiveVideo',
        {'broadcast_id': broadcasterId, 'request_id': requestId});
  }

  static Future<dynamic> removeLiveVideo(
      String broadcasterId, String requestId) async {
    return await _channel.invokeMethod('removeLiveVideo',
        {'broadcast_id': broadcasterId, 'request_id': requestId});
  }

  static Future<dynamic> leftLiveVideo(
      String broadcasterId, String requestId) async {
    return await _channel.invokeMethod('leftLiveVideo',
        {'broadcast_id': broadcasterId, 'request_id': requestId});
  }

  static Future<dynamic> sendLSChat(String lPin, String messageText) async {
    return await _channel.invokeMethod(
        'sendLSChat', {'l_pin': lPin, 'message_text': messageText});
  }

  static Future<dynamic> sendLSEmotion(String lPin, String emotion) async {
    return await _channel
        .invokeMethod('sendLSEmotion', {'l_pin': lPin, 'emotion': emotion});
  }

  static Future<dynamic> updateLSTitle(String title) async {
    return await _channel.invokeMethod('updateLSTitle', {'title': title});
  }

  static Future<String> sendLoginEmail(String email) async {
    return await _channel.invokeMethod('sendLoginEmail', {'email': email});
  }

  static Future<String> sendVerificationEmail(
      String email, String token) async {
    return await _channel.invokeMethod(
        'sendVerificationEmail', {'email': email, 'token': token});
  }

  static Future<String> getListSchool() async {
    return await _channel.invokeMethod('getListSchool');
  }

  static Future<String> getListSchoolByName(String keyword) async {
    return await _channel
        .invokeMethod('getListSchoolByName', {"keyword": keyword});
  }

  static Future<String> getListClassName(
      String schoolId, String level, String major, String classType) async {
    return await _channel.invokeMethod('getListClass', {
      "school_id": schoolId,
      "level": level,
      "major": major,
      "class_type": classType
    });
  }

  static Future<String> getRequestApprovalTeacher(
      String f_pin,
      String l_pin,
      String schoolId,
      String level,
      String major,
      String stage,
      String type,
      String className,
      String email,
      String name,
      String msisdn) async {
    return await _channel.invokeMethod('approvalRequestTeacher', {
      "f_pin": f_pin,
      "l_pin": l_pin,
      "school_id": schoolId,
      "level": level,
      "major": major,
      "type": type,
      "class_name": className,
      "email": email,
      "name": name,
      "stage": stage,
      "msisdn": msisdn
    });
  }

  static Future<String> requestStudent(
      String f_pin,
      String l_pin,
      String schoolId,
      String groupId,
      String email,
      String name,
      String msisdn) async {
    return await _channel.invokeMethod('requestStudent', {
      "f_pin": f_pin,
      "l_pin": l_pin,
      "school_id": schoolId,
      "group_id": groupId,
      "email": email,
      "name": name,
      "msisdn": msisdn
    });
  }

  static Future<String> getUpdateUser(
      String role,
      String image,
      String password,
      String email,
      String data,
      String name,
      String msisdn) async {
    return await _channel.invokeMethod('getUpdateUser', {
      "role": role,
      "image": image,
      "password": password,
      "email": email,
      "data": data,
      "name": name,
      "msisdn": msisdn
    });
  }

  static int _getCamera(CameraInfo camera) {
    switch (camera) {
      case CameraInfo.CAMERA_FACING_FRONT:
        return 1;
      default:
        return 0;
    }
  }

  static Future<dynamic> updatePassword(
      String password, String oldPassword) async {
    return await _channel.invokeMethod(
        'changeUserPwd', {'password': password, 'oldpassword': oldPassword});
  }

  static Future<dynamic> updateUserName(
      String firstname, String lastname) async {
    return await _channel.invokeMethod(
        'changeUserName', {'firstname': firstname, 'lastname': lastname});
  }

  static Future<dynamic> updateUserStatus(String status) async {
    return await _channel.invokeMethod('changeUserStatus', {'status': status});
  }

  static Future<dynamic> setAcceptCall(bool acceptCall) async {
    return await _channel
        .invokeMethod('setAcceptCall', {'accept_call': acceptCall});
  }

  static Future<dynamic> setPrivate(bool private) async {
    return await _channel.invokeMethod('setPrivate', {'private': private});
  }

  static Future<dynamic> getLSData(String l_pin) async {
    return await _channel.invokeMethod('getLSData', {'l_pin': l_pin});
  }

  static Future<dynamic> sendCreateLS(String title, String type,
      String typeValue, String category, String blog_id, String data) async {
    return await _channel.invokeMethod('createLS', {
      'title': title,
      'type': type,
      'typeValue': typeValue,
      'category': category,
      'blog_id': blog_id,
      'data': data
    });
  }

  static Future<dynamic> initiateACRoom(String deviceId) async {
    return await _channel
        .invokeMethod('initiateACRoom', {'device_id': deviceId});
  }

  static Future<dynamic> joinACRoom(String deviceId) async {
    return await _channel.invokeMethod('joinACRoom', {'device_id': deviceId});
  }

  static Future<dynamic> initiateVCRoom(String deviceId, int cameraId) async {
    return await _channel.invokeMethod(
        'initiateVCRoom', {'device_id': deviceId, 'camera_id': cameraId});
  }

  static Future<dynamic> joinVCRoom(String deviceId, int cameraId) async {
    return await _channel.invokeMethod(
        'joinVCRoom', {'device_id': deviceId, 'camera_id': cameraId});
  }

  static Future<dynamic> requestLiveVideo(String f_pin) async {
    return await _channel.invokeMethod('requestLiveVideo', {'f_pin': f_pin});
  }

  static Future<dynamic> createCRoom(String blog_id, String data) async {
    return await _channel
        .invokeMethod('createCRoom', {'blog_id': blog_id, 'data': data});
  }

  static Future<dynamic> startCRoom(String blog_id, String time) async {
    return await _channel
        .invokeMethod('startCRoom', {'blog_id': blog_id, 'time': time});
  }

  static Future<dynamic> joinCRoom(String blog_id) async {
    return await _channel.invokeMethod('joinCRoom', {'blog_id': blog_id});
  }

  static Future<dynamic> endCRoom(String blog_id) async {
    return await _channel.invokeMethod('endCRoom', {'blog_id': blog_id});
  }

  static Future<dynamic> startScreenSharing(String deviceId) async {
    return await _channel
        .invokeMethod('startScreenSharing', {'device_id': deviceId});
  }

  static Future<dynamic> joinScreenSharing(String deviceId) async {
    return await _channel
        .invokeMethod('joinScreenSharing', {'device_id': deviceId});
  }

  static Future<dynamic> endScreenSharing(String deviceId) async {
    return await _channel
        .invokeMethod('endScreenSharing', {'device_id': deviceId});
  }

  static Future<dynamic> initScreenSharing(String deviceId) async {
    return await _channel
        .invokeMethod('initScreenSharing', {'device_id': deviceId});
  }

  static Future<dynamic> leaveScreenSharing() async {
    return await _channel.invokeMethod('leaveScreenSharing');
  }

  static Future<dynamic> sendToBot(
      String text, String type, String blog_id) async {
    return await _channel.invokeMethod(
        'sendToBot', {'text': text, 'type': type, 'blog_id': blog_id});
  }

  static Future<dynamic> submitPost(
      String title,
      String description,
      String link,
      String type,
      String thumbId,
      String privacy,
      String fileId,
      String category,
      String fileType,
      String author_name,
      String author_thumb,
      String pricing,
      String pricing_money) async {
    return await _channel.invokeMethod('submitPost', {
      'title': title,
      'description': description,
      'link': link,
      'type': type,
      'thumb_id': thumbId,
      'privacy': privacy,
      'file_id': fileId,
      'category': category,
      'file_type': fileType,
      'author_name': author_name,
      'author_thumb': author_thumb,
      'pricing': pricing,
      'pricing_money': pricing_money
    });
  }

  static Future<dynamic> submitPostInvitation(
      String title,
      String description,
      String link,
      String type,
      String thumbId,
      String privacy,
      String fileId,
      String category,
      String fileType,
      String author_name,
      String author_thumb,
      String target,
      String members,
      String pricing,
      String pricing_money) async {
    return await _channel.invokeMethod('submitPostInvitation', {
      'title': title,
      'description': description,
      'link': link,
      'type': type,
      'thumb_id': thumbId,
      'privacy': privacy,
      'file_id': fileId,
      'category': category,
      'file_type': fileType,
      'author_name': author_name,
      'author_thumb': author_thumb,
      'target': target,
      'members': members,
      'pricing': pricing,
      'pricing_money': pricing_money
    });
  }

  static Future<dynamic> submitPostGroup(
      String title,
      String description,
      String link,
      String type,
      String thumbId,
      String privacy,
      String fileId,
      String category,
      String fileType,
      String author_name,
      String author_thumb,
      String target,
      String groups,
      String topics,
      String pricing,
      String pricing_money) async {
    return await _channel.invokeMethod('submitPostGroup', {
      'title': title,
      'description': description,
      'link': link,
      'type': type,
      'thumb_id': thumbId,
      'privacy': privacy,
      'file_id': fileId,
      'category': category,
      'file_type': fileType,
      'author_name': author_name,
      'author_thumb': author_thumb,
      'target': target,
      'groups': groups,
      'topics': topics,
      'pricing': pricing,
      'pricing_money': pricing_money,
    });
  }

  static Future<dynamic> submitPostEdu(
      String title,
      String description,
      String link,
      String type,
      String thumbId,
      String privacy,
      String fileId,
      String category,
      String fileType,
      String author_name,
      String author_thumb,
      String level_edu,
      String materi_edu,
      String finaltest_edu,
      String pricing,
      String pricing_money,
      String question) async {
    return await _channel.invokeMethod('submitPostEdu', {
      'title': title,
      'description': description,
      'link': link,
      'type': type,
      'thumb_id': thumbId,
      'privacy': privacy,
      'file_id': fileId,
      'category': category,
      'file_type': fileType,
      'author_name': author_name,
      'author_thumb': author_thumb,
      'level_edu': level_edu,
      'materi_edu': materi_edu,
      'finaltest_edu': finaltest_edu,
      'pricing': pricing,
      'pricing_money': pricing_money,
      'question': question
    });
  }

  static Future<dynamic> submitPostEduInvitation(
      String title,
      String description,
      String link,
      String type,
      String thumbId,
      String privacy,
      String fileId,
      String category,
      String fileType,
      String author_name,
      String author_thumb,
      String target,
      String members,
      String level_edu,
      String materi_edu,
      String finaltest_edu,
      String pricing,
      String pricing_money,
      String question) async {
    return await _channel.invokeMethod('submitPostEduInvitation', {
      'title': title,
      'description': description,
      'link': link,
      'type': type,
      'thumb_id': thumbId,
      'privacy': privacy,
      'file_id': fileId,
      'category': category,
      'file_type': fileType,
      'author_name': author_name,
      'author_thumb': author_thumb,
      'target': target,
      'members': members,
      'level_edu': level_edu,
      'materi_edu': materi_edu,
      'finaltest_edu': finaltest_edu,
      'pricing': pricing,
      'pricing_money': pricing_money,
      'question': question
    });
  }

  static Future<dynamic> submitPostEduGroup(
      String title,
      String description,
      String link,
      String type,
      String thumbId,
      String privacy,
      String fileId,
      String category,
      String fileType,
      String author_name,
      String author_thumb,
      String target,
      String groups,
      String topics,
      String level_edu,
      String materi_edu,
      String finaltest_edu,
      String pricing,
      String pricing_money,
      String question) async {
    return await _channel.invokeMethod('submitPostEduGroup', {
      'title': title,
      'description': description,
      'link': link,
      'type': type,
      'thumb_id': thumbId,
      'privacy': privacy,
      'file_id': fileId,
      'category': category,
      'file_type': fileType,
      'author_name': author_name,
      'author_thumb': author_thumb,
      'target': target,
      'groups': groups,
      'topics': topics,
      'level_edu': level_edu,
      'materi_edu': materi_edu,
      'finaltest_edu': finaltest_edu,
      'pricing': pricing,
      'pricing_money': pricing_money,
      'question': question
    });
  }

  static Future<dynamic> createStory(String post_id, String title) async {
    return await _channel
        .invokeMethod('createStory', {'post_id': post_id, 'title': title});
  }

  static Future<dynamic> updateStory(
      String story_id, String post_id, String title) async {
    return await _channel.invokeMethod('updateStory',
        {'story_id': story_id, 'post_id': post_id, 'title': title});
  }

  static Future<dynamic> changeUserImage(String thumb_id) async {
    return await _channel
        .invokeMethod('changeUserImage', {'thumb_id': thumb_id});
  }

  static Future<dynamic> uploadImage(String thumb_id) async {
    return await _channel.invokeMethod('uploadImage', {'thumb_id': thumb_id});
  }

  static Future<dynamic> sendOTPMsisdn(String msisdn, String demo) async {
    return await _channel
        .invokeMethod('sendOTPSignup', {'msisdn': msisdn, 'demo': demo});
  }

  static Future<dynamic> verifyOTP(String msisdn, String otp) async {
    return await _channel
        .invokeMethod('verifyOTP', {'msisdn': msisdn, 'otp': otp});
  }

  static Future<dynamic> signInOTP(
      String f_pin, String f_name, String l_name, String thumb_id) async {
    return await _channel.invokeMethod('signinOTP', {
      'f_pin': f_pin,
      'f_name': f_name,
      'l_name': l_name,
      'thumb_id': thumb_id
    });
  }

  static Future<dynamic> changeUserConnection(String f_pin) async {
    return await _channel.invokeMethod('changeConnection', {'f_pin': f_pin});
  }

  static Future<dynamic> sendOTPChangeMSISDN(String newMsisdn) async {
    return await _channel
        .invokeMethod('sendOTPChangeMSISDN', {'msisdn': newMsisdn});
  }

  static Future<dynamic> changePersonMSISDN(String newMsisdn) async {
    return await _channel
        .invokeMethod('changePersonMSISDN', {'msisdn': newMsisdn});
  }

  static Future<dynamic> saveToGallery(bool save) async {
    return await _channel.invokeMethod('saveToGallery', {'save': save});
  }

  static Future<dynamic> vibrate() async {
    return await _channel.invokeMethod('vibrate');
  }

  static Future<dynamic> stopVibrate() async {
    return await _channel.invokeMethod('stopVibrate');
  }

  static Future<dynamic> setVibrate(bool vibrate) async {
    return await _channel.invokeMethod('setVibrate', {'vibrate': vibrate});
  }

  static Future<dynamic> deletePostTimeline(
      String post_id, String last_update, String ec_date) async {
    return await _channel.invokeMethod('deletePost',
        {'post_id': post_id, 'last_update': last_update, 'ec_date': ec_date});
  }

  static Future<dynamic> isInitiatorJoin(
      String broadcast_id, String category_id, String blog_id) async {
    return await _channel.invokeMethod('isInitiatorJoin', {
      'broadcast_id': broadcast_id,
      'category_id': category_id,
      'blog_id': blog_id
    });
  }

  static Future<dynamic> startLPInvited(String title, String type,
      String typeValue, String category_id, String blog_id) async {
    return await _channel.invokeMethod('startLPInvited', {
      'title': title,
      'type': type,
      'typeValue': typeValue,
      'category_id': category_id,
      'blog_id': blog_id
    });
  }

  static Future<String> getRequestStudent(String formId) async {
    return await _channel
        .invokeMethod('getRequestStudent', {'form_id': formId});
  }

  static Future<String> approveRequestStudent(
      String l_pin, String formId) async {
    return await _channel.invokeMethod(
        'approveRequestStudent', {"l_pin": l_pin, "form_id": formId});
  }

  static Future<String> getRequestTeacher(String formId) async {
    return await _channel
        .invokeMethod('getRequestTeacher', {'form_id': formId});
  }

  static Future<String> approveRequestTeacher(
      String l_pin, String formId) async {
    return await _channel.invokeMethod(
        'approveRequestTeacher', {"l_pin": l_pin, "form_id": formId});
  }

  static Future<String> getListSubAccount() async {
    return await _channel.invokeMethod("getListSubAccount");
  }

  static Future<String> quizDetail(String post_id, String l_pin) async {
    return await _channel
        .invokeMethod("quizDetail", {"l_pin": l_pin, "post_id": post_id});
  }

  static Future<String> quizAnswer(String post_id, String data) async {
    return await _channel
        .invokeMethod("quizAnswer", {"data": data, "post_id": post_id});
  }

  static Future<String> quizScoring(
      String post_id, String l_pin, String score) async {
    return await _channel.invokeMethod(
        "quizScoring", {"l_pin": l_pin, "post_id": post_id, "score": score});
  }

  static void Function(int progress, String code)? onLoginProgress;

  static void Function(double progress, String fileName)? onDownloadProgress;

  static void Function(double progress, String fileName)? onUploadProgress;

  static void Function(double progress, String fileName)? onUploadFileProgress;

  static void _eventListener(dynamic event) {
    print('_eventListener $event');
    if (event == null) return;
    final Map<dynamic, dynamic> map = event;

    switch (map['event']) {
      case 'loginProgress':
        if (onLoginProgress != null) {
          onLoginProgress!(map['progress'], map['code']);
        }
        break;

      case 'downloadProgress':
        if (onDownloadProgress != null) {
          onDownloadProgress!(map['progress'], map['fileName']);
        }
        break;

      case 'upload':
        if (onUploadProgress != null) {
          onUploadProgress!(map['progress'], map['name']);
        }
        break;

      case 'receiveMessage':
        if (onReceiveMessage != null) {
          onReceiveMessage!(map);
        }
        break;

      case 'statusMessage':
        if (onMessage != null) {
          onMessage!(map);
        }
        break;

      case 'personInfoCallback':
        if (onStatus != null) {
          onStatus!(map["state"], map["message"]);
        }
        break;

      case 'typingStatus':
        if (onTyping != null) {
          onTyping!(map);
        }
        break;

      case 'group':
        if (onGroup != null) {
          onGroup!(map['code'], map['f_pin'], map['groupId']);
        }
        break;

      case 'topic':
        if (onTopic != null) {
          onTopic!(map['code'], map['f_pin'], map['topicId']);
        }
        break;

      case 'incomingCall':
        if (incomingCall != null) {
          incomingCall!(map["state"], map["message"]);
        }
        break;

      case 'statusCall':
        if (statusCall != null) {
          statusCall!(map["state"], map["message"]);
        }
        break;

      case 'startSS':
        if (startSS != null) {
          startSS!(map["state"], map["message"]);
        }
        break;

      case 'joinSS':
        if (joinSS != null) {
          joinSS!(map["state"], map["message"]);
        }
        break;

      case 'member':
        if (onMemberGroup != null) {
          onMemberGroup!(
              map['code'], map['f_pin'], map['groupId'], map['member']);
        }
        break;

      case 'startLS':
        if (onStartLS != null) {
          onStartLS!(map["state"], map["message"]);
        }
        break;

      case 'joinLS':
        if (onJoinLS != null) {
          onJoinLS!(map["state"], map["message"]);
        }
        break;
      case 'uploadProgress':
        if (onUploadFileProgress != null) {
          onUploadFileProgress!(map['progress'], map['name']);
        }
        break;
      case 'loginProcess':
        if (onLoginProcess != null) {
          onLoginProcess!(map['message'], map['status']);
        }
        break;
      case 'postTimeline':
        if (onPostUpdate != null) {
          onPostUpdate!(map['status'], map['message']);
        }
        break;
    }
  }

  static Future<dynamic> sendMessage(
      {required String lPin,
      required String messageScopeId,
      required String status,
      required String messageText,
      String credential = "",
      String attachmentFlag = "",
      String exBlogId = "",
      String messageLargeText = "",
      String exFormat = "",
      String imageId = "",
      String audioId = "",
      String videoId = "",
      String fileId = "",
      String thumbId = "",
      String reffId = "",
      String readReceipts = "",
      String chatId = "",
      String isCallCenter = "0",
      String callCenterId = ""}) async {
    return await _channel.invokeMethod('sendMessage', {
      'l_pin': lPin,
      'message_scope_id': messageScopeId,
      'status': status,
      'message_text': messageText,
      'credential': credential,
      'attachment_flag': attachmentFlag,
      'ex_blog_id': exBlogId,
      'message_large_text': messageLargeText,
      'ex_format': exFormat,
      'image_id': imageId,
      'audio_id': audioId,
      'video_id': videoId,
      'file_id': fileId,
      'thumb_id': thumbId,
      'reff_id': reffId,
      'read_receipts': readReceipts,
      'chat_id': chatId,
      'is_call_center': isCallCenter,
      'call_center_id': callCenterId
    });
  }

  static Future<void> sendReadMessageStatus(String fPin, String chatId,
      String messageScopeId, String messageId) async {
    return await _channel.invokeMethod('sendReadMessageStatus', {
      "f_pin": fPin,
      "chat_id": chatId,
      "message_scope_id": messageScopeId,
      "message_id": messageId,
    });
  }

  static Future<void> deleteMessage(String lPin, String chatId, String type,
      String messageScopeId, String messageId) async {
    return await _channel.invokeMethod('deleteMessage', {
      "l_pin": lPin,
      "chat": chatId,
      "type": type,
      "scope": messageScopeId,
      "message_id": messageId,
    });
  }

  static Future<void> sendTyping(
      String lPin, String messageScopeId, String status) async {
    return await _channel.invokeMethod('sendTyping', {
      "l_pin": lPin,
      "scope": messageScopeId,
      "status": status,
    });
  }

  static Future<void> getBuddyInfo(String lPin) async {
    return await _channel.invokeMethod('getBuddyInfo', {
      "l_pin": lPin,
    });
  }

  static Future<void> sendAckmessage(
      String lPin,
      String fPin,
      String serverDate,
      String messageScopeId,
      String messageId,
      String longitude,
      String latitude,
      String description) async {
    return await _channel.invokeMethod('sendAckMessage', {
      "l_pin": lPin,
      "f_pin": fPin,
      "server_date": serverDate,
      "scope": messageScopeId,
      "message_id": messageId,
      "longitude": longitude,
      "latitude": latitude,
      "description": description,
    });
  }

  static Future<void> pullBatchBuddyList(String p_f_pin) async {
    return await _channel
        .invokeMethod("pullBatchBuddy", {"p_f_pin": p_f_pin, "last_update": 0});
  }

  static Future<void> sendReport(String post_id, String reason,
      {String report_date = ""}) async {
    if (report_date == null || report_date.isEmpty) {
      report_date = DateTime.now().millisecondsSinceEpoch.toString();
    }
    return await _channel.invokeMethod('sendReport', {
      "post_id": post_id,
      "reason": reason,
      "report_date": report_date,
    });
  }

  static Future<String> getComment(String post_id) async {
    return await _channel.invokeMethod('getComment', {
      "post_id": post_id,
    });
  }

  static Future<String> sendComment(
      String comment_id, String post_id, String ref_comment_id, String comment,
      {String image = "", String name = "", String commentDate = ""}) async {
    if (commentDate == null || commentDate.isEmpty) {
      commentDate = DateTime.now().millisecondsSinceEpoch.toString();
    }
    return await _channel.invokeMethod('sendComment', {
      "image": image,
      "name": name,
      "comment_id": comment_id,
      "post_id": post_id,
      "ref_comment_id": ref_comment_id,
      "comment": comment,
      "commentDate": commentDate,
    });
  }

  static Future<String> sendFollow(String post_id, String l_pin) async {
    return await _channel.invokeMethod('sendFollow', {
      "post_id": post_id,
      "l_pin": l_pin,
      "followDate": DateTime.now().millisecondsSinceEpoch.toString(),
    });
  }

  static Future<String> sendUnFollow(String post_id, String l_pin) async {
    return await _channel.invokeMethod('sendUnFollow', {
      "post_id": post_id,
      "l_pin": l_pin,
    });
  }

  static Future<String> deleteComment(
      String comment_id, String post_id, String ref_comment_id) async {
    return await _channel.invokeMethod('deleteComment', {
      "comment_id": comment_id,
      "post_id": post_id,
      "ref_comment_id": ref_comment_id,
    });
  }

  static Future<String> receiveUpdateComment(
      String post_id, String status) async {
    return await _channel.invokeMethod('receiveUpdateComment', {
      "post_id": post_id,
      "status": status, // 1: open, 2: close editor
    });
  }

  static Future<String> sendReaction(
      String post_id, String flag_reaction, String beforeFlagChanged,
      {String last_update = ""}) async {
    if (last_update == null || last_update.isEmpty) {
      last_update = DateTime.now().millisecondsSinceEpoch.toString();
    }
    return await _channel.invokeMethod('sendReaction', {
      "post_id": post_id,
      "flag_reaction": flag_reaction, // 0: unlike, 1: like, -1: dislike
      "last_update": last_update,
      "beforeFlagChanged": beforeFlagChanged, // 0: unlike, 1: like, -1: dislike
    });
  }

  static Future<void> clearChat(String lPin, String chatId, String fPin,
      String idSummary, String scope) async {
    return await _channel.invokeMethod('clearChat', {
      "l_pin": lPin,
      "chat_id": chatId,
      "f_pin": fPin,
      "id_summary": idSummary,
      "scope": scope
    });
  }

  static Future<void> addLink(
      String idLink, String link, String dataLink, int retry) async {
    return await _channel.invokeMethod('addLink', {
      "id_link": idLink,
      "link": link,
      "data_link": dataLink,
      "retry": retry
    });
  }

  static Future<void> addFavoriteMessage(String messageId) async {
    return await _channel
        .invokeMethod('addMessageFavorite', {"message_id": messageId});
  }

  static Future<void> deleteMessageFavorite(String messageId) async {
    return await _channel
        .invokeMethod('deleteMessageFavorite', {"message_id": messageId});
  }

  static Future<void> initWhiteboard(String lPin) async {
    return await _channel.invokeMethod('initWhiteboard', {"l_pin": lPin});
  }

  static Future<void> joinWhiteboard(String lPin) async {
    return await _channel.invokeMethod('joinWhiteboard', {"l_pin": lPin});
  }

  static Future<String> blockUser(String lPin, String block) async {
    return await _channel
        .invokeMethod('blockUser', {"l_pin": lPin, "block": block});
  }

  static Future<String> unblockUser(String lPin, String block) async {
    return await _channel
        .invokeMethod('unblockUser', {"l_pin": lPin, "block": block});
  }

  static Future<dynamic> getOpenGroups(
      String pAccount, String offset, String search) async {
    return await _channel.invokeMethod('getOpenGroups',
        {"p_account": pAccount, "offset": offset, "search": search});
  }

  static Widget createBoardView(Function(int viewId) created, {Key? key}) {
    return UiKitView(
      key: key,
      viewType: 'BoardView',
      onPlatformViewCreated: (viewId) {
        if (created != null) {
          created(viewId);
        }
      },
    );
  }

  static Future<void> clearWhiteboard() async {
    await _channel.invokeMethod('clearWhiteboard');
  }

  static Future<void> terminateWhiteboard() async {
    await _channel.invokeMethod('terminateWhiteboard');
  }

  static Future<void> setWBPenSize(double size) async {
    await _channel.invokeMethod('setWBPenSize', {'size': size});
  }

  static Future<void> setWBPenColor(int color) async {
    await _channel.invokeMethod('setWBPenColor', {'color': color});
  }

  static Future<String> createSubGroup(
      String groupName, String parent, String level) async {
    return await _channel.invokeMethod('createSubGroup',
        {'group_name': groupName, 'parent': parent, 'level': level});
  }

  static Future<String> submitSchool(
      String f_pin,
      String schoolId,
      String schoolName,
      String level,
      String stage,
      String class_,
      String major,
      String name) async {
    return await _channel.invokeMethod('submitSchool', {
      'f_pin': f_pin,
      'school_id': schoolId,
      'school_name': schoolName,
      'level': level,
      'stage': stage,
      'class': class_,
      'major': major,
      'name': name
    });
  }

  static Future<String> requestCallCenter(int pChannel) async {
    return await _channel
        .invokeMethod('requestCallCenter', {'p_channel': pChannel});
  }

  static Future<String> requestEmailCallCenter(int pChannel) async {
    return await _channel
        .invokeMethod('requestEmailCallCenter', {'p_channel': pChannel});
  }

  static Future<String> acceptRequestCallCenter(
      String lPin, String channel) async {
    return await _channel.invokeMethod(
        'acceptRequestCallCenter', {'l_pin': lPin, 'channel': channel});
  }

  static Future<String> endRequestCallCenter(
      String lPin, String complaintId) async {
    return await _channel.invokeMethod(
        'endRequestCallCenter', {'l_pin': lPin, 'complaint_id': complaintId});
  }

  static Future<String> getFeatureAccess(String key) async {
    return await _channel.invokeMethod('getFeatureAccess', {'key': key});
  }

  static Future<String> broadcastMessage(
      String title,
      String broadcastFlag,
      String message,
      int startingDate,
      int endingDate,
      String destination,
      String data,
      String categoryFlag,
      String thumbId,
      String fileId,
      String notificationType,
      String link) async {
    return await _channel.invokeMethod('broadcastMessage', {
      'title': title,
      'broadcast_flag': broadcastFlag,
      'message': message,
      'starting_date': startingDate,
      'ending_date': endingDate,
      'destination': destination,
      'data': data,
      'category_flag': categoryFlag,
      'notification_type': notificationType,
      'link': link,
      'thumb_id': thumbId,
      'file_id': fileId
    });
  }

  static Future<String> timeOutRequestCallCenter(
      String lPin, String channel) async {
    return await _channel.invokeMethod(
        'timeOutRequestCallCenter', {'l_pin': lPin, 'channel': channel});
  }

  static Future<String> managementContactCenter(
      String lPin, String userType) async {
    return await _channel.invokeMethod(
        'managementContactCenter', {'l_pin': lPin, 'user_type': userType});
  }

  static Future<String> signInCreator(String pName, String pPassword) async {
    return await _channel.invokeMethod(
        'signInCreator', {'p_name': pName, 'p_password': pPassword});
  }

  static Future<String> signInAdmin(String pName, String pPassword) async {
    return await _channel.invokeMethod(
        'signInAdmin', {'p_name': pName, 'p_password': pPassword});
  }

  static Future<String> signInInternal(String pName, String pPassword) async {
    return await _channel.invokeMethod(
        'signInInternal', {'p_name': pName, 'p_password': pPassword});
  }

  static Future<String> changePasswordAdmin(
      String pFPin, String pwdEn, String pwdOld) async {
    return await _channel.invokeMethod('changePasswordAdmin',
        {'p_f_pin': pFPin, 'pwd_en': pwdEn, 'pwd_old': pwdOld});
  }

  static Future<String> changePasswordInternal(
      String pFPin, String pwdEn, String pwdOld) async {
    return await _channel.invokeMethod('changePasswordInternal',
        {'p_f_pin': pFPin, 'pwd_en': pwdEn, 'pwd_old': pwdOld});
  }

  static Future<String> queuingCallCenter(int pChannel) async {
    return await _channel
        .invokeMethod('queuingCallCenter', {'p_channel': pChannel});
  }

  static Future<String> statusContactCenter(String pComplaintId) async {
    return await _channel
        .invokeMethod('statusContactCenter', {'p_complaint_id': pComplaintId});
  }

  static Future<String> setLiveCenterSession(String f_pin, String status,
      {String complaintId = ""}) async {
    return await _channel.invokeMethod('setLiveCenterSession',
        {'f_pin': f_pin, 'status': status, 'complaint_id': complaintId});
  }

  static Future<String> setCallCenterDateStart(
      String f_pin, String date) async {
    return await _channel
        .invokeMethod('setCallCenterDateStart', {'f_pin': f_pin, 'date': date});
  }

  static Future<String> setCallCenterData(String f_pin, String data) async {
    return await _channel
        .invokeMethod('setCallCenterData', {'f_pin': f_pin, 'data': data});
  }

  static Future<String> setCCSessionWelcome(String f_pin) async {
    return await _channel.invokeMethod('setCallCenterData', {'f_pin': f_pin});
  }
}

enum CameraInfo { CAMERA_FACING_FRONT, CAMERA_FACING_BACK }
