import 'dart:collection';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_translate/flutter_translate.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ChangeState with ChangeNotifier {
  List<Map<dynamic, dynamic>> _dataContact = [], _srcContact = [];

  List<Map<dynamic, dynamic>> get getDataContact => _dataContact;

  List<Map<dynamic, dynamic>> get getSrcContact => _srcContact;

  set dataContact(Map<dynamic, dynamic> value) {
    _dataContact.add(value);
    notifyListeners();
  }

  set srcContact(List<Map<dynamic, dynamic>> value) {
    _srcContact.addAll(value);
    notifyListeners();
  }

  set addSrcContact(Map<dynamic, dynamic> value) {
    _srcContact.add(value);
    notifyListeners();
  }

  set clearSrcContact(bool value) {
    _srcContact.clear();
    notifyListeners();
  }
}

class AddCall extends StatelessWidget {
  final LinkedHashSet<String> destinationList;

  AddCall(this.destinationList);

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
        create: (_) => ChangeState(),
        child: Consumer<ChangeState>(
            builder: (context, changeState, _) =>
                AddCalls(changeState, destinationList)));
  }
}

class AddCalls extends StatefulWidget {
  final LinkedHashSet<String> destinationList;
  final ChangeState changeState;
  bool isCall = false;

  AddCalls(this.changeState, this.destinationList);

  _AddCalls createState() => _AddCalls();
}

class _AddCalls extends State<AddCalls> {
  TextEditingController _editingController = new TextEditingController();

  _searchPerson(name) async {
    if (name.toString().isNotEmpty) {
      widget.changeState.clearSrcContact = true;
      for (int i = 0; i < widget.changeState.getDataContact.length; i++) {
        if (widget.changeState.getDataContact[i]["name"]
            .toString()
            .toLowerCase()
            .contains(name.toString().toLowerCase())) {
          widget.changeState.addSrcContact =
              widget.changeState.getDataContact[i];
        }
      }
    } else {
      widget.changeState.clearSrcContact = true;
      widget.changeState.srcContact = widget.changeState.getDataContact;
    }
  }

  @override
  void initState() {
    getData();
    super.initState();
  }

  void getData() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String idMe = prefs.getString("hasLogin")!;
    List<dynamic> resultQuery = await DBHelper().queryCustom(
        "SELECT f_pin, first_name,official_account, image_id, device_id, offline_mode FROM BUDDY where official_account<>'1' and f_pin <> '$idMe'");
//    List<dynamic> resultQuery = dummyAccounts;
    for (var i = 0; i < resultQuery.length; i++) {
      if (!widget.destinationList.contains(resultQuery[i]["f_pin"])) {
        widget.changeState.dataContact = {
          "f_pin": resultQuery[i]["f_pin"],
          "name": resultQuery[i]["first_name"],
          "picture": resultQuery[i]["image_id"],
          "isVerified": resultQuery[i]["official_account"] == "2" ? "1" : "0",
          "isOfficial": resultQuery[i]["official_account"] == "1" ? "1" : "0",
          "deviceId": resultQuery[i]["device_id"],
          "isOffline": resultQuery[i]["offline_mode"]
        };
      }
    }
    widget.changeState.srcContact = widget.changeState.getDataContact;
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
          automaticallyImplyLeading: true,
          backgroundColor: AppColor.primaryColor,
          title: Text("Add Call")),
      body: Container(
        child: Column(
          children: <Widget>[
            Container(
              decoration: BoxDecoration(
                  color: Colors.grey[100],
                  border: Border(
                      bottom: BorderSide(color: Colors.grey[200]!, width: 2.0))),
              padding: EdgeInsets.all(2.0),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: <Widget>[
                  Container(
                    width: MediaQuery.of(context).size.width * 0.98,
                    child: Theme(
                      child: TextField(
                          controller: _editingController,
                          onChanged: (val) async {
                            await _searchPerson(val);
                          },
                          cursorColor: AppColor.primaryColor,
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: Colors.white,
                              contentPadding: EdgeInsets.all(5.0),
                              prefixIcon: Icon(Icons.search),
                              focusedBorder: OutlineInputBorder(
                                  borderRadius: BorderRadius.circular(3.0),
                                  borderSide: BorderSide(color: Colors.grey)),
                              border: OutlineInputBorder(
                                  borderRadius: BorderRadius.circular(3.0),
                                  borderSide: BorderSide(color: Colors.grey)))),
                      data:
                          Theme.of(context).copyWith(primaryColor: Colors.grey),
                    ),
                  )
                ],
              ),
            ),
            Flexible(
              child: widget.changeState.getSrcContact.length == 0 &&
                      _editingController.text.trim().isNotEmpty
                  ? Container(
                      alignment: Alignment.topCenter,
                      margin: EdgeInsets.only(top: 15.0),
                      child: Text(
                        "Search not found",
                        textAlign: TextAlign.center,
                        style: TextStyle(fontSize: 12),
                      ))
                  : ListView.builder(
                      itemCount: widget.changeState.getSrcContact.length,
                      itemBuilder: (builder, index) {
                        return GestureDetector(
                          onTap: () {
                            if (widget.changeState
                                .getSrcContact[index]["isOffline"] == "1") {
                              Fluttertoast.showToast(
                                  msg: "Unable to call because user disable calls");
                            }
                            else {
                              Navigator.pop(context, [
                                widget.changeState.getSrcContact[index]["name"],
                                widget.changeState.getSrcContact[index]["f_pin"]
                              ]);
                            }
                          },
                          child: ListTile(
                            leading: Container(
                              width: 45,
                              height: 45,
                              decoration: BoxDecoration(
                                  shape: BoxShape.circle,
                                  image: DecorationImage(
                                      fit: BoxFit.cover,
                                      // image: Image.asset(
                                      //   "assets/images/ic_person_boy.png",
                                      // ).image
                                      image: LocalImage(widget.changeState
                                          .getSrcContact[index]["picture"]))),
                            ),
                            title: Row(
                              children: <Widget>[
                                widget.changeState.getSrcContact[index]
                                            ["isVerified"] ==
                                        "1"
                                    ? Image.asset(
                                        "assets/images/ic_verified_flag.png",
                                        width: 15,
                                        height: 15, package: "palio_lite"
                                      )
                                    : widget.changeState.getSrcContact[index]
                                                ["isOfficial"] ==
                                            "1"
                                        ? Image.asset(
                                            "assets/images/ic_official_flag.png",
                                            width: 15,
                                            height: 15, package: "palio_lite"
                                          )
                                        : Container(),
                                Padding(
                                  padding: EdgeInsets.only(
                                    left: 8.0,
                                  ),
                                  child: Text(
                                    widget.changeState.getSrcContact[index]
                                        ["name"],
                                  ),
                                )
                              ],
                            ),
                          ),
                        );
                      }),
            )
          ],
        ),
      ),
    );
  }
}
