import 'dart:async';
import 'dart:collection';
import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_translate/flutter_translate.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/call/add_call.dart';
import 'package:palio_lite/call/widgets/timer_controller.dart';
import 'package:palio_lite/call/widgets/zoom.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../shared_obj.dart';

class AudioCall extends StatefulWidget {
  final int isInitiatior;
  String name;
  final String image;
  final String deviceId;
  final bool isContactCenter;

  AudioCall(
      {required this.isInitiatior,
      required this.name,
      required this.image,
      required this.deviceId,
      this.isContactCenter = false});

  _AudioCall createState() => _AudioCall();
}

class _AudioCall extends State<AudioCall> {
  static int? isInitiator;
  Widget _zoomView = Zoom();
  bool muted = false,
      speaker = false,
      accept = false,
      end = false,
      busy = false;
  bool vibrate = false;
  int status = 0, screenSharingStatus = 0;
  List<dynamic> personData = [];
  double animatedWidth = 120.0;
  double opac = 1.0;
  String textHistory = "";
  static const MAX_DEST = 3;

  LinkedHashSet<String> destinationList = LinkedHashSet();
  LinkedHashSet<String> nameList = LinkedHashSet();
  LinkedHashSet<String> imageList = LinkedHashSet();
  var _audioCallIndex = <int>[];

  Timer? _timer;

  String title = "";
  String deviceid = "";
  var controller = TimerController();

  _getPersonData() async {
    personData = await DBHelper().queryCustom(
        "SELECT f_pin, first_name, last_name, image_id,connected, offline_mode FROM BUDDY");
  }

  Future<String> getFPIN() async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.getString('hasLogin') ?? "";
  }

  void initVibrate(bool vibrates) {
    _timer = new Timer.periodic(Duration(seconds: 1), (Timer timer) async {
      if (vibrates) {
        API.vibrate();
      }
    });
  }

  Future<bool> getVibration() async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    vibrate = prefs.getBool("vibrate") ?? false;
    return vibrate;
  }

  void statusCall() {
    API.statusCall = (int state, String message) async {
      // state
      // 21 incoming
      // 22 connected
      // terminate

      if (mounted) {
        print('state: $state, message: $message');

        switch (state) {
          case 21:
            if (destinationList.length >= MAX_DEST) {
              print("You can't add more call");
            }
            break;

          case 22:
            if (destinationList.length < MAX_DEST) {
              String deviceId = message.split(",")[1];
              for (int i = 0; i < personData.length; i++) {
                if (personData[i]['f_pin'] == deviceId &&
                    !destinationList.contains(personData[i]['f_pin'])) {
                  destinationList.add(personData[i]['f_pin']);
                  imageList.add(personData[i]["image_id"]);
                  nameList.add(personData[i]["first_name"]);
                }
              }
            }
            title = "Connected";
            controller.start();

            setState(() {
              accept = true;
            });
            break;

          case 28:
          case -4:
            print("Destination Length : " + nameList.length.toString());
            String deviceId = message.split(",")[0];
            print("Device id : " + deviceId);
            String attachmentFlag;
            if (textHistory == null && state == 28) {
              textHistory = "Incoming VoIP Call";
              attachmentFlag = "12";
            } else if (textHistory == null && state == -4) {
              textHistory = "Missed VoIP Call";
              attachmentFlag = "19";
            } else {
              attachmentFlag = "19";
            }
            String myFpin = await getFPIN();
            var result;
            if (deviceId != myFpin && widget.isInitiatior == 1) {
              result = await API.sendMessage(
                  lPin: deviceId,
                  messageText: textHistory,
                  attachmentFlag: attachmentFlag,
                  messageScopeId: "3",
                  status: "2");
            }
            if (destinationList.length >= 2) {
              for (int i = 0; i < personData.length; i++) {
                if (personData[i]['f_pin'] == deviceId) {
                  destinationList.remove(personData[i]["f_pin"]);
                  imageList.remove(personData[i]["image_id"]);
                  nameList.remove(personData[i]["first_name"]);
                }
              }
              setState(() {});
            } else if ((destinationList.length == 1 &&
                    destinationList.contains(deviceId)) ||
                destinationList.length == 0) {
              SharedObj.stopRingtone();
              controller.stop();
              title = "End Calling";
              end = true;
              if (state == -4) busy = true;
              setState(() {});
              Future.delayed(Duration(seconds: 1), () {
                if (mounted) {
                  if (_timer != null) {
                    _timer?.cancel();
                  }
                  // API.leaveScreenSharing();
                  API.terminateCall(deviceId);
                  Navigator.of(context).pop({
                    "messageId": result,
                    "textHistory": "$textHistory",
                    "attachmentFlag": "$attachmentFlag"
                  });
                }
              });
            }

            break;
        }
      }
    };
    API.startSS = (int state, String message) {
      if (mounted) {
        print('ss start state: $state, message: $message');
        switch (state) {
          case 1:
            setState(() {});
            break;
        }
      }
    };
    API.joinSS = (int state, String message) {
      if (mounted) {
        print('ss join state: $state, message: $message');
        switch (state) {
          case 1:
            if (message.isEmpty) {
              print("init ss initiator");
              // API.initScreenSharing(widget.deviceId);
            }
            print("init ss join");
            if (screenSharingStatus != 3) {
              screenSharingStatus = 1;
              setState(() {});
            }
            break;
          case 88:
            screenSharingStatus = 0;
            setState(() {});
            // API.leaveScreenSharing();
            break;
        }
      }
    };
  }

  @override
  void dispose() {
    SharedObj.stopRingtone();
    SharedObj.clearNotificationCall();
    controller.dispose();
    nameList.clear();
    imageList.clear();
    deviceid = "";
    if (_timer != null) {
      _timer?.cancel();
    }
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    _getPersonData();
    deviceid = widget.deviceId;
    bool offline = false;
    for (var i = 0; i < personData.length; i++) {
      if (personData[i]["f_pin"] == deviceid) {
        offline = personData[i]["offline_mode"] == '1';
        break;
      }
    }
    // offline = true;
    if (offline) {
      Fluttertoast.showToast(msg: "Unable to call because user disable calls");
      Navigator.pop(context);
    } else {
      getVibration();
      setState(() {});
//    _getPersonData();
      statusCall();
      isInitiator = widget.isInitiatior;
      if (isInitiator == 1) {
        API.initiateAudioCall(widget.deviceId);
      } else {
        getVibration().then((value) {
          if (value != null) {
            if (value) {
              initVibrate(value);
            }
          }
        });
      }

      controller.addListener(() {
        setState(() {
          title = controller.value;
        });
      });

      title = "VoIP Call";
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        backgroundColor: AppColor.primaryColor,
        automaticallyImplyLeading: false,
        title: Text(
          title,
          style: TextStyle(color: Colors.white),
        ),
        centerTitle: true,
        elevation: 0.0,
      ),
      body: Center(
          child: Stack(
        children: <Widget>[
          Container(
            height: MediaQuery.of(context).size.height * 0.75,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                _profile(),
                Visibility(
                  visible: accept && !end,
                  child: _toolbar2(),
                ),
                _Views()
              ],
            ),
          ),
          // _poweredBy(),
          bottom(),
          // _screenShareView(screenSharingStatus == 2 ? true : false),
          // Visibility(
          //   child: _screenShareView(screenSharingStatus == 2 ? true ? false),
          //   visible: (screenSharingStatus == 2),
          // ),
          // Visibility(
          //   child: _toolbarSS(),
          //   visible: (screenSharingStatus == 2),
          // ),
          Align(
            alignment: Alignment.center,
            child: _status(),
          ),
        ],
      )),
    );
  }

  Widget _remoteOtherView(String name, String image) {
    return GestureDetector(
      child: Container(
        width: 100,
        height: 100,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          children: <Widget>[
            Container(
                width: 80,
                height: 80,
//              color: Colors.grey,
                decoration: BoxDecoration(
                    image: DecorationImage(
                        image: image.trim().isEmpty
                            ? AssetImage("assets/images/default_pp.png",
                                package: "palio_lite")
                            : LocalImage(image) as ImageProvider))),
            Container(
              padding: EdgeInsets.only(top: 5),
              alignment: Alignment.bottomCenter,
              child: Visibility(
                visible: true,
                child: Text(
                  "$name",
                  style: TextStyle(color: Colors.blueAccent, fontSize: 12),
                ),
              ),
            ),
          ],
        ),
//        child: Stack(
//          children: <Widget>[
//            Container(
//              padding: EdgeInsets.only(top: 5),
//              alignment: Alignment.bottomCenter,
//              child: Visibility(
//                visible: true,
//                child: Text(
//                  "$name",
//                  style: TextStyle(color: Colors.blueAccent, fontSize: 10),
//                ),
//              ),
//            ),
//            //          Remote()
//            Container(
//                decoration: BoxDecoration(
//                    image: DecorationImage(
//                        image: Image.asset("assets/images/ic_person_girl.png")
//                            .image)))
//          ],
//        ),
      ),
      onTap: () {
        widget.name = name;
        setState(() {});
      },
    );
  }

  Widget _Views() {
    return Padding(
      padding: EdgeInsets.only(right: 5.0),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[
          Expanded(
            child: SingleChildScrollView(
              physics: nameList.length >= 3
                  ? ClampingScrollPhysics()
                  : NeverScrollableScrollPhysics(),
              scrollDirection: Axis.horizontal,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: <Widget>[
                  Padding(
                      padding: EdgeInsets.only(top: 3.0),
                      child: Visibility(
                        visible: destinationList.length >= 2,
                        child: _remoteOtherView(
                            nameList.length >= 2 ? nameList.elementAt(1) : "",
                            imageList.length >= 2
                                ? imageList.elementAt(1)
                                : ""),
                      )),
                  Padding(
                      padding: EdgeInsets.only(top: 3.0),
                      child: Visibility(
                        visible: destinationList.length >= 3,
                        child: _remoteOtherView(
                          nameList.length >= 3 ? nameList.elementAt(2) : "",
                          imageList.length >= 3 ? imageList.elementAt(2) : "",
                        ),
                      )),
                  // Padding(
                  //     padding: EdgeInsets.only(top: 3.0),
                  //     child: Visibility(
                  //       visible: destinationList.length >= 4,
                  //       child: _remoteOtherView(
                  //         nameList.length >= 4 ? nameList.elementAt(3) : "",
                  //         imageList.length >= 4 ? imageList.elementAt(3) : "",
                  //       ),
                  //     )),
                  // Padding(
                  //     padding: EdgeInsets.only(top: 3.0),
                  //     child: Visibility(
                  //       visible: destinationList.length >= 5,
                  //       child: _remoteOtherView(
                  //         nameList.length >= 5 ? nameList.elementAt(4) : "",
                  //         imageList.length >= 5 ? imageList.elementAt(4) : "",
                  //       ),
                  //     )),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }

  Widget _toolbar2() {
    return Container(
      height: 160,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Container(
            width: MediaQuery.of(context).size.width,
            height: 80,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: <Widget>[
                GestureDetector(
                  onTap: () {
                    if (mounted) {
                      setState(() {
                        speaker = !speaker;
                        print("Speakerphone " + speaker.toString());
                      });
                    }
                    API.setSpeaker(speaker);
                  },
                  child: Container(
                    width: MediaQuery.of(context).size.width / 3,
                    decoration: BoxDecoration(
                      border: Border(
                        top: BorderSide(color: Colors.grey, width: 1.0),
                        bottom: BorderSide(color: Colors.grey, width: 1.0),
                      ),
                    ),
                    alignment: Alignment.center,
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: <Widget>[
                        Container(
                          child: Icon(
                            Icons.volume_mute,
                            color: speaker ? Colors.green : Colors.grey,
                            size: 30,
                          ),
                        ),
                        Text(
                          "Speaker",
                          style:
                              TextStyle(color: Colors.blueAccent, fontSize: 12),
                        ),
                      ],
                    ),
                  ),
                ),
                GestureDetector(
                  onTap: () async {
                    if (destinationList.length < 16 &&
                        !widget.isContactCenter) {
                      final dest = await Navigator.push(
                          context,
                          PageTransition(
                              type: PageTransitionType.fade,
                              child: AddCall(destinationList)));
                      if (dest != null) {
                        List<dynamic> res = await DBHelper().queryCustom(
                            "SELECT offline_mode FROM BUDDY where f_pin = '${dest[1]}'");
                        if (res[0]["offline_mode"] == "1") {
                          Fluttertoast.showToast(
                              msg: "Unable to call because user disable calls");
                        } else {
                          API.addAudioCall(dest[1]);
                        }
                      }

                      setState(() {});
                    }
                  },
                  child: Container(
                    width: MediaQuery.of(context).size.width / 3,
                    decoration: BoxDecoration(
                      border: !widget.isContactCenter
                          ? Border.all(color: Colors.grey, width: 1.0)
                          : Border(left: BorderSide(width: 1.0)),
                    ),
                    alignment: Alignment.center,
                    child: widget.isContactCenter
                        ? Container()
                        : Column(
                            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: <Widget>[
                              Container(
                                child: Icon(
                                  Icons.call,
                                  color: Colors.grey,
                                  size: 30,
                                ),
                              ),
                              Text(
                                "Add Call",
                                style: TextStyle(
                                    color: Colors.blueAccent, fontSize: 12),
                              ),
                            ],
                          ),
                  ),
                ),
                GestureDetector(
                  onTap: () {
                    // if (mounted) {
                    //   setState(() {
                    //     muted = !muted;
                    //     print("Mute Microphone " + muted.toString());
                    //   });
                    // }
                    // API.muteMicrophone(muted);
                  },
                  child: Container(
                    width: MediaQuery.of(context).size.width / 3,
                    // decoration: BoxDecoration(
                    //   border: Border(
                    //     top: BorderSide(color: Colors.grey, width: 1.0),
                    //     bottom: BorderSide(color: Colors.grey, width: 1.0),
                    //   ),
                    // ),
                    alignment: Alignment.center,
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: <Widget>[
                        // Container(
                        //   child: Icon(
                        //     muted ? Icons.mic_off : Icons.mic,
                        //     color: Colors.grey,
                        //     size: 30,
                        //   ),
                        // ),
                        // Text(
                        //   "Mute",
                        //   style:
                        //   TextStyle(color: Colors.blueAccent, fontSize: 12),
                        // ),
                      ],
                    ),
                  ),
                ),
              ],
            ),
          ),
          // Container(
          //   width: MediaQuery.of(context).size.width,
          //   height: 80,
          //   child: Row(
          //     mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          //     children: <Widget>[
          //       GestureDetector(
          //         onTap: () {
          //           if (mounted) {
          //             if (screenSharingStatus == 0) {
          //               print(widget.deviceId + " " + widget.name);
          //               API.startScreenSharing(widget.deviceId);
          //               screenSharingStatus = 3;
          //               print("start screen sharing");
          //             } else if (screenSharingStatus == 1) {
          //               print(widget.deviceId + " " + widget.name);
          //               API.joinScreenSharing(widget.deviceId);
          //               screenSharingStatus = 2;
          //               print("join screen sharing");
          //             } else if (screenSharingStatus == 3) {
          //               print(widget.deviceId + " " + widget.name);
          //               API.endScreenSharing(widget.deviceId);
          //               screenSharingStatus = 0;
          //               print("end screen sharing");
          //             }
          //             setState(() {});
          //           }
          //         },
          //         child: Container(
          //           width: MediaQuery.of(context).size.width / 3,
          //           decoration: BoxDecoration(
          //             border: Border(
          //               right: BorderSide(color: Colors.grey, width: 1.0),
          //               bottom: BorderSide(color: Colors.grey, width: 1.0),
          //             ),
          //           ),
          //           alignment: Alignment.center,
          //           child: Column(
          //             mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          //             children: <Widget>[
          //               Container(
          //                 child: Icon(
          //                   (screenSharingStatus == 3)
          //                       ? Icons.stop
          //                       : Icons.screen_share,
          //                   color: (screenSharingStatus == 1)
          //                       ? Colors.amber
          //                       : (screenSharingStatus == 3)
          //                           ? Colors.red
          //                           : Colors.grey,
          //                   size: 30,
          //                 ),
          //               ),
          //               Text(
          //                 "Screen Sharing",
          //                 style:
          //                     TextStyle(color: Colors.blueAccent, fontSize: 12),
          //               ),
          //             ],
          //           ),
          //         ),
          //       ),
          //       Container(
          //         width: MediaQuery.of(context).size.width / 3,
          //         alignment: Alignment.center,
          //       ),
          //       Container(
          //         width: MediaQuery.of(context).size.width / 3,
          //         alignment: Alignment.center,
          //       ),
          //     ],
          //   ),
          // ),
        ],
      ),
    );
  }

  Widget _profile() {
    return Center(
      child: Column(
        children: <Widget>[
          Container(
              height: 120,
              width: 120,
              decoration: BoxDecoration(
                  shape: BoxShape.circle,
                  image: DecorationImage(
                      fit: BoxFit.cover,
                      image: widget.image.trim().isEmpty
                          ? AssetImage("assets/images/default_pp.png",
                              package: "palio_lite")
                          : LocalImage(widget.image) as ImageProvider))),
          SizedBox(
            height: 20,
          ),
          Text(
            widget.name,
            style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold),
          )
        ],
      ),
    );
  }

  Widget _toolbar() {
    if (isInitiator == 1) {
      return _toolbarOnGoing();
    } else if (accept) {
      return _toolbarOnGoing();
    } else {
      return _toolbarReceive();
    }
  }

  // Widget _screenShareView(bool isSS) {
  //   return Align(
  //     alignment: Alignment.center,
  //     child: Container(
  //       color: isSS ? Colors.black : Colors.transparent,
  //       width: isSS ? MediaQuery.of(context).size.width : 1.0,
  //       height: isSS ? MediaQuery.of(context).size.height : 1.0,
  //       child: Zoom(),
  //     ),
  //   );
  // }
  //
  // Widget _toolbarSS() {
  //   return Container(
  //       alignment: Alignment.bottomCenter,
  //       padding: EdgeInsets.symmetric(vertical: 18),
  //       child: Row(
  //         mainAxisAlignment: MainAxisAlignment.center,
  //         children: <Widget>[
  //           RawMaterialButton(
  //             onPressed: () {
  //               print("Terminate call receive : " + widget.deviceId);
  //               // end = true;
  //               API.leaveScreenSharing();
  //               screenSharingStatus = 0;
  //               setState(() {});
  //               // API.terminateCall(deviceid);
  //               // Navigator.of(context).pop();
  //             },
  //             child: new Icon(
  //               Icons.stop,
  //               color: Colors.white,
  //               size: 20.0,
  //             ),
  //             shape: new CircleBorder(),
  //             elevation: 2.0,
  //             fillColor: Colors.red,
  //             padding: const EdgeInsets.all(12.0),
  //           ),
  //         ],
  //       ));
  // }

  Widget _toolbarOnGoing() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        RawMaterialButton(
          onPressed: () {
            showDialog(
                context: context,
                builder: (context) {
                  return CupertinoAlertDialog(
                      insetAnimationCurve: Curves.easeInBack,
                      title: Text("Are you sure to end this call?"),
                      actions: <Widget>[
                        CupertinoDialogAction(
                          onPressed: () {
                            Navigator.pop(context);
                          },
                          child: Text("Cancel",
                              style: TextStyle(
                                  color: AppColor.primaryColor, fontSize: 20)),
                        ),
                        CupertinoDialogAction(
                          onPressed: () {
                            Navigator.pop(context);
                            print(
                                "Terminate call receive : " + widget.deviceId);
                            end = true;
                            setState(() {});
                            if (screenSharingStatus == 2) {
                              // API.endScreenSharing(deviceid);
                            } else {
                              // API.leaveScreenSharing();
                            }
                            API.terminateCall(deviceid);
                            Navigator.pop(context);
                          },
                          child: Text("Yes",
                              style: TextStyle(
                                  color: AppColor.primaryColor, fontSize: 20)),
                        )
                      ]);
                });
          },
          child: new Icon(
            Icons.call_end,
            color: Colors.white,
            size: 20.0,
          ),
          shape: new CircleBorder(),
          elevation: 2.0,
          fillColor: Colors.red,
          padding: const EdgeInsets.all(12.0),
        ),
      ],
    );
  }

  Widget _status() {
    return Visibility(
      visible: !accept || end || busy,
      child: Align(
        alignment: Alignment.center,
        child: Container(
          padding: EdgeInsets.all(5),
          color: Colors.black26,
          child: Text(
            (isInitiator == 1 && !accept && !end)
                ? "Waiting for answer"
                : (isInitiator == 0 && !accept && !end)
                    ? "Incoming Audio Call"
                    : (end && busy)
                        ? "Busy"
                        : end
                            ? "Audio Call Finished"
                            : "",
            style: TextStyle(color: Colors.white, fontSize: 13),
          ),
        ),
      ),
    );
  }

  Widget bottom() {
    return Container(
      alignment: Alignment.bottomCenter,
      padding: EdgeInsets.symmetric(vertical: 18),
      // child: _toolbar(),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          _toolbar(),
          // Align(
          //   alignment: Alignment.bottomRight,
          //   child: _poweredBy(),
          // ),
        ],
      ),
    );
  }

  Widget _poweredBy() {
    return Container(
      alignment: Alignment.bottomRight,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.end,
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          Container(
            height: 25,
            alignment: Alignment.centerRight,
            child: Text(
              translate("poweredByPalio"),
              style: TextStyle(fontSize: 15),
            ),
          ),
          SizedBox(
            width: 5,
          ),
          Container(
            width: 25,
            height: 25,
//              color: Colors.grey,
            decoration: BoxDecoration(
                image: DecorationImage(
                    image: AssetImage("assets/images/palio.png",
                        package: "palio_lite"))),
          )
        ],
      ),
    );
  }

  Widget _toolbarReceive() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
      children: <Widget>[
        RawMaterialButton(
          onPressed: () {
            showDialog(
                context: context,
                builder: (context) {
                  return CupertinoAlertDialog(
                      insetAnimationCurve: Curves.easeInBack,
                      title: Text("Are you sure to end this call?"),
                      actions: <Widget>[
                        CupertinoDialogAction(
                          onPressed: () {
                            Navigator.pop(context);
                          },
                          child: Text("Cancel",
                              style: TextStyle(
                                  color: AppColor.primaryColor, fontSize: 20)),
                        ),
                        CupertinoDialogAction(
                          onPressed: () {
                            SharedObj.stopRingtone();
                            Navigator.pop(context);
                            print(
                                "Terminate call receive : " + widget.deviceId);
                            end = true;
                            if (_timer != null) {
                              _timer?.cancel();
                            }
                            setState(() {});
                            API.terminateCall(deviceid);
                            Navigator.pop(context);
                          },
                          child: Text("Yes",
                              style: TextStyle(
                                  color: AppColor.primaryColor, fontSize: 20)),
                        )
                      ]);
                });
          },
          child: new Icon(
            Icons.call_end,
            color: Colors.white,
            size: 20.0,
          ),
          shape: new CircleBorder(),
          elevation: 2.0,
          fillColor: Colors.red,
          padding: const EdgeInsets.all(12.0),
        ),
        RawMaterialButton(
          onPressed: () {
            setState(() {
              SharedObj.stopRingtone();
              if (_timer != null) {
                _timer?.cancel();
              }
              API.receiveAudioCall(widget.deviceId);
              accept = true;
            });
          },
          child: new Icon(
            Icons.call,
            color: Colors.white,
            size: 20.0,
          ),
          shape: new CircleBorder(),
          elevation: 2.0,
          fillColor: Colors.blueAccent,
          padding: const EdgeInsets.all(12.0),
        )
      ],
    );
  }
}
