import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/call/audio_call.dart';
import 'package:palio_lite/call/video_call.dart';
import 'package:palio_lite/chat/add_friend_screen.dart';
import 'package:palio_lite/chat/pa_profile.dart';

//import 'package:un_ios_app/Screen/video_call_new.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/conference/add_vidconf_room.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:palio_lite/palio_button.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ChangeState with ChangeNotifier {
  List<Map<dynamic, dynamic>> _dataContact = [], _srcContact = [];

  List<Map<dynamic, dynamic>> get getDataContact => _dataContact;

  List<Map<dynamic, dynamic>> get getSrcContact => _srcContact;

  set dataContact(Map<dynamic, dynamic> value) {
    _dataContact.add(value);
    notifyListeners();
  }

  set srcContact(List<Map<dynamic, dynamic>> value) {
    _srcContact.addAll(value);
    notifyListeners();
  }

  set addSrcContact(Map<dynamic, dynamic> value) {
    _srcContact.add(value);
    notifyListeners();
  }

  set clearSrcContact(bool value) {
    _srcContact.clear();
    notifyListeners();
  }

  set clearDataContact(bool value) {
    _dataContact.clear();
    notifyListeners();
  }
}

class NewCall extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
        create: (_) => ChangeState(),
        child: Consumer<ChangeState>(
            builder: (context, changeState, _) => NewCalls(changeState)));
  }
}

class NewCalls extends StatefulWidget {
  final ChangeState changeState;

  NewCalls(this.changeState);

  _NewCalls createState() => _NewCalls();
}

class _NewCalls extends State<NewCalls> {
  TextEditingController _editingController = new TextEditingController();
  TextEditingController _groupController = new TextEditingController();

  _searchPerson(name) async {
    if (name.toString().isNotEmpty) {
      widget.changeState.clearSrcContact = true;
      for (int i = 0; i < widget.changeState.getDataContact.length; i++) {
        if (widget.changeState.getDataContact[i]["name"]
            .toString()
            .toLowerCase()
            .contains(name.toString().toLowerCase())) {
          widget.changeState.addSrcContact =
              widget.changeState.getDataContact[i];
        }
      }
    } else {
      widget.changeState.clearSrcContact = true;
      widget.changeState.srcContact = widget.changeState.getDataContact;
    }
  }

  String idMe = "";

  Future<String> getLogin() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("hasLogin")) {
      idMe = prefs.getString("hasLogin") ?? "";
      API.pullBatchBuddyList(idMe);
      // _timerRefreshBuddy = Timer(Duration(seconds: 2), () {
      //   getData();
      //   if (_editingController.text.isNotEmpty) {
      //     _searchPerson(_editingController.text);
      //   }
      //   API.pullBatchBuddyList(idMe);
      // });
    }
    return idMe;
  }

  @override
  void initState() {
    getLogin();
    getData();
    super.initState();
  }

  void getData() async {
    if (widget.changeState.getDataContact.length > 0) {
      widget.changeState.clearDataContact = true;
      widget.changeState.clearSrcContact = true;
    }
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    String idMe = prefs.getString("hasLogin")!;
    List<dynamic> resultQuery = await DBHelper().queryCustom(
        "SELECT f_pin, first_name,official_account, image_id, device_id, offline_mode, user_type FROM BUDDY where official_account<>'1' and f_pin <> '$idMe' and f_pin <> '02121368af'");
//    List<dynamic> resultQuery = dummyAccounts;
    for (var i = 0; i < resultQuery.length; i++) {
      if (idMe.isNotEmpty && idMe != resultQuery[i]["f_pin"]) {
        widget.changeState.dataContact = {
          "f_pin": resultQuery[i]["f_pin"],
          "name": resultQuery[i]["first_name"],
          "picture": resultQuery[i]["image_id"],
          "isVerified": resultQuery[i]["official_account"] == "2" ? "1" : "0",
          "isOfficial": resultQuery[i]["official_account"] == "1" ? "1" : "0",
          "deviceId": resultQuery[i]["device_id"],
          "isOffline": resultQuery[i]["offline_mode"],
          "user_type": resultQuery[i]["user_type"]
        };
      }
    }
    widget.changeState.srcContact = widget.changeState.getDataContact;
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        automaticallyImplyLeading: true,
        backgroundColor: AppColor.primaryColor,
        title: Text("Calling"),
        iconTheme: IconThemeData(color: Colors.white),
        actions: [
          IconButton(
            icon: ImageIcon(
                AssetImage("assets/images/ic_vroom.png", package: "palio_lite"),
                color: Colors.white),
            onPressed: () async {
              Navigator.push(
                  context,
                  PageTransition(
                      type: PageTransitionType.fade,
                      child: AddConferenceRoom()));
            },
          ),
          PopupMenuButton(
            icon: Icon(CupertinoIcons.add),
            itemBuilder: (context) => [
              PopupMenuItem(
                  child: ListTile(
                leading: Icon(Icons.group_add, color: Colors.black),
                title: Text("Create Group"),
                contentPadding: EdgeInsets.all(0.0),
                dense: true,
                onTap: () {
                  Navigator.pop(context);
                  showDialog(
                      context: context,
                      builder: (_) {
                        return CupertinoAlertDialog(
                          title: Row(
                            mainAxisAlignment: MainAxisAlignment.start,
                            children: <Widget>[
                              Image.asset(
                                "assets/images/groupicon.png",
                                width: 25,
                                height: 25,
                                package: "palio_lite",
                              ),
                              Text("Create Group",
                                  style:
                                      TextStyle(color: AppColor.primaryColor))
                            ],
                          ),
                          content: Material(
                            color: Colors.transparent,
                            child: TextField(
                              cursorColor: AppColor.primaryColor,
                              controller: _groupController,
                              decoration: InputDecoration(
                                  fillColor: Colors.transparent,
                                  focusedBorder: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFF364656))),
                                  border: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFF364656))),
                                  hintText: "Set Group Name",
                                  filled: true),
                            ),
                          ),
                          actions: <Widget>[
                            FlatButton(
                              child: Text(
                                "Cancel",
                                style: TextStyle(color: AppColor.primaryColor),
                              ),
                              onPressed: () {
                                Navigator.of(context).pop();
                              },
                            ),
                            FlatButton(
                              child: Text(
                                "CREATE GROUP",
                                style: TextStyle(color: AppColor.primaryColor),
                              ),
                              onPressed: () async {
                                Navigator.pop(context);
                                String result = await API
                                    .createGroup(_groupController.text);
                                if (result != "00") {
                                  Fluttertoast.showToast(
                                      msg: "CREATE GROUP FAILED",
                                      toastLength: Toast.LENGTH_SHORT);
                                }
                              },
                            )
                          ],
                        );
                      });
                },
              )),
              PopupMenuItem(
                  child: ListTile(
                leading:
                    Icon(CupertinoIcons.person_add_solid, color: Colors.black),
                title: Text("Add Friend"),
                contentPadding: EdgeInsets.all(0.0),
                dense: true,
                onTap: () async {
                  Navigator.pop(context);
                  Navigator.pushReplacement(
                      context,
                      PageTransition(
                          type: PageTransitionType.fade,
                          child: AddFriendScreen(afterLogin: true)));
                },
              )),
              PopupMenuItem(
                  child: ListTile(
                leading: Icon(CupertinoIcons.mail_solid, color: Colors.black),
                title: Text("Email Configuration"),
                contentPadding: EdgeInsets.all(0.0),
                dense: true,
              ))
            ],
          ),
        ],
      ),
      body: Container(
        child: Column(
          children: <Widget>[
            Container(
              decoration: BoxDecoration(
                  color: Colors.grey[100],
                  border: Border(
                      bottom:
                          BorderSide(color: Colors.grey[200]!, width: 2.0))),
              padding: EdgeInsets.all(2.0),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: <Widget>[
                  Container(
                    width: MediaQuery.of(context).size.width * 0.98,
                    child: Theme(
                      child: TextField(
                          controller: _editingController,
                          onChanged: (val) async {
                            await _searchPerson(val);
                          },
                          cursorColor: AppColor.primaryColor,
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: Colors.white,
                              contentPadding: EdgeInsets.all(5.0),
                              prefixIcon: Icon(Icons.search),
                              focusedBorder: OutlineInputBorder(
                                  borderRadius: BorderRadius.circular(3.0),
                                  borderSide: BorderSide(color: Colors.grey)),
                              border: OutlineInputBorder(
                                  borderRadius: BorderRadius.circular(3.0),
                                  borderSide: BorderSide(color: Colors.grey)))),
                      data:
                          Theme.of(context).copyWith(primaryColor: Colors.grey),
                    ),
                  )
                ],
              ),
            ),
            Flexible(
              child: ListView.builder(
                  itemCount: widget.changeState.getSrcContact.length,
                  itemBuilder: (builder, index) {
                    return ListTile(
                      onTap: () async {
                        List<dynamic> resultQuery = await DBHelper().queryCustom(
                            "SELECT * FROM BUDDY where f_pin='${widget.changeState.getSrcContact[index]["f_pin"]}'");
                        Navigator.pushReplacement(
                            context,
                            PageTransition(
                                type: PageTransitionType.fade,
                                child: PAProfile(
                                  profileDet: {
                                    'f_pin': widget.changeState
                                        .getSrcContact[index]["f_pin"],
                                    'myProfile': false,
                                    'name': resultQuery[0]["first_name"],
                                    'follower': 0,
                                    'official_flag': resultQuery[0]
                                                ["official_account"] ==
                                            "1"
                                        ? 1
                                        : resultQuery[0]["official_account"] ==
                                                "2"
                                            ? 0
                                            : 2,
                                    'isFriend': 1,
                                    'post': 0,
                                    'following': 0,
                                    'isOffline': resultQuery[0]["offline_mode"],
                                    'isMuted': resultQuery[0]["muted"],
                                    'industry': resultQuery[0]
                                        ["bidang_industri"],
                                    'email_address': resultQuery[0]["email"],
                                    'phone_number': resultQuery[0]["msisdn"],
                                    'website': resultQuery[0]["web"],
                                    'address': resultQuery[0]["address"],
                                    'status': resultQuery[0]["quote"],
                                    'isOnline':
                                        int.parse(resultQuery[0]["connected"]),
                                    'image': resultQuery[0]["image_id"],
                                    'official_account': resultQuery[0]
                                        ["official_account"],
                                    'user_type': resultQuery[0]["user_type"]
                                  },
                                )));
                      },
                      leading: Container(
                        width: 45,
                        height: 45,
                        decoration: BoxDecoration(
                            shape: BoxShape.circle,
                            color: Colors.grey,
                            image: DecorationImage(
                                fit: BoxFit.cover,
                                image: widget.changeState
                                        .getSrcContact[index]["picture"]
                                        .trim()
                                        .isEmpty
                                    ? AssetImage("assets/images/default_pp.png",
                                        package: "palio_lite")
                                    : LocalImage(
                                        widget.changeState.getSrcContact[index]
                                            ["picture"]) as ImageProvider)),
                      ),
                      title: Row(
                        children: <Widget>[
                          widget.changeState.getSrcContact[index]
                                      ["isVerified"] ==
                                  "1"
                              ? Image.asset(
                                  "assets/images/ic_verified_flag.png",
                                  width: 15,
                                  height: 15,
                                  package: "palio_lite")
                              : widget.changeState.getSrcContact[index]
                                          ["isOfficial"] ==
                                      "1"
                                  ? Image.asset(
                                      "assets/images/ic_official_flag.png",
                                      width: 15,
                                      height: 15,
                                      package: "palio_lite")
                                  : widget.changeState.getSrcContact[index]
                                              ["user_type"] ==
                                          "23"
                                      ? Image.asset(
                                          "assets/images/ic_internal.png",
                                          width: 15,
                                          height: 15,
                                          package: "palio_lite")
                                      : widget.changeState.getSrcContact[index]
                                                  ["user_type"] ==
                                              "24"
                                          ? Image.asset(
                                              "assets/images/call_center.png",
                                              width: 15,
                                              height: 15,
                                              package: "palio_lite")
                                          : Container(),
                          Padding(
                            padding: EdgeInsets.only(
                              left: 8.0,
                            ),
                            child: Text(
                              widget.changeState.getSrcContact[index]["name"],
                            ),
                          )
                        ],
                      ),
                      trailing: Container(
                        width: 96,
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceAround,
                          children: <Widget>[
                            IconButton(
                              icon: Icon(Icons.call,
                                  color: AppColor.primaryColor),
                              onPressed: () async {
                                var resultQueryBuddy = await DBHelper().queryCustom(
                                    "SELECT ex_block FROM BUDDY WHERE f_pin=='${widget.changeState.getSrcContact[index]["f_pin"]}'");
                                if (widget.changeState.getSrcContact[index]
                                        ["isOffline"] ==
                                    "1") {
                                  Fluttertoast.showToast(
                                      msg:
                                          "Unable to call because user disable calls");
                                } else if (resultQueryBuddy[0]["ex_block"] ==
                                        "1" ||
                                    resultQueryBuddy[0]["ex_block"] == "-1") {
                                  if (resultQueryBuddy[0]["ex_block"] == "-1") {
                                    Fluttertoast.showToast(
                                        msg: "You are blocked by this user");
                                  } else {
                                    Fluttertoast.showToast(
                                        msg: "You block this user");
                                  }
                                } else {
                                  await Navigator.push(
                                      context,
                                      PageTransition(
                                          type: PageTransitionType.fade,
                                          child: AudioCall(
                                            isInitiatior: 1,
                                            deviceId: widget.changeState
                                                .getSrcContact[index]["f_pin"],
                                            name: widget.changeState
                                                .getSrcContact[index]["name"],
                                            image: widget.changeState
                                                    .getSrcContact[index]
                                                ["picture"],
                                          )));
                                  PalioButtonState statePalio =
                                      PalioButtonState();
                                  statePalio.incomingCall(context);
                                }
                              },
                            ),
                            IconButton(
                              icon: Icon(Icons.videocam,
                                  color: AppColor.primaryColor),
                              onPressed: () async {
                                var resultQueryBuddy = await DBHelper().queryCustom(
                                    "SELECT ex_block FROM BUDDY WHERE f_pin=='${widget.changeState.getSrcContact[index]["f_pin"]}'");
                                if (widget.changeState.getSrcContact[index]
                                        ["isOffline"] ==
                                    "1") {
                                  Fluttertoast.showToast(
                                      msg:
                                          "Unable to call because user disable calls");
                                } else if (resultQueryBuddy[0]["ex_block"] ==
                                        "1" ||
                                    resultQueryBuddy[0]["ex_block"] == "-1") {
                                  if (resultQueryBuddy[0]["ex_block"] == "-1") {
                                    Fluttertoast.showToast(
                                        msg: "You are blocked by this user");
                                  } else {
                                    Fluttertoast.showToast(
                                        msg: "You block this user");
                                  }
                                } else {
                                  await Navigator.push(
                                      context,
                                      PageTransition(
                                          type: PageTransitionType.fade,
                                          child: VideoCall(
                                            isInitiatior: 1,
                                            deviceId: widget.changeState
                                                .getSrcContact[index]["f_pin"],
                                            cameraId: "1",
                                            name: widget.changeState
                                                .getSrcContact[index]["name"],
                                            image: widget.changeState
                                                    .getSrcContact[index]
                                                ["picture"],
                                          )));
                                  PalioButtonState statePalio =
                                      PalioButtonState();
                                  statePalio.incomingCall(context);
                                }
                              },
                            ),
                          ],
                        ),
                      ),
                    );
                  }),
            )
          ],
        ),
      ),
    );
  }
}
