import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';

class ChangeState with ChangeNotifier {
  bool _recommendFriend = true, _loading = true;
  int _exits = 0;
  List<dynamic> _listContact = [];
  List<dynamic> _temporaryContact = [];
  Map<int, String> _chooseFriend = Map<int, String>();

  bool get recommendFriend => _recommendFriend;

  bool get loading => _loading;

  int get exits => _exits;

  Map<int, dynamic> get getChooseFriend => _chooseFriend;

  List<dynamic> get getListContact => _listContact;

  List<dynamic> get getTemporaryContact => _temporaryContact;

  set recommendFriend(bool value) {
    _recommendFriend = value;
    notifyListeners();
  }

  set loading(bool value) {
    _loading = value;
    notifyListeners();
  }

  set exits(int value) {
    _exits = value;
    notifyListeners();
  }

  set chooseFriend(var value) {
    _chooseFriend[value["index"]] = value["data"];
    notifyListeners();
  }

  set removeChooseFriend(int value) {
    _chooseFriend.remove(value);
  }

  set clearChooseFriend(int value) {
    _chooseFriend.clear();
  }

  set temporaryContact(var value) {
    _temporaryContact.add(value);
    notifyListeners();
  }

  set listContact(var value) {
    _listContact.addAll(value);
    notifyListeners();
  }

  set removeListContact(int value) {
    _listContact.removeAt(value);
    notifyListeners();
  }

  set clearListContact(var value) {
    _listContact.clear();
    notifyListeners();
  }

  set updateListContact(var value) {
    _listContact[value["index"]]["check"] = value["check"];
    notifyListeners();
  }
}

class AddFriendScreen extends StatelessWidget {
  final bool afterLogin;

  AddFriendScreen({this.afterLogin = false});

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
        create: (_) => ChangeState(),
        child: Consumer<ChangeState>(
            builder: (context, changeState, _) =>
                AddFriendScreens(afterLogin, changeState)));
  }
}

class AddFriendScreens extends StatefulWidget {
  final bool afterLogin;
  final ChangeState changeState;

  AddFriendScreens(this.afterLogin, this.changeState);

  @override
  _AddFriendScreensState createState() => _AddFriendScreensState();
}

class _AddFriendScreensState extends State<AddFriendScreens> {
  TextEditingController _editingController = TextEditingController();
  bool scan = false;
  String scanResult = '';
  String idMe = "";

  Future<bool> _onWillPop() async {
    if (widget.changeState.exits == 1) {
      exit(0);
    }
    Fluttertoast.showToast(
        msg: "Press again to exit app", toastLength: Toast.LENGTH_LONG);
    widget.changeState.exits = 1;
    Timer(Duration(seconds: 4), () {
      widget.changeState.exits = 0;
    });
    return Future.value(false);
  }

  Future<String> getLogin() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("hasLogin")) {
      idMe = prefs.getString("hasLogin")!;
    }
    return idMe;
  }

  @override
  void initState() {
    getLogin();
    getPeople(0);
    _editingController.addListener(textListener);
    super.initState();
  }

  void textListener() {
    if (_editingController.text.trim().isEmpty) {
      widget.changeState.clearListContact = true;
      widget.changeState.listContact = widget.changeState.getTemporaryContact;
    }
  }

  void getPeople(int offset) async {
    String resultQuerySuggestion = await API.suggestionPerson("0");
    List<dynamic> listSuggestion = jsonDecode(resultQuerySuggestion);

    for (var i = 0; i < listSuggestion.length; i++) {
      widget.changeState.temporaryContact = {
        "f_pin": listSuggestion[i]["A00"],
        "name": listSuggestion[i]["A02"],
        "image": listSuggestion[i]["A74"],
        "check": false
      };
    }
    widget.changeState.listContact = widget.changeState.getTemporaryContact;
    widget.changeState.loading = false;
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        onWillPop:
            widget.afterLogin == false ? _onWillPop : () => Future.value(true),
        child: Scaffold(
          backgroundColor: Colors.white,
          appBar: AppBar(
              automaticallyImplyLeading:
                  widget.afterLogin == false ? false : true,
              backgroundColor: AppColor.primaryColor,
              title: Text(
                "Add Friends",
                style: TextStyle(color: Colors.white),
              )),
          body: Stack(
            children: <Widget>[
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisAlignment: MainAxisAlignment.start,
                children: <Widget>[
                  Visibility(
                    visible: scan,
                    child: Container(
                      height: MediaQuery.of(context).size.height * 0.4,
                      width: MediaQuery.of(context).size.width,
//                      child: QRView(
//                        onQRViewCreated: (controller) {
//                          controller.scannedDataStream.listen((event) {
//                            setState(() {
//                              scanResult = event;
//                              print("result "+event);
//                            });
//                          });
//                        },
//                      ),
                    ),
                  ),
                  Container(
                    decoration: BoxDecoration(
                        color: Colors.grey[100],
                        border: Border(
                            bottom: BorderSide(
                                color: Colors.grey[200]!, width: 2.0))),
                    padding: EdgeInsets.all(10.0),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceAround,
                      children: <Widget>[
                        Container(
                          width: MediaQuery.of(context).size.width * 0.8,
                          child: Theme(
                            child: TextField(
                                controller: _editingController,
                                onSubmitted: (text) async {
                                  if (text.trim().isNotEmpty) {
                                    widget.changeState.clearChooseFriend = 1;
                                    var resultSearch =
                                        await API.searchFriend(text, "40");
                                    List<dynamic> listSuggestion =
                                        jsonDecode(resultSearch);
                                    List<dynamic> temporaryContact = [];
                                    for (var i = 0;
                                        i < listSuggestion.length;
                                        i++) {
                                      if (idMe.isNotEmpty &&
                                          idMe != listSuggestion[i]["A00"]) {
                                        temporaryContact.add({
                                          "f_pin": listSuggestion[i]["A00"],
                                          "name": listSuggestion[i]["A02"],
                                          "image": listSuggestion[i]["A74"],
                                          "check": false
                                        });
                                      }
                                    }
                                    widget.changeState.clearListContact = true;
                                    widget.changeState.listContact =
                                        temporaryContact;
                                  }
                                },
                                cursorColor: AppColor.primaryColor,
                                decoration: InputDecoration(
                                    filled: true,
                                    fillColor: Colors.white,
                                    hintText: "Input your Friend's PIN or Name",
                                    contentPadding: EdgeInsets.all(5.0),
                                    prefixIcon: Icon(Icons.search),
                                    focusedBorder: OutlineInputBorder(
                                        borderRadius:
                                            BorderRadius.circular(3.0),
                                        borderSide:
                                            BorderSide(color: Colors.grey)),
                                    border: OutlineInputBorder(
                                        borderRadius:
                                            BorderRadius.circular(3.0),
                                        borderSide:
                                            BorderSide(color: Colors.grey)))),
                            data: Theme.of(context)
                                .copyWith(primaryColor: Colors.grey),
                          ),
                        ),
                        InkWell(
                          onTap: () {
                            if (mounted) {
                              setState(() {
                                print("change state scan");
                                scan = !scan;
                              });
                            }
                          },
                          child: Icon(
                            Icons.filter_center_focus,
                            size: 40,
                            color: Colors.black,
                          ),
                        )
                      ],
                    ),
                  ),
                  widget.changeState.loading
                      ? CupertinoActivityIndicator()
                      : widget.changeState.getListContact.length == 0
                          ? Container(
                              margin: EdgeInsets.only(left: 10.0, top: 10.0),
                              child: Text(
                                  "'${_editingController.text}' " + "Not Found",
                                  style: TextStyle(
                                      color: Colors.grey, fontSize: 16)),
                            )
                          : Flexible(
                              child: SingleChildScrollView(
                              child: Container(
                                padding: EdgeInsets.only(
                                    left: 10.0, right: 10.0, top: 15.0),
                                child: Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: <Widget>[
                                    widget.changeState.recommendFriend
                                        ? Container(
                                            decoration: BoxDecoration(
                                                border: Border.all(
                                                    width: 1.0,
                                                    color: Colors.grey[300]!),
                                                borderRadius:
                                                    BorderRadius.circular(
                                                        20.0)),
                                            padding: EdgeInsets.only(
                                                left: 15.0,
                                                top: 20.0,
                                                right: 15.0,
                                                bottom: 20.0),
                                            margin: EdgeInsets.all(5.0),
                                            child: Column(
                                              crossAxisAlignment:
                                                  CrossAxisAlignment.start,
                                              children: <Widget>[
                                                Text(
                                                  "Do you want to get recommendations from the contact list?",
                                                  style: TextStyle(
                                                      color: Colors.grey,
                                                      fontSize: 18,
                                                      fontWeight:
                                                          FontWeight.w300),
                                                ),
                                                SizedBox(height: 5.0),
                                                Container(
                                                    margin: EdgeInsets.only(
                                                        right: 20.0),
                                                    alignment:
                                                        Alignment.centerRight,
                                                    child: InkWell(
                                                      onTap: () {
                                                        widget.changeState
                                                                .recommendFriend =
                                                            !widget.changeState
                                                                .recommendFriend;
                                                      },
                                                      child: Text(
                                                        "Yes",
                                                        style: TextStyle(
                                                            color: Colors.grey,
                                                            fontSize: 15),
                                                      ),
                                                    ))
                                              ],
                                            ),
                                          )
                                        : Container(),
                                    Padding(
                                      padding: EdgeInsets.only(
                                          top:
                                              widget.changeState.recommendFriend
                                                  ? 10.0
                                                  : 5.0,
                                          bottom: 10.0),
                                      child: Text("People you may know",
                                          style: TextStyle(
                                              fontSize: 17,
                                              color: Colors.grey[600])),
                                    ),
                                    ListView.builder(
                                        itemCount: widget
                                            .changeState.getListContact.length,
                                        shrinkWrap: true,
                                        physics: NeverScrollableScrollPhysics(),
                                        itemBuilder: (_, i) {
                                          return Column(
                                            crossAxisAlignment:
                                                CrossAxisAlignment.start,
                                            children: <Widget>[
                                              FlatButton(
                                                child: Row(
                                                  mainAxisAlignment:
                                                      MainAxisAlignment
                                                          .spaceBetween,
                                                  children: <Widget>[
                                                    Row(
                                                      mainAxisAlignment:
                                                          MainAxisAlignment
                                                              .start,
                                                      crossAxisAlignment:
                                                          CrossAxisAlignment
                                                              .center,
                                                      children: <Widget>[
                                                        Container(
                                                          decoration: BoxDecoration(
                                                              color: Color(
                                                                  0xFFababab),
                                                              image: DecorationImage(
                                                                  image: LocalImage(widget
                                                                          .changeState
                                                                          .getListContact[i]
                                                                      [
                                                                      "image"]),
                                                                  fit: BoxFit
                                                                      .cover)),
                                                          height: 100,
                                                          width: 130,
                                                        ),
                                                        SizedBox(width: 10.0),
                                                        Text(
                                                          widget.changeState
                                                                  .getListContact[
                                                              i]["name"],
                                                          style: TextStyle(
                                                              fontWeight:
                                                                  FontWeight
                                                                      .w400),
                                                        )
                                                      ],
                                                    ),
                                                    Checkbox(
                                                      value: widget.changeState
                                                              .getListContact[i]
                                                          ["check"],
                                                      onChanged: (change) {
                                                        if (change!) {
                                                          widget.changeState
                                                              .chooseFriend = {
                                                            "index": i,
                                                            "data": widget
                                                                    .changeState
                                                                    .getListContact[
                                                                i]["f_pin"]
                                                          };
                                                        } else {
                                                          widget.changeState
                                                              .removeChooseFriend = i;
                                                        }
                                                        widget.changeState
                                                            .updateListContact = {
                                                          "index": i,
                                                          "check": change
                                                        };
                                                      },
                                                      activeColor:
                                                          AppColor.primaryColor,
                                                    )
                                                  ],
                                                ),
                                                onPressed: () {
                                                  if (widget.changeState
                                                          .getListContact[i]
                                                      ["check"]) {
                                                    widget.changeState
                                                        .updateListContact = {
                                                      "index": i,
                                                      "check": false
                                                    };
                                                    widget.changeState
                                                        .removeChooseFriend = i;
                                                  } else {
                                                    widget.changeState
                                                        .updateListContact = {
                                                      "index": i,
                                                      "check": true
                                                    };
                                                    widget.changeState
                                                        .chooseFriend = {
                                                      "index": i,
                                                      "data": widget.changeState
                                                              .getListContact[i]
                                                          ["f_pin"]
                                                    };
                                                  }
                                                },
                                                padding: EdgeInsets.all(0.0),
                                              ),
                                              SizedBox(
                                                height: 5.0,
                                              ),
                                              Divider(
                                                  color: Colors.grey,
                                                  height: 1.0),
                                              SizedBox(
                                                height: 5.0,
                                              )
                                            ],
                                          );
                                        })
                                  ],
                                ),
                              ),
                            ))
                ],
              ),
              widget.changeState.getChooseFriend.length > 0
                  ? Align(
                      alignment: Alignment.bottomCenter,
                      child: Material(
                        color: Colors.transparent,
                        child: Container(
                          alignment: Alignment.centerRight,
                          width: double.infinity,
                          padding: EdgeInsets.all(10.0),
                          height: 40,
                          color: Colors.black45,
                          child: InkWell(
                            onTap: () {
                              showDialog(
                                  context: context,
                                  builder: (_) {
                                    return CupertinoAlertDialog(
                                      title: Text("Add Friends",
                                          style: TextStyle(
                                              color: AppColor.primaryColor)),
                                      content: Material(
                                        color: Colors.transparent,
                                        child: Text(
                                            "Do you want to make selected users as friends",
                                            style: TextStyle(
                                                color: AppColor.primaryColor)),
                                      ),
                                      actions: <Widget>[
                                        FlatButton(
                                          child: Text(
                                            "Cancel",
                                            style: TextStyle(
                                                color: AppColor.primaryColor),
                                          ),
                                          onPressed: () {
                                            Navigator.of(context).pop();
                                          },
                                        ),
                                        FlatButton(
                                          child: Text(
                                            "NEXT",
                                            style: TextStyle(
                                                color: AppColor.primaryColor),
                                          ),
                                          onPressed: () async {
                                            // get list from db
                                            // List<dynamic> resultQueryFriend = await DBHelper().queryCustom(
                                            //     "SELECT f_pin, first_name,last_name,official_account, image_id FROM BUDDY where type <> '1' and f_pin <> '$idMe'");
                                            for (var i = 0;
                                                i <
                                                    widget.changeState
                                                        .getChooseFriend.length;
                                                i++) {
                                              // check if friend exists in list
                                              var resultAdd =
                                                  await API.addFriend(widget
                                                      .changeState
                                                      .getChooseFriend
                                                      .values
                                                      .elementAt(i));
                                              if (resultAdd == "00" ||
                                                  !widget.afterLogin) {
                                                Navigator.pop(context);
                                                Fluttertoast.showToast(
                                                    msg:
                                                        "New friends added successfully",
                                                    toastLength:
                                                        Toast.LENGTH_LONG);
                                                widget.changeState
                                                        .removeListContact =
                                                    widget.changeState
                                                        .getChooseFriend.keys
                                                        .elementAt(i);
                                                widget.changeState
                                                    .clearChooseFriend = 1;
                                              }
                                            }
                                          },
                                        )
                                      ],
                                    );
                                  });
                            },
                            child: Text(
                              "CHOOSE",
                              style: TextStyle(color: Colors.white),
                            ),
                          ),
                        ),
                      ),
                    )
                  : Container()
            ],
          ),
        ));
  }
}
