import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:file_picker/file_picker.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:fluttertoast/fluttertoast.dart';

// import 'package:geolocator/geolocator.dart';
// import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:image/image.dart' as image;
import 'package:image_picker/image_picker.dart';
import 'package:intl/intl.dart';
import 'package:open_file/open_file.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/chat/group_detail.dart';
import 'package:palio_lite/chat/pa_profile.dart';
import 'package:palio_lite/chat/widgets/buble_chat_group.dart';
import 'package:palio_lite/chat/widgets/custom_scroll_to_index.dart';
import 'package:palio_lite/chat/widgets/forward_chat.dart';
import 'package:palio_lite/chat/widgets/link_previewer.dart';
import 'package:palio_lite/chat/widgets/message_info.dart';
import 'package:palio_lite/chat/widgets/my_text_selection.dart';
import 'package:palio_lite/chat/widgets/notifications.dart';
import 'package:palio_lite/chat/widgets/rich_text_field.dart';
import 'package:palio_lite/chat/widgets/show_date.dart';
import 'package:palio_lite/chat/widgets/tap_to_bottom.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/chat/widgets/widget_sticker.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:palio_lite/palio_button.dart';
import 'package:path/path.dart' as path;
import 'package:path_provider/path_provider.dart';
import 'package:photo_view/photo_view.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:video_compress/video_compress.dart';
import 'package:video_thumbnail/video_thumbnail.dart';

class ChatGroupPage extends StatelessWidget {
  final String nameGroup;
  final String nameTopic;
  final String image;
  final String idGroup;
  final String identifier;
  final bool refresh;

  ChatGroupPage(this.nameGroup, this.nameTopic, this.image,
      {required this.idGroup, required this.identifier, this.refresh = false});

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: [
        ChangeNotifierProvider(create: (_) => ListMessage()),
        ChangeNotifierProvider(create: (_) => SelectedChat()),
        ChangeNotifierProvider(create: (_) => Attachment()),
      ],
      child: Consumer<SelectedChat>(
        builder: (context, selectChat, _) => Consumer<Attachment>(
          builder: (context, attachment, _) => Consumer<ListMessage>(
              builder: (context, select, _) => ChatGroupPages(
                    idGroup,
                    nameGroup,
                    nameTopic,
                    image,
                    selectChat,
                    attachment,
                    select,
                    identifier: identifier,
                    refresh: refresh,
                  )),
        ),
      ),
    );
  }
}

class ChatGroupPages extends StatefulWidget {
  final String idGroup;
  final String nameGroup;
  final String nameTopic;
  final String image;
  final SelectedChat selectChat;
  final Attachment attachment;
  final ListMessage select;
  final String identifier;
  final bool refresh;

  ChatGroupPages(this.idGroup, this.nameGroup, this.nameTopic, this.image,
      this.selectChat, this.attachment, this.select,
      {required this.identifier, required this.refresh});

  @override
  _ChatGroupPagesState createState() => _ChatGroupPagesState();
}

class _ChatGroupPagesState extends State<ChatGroupPages> {
  AutoScrollController _scrollController =
      AutoScrollController(initialScrollOffset: 0.0);
  TextEditingController _controllerEditor = TextEditingController();
  TextEditingController _controllerSearch = TextEditingController();
  Timer? _timer2, _timerSendTyping, _timerReceiveTyping, _timerLink;
  final MyTextSelectionControls _myExtendedMaterialTextSelectionControls =
      MyTextSelectionControls();

  String language = "";

  Future<String> getUserName(String f_pin) async {
    String name = "";
    var resultQueryBuddy = await DBHelper().queryCustom(
        "SELECT first_name, last_name FROM BUDDY WHERE f_pin = '$f_pin'");
    print("name " + resultQueryBuddy[0]['first_name']);
    print("last name " + resultQueryBuddy[0]['last_name']);
    name = resultQueryBuddy[0]['first_name'] +
        " " +
        resultQueryBuddy[0]['last_name'];
    return name;
  }

  Future<String> getLanguage() async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    language = prefs.getString("language") ?? "";
    return language;
  }

  String toNormalString(String stupidString) {
    String result = stupidString.replaceAll("+", " ");
    return result;
  }

  Future<bool> getAutoDownload() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("autoDownload")) {
      return prefs.getBool("autoDownload") ?? false;
    }
    return false;
  }

  Widget _textComposerWidget(Attachment attachment) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Container(
          width: MediaQuery.of(context).size.width > 400
              ? MediaQuery.of(context).size.width * 0.85
              : MediaQuery.of(context).size.width * 0.83,
          decoration: BoxDecoration(
              color: Colors.white, borderRadius: BorderRadius.circular(8.0)),
          margin: EdgeInsets.only(left: 5.0, top: 5.0, bottom: 5.0),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              attachment.getReplyContent != null
                  ? Stack(
                      children: <Widget>[
                        Container(
                          color: Color(0xFFECF0F1),
                          constraints:
                              BoxConstraints(minHeight: 30, maxHeight: 80),
                          margin: EdgeInsets.all(5.0),
                          padding: EdgeInsets.all(8.0),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: <Widget>[
                              Expanded(
                                child: Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  mainAxisSize: MainAxisSize.min,
                                  children: <Widget>[
                                    Text(attachment.getReplyContent["fromName"],
                                        style: TextStyle(
                                            color: Colors.black54,
                                            fontWeight: FontWeight.bold)),
                                    Wrap(
                                      children: <Widget>[
                                        Text(
                                          attachment.getReplyContent[
                                                      "attachment_type"] ==
                                                  "11"
                                              ? "Attachment sticker"
                                              : attachment.getReplyContent[
                                                              "attachment_type"] ==
                                                          "1" &&
                                                      attachment
                                                          .getReplyContent[
                                                              "text"]
                                                          .isEmpty
                                                  ? 'Attaching picture'
                                                  : attachment.getReplyContent[
                                                                  "attachment_type"] ==
                                                              "2" &&
                                                          attachment
                                                              .getReplyContent[
                                                                  "text"]
                                                              .isEmpty
                                                      ? 'Attaching video'
                                                      : attachment.getReplyContent[
                                                                      "attachment_type"] ==
                                                                  "6" &&
                                                              attachment
                                                                  .getReplyContent[
                                                                      "text"]
                                                                  .isEmpty
                                                          ? "Attachment document"
                                                          : attachment.getReplyContent["attachment_type"] ==
                                                                      "-1" &&
                                                                  attachment
                                                                      .getReplyContent[
                                                                          "text"]
                                                                      .isEmpty
                                                              ? "Attachment audio"
                                                              : attachment.getReplyContent["attachment_type"] == "3" &&
                                                                      attachment
                                                                          .getReplyContent["text"]
                                                                          .isEmpty
                                                                  ? "Attachment location"
                                                                  : attachment.getReplyContent["text"],
                                          maxLines: 2,
                                          overflow: TextOverflow.ellipsis,
                                          style:
                                              TextStyle(color: Colors.black54),
                                        )
                                      ],
                                    )
                                  ],
                                ),
                              ),
                              attachment.getReplyContent["attachment_type"] != "0" &&
                                      attachment.getReplyContent["attachment_type"] !=
                                          "11"
                                  ? Container(
                                      margin: EdgeInsets.only(left: 10.0),
                                      width: 64,
                                      height: 64,
                                      decoration: BoxDecoration(
                                          image: attachment.getReplyContent["attachment_type"] == "-1" ||
                                                  attachment.getReplyContent[
                                                          "attachment_type"] ==
                                                      "6"
                                              ? null
                                              : DecorationImage(
                                                  fit: BoxFit.cover,
                                                  image: attachment.getReplyContent["attachment_type"] == "3"
                                                      ? NetworkImage(
                                                          attachment.getReplyContent[
                                                              "thumb_location"])
                                                      : FileImage(File(attachment.getReplyContent["thumb_id"]))
                                                          as ImageProvider)),
                                      child: attachment.getReplyContent["attachment_type"] == "-1"
                                          ? Icon(
                                              Icons.volume_up,
                                              color: Colors.grey,
                                              size: 60,
                                            )
                                          : attachment.getReplyContent["attachment_type"] == "6"
                                              ? Icon(Icons.insert_drive_file, color: Colors.green, size: 60)
                                              : Container())
                                  : attachment.getReplyContent["attachment_type"] == "11"
                                      ? Container(
                                          margin: EdgeInsets.only(left: 10.0),
                                          width: 64,
                                          height: 64,
                                          child: Image.asset(
                                              "assets/stickers/${attachment.getReplyContent["text"]}",
                                              package: "palio_lite"),
                                        )
                                      : Container(height: 0.0, width: 0.0)
                            ],
                          ),
                        ),
                        Align(
                          alignment: Alignment.topRight,
                          child: Container(
                            margin: EdgeInsets.only(top: 8, right: 8),
                            child: Material(
                              color: Colors.transparent,
                              child: InkWell(
                                onTap: () {
                                  widget.attachment.replyContent = null;
                                },
                                child: Icon(Icons.cancel, color: Colors.white),
                              ),
                            ),
                          ),
                        )
                      ],
                    )
                  : Container(),
              RichExtendedTextField(
                isRTF: true,
                minLines: 1,
                maxLines: 6,
                selectionControls: _myExtendedMaterialTextSelectionControls,
                cursorColor: AppColor.primaryColor,
                controller: _controllerEditor,
                onTap: () {
                  if (widget.attachment.clickedSticker) {
                    widget.attachment.clickedSticker = false;
                  }
                },
                onChanged: (text) {
                  textListener();
                  checkMention(text);
                },
                decoration: InputDecoration(
                    filled: true,
                    fillColor: Colors.white,
                    contentPadding: EdgeInsets.all(20),
                    hintText: 'Type Message',
                    border: InputBorder.none),
              )
            ],
          ),
        ),
        Container(
          margin: EdgeInsets.only(left: 5.0, right: 5.0, top: 10.0),
          decoration: BoxDecoration(
              color: AppColor.primaryColor, shape: BoxShape.circle),
          width: 45,
          height: 45,
          child: Material(
            color: Colors.transparent,
            child: InkWell(
              onTap: () async {
                SharedPreferences prefs = await SharedPreferences.getInstance();
                String idMe = prefs.getString("hasLogin")!;
                var temp_text = _controllerEditor.text;
                if (temp_text.contains("@")) {
                  for (int i = 0;
                      i < widget.attachment.getListMembers.length;
                      i++) {
                    if (temp_text.contains(
                        "@${widget.attachment.getListMembers.values.elementAt(i)} ")) {
                      temp_text = temp_text.replaceAll(
                          "@${widget.attachment.getListMembers.values.elementAt(i)} ",
                          "${widget.attachment.getListMembers.keys.elementAt(i)} ");
                    }
                  }
                }
                if (attachment.getListImage.length > 0) {
                  for (var j = 0; j < attachment.getListImage.length; j++) {
                    attachment.getListImage[j].copy(
                        "${attachment.path.path}/${path.basename(attachment.getListImage[j].path)}");
                    final realImage = image.decodeImage(
                        attachment.getListImage[j].readAsBytesSync());
                    final thumbImage =
                        image.copyResize(realImage!, width: 480, height: 640);
                    File newThumb =
                        new File('${attachment.path.path}/thumbnail.png')
                          ..writeAsBytesSync(image.encodePng(thumbImage));
                    newThumb.copy(
                        "${attachment.path.path}/THUMB_${path.basename(attachment.getListImage[j].path)}");
                    var resultSend = await API.sendMessage(
                        lPin: widget.idGroup,
                        chatId: widget.identifier,
                        messageScopeId: "4",
                        status: "2",
                        messageText: j == 0 ? temp_text : "",
                        imageId: path.basename(attachment.getListImage[j].path),
                        reffId: attachment.getReplyContent != null
                            ? attachment.getReplyContent["message_id"]
                            : "",
                        thumbId: "THUMB_" +
                            path.basename(attachment.getListImage[j].path),
                        readReceipts: attachment.getAckActive ? "8" : "",
                        attachmentFlag: "1");
                    var formatTime = new DateFormat('Hm');
                    String? refId,
                        refName,
                        refText,
                        refType,
                        refThumbId,
                        refThumbLoc;
                    if (attachment.getReplyContent != null) {
                      refId = attachment.getReplyContent["message_id"];
                      refName = attachment.getReplyContent["fromName"];
                      refText = attachment.getReplyContent["attachment_type"] ==
                                  "3" ||
                              attachment.getReplyContent["attachment_type"] ==
                                  "-1"
                          ? ""
                          : attachment.getReplyContent["text"];
                      refType = attachment.getReplyContent["attachment_type"];
                      refThumbId = attachment.getReplyContent["thumb_id"];
                      refThumbLoc =
                          attachment.getReplyContent["thumb_location"];
                    }
                    var resultQueryBuddy = await DBHelper().queryCustom(
                        "SELECT first_name, last_name, image_id FROM BUDDY WHERE f_pin=='$idMe'");
                    widget.select.addChat = Chat(
                        messageId: resultSend,
                        isOriginator: true,
                        text: temp_text,
                        statusRead: "2",
                        fileName:
                            path.basename(attachment.getListImage[j].path),
                        thumbName: attachment.getListImage[j].path,
                        refFrom: refName,
                        refType: refType,
                        fName: resultQueryBuddy[0]["first_name"] +
                            " " +
                            resultQueryBuddy[0]["last_name"],
                        fImage: resultQueryBuddy[0]["image_id"],
                        refText: refText,
                        refThumbLoc: refThumbLoc,
                        refId: refId,
                        refThumbId: refThumbId,
                        attachmentType: "1",
                        ack: widget.attachment.getAckActive ? "8" : "",
                        time: formatTime.format(DateTime.now()),
                        date: DateTime.now());
                  }
                  _controllerEditor.clear();
                  attachment.clearListImage = true;
                  widget.attachment.refresh = true;
                  attachment.replyContent = null;
                  Future.delayed(Duration(milliseconds: 50), () {
                    _scrollController.animateTo(0.0,
                        duration: Duration(milliseconds: 1),
                        curve: Curves.easeOut);
                  });
                } else if (attachment.getVideo != null) {
                  attachment.getVideo.copy(
                      "${attachment.path.path}/${path.basename(attachment.getVideo.path)}");
                  var resultSend = await API.sendMessage(
                      lPin: widget.idGroup,
                      chatId: widget.identifier,
                      messageScopeId: "4",
                      status: "2",
                      reffId: attachment.getReplyContent != null
                          ? attachment.getReplyContent["message_id"]
                          : "",
                      messageText: temp_text,
                      videoId: path.basename(attachment.getVideo.path),
                      thumbId: path.basename(attachment.getThumbnailVideo.path),
                      readReceipts: attachment.getAckActive ? "8" : "",
                      attachmentFlag: "2");
                  var formatTime = new DateFormat('Hm');
                  String? refId,
                      refName,
                      refText,
                      refType,
                      refThumbId,
                      refThumbLoc;
                  if (attachment.getReplyContent != null) {
                    refId = attachment.getReplyContent["message_id"];
                    refName = attachment.getReplyContent["fromName"];
                    refText =
                        attachment.getReplyContent["attachment_type"] == "3" ||
                                attachment.getReplyContent["attachment_type"] ==
                                    "-1"
                            ? ""
                            : attachment.getReplyContent["text"];
                    refType = attachment.getReplyContent["attachment_type"];
                    refThumbId = attachment.getReplyContent["thumb_id"];
                    refThumbLoc = attachment.getReplyContent["thumb_location"];
                  }
                  var resultQueryBuddy = await DBHelper().queryCustom(
                      "SELECT first_name, last_name, image_id FROM BUDDY WHERE f_pin=='$idMe'");
                  widget.select.addChat = Chat(
                      messageId: resultSend,
                      isOriginator: true,
                      text: temp_text,
                      statusRead: "1",
                      fileName: path.basename(attachment.getVideo.path),
                      thumbName: attachment.getThumbnailVideo.path,
                      attachmentType: "2",
                      refFrom: refName,
                      refType: refType,
                      refText: refText,
                      fName: resultQueryBuddy[0]["first_name"] +
                          " " +
                          resultQueryBuddy[0]["last_name"],
                      fImage: resultQueryBuddy[0]["image_id"],
                      refThumbLoc: refThumbLoc!,
                      refId: refId,
                      refThumbId: refThumbId,
                      ack: widget.attachment.getAckActive ? "8" : "",
                      time: formatTime.format(DateTime.now()),
                      date: DateTime.now());
                  _controllerEditor.clear();
                  attachment.video = null;
                  widget.attachment.refresh = true;
                  attachment.replyContent = null;
                  Future.delayed(Duration(milliseconds: 50), () {
                    _scrollController.animateTo(0.0,
                        duration: Duration(milliseconds: 1),
                        curve: Curves.easeOut);
                  });
                } else if (attachment.getFile != null) {
                  attachment.getFile.copy(
                      "${attachment.path.path}/${path.basename(attachment.getFile.path)}");
                  var resultSend = await API.sendMessage(
                      lPin: widget.idGroup,
                      chatId: widget.identifier,
                      messageScopeId: "4",
                      status: "2",
                      reffId: attachment.getReplyContent != null
                          ? attachment.getReplyContent["message_id"]
                          : "",
                      messageText: temp_text,
                      fileId: path.basename(attachment.getFile.path),
                      readReceipts: attachment.getAckActive ? "8" : "",
                      attachmentFlag: "6");
                  var formatTime = new DateFormat('Hm');
                  String? refId,
                      refName,
                      refText,
                      refType,
                      refThumbId,
                      refThumbLoc;
                  if (attachment.getReplyContent != null) {
                    refId = attachment.getReplyContent["message_id"];
                    refName = attachment.getReplyContent["fromName"];
                    refText =
                        attachment.getReplyContent["attachment_type"] == "3" ||
                                attachment.getReplyContent["attachment_type"] ==
                                    "-1"
                            ? ""
                            : attachment.getReplyContent["text"];
                    refType = attachment.getReplyContent["attachment_type"];
                    refThumbId = attachment.getReplyContent["thumb_id"];
                    refThumbLoc = attachment.getReplyContent["thumb_location"];
                  }
                  var resultQueryBuddy = await DBHelper().queryCustom(
                      "SELECT first_name, last_name, image_id FROM BUDDY WHERE f_pin=='$idMe'");
                  widget.select.addChat = Chat(
                      messageId: resultSend,
                      isOriginator: true,
                      text: temp_text,
                      statusRead: "2",
                      fileName: path.basename(attachment.getFile.path),
                      attachmentType: "6",
                      refFrom: refName,
                      refType: refType,
                      refText: refText,
                      refThumbLoc: refThumbLoc!,
                      refId: refId,
                      fName: resultQueryBuddy[0]["first_name"] +
                          " " +
                          resultQueryBuddy[0]["last_name"],
                      fImage: resultQueryBuddy[0]["image_id"],
                      refThumbId: refThumbId,
                      ack: widget.attachment.getAckActive ? "8" : "",
                      time: formatTime.format(DateTime.now()),
                      date: DateTime.now());
                  _controllerEditor.clear();
                  attachment.file = null;
                  widget.attachment.refresh = true;
                  attachment.replyContent = null;
                  Future.delayed(Duration(milliseconds: 50), () {
                    _scrollController.animateTo(0.0,
                        duration: Duration(milliseconds: 1),
                        curve: Curves.easeOut);
                  });
                } else if (_controllerEditor.text.trim().isNotEmpty) {
                  var resultSend = await API.sendMessage(
                      lPin: widget.idGroup,
                      chatId: widget.identifier,
                      messageScopeId: "4",
                      status: "2",
                      reffId: attachment.getReplyContent != null
                          ? attachment.getReplyContent["message_id"]
                          : "",
                      readReceipts: attachment.getAckActive ? "8" : "",
                      messageText: temp_text);
                  var formatTime = new DateFormat('Hm');
                  String? refId,
                      refName,
                      refText,
                      refType,
                      refThumbId,
                      refThumbLoc;
                  if (attachment.getReplyContent != null) {
                    refId = attachment.getReplyContent["message_id"];
                    refName = attachment.getReplyContent["fromName"];
                    refText =
                        attachment.getReplyContent["attachment_type"] == "3" ||
                                attachment.getReplyContent["attachment_type"] ==
                                    "-1"
                            ? ""
                            : attachment.getReplyContent["text"];
                    refType = attachment.getReplyContent["attachment_type"];
                    refThumbId = attachment.getReplyContent["thumb_id"];
                    refThumbLoc = attachment.getReplyContent["thumb_location"];
                  }
                  var resultQueryBuddy = await DBHelper().queryCustom(
                      "SELECT first_name, last_name, image_id FROM BUDDY WHERE f_pin=='$idMe'");
                  widget.select.addChat = Chat(
                      messageId: resultSend,
                      isOriginator: true,
                      text: temp_text,
                      statusRead: "1",
                      attachmentType: "0",
                      refFrom: refName,
                      refType: refType,
                      refText: refText,
                      refThumbLoc: refThumbLoc,
                      refId: refId,
                      fName: resultQueryBuddy[0]["first_name"] +
                          " " +
                          resultQueryBuddy[0]["last_name"],
                      fImage: resultQueryBuddy[0]["image_id"],
                      refThumbId: refThumbId!,
                      ack: widget.attachment.getAckActive ? "8" : "",
                      time: formatTime.format(DateTime.now()),
                      date: DateTime.now());
                  _controllerEditor.clear();
                  widget.attachment.refresh = true;
                  attachment.replyContent = null;
                  Future.delayed(Duration(milliseconds: 50), () {
                    _scrollController.animateTo(0.0,
                        duration: Duration(milliseconds: 1),
                        curve: Curves.easeOut);
                  });
                } else {
                  Fluttertoast.showToast(
                      msg: "Type Message", toastLength: Toast.LENGTH_SHORT);
                }
                if (widget.attachment.textLink != null) {
                  attachment.textLink = null;
                }
                if (widget.attachment.isMention) {
                  widget.attachment.isMention = false;
                }
                if (widget.selectChat.getSelectedChat.length > 0) {
                  widget.selectChat.getSelectedChat.forEach((k, v) {
                    int idx = widget.select.getChat.indexWhere(
                        (element) => element.messageId == v.messageId);
                    widget.select.falseSelected = idx;
                  });
                  widget.attachment.cantForward = 0;
                  widget.attachment.justDelete = 0;
                  if (widget.attachment.disableFavoriteMessage) {
                    widget.attachment.disableFavoriteMessage = false;
                  }
                  widget.selectChat.clearList = true;
                }
              },
              splashColor: Colors.yellow,
              child: Icon(
                Icons.send,
                size: 25,
                color: Colors.white,
              ),
            ),
          ),
        )
      ],
    );
  }

  Widget clip(Attachment attachment) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: <Widget>[
        Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            Container(
              margin: EdgeInsets.only(top: 5.0),
              width: 40,
              height: 40,
              child: Material(
                color: Colors.transparent,
                child: InkWell(
                  onTap: () async {
                    if (FocusScope.of(context).hasFocus) {
                      FocusScope.of(context).unfocus();
                      await Future.delayed(Duration(milliseconds: 100));
                    }
                    dialogAckConfidential(attachment);
                  },
                  splashColor: Colors.blueGrey,
                  child: Stack(
                    alignment: Alignment.center,
                    children: <Widget>[
                      attachment.getAckActive
                          ? Image.asset(
                              "assets/images/twsn_menulist_ack_12.png",
                              package: "palio_lite")
                          : Image.asset(
                              "assets/images/twsn_menulist_ack_11.png",
                              package: "palio_lite"),
                    ],
                  ),
                ),
              ),
            ),
            Container(
              margin: EdgeInsets.only(top: 5.0),
              width: 40,
              height: 40,
              child: Material(
                color: Colors.transparent,
                child: InkWell(
                  onTap: () async {
                    if (FocusScope.of(context).hasFocus) {
                      FocusScope.of(context).unfocus();
                      await Future.delayed(Duration(milliseconds: 100));
                    }
                    if (attachment.clickedSticker) {
                      attachment.clickedSticker = false;
                    } else {
                      attachment.clickedSticker = true;
                    }
                  },
                  splashColor: Colors.blueGrey,
                  child: Stack(
                    alignment: Alignment.center,
                    children: <Widget>[
                      Image.asset("assets/images/od_hex_base.png",
                          package: "palio_lite"),
                      Icon(
                        Icons.mood,
                        size: 20,
                        color: Colors.grey[300],
                      )
                    ],
                  ),
                ),
              ),
            ),
          ],
        ),
        Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            attachment.getFile != null
                ? Row(
                    mainAxisAlignment: MainAxisAlignment.start,
                    children: <Widget>[
                      Container(
                        constraints:
                            BoxConstraints(maxWidth: 120, minWidth: 60),
                        child: Text(
                          path
                              .basename(attachment.getFile.absolute.path)
                              .split(".")[0],
                          style: TextStyle(fontWeight: FontWeight.bold),
                          overflow: TextOverflow.ellipsis,
                        ),
                      ),
                      Text(
                        ".${path.basename(attachment.getFile.absolute.path).split(".")[1]}",
                        style: TextStyle(fontWeight: FontWeight.bold),
                      )
                    ],
                  )
                : Container(),
            SizedBox(width: 5),
            Container(
              margin: EdgeInsets.only(top: 5.0, right: 5.0),
              width: 40,
              height: 40,
              child: Material(
                color: Colors.transparent,
                child: InkWell(
                  onTap: () async {
                    if (FocusScope.of(context).hasFocus) {
                      FocusScope.of(context).unfocus();
                      await Future.delayed(Duration(milliseconds: 100));
                    }
                    dialogAttachment(attachment);
                  },
                  splashColor: Colors.blueGrey,
                  child: attachment.getVideo != null
                      ? Image.asset("assets/images/od_video.png",
                          package: "palio_lite")
                      : attachment.getListImage.length > 0
                          ? Image.asset("assets/images/od_images.png",
                              package: "palio_lite")
                          : attachment.getFile != null
                              ? Image.asset("assets/images/od_file.png",
                                  package: "palio_lite")
                              : Stack(
                                  alignment: Alignment.center,
                                  children: <Widget>[
                                    Image.asset("assets/images/od_hex_base.png",
                                        package: "palio_lite"),
                                    Image.asset(
                                        "assets/images/od_icon_attach.png",
                                        package: "palio_lite")
                                  ],
                                ),
                ),
              ),
            ),
          ],
        )
      ],
    );
  }

  Widget screenImageList(Attachment attachment) {
    return Container(
      margin: EdgeInsets.only(bottom: 20, right: 10),
      child: SizedBox(
        height: 70,
        child: ListView.builder(
            shrinkWrap: true,
            reverse: true,
            physics: ClampingScrollPhysics(),
            scrollDirection: Axis.horizontal,
            itemCount: attachment.getListImage.length,
            itemBuilder: (_, i) {
              return Container(
                margin: EdgeInsets.only(right: 20),
                width: 100,
                color: Colors.transparent,
                child: Container(
                  padding: EdgeInsets.all(2.0),
                  decoration: BoxDecoration(
                      color: Colors.white,
                      border: Border.all(color: Colors.black),
                      borderRadius: BorderRadius.circular(5.0)),
                  child: Stack(
                    alignment: Alignment.topRight,
                    children: <Widget>[
                      Material(
                        color: Colors.transparent,
                        child: InkWell(
                          splashColor: Colors.grey,
                          onTap: () {
                            Navigator.push(
                                context,
                                PageTransition(
                                    type: PageTransitionType.fade,
                                    child: Stack(
                                      children: <Widget>[
                                        Center(
                                          child: PhotoView(
                                              imageProvider: FileImage(
                                                  attachment.getListImage[i]),
                                              backgroundDecoration:
                                                  BoxDecoration(
                                                      color: Colors.blueGrey)),
                                        ),
                                        Align(
                                          alignment: Alignment.topLeft,
                                          child: Container(
                                            child: Material(
                                              color: Colors.transparent,
                                              child: GestureDetector(
                                                child: Row(
                                                  mainAxisAlignment:
                                                      MainAxisAlignment.start,
                                                  children: <Widget>[
                                                    Icon(Icons.arrow_back_ios,
                                                        color: Colors.white),
                                                    Text("Back",
                                                        style: TextStyle(
                                                            color: Colors.white,
                                                            fontSize: 18))
                                                  ],
                                                ),
                                                onTap: () =>
                                                    Navigator.pop(context),
                                              ),
                                            ),
                                            margin: EdgeInsets.only(
                                                top: 40.0, left: 5.0),
                                          ),
                                        )
                                      ],
                                    )));
                          },
                          child: Container(
                            decoration: BoxDecoration(
                                image: DecorationImage(
                                    fit: BoxFit.cover,
                                    image:
                                        FileImage(attachment.getListImage[i]))),
                          ),
                        ),
                      ),
                      ClipOval(
                        child: Material(
                          color: Colors.white,
                          child: InkWell(
                            splashColor: Colors.grey,
                            onTap: () {
                              attachment.removeListImage = i;
                            },
                            child: Container(
                              width: 25,
                              height: 25,
                              child: Icon(
                                Icons.close,
                                color: Colors.grey,
                              ),
                            ),
                          ),
                        ),
                      )
                    ],
                  ),
                ),
              );
            }),
      ),
    );
  }

  Widget screenVideo(Attachment attachment) {
    return Align(
      alignment: Alignment.centerRight,
      child: Stack(
        alignment: Alignment.topRight,
        children: <Widget>[
          Material(
            color: Colors.transparent,
            child: InkWell(
              onTap: () {
                OpenFile.open(attachment.getVideo.absolute.path);
              },
              child: Container(
                margin: EdgeInsets.only(right: 5.0),
                height: 90,
                width: 80,
                decoration: BoxDecoration(
                    image: DecorationImage(
                        fit: BoxFit.cover,
                        image: FileImage(attachment.getThumbnailVideo))),
              ),
            ),
          ),
          Padding(
            padding: const EdgeInsets.only(right: 5.0),
            child: ClipOval(
              child: Material(
                color: Colors.white,
                child: InkWell(
                  splashColor: Colors.grey,
                  onTap: () {
                    attachment.video = null;
                    attachment.thumbnailVideo = null;
                  },
                  child: Container(
                    width: 25,
                    height: 25,
                    child: Icon(
                      Icons.close,
                      color: Colors.grey,
                    ),
                  ),
                ),
              ),
            ),
          )
        ],
      ),
    );
  }

  void dialogAckConfidential(Attachment attachment) {
    showGeneralDialog(
        context: context,
        barrierDismissible: true,
        barrierLabel:
            MaterialLocalizations.of(context).modalBarrierDismissLabel,
        barrierColor: Colors.black45,
        transitionDuration: const Duration(milliseconds: 200),
        pageBuilder: (BuildContext buildContext, Animation animation,
            Animation secondaryAnimation) {
          return Center(
            child: Container(
              width: MediaQuery.of(context).size.width * 0.9,
              height: 130,
              padding: EdgeInsets.all(10.0),
              decoration: BoxDecoration(
                  border: Border.all(color: Colors.white, width: 1.0),
                  color: Colors.black54),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Material(
                    color: Colors.transparent,
                    child: Text("Message Mode",
                        style: TextStyle(color: Colors.white, fontSize: 16)),
                  ),
                  Material(
                    color: Colors.transparent,
                    child: Text("Choose Message Mode",
                        style: TextStyle(color: Colors.white, fontSize: 14)),
                  ),
                  SizedBox(
                    height: 5.0,
                  ),
                  Container(
                    margin: EdgeInsets.only(left: 30),
                    child: Material(
                      color: Colors.transparent,
                      child: InkWell(
                        onTap: () {
                          attachment.ackActive = !attachment.getAckActive;
                          Navigator.pop(context);
                        },
                        splashColor: Colors.blueAccent,
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.start,
                          children: <Widget>[
                            Container(
                              width: 50,
                              height: 50,
                              child: Image.asset(
                                  "assets/images/twsn_menulist_ack_12.png",
                                  package: "palio_lite"),
                            ),
                            Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                                Text("Confirmation Message",
                                    style: TextStyle(
                                        color: Colors.white, fontSize: 14)),
                                Container(
                                  constraints: BoxConstraints(
                                      maxWidth: MediaQuery.of(context)
                                                  .size
                                                  .width >
                                              400
                                          ? MediaQuery.of(context).size.width *
                                              0.65
                                          : MediaQuery.of(context).size.width *
                                              0.6),
                                  child: Wrap(
                                    children: <Widget>[
                                      Text(
                                          "Request to get confirmation from the recipient including the user's location information",
                                          style: TextStyle(
                                              color: Colors.white,
                                              fontSize: 9)),
                                    ],
                                  ),
                                )
                              ],
                            )
                          ],
                        ),
                      ),
                    ),
                  )
                ],
              ),
            ),
          );
        });
  }

  void dialogAttachment(Attachment attachment) {
    showGeneralDialog(
        context: context,
        barrierDismissible: true,
        barrierLabel:
            MaterialLocalizations.of(context).modalBarrierDismissLabel,
        barrierColor: Colors.black45,
        transitionDuration: const Duration(milliseconds: 200),
        pageBuilder: (BuildContext buildContext, Animation animation,
            Animation secondaryAnimation) {
          return Center(
            child: Container(
              width: MediaQuery.of(context).size.width * 0.9,
              height: 220,
              padding: EdgeInsets.all(10.0),
              decoration: BoxDecoration(
                  border: Border.all(color: Colors.white, width: 1.0),
                  color: Colors.black54),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Material(
                    color: Colors.transparent,
                    child: Text("Attachment",
                        style: TextStyle(color: Colors.white, fontSize: 16)),
                  ),
                  SizedBox(height: 5.0),
                  Container(
                    margin: EdgeInsets.only(left: 25),
                    child: Material(
                      color: Colors.transparent,
                      child: InkWell(
                        onTap: () {
                          if (attachment.getListImage.length < 5) {
                            Navigator.pop(context);
                            showDialog(
                                barrierDismissible: true,
                                context: context,
                                builder: (BuildContext context) {
                                  return GestureDetector(
                                    child: CupertinoAlertDialog(
                                      content: Material(
                                        color: Colors.transparent,
                                        child: Column(
                                          crossAxisAlignment:
                                              CrossAxisAlignment.start,
                                          children: <Widget>[
                                            ListTile(
                                              title: Text("Take Photo"),
                                              onTap: () {
                                                Navigator.pop(context);
                                                takePhoto(attachment);
                                              },
                                            ),
                                            ListTile(
                                              title:
                                                  Text("Select from Gallery"),
                                              onTap: () {
                                                Navigator.pop(context);
                                                pickImages(attachment);
                                              },
                                            )
                                          ],
                                        ),
                                      ),
                                    ),
                                  );
                                });
                          } else {
                            Fluttertoast.showToast(
                                msg: "Maximum 5 images",
                                toastLength: Toast.LENGTH_SHORT);
                          }
                        },
                        splashColor: Colors.blueAccent,
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.start,
                          children: <Widget>[
                            Container(
                              width: 50,
                              height: 50,
                              child: Image.asset("assets/images/od_images.png",
                                  package: "palio_lite"),
                            ),
                            Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                                Text("Image",
                                    style: TextStyle(
                                        color: Colors.white, fontSize: 14)),
                                Text("Send image attachment",
                                    style: TextStyle(
                                        color: Colors.white, fontSize: 9))
                              ],
                            )
                          ],
                        ),
                      ),
                    ),
                  ),
                  SizedBox(height: 5.0),
                  Container(
                    margin: EdgeInsets.only(left: 25),
                    child: Material(
                      color: Colors.transparent,
                      child: InkWell(
                        onTap: () {
                          Navigator.pop(context);
                          showDialog(
                              barrierDismissible: true,
                              context: context,
                              builder: (BuildContext context) {
                                return GestureDetector(
                                  child: CupertinoAlertDialog(
                                    content: Material(
                                      color: Colors.transparent,
                                      child: Column(
                                        crossAxisAlignment:
                                            CrossAxisAlignment.start,
                                        children: <Widget>[
                                          ListTile(
                                            title: Text("Take Video"),
                                            onTap: () {
                                              Navigator.pop(context);
                                              takePickVideo(
                                                  "camera", attachment);
                                            },
                                          ),
                                          ListTile(
                                            title: Text("Select from Gallery"),
                                            onTap: () {
                                              Navigator.pop(context);
                                              takePickVideo(
                                                  "gallery", attachment);
                                            },
                                          )
                                        ],
                                      ),
                                    ),
                                  ),
                                );
                              });
                        },
                        splashColor: Colors.blueAccent,
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.start,
                          children: <Widget>[
                            Container(
                              width: 50,
                              height: 50,
                              child: Image.asset("assets/images/od_video.png",
                                  package: "palio_lite"),
                            ),
                            Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                                Text("Video",
                                    style: TextStyle(
                                        color: Colors.white, fontSize: 14)),
                                Text("Send video attachment",
                                    style: TextStyle(
                                        color: Colors.white, fontSize: 9))
                              ],
                            )
                          ],
                        ),
                      ),
                    ),
                  ),
                  SizedBox(height: 5.0),
                  // Container(
                  //   margin: EdgeInsets.only(left: 25),
                  //   child: Material(
                  //     color: Colors.transparent,
                  //     child: InkWell(
                  //       onTap: () {
                  //         Navigator.pop(context);
                  //         pickLocation();
                  //       },
                  //       splashColor: Colors.blueAccent,
                  //       child: Row(
                  //         mainAxisAlignment: MainAxisAlignment.start,
                  //         children: <Widget>[
                  //           Container(
                  //             width: 50,
                  //             height: 50,
                  //             child:
                  //                 Image.asset("assets/images/od_location.png"),
                  //           ),
                  //           Column(
                  //             crossAxisAlignment: CrossAxisAlignment.start,
                  //             children: <Widget>[
                  //               Text(translate("location"),
                  //                   style: TextStyle(
                  //                       color: Colors.white, fontSize: 14)),
                  //               Text(translate("descLocation"),
                  //                   style: TextStyle(
                  //                       color: Colors.white, fontSize: 9))
                  //             ],
                  //           )
                  //         ],
                  //       ),
                  //     ),
                  //   ),
                  // ),
                  // SizedBox(height: 5.0),
                  Container(
                    margin: EdgeInsets.only(left: 25),
                    child: Material(
                      color: Colors.transparent,
                      child: InkWell(
                        onTap: () {
                          Navigator.pop(context);
                          pickFile(attachment);
                        },
                        splashColor: Colors.blueAccent,
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.start,
                          children: <Widget>[
                            Container(
                              width: 50,
                              height: 50,
                              child: Image.asset("assets/images/od_file.png",
                                  package: "palio_lite"),
                            ),
                            Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                                Text("File",
                                    style: TextStyle(
                                        color: Colors.white, fontSize: 14)),
                                Text("Send file attachment",
                                    style: TextStyle(
                                        color: Colors.white, fontSize: 9))
                              ],
                            )
                          ],
                        ),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          );
        });
  }

  void takePhoto(Attachment attachment) async {
    PickedFile? image = await ImagePicker.platform.pickImage(
      source: ImageSource.camera,
      imageQuality: 100,
      maxHeight: 720,
      maxWidth: 1280,
    );
    if (image == null) {
      return;
    } else {
      if (attachment.getVideo != null) {
        attachment.video = null;
        attachment.thumbnailVideo = null;
      }
      attachment.addListImage = File(image.path);
    }
  }

  void pickImages(Attachment attachment) async {
    List<PickedFile>? image = await ImagePicker.platform
        .pickMultiImage(imageQuality: 100, maxHeight: 720, maxWidth: 1280);
    if (image == null) {
      return;
    } else {
      if (attachment.getVideo != null) {
        attachment.video = null;
        attachment.thumbnailVideo = null;
      } else if (attachment.getFile != null) {
        attachment.file = null;
      }
      List<File> listImage = [];
      for (var i = 0; i < 5; i++) {
        if (File(image[i].path).lengthSync() * 0.001 >= 250.0) {
          Fluttertoast.showToast(
              msg: "You can only attach file that are less than 250 KB");
          return null;
        }
        listImage.add(File(image[i].path));
      }
      attachment.listImage = listImage;
    }
  }

  void takePickVideo(String from, Attachment attachment) async {
    PickedFile? pickedFile = await ImagePicker().getVideo(
        source: from == "camera" ? ImageSource.camera : ImageSource.gallery);
    File? pickVideo;
    if (pickedFile != null) {
      pickVideo = File(pickedFile.path);
    }
    if (pickVideo == null) {
      return;
    } else {
      await VideoCompress.setLogLevel(0);
      final info = await VideoCompress.compressVideo(
        pickVideo.path,
        quality: VideoQuality.MediumQuality,
        deleteOrigin: false,
        includeAudio: true,
      );
      if (info != null) {
        pickVideo = info.file;
      }
      if (attachment.getListImage.length > 0) {
        attachment.clearListImage = true;
      } else if (attachment.getFile != null) {
        attachment.file = null;
      }
      if (pickVideo!.lengthSync() * 0.001 >= 2500.0) {
        Fluttertoast.showToast(
            msg: "You can only attach file that are less than 2.5 MB");
        return null;
      }
      var myPath = await getApplicationDocumentsDirectory();
      String thumbPath = myPath.path;
      var videoThumb = await VideoThumbnail.thumbnailFile(
        video: pickVideo.path,
        thumbnailPath: thumbPath,
        imageFormat: ImageFormat.JPEG,
        maxHeight: 720,
        maxWidth: 1280,
        quality: 80,
      );
      attachment.thumbnailVideo = File(videoThumb!);
      attachment.video = pickVideo;
    }
  }

  // void pickLocation() async {
  //   Position position = await Geolocator()
  //       .getCurrentPosition(desiredAccuracy: LocationAccuracy.high);
  //   if (position == null) {
  //     Fluttertoast.showToast(
  //         msg: translate("checkGPS"), toastLength: Toast.LENGTH_SHORT);
  //   } else {
  //     var result = await Navigator.push(
  //         context,
  //         PageTransition(
  //             type: PageTransitionType.fade,
  //             child: MapPicker(
  //               "AIzaSyB3RM2rmLCsLzuymu6IU_0kx-GNmoAaY6g",
  //               LatLng(position.latitude, position.longitude),
  //             )));
  //     if (result != null) {
  //       Position position = result["latlng"];
  //       var resultSend = await API.sendMessage(
  //           lPin: widget.idGroup,
  //           chatId: widget.identifier,
  //           messageScopeId: "4",
  //           status: "2",
  //           attachmentFlag: "3",
  //           readReceipts: widget.attachment.getAckActive ? "8" : "",
  //           messageText:
  //               "Share location ${result["namePlace"]} lat/lng: (${position.latitude},${position.longitude})");
  //       var formatTime = new DateFormat('Hm');
  //       String refId, refName, refText, refType, refThumbId, refThumbLoc;
  //       if (widget.attachment.getReplyContent != null) {
  //         refId = widget.attachment.getReplyContent["message_id"];
  //         refName = widget.attachment.getReplyContent["fromName"];
  //         refText = widget.attachment.getReplyContent["attachment_type"] ==
  //                     "3" ||
  //                 widget.attachment.getReplyContent["attachment_type"] == "-1"
  //             ? ""
  //             : widget.attachment.getReplyContent["text"];
  //         refType = widget.attachment.getReplyContent["attachment_type"];
  //         refThumbId = widget.attachment.getReplyContent["thumb_id"];
  //         refThumbLoc = widget.attachment.getReplyContent["thumb_location"];
  //       }
  //       widget.select.addChat = Chat(
  //           messageId: resultSend,
  //           isOriginator: true,
  //           text:
  //               "Share location ${result["namePlace"]} lat/lng: (${position.latitude},${position.longitude})",
  //           statusRead: "1",
  //           attachmentType: "3",
  //           refThumbLoc: refThumbLoc,
  //           refThumbId: refThumbId,
  //           refText: refText,
  //           refFrom: refName,
  //           refId: refId,
  //           refType: refType,
  //           ack: widget.attachment.getAckActive ? "8" : "",
  //           time: formatTime.format(DateTime.now()),
  //           date: DateTime.now());
  //     }
  //   }
  // }

  void pickFile(Attachment attachment) async {
    FilePickerResult? result = await FilePicker.platform
        .pickFiles(type: FileType.any, allowMultiple: false);
    File? pickFile;
    if (result != null && result.files.single != null) {
      pickFile = File(result.files.single.path!);
    }
    if (pickFile == null) {
      return;
    } else {
      if (pickFile.lengthSync() * 0.001 >= 250.0) {
        Fluttertoast.showToast(
            msg: "You can only attach file that are less than 250 KB");
        return null;
      }
      if (attachment.getVideo != null) {
        attachment.video = null;
        attachment.thumbnailVideo = null;
      } else if (attachment.getListImage.length > 0) {
        attachment.clearListImage = true;
      }
      attachment.file = pickFile;
    }
  }

  void onMessageReceiver(ListMessage listMessage, Attachment attachment) {
    API.onReceiveMessage = (dynamic result) async {
      if (mounted) {
        String idGroup = result["A01"];
        String topicGroup = result["BA"];
        SharedPreferences prefs = await SharedPreferences.getInstance();
        String idMe = prefs.getString("hasLogin")!;
        if (idGroup == widget.idGroup && result["A00"] != idMe) {
          if ((widget.identifier.isEmpty && topicGroup == null) ||
              topicGroup == widget.identifier) {
            API.sendReadMessageStatus(
                result["A00"], widget.identifier, result["A06"], result["A18"]);
            String? fileName, thumbName, attachmentFlag, unDownloadThumb;
            bool auto = await getAutoDownload();
            if (result["A57"] != null) {
              fileName = result["A57"];
              thumbName = "${attachment.path.path}/${result["A74"]}";
              attachmentFlag = "1";
              if (auto) {
                Fluttertoast.showToast(
                    msg: "Downloading...", toastLength: Toast.LENGTH_SHORT);
                API.download(result["A57"]);
              }
            } else if (result["A47"] != null) {
              fileName = result["A47"];
              thumbName = "${attachment.path.path}/${result["A74"]}";
              unDownloadThumb = result["A74"];
              if (!File("${attachment.path.path}/${result["A74"]}")
                  .existsSync()) {
                API.download(result["A74"]);
              }
              attachmentFlag = "2";
              if (auto) {
                Fluttertoast.showToast(
                    msg: "Downloading...", toastLength: Toast.LENGTH_SHORT);
                API.download(result["A47"]);
              }
            } else if (result["BN"] != null) {
              fileName = result["BN"];
              attachmentFlag = "6";
              if (auto) {
                Fluttertoast.showToast(
                    msg: "Downloading...", toastLength: Toast.LENGTH_SHORT);
                API.download(result["BN"]);
              }
            } else if (result["A63"] != null) {
              fileName = result["A63"];
              attachmentFlag = "-1";
              if (auto) {
                Fluttertoast.showToast(
                    msg: "Downloading...", toastLength: Toast.LENGTH_SHORT);
                API.download(result["A63"]);
              }
            } else if (result["A07"].contains("Share%20location%20")) {
              attachmentFlag = "3";
            } else if (result["A149"].toString() == "11") {
              attachmentFlag = "11";
            } else {
              attachmentFlag = "0";
            }
            String? refId, refName, refText, refType, refThumbId, refThumbLoc;
            if (result["A121"] != null) {
              var resultReplyMessage = await DBHelper().queryCustom(
                  "SELECT * FROM MESSAGE WHERE message_id='${result["A121"]}'");
              if (resultReplyMessage.length != 0) {
                String? imageLocation;
                if (resultReplyMessage[0]["message_text"]
                    .contains("lat/lng: (")) {
                  String textDecode = Uri.decodeQueryComponent(
                      Uri.decodeFull(resultReplyMessage[0]["message_text"]));
                  int idx = textDecode.indexOf("(");
                  var textLatLng = textDecode
                      .substring(idx + 1, textDecode.trim().length - 1)
                      .split(",");
                  imageLocation =
                      "https://maps.googleapis.com/maps/api/staticmap?center=" +
                          textLatLng[0] +
                          "," +
                          textLatLng[1].trimLeft() +
                          "&zoom=15&size=" +
                          300.toString() +
                          "x" +
                          150.toString() +
                          "&maptype=normal&markers=color:red|" +
                          textLatLng[0] +
                          "," +
                          textLatLng[1].trimLeft() +
                          "&key=AIzaSyC79oNmadezP9O_4Q_Ud2bbuJ_L7M--y1s";
                }
                refId = result["A121"];
                var resultQueryBuddyreff;
                SharedPreferences prefs = await SharedPreferences.getInstance();
                String idMe = prefs.getString("hasLogin")!;
                if (resultReplyMessage[0]["f_pin"] != idMe) {
                  resultQueryBuddyreff = await DBHelper().queryCustom(
                      "SELECT first_name, last_name, image_id FROM BUDDY WHERE f_pin='${resultReplyMessage[0]["f_pin"]}'");
                }
                refName = resultReplyMessage[0]["f_pin"] == idMe
                    ? "You"
                    : resultQueryBuddyreff[0]["first_name"] +
                        " " +
                        resultQueryBuddyreff[0]["last_name"];
                refText = resultReplyMessage[0]["message_text"]
                            .contains("lat/lng: (") ||
                        resultReplyMessage[0]["audio_id"] != ""
                    ? ""
                    : resultReplyMessage[0]["message_text"];
                if (resultReplyMessage[0]["image_id"] != "") {
                  fileName = resultReplyMessage[0]["image_id"];
                  thumbName =
                      "${attachment.path.path}/${resultReplyMessage[0]["thumb_id"]}";
                  refType = "1";
                } else if (resultReplyMessage[0]["video_id"] != "") {
                  fileName = resultReplyMessage[0]["video_id"];
                  thumbName =
                      "${attachment.path.path}/${resultReplyMessage[0]["thumb_id"]}";
                  refType = "2";
                } else if (resultReplyMessage[0]["file_id"] != "") {
                  fileName = resultReplyMessage[0]["file_id"];
                  refType = "6";
                } else if (resultReplyMessage[0]["audio_id"] != "") {
                  fileName = resultReplyMessage[0]["audio_id"];
                  refType = "-1";
                } else if (resultReplyMessage[0]["message_text"]
                    .contains("lat/lng: (")) {
                  refType = "3";
                } else if (resultReplyMessage[0]["attachment_flag"]
                        .toString() ==
                    "11") {
                  refType = "11";
                } else {
                  refType = "0";
                }
                refThumbId = resultReplyMessage[0]["thumb_id"];
                refThumbLoc = imageLocation;
              }
            }
            DateTime dateTime =
                DateTime.fromMillisecondsSinceEpoch(int.parse(result["A19"]));
            var formatTime = new DateFormat('Hm');
            var resultQueryBuddy = await DBHelper().queryCustom(
                "SELECT first_name, last_name, image_id FROM BUDDY WHERE f_pin='${result["A00"]}'");

            var text = result["A07"];
            if (result["A149"] == "24") {
              var parseVCRMessage = json.decode(text);
              print(parseVCRMessage);
              var by = parseVCRMessage['by'];
              String name = await getUserName(by);
              String student = await getUserName(idMe);
              String titleQuiz =
                  toNormalString(parseVCRMessage['title'].toString());
              String descQuiz =
                  toNormalString(parseVCRMessage['description'].toString());
              if (language == "en_US") {
                String pricing = parseVCRMessage['pricing'].toString() == "0"
                    ? "Free"
                    : "Paid";
                text =
                    "Quiz\nTitle : ${titleQuiz} \nDescription : ${descQuiz} \nEducator: $name \nStudent: $student";
              } else {
                String pricing = parseVCRMessage['pricing'].toString() == "0"
                    ? "Gratis"
                    : "Berbayar";
                text =
                    "Kuis\nJudul : ${titleQuiz} \nDeskripsi : ${descQuiz} \nPendidik: $name \nPelajar: $student";
              }

              fileName = "";
              thumbName = "assets/images/seminar_wpr.png";
              attachmentFlag = "24";
            }

            Chat message = Chat(
                text: text,
                messageId: result["A18"],
                time: formatTime.format(dateTime),
                attachmentType: attachmentFlag,
                statusRead: result["A15"],
                fId: result["A00"],
                fName: !result["A18"].contains("-")
                    ? resultQueryBuddy[0]["first_name"] +
                        " " +
                        resultQueryBuddy[0]["last_name"]
                    : "",
                fImage: !result["A18"].contains("-")
                    ? resultQueryBuddy[0]["image_id"]
                    : "",
                refThumbId: refThumbId,
                refId: refId,
                unDownloadThumb: unDownloadThumb,
                refThumbLoc: refThumbLoc,
                refText: refText,
                refType: refType,
                refFrom: refName,
                fileName: fileName,
                thumbName: thumbName,
                ack: result["Bf"],
                date: dateTime,
                credential: result["A118"],
                isOriginator: false);
            if (!attachment.isNotBottom) {
              listMessage.addChat = message;
            } else {
              listMessage.addNewMessages = message;
            }
            await API.updateCounter(widget.identifier.isNotEmpty
                ? widget.identifier
                : widget.idGroup);
          }
          if (widget.selectChat.getSelectedChat.length > 0) {
            widget.selectChat.getSelectedChat.forEach((k, v) {
              int idx = widget.select.getChat
                  .indexWhere((element) => element.messageId == v.messageId);
              widget.select.falseSelected = idx;
            });
            widget.attachment.cantForward = 0;
            widget.attachment.justDelete = 0;
            if (widget.attachment.disableFavoriteMessage) {
              widget.attachment.disableFavoriteMessage = false;
            }
            widget.selectChat.clearList = true;
          }
        } else {
          Notifications.showNotifications(result, 0, context, level: 2);
        }
      }
      if (!widget.attachment.refresh) {
        widget.attachment.refresh = true;
      }
    };
  }

  void onMessage(ListMessage listMessage, Attachment attachment) {
    API.onMessage = (dynamic result) async {
      if (mounted) {
        SharedPreferences prefs = await SharedPreferences.getInstance();
        String idMe = prefs.getString("hasLogin")!;
        if (result["A00"] == idMe || result["A01"] == idMe) {
          if (!result["A18"].contains("-2,")) {
            var statusMessage = await DBHelper().minStatusGroup(result["A18"]);
            int idx =
                listMessage.getChat.indexOf(Chat(messageId: result["A18"]));
            if (idx != -1) {
              if (result["D01"] == "1") {
                listMessage.updateStatusDelete = {
                  "index": idx,
                  "value": "🚫 _This message was deleted_"
                };
              } else {
                listMessage.updateStatus = {
                  "index": idx,
                  "value": statusMessage[0]["MIN(status)"].toString()
                };
              }
            }
          } else {
            int idx = listMessage.getChat
                .indexOf(Chat(messageId: result["A18"].split(",")[1]));
            var statusMessage =
                await DBHelper().minStatusGroup(result["A18"].split(",")[1]);
            if (idx != -1) {
              listMessage.updateStatus = {
                "index": idx,
                "value": statusMessage[0]["MIN(status)"].toString()
              };
            }
          }
        }
        if (!attachment.refresh) {
          attachment.refresh = true;
        }
      }
    };
  }

  Future<bool> onWillPop() async {
    if (widget.attachment.isSearch) {
      _controllerSearch.clear();
      widget.attachment.isSuffix = false;
      widget.attachment.isSearch = false;
      widget.attachment.textSearch = null;
      widget.attachment.indexLastSearch = null;
    } else if (widget.selectChat.getSelectedChat.length > 0) {
      widget.selectChat.getSelectedChat.forEach((k, v) {
        int idx = widget.select.getChat
            .indexWhere((element) => element.messageId == v.messageId);
        widget.select.falseSelected = idx;
      });
      widget.attachment.cantForward = 0;
      widget.attachment.justDelete = 0;
      if (widget.attachment.disableFavoriteMessage) {
        widget.attachment.disableFavoriteMessage = false;
      }
      widget.selectChat.clearList = true;
    } else {
      PalioButtonState statePalio = PalioButtonState();
      statePalio.receiveMessage(context);
      Navigator.pop(context);
    }
    return Future.value(false);
  }

  Future<void> getChat(
      ListMessage listMessage, Attachment attachment, int limit,
      {int end = 40, bool fromMessageInfo = false}) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    attachment.path = await getApplicationDocumentsDirectory();
    String idMe = prefs.getString("hasLogin")!;
    List<dynamic> resultQuery;
    if (widget.identifier != "") {
      resultQuery = await DBHelper().queryCustom(
          "SELECT * FROM MESSAGE where chat_id='${widget.identifier}' order by server_date desc limit $limit,$end");
    } else {
      resultQuery = await DBHelper().queryCustom(
          "SELECT * FROM MESSAGE where chat_id = '' AND l_pin='${widget.idGroup}' order by server_date desc limit $limit,$end");
    }
    List<Chat> temporaryChat = [];
    for (var i = 0; i < resultQuery.length; i++) {
      var resultQueryBuddy = await DBHelper().queryCustom(
          "SELECT first_name, last_name, image_id FROM BUDDY WHERE f_pin='${resultQuery[i]["f_pin"]}'");
      DateTime dateTime =
          DateTime.fromMillisecondsSinceEpoch(resultQuery[i]["server_date"]);
      var formatTime = new DateFormat('Hm');
      if (resultQuery[i]["f_pin"] != idMe) {
        if (resultQuery[i]["status"] != "4" &&
            !resultQuery[i]["message_text"].contains("🚫") &&
            resultQuery[i]["status"] != "8") {
          API.sendReadMessageStatus(resultQuery[i]["f_pin"], widget.identifier,
              resultQuery[i]["message_scope_id"], resultQuery[i]["message_id"]);
        }
      }
      String? fileName, thumbName, attachmentFlag, unDownloadThumb;
      double? progress;
      bool auto = await getAutoDownload();
      if (resultQuery[i]["image_id"].trim().isNotEmpty) {
        fileName = resultQuery[i]["image_id"];
        thumbName = "${attachment.path.path}/${resultQuery[i]["thumb_id"]}";
        attachmentFlag = "1";
        if (File("${attachment.path.path}/${resultQuery[i]["image_id"]}")
            .existsSync()) {
          progress = 100.0;
        } else {
          if (auto) {
            API.download(resultQuery[i]["image_id"]);
          }
        }
      } else if (resultQuery[i]["video_id"].trim().isNotEmpty) {
        fileName = resultQuery[i]["video_id"];
        thumbName = "${attachment.path.path}/${resultQuery[i]["thumb_id"]}";
        attachmentFlag = "2";
        if (File("${attachment.path.path}/${resultQuery[i]["video_id"]}")
            .existsSync()) {
          progress = 100.0;
        } else {
          if (auto) {
            API.download(resultQuery[i]["video_id"]);
          }
        }
        if (!File("${attachment.path.path}/${resultQuery[i]["thumb_id"]}")
            .existsSync()) {
          thumbName = "assets/images/cuc_scope_image.png";
          unDownloadThumb = resultQuery[i]["thumb_id"];
          API.download(resultQuery[i]["thumb_id"]);
        }
      } else if (resultQuery[i]["file_id"].trim().isNotEmpty) {
        fileName = resultQuery[i]["file_id"];
        attachmentFlag = "6";
        if (File("${attachment.path.path}/${resultQuery[i]["file_id"]}")
            .existsSync()) {
          progress = 100.0;
        } else {
          if (auto) {
            API.download(resultQuery[i]["file_id"]);
          }
        }
      } else if (resultQuery[i]["audio_id"].trim().isNotEmpty) {
        fileName = resultQuery[i]["audio_id"];
        attachmentFlag = "-1";
        if (File("${attachment.path.path}/${resultQuery[i]["audio_id"]}")
            .existsSync()) {
          progress = 100.0;
        } else {
          if (auto) {
            API.download(resultQuery[i]["audio_id"]);
          }
        }
      } else if (resultQuery[i]["message_text"].contains("lat/lng: (")) {
        attachmentFlag = "3";
      } else if (resultQuery[i]["attachment_flag"].toString() == "11") {
        attachmentFlag = "11";
      } else {
        attachmentFlag = "0";
      }
      String? refId, refName, refText, refType, refThumbId, refThumbLoc;
      if (resultQuery[i]["reff_id"].isNotEmpty) {
        var resultReplyMessage = await DBHelper().queryCustom(
            "SELECT * FROM MESSAGE WHERE message_id='${resultQuery[i]["reff_id"]}'");
        if (resultReplyMessage.length != 0) {
          String? imageLocation;
          if (resultReplyMessage[0]["message_text"].contains("lat/lng: (")) {
            String textDecode = Uri.decodeQueryComponent(
                Uri.decodeFull(resultReplyMessage[0]["message_text"]));
            int idx = textDecode.indexOf("(");
            var textLatLng = textDecode
                .substring(idx + 1, textDecode.trim().length - 1)
                .split(",");
            imageLocation =
                "https://maps.googleapis.com/maps/api/staticmap?center=" +
                    textLatLng[0] +
                    "," +
                    textLatLng[1].trimLeft() +
                    "&zoom=15&size=" +
                    300.toString() +
                    "x" +
                    150.toString() +
                    "&maptype=normal&markers=color:red|" +
                    textLatLng[0] +
                    "," +
                    textLatLng[1].trimLeft() +
                    "&key=AIzaSyC79oNmadezP9O_4Q_Ud2bbuJ_L7M--y1s";
          }
          refId = resultQuery[i]["reff_id"];
          var resultQueryBuddyreff;
          if (resultReplyMessage[0]["f_pin"] != idMe) {
            resultQueryBuddyreff = await DBHelper().queryCustom(
                "SELECT first_name, last_name, image_id FROM BUDDY WHERE f_pin='${resultReplyMessage[0]["f_pin"]}'");
          }
          refName = resultReplyMessage[0]["f_pin"] == idMe
              ? "You"
              : resultQueryBuddyreff[0]["first_name"] +
                  " " +
                  resultQueryBuddyreff[0]["last_name"];
          refText =
              resultReplyMessage[0]["message_text"].contains("lat/lng: (") ||
                      resultReplyMessage[0]["audio_id"] != ""
                  ? ""
                  : resultReplyMessage[0]["message_text"];
          if (resultReplyMessage[0]["image_id"] != "") {
            fileName = resultReplyMessage[0]["image_id"];
            thumbName =
                "${attachment.path.path}/${resultReplyMessage[0]["thumb_id"]}";
            refType = "1";
          } else if (resultReplyMessage[0]["video_id"] != "") {
            fileName = resultReplyMessage[0]["video_id"];
            thumbName =
                "${attachment.path.path}/${resultReplyMessage[0]["thumb_id"]}";
            refType = "2";
          } else if (resultReplyMessage[0]["file_id"] != "") {
            fileName = resultReplyMessage[0]["file_id"];
            refType = "6";
          } else if (resultReplyMessage[0]["audio_id"] != "") {
            fileName = resultReplyMessage[0]["audio_id"];
            refType = "-1";
          } else if (resultReplyMessage[0]["message_text"]
              .contains("lat/lng: (")) {
            refType = "3";
          } else if (resultReplyMessage[0]["attachment_flag"].toString() ==
              "11") {
            refType = "11";
          } else {
            refType = "0";
          }
          refThumbId = resultReplyMessage[0]["thumb_id"];
          refThumbLoc = imageLocation;
        }
      }
      String text = resultQuery[i]["message_text"];
      if (resultQuery[i]["attachment_flag"].toString() == "30") {
        var parseLSMessage = jsonDecode(text);
        var title = parseLSMessage['title'];
        var desc = parseLSMessage['description'].trim().isEmpty
            ? "-"
            : parseLSMessage['description'];
        var by = parseLSMessage['by'];
        String name = await getUserName(by);
        var pricing = parseLSMessage['pricing'].toString();
        text =
            "Content\nTitle: $title\nDescription: $desc\nCreator: $name\nContent Conditions: Theory -- Materi";
      } else if (resultQuery[i]["attachment_flag"].toString() == "24") {
        var parseVCRMessage = json.decode(text);
        print(parseVCRMessage);
        print("pricing " + parseVCRMessage['pricing'].toString());
        var by = parseVCRMessage['by'];
        String name = await getUserName(by);
        String student = await getUserName(idMe);
        String titleQuiz = toNormalString(parseVCRMessage['title'].toString());
        String descQuiz =
            toNormalString(parseVCRMessage['description'].toString());
        if (language == "en_US") {
          String pricing =
              parseVCRMessage['pricing'].toString() == "0" ? "Free" : "Paid";
          text =
              "Quiz\nTitle : ${titleQuiz} \nDescription : ${descQuiz} \nEducator: $name \nStudent: $student";
        } else {
          String pricing = parseVCRMessage['pricing'].toString() == "0"
              ? "Gratis"
              : "Berbayar";
          text =
              "Kuis\nJudul : ${titleQuiz} \nDeskripsi : ${descQuiz} \nPendidik: $name \nPelajar: $student";
        }
        fileName = "";
        thumbName = "assets/images/seminar_wpr.png";
        attachmentFlag = "24";
      }
      var statusMessage =
          await DBHelper().minStatusGroup(resultQuery[i]["message_id"]);
      var resultFavorite = await DBHelper().queryCustom(
          "SELECT message_id FROM MESSAGE_FAVORITE WHERE message_id='${resultQuery[i]["message_id"]}'");
      Chat message = Chat(
          text: text,
          messageId: resultQuery[i]["message_id"],
          time: formatTime.format(dateTime),
          fId: resultQuery[i]["f_pin"],
          fName: !resultQuery[i]["message_id"].contains("-") &&
                  resultQueryBuddy.length > 0
              ? resultQueryBuddy[0]["first_name"] +
                  " " +
                  resultQueryBuddy[0]["last_name"]
              : "Bot",
          fImage: !resultQuery[i]["message_id"].contains("-") &&
                  resultQueryBuddy.length > 0
              ? resultQueryBuddy[0]["image_id"]
              : "",
          refId: refId,
          refFrom: refName,
          refText: refText,
          refThumbId: refThumbId,
          refType: refType,
          unDownloadThumb: unDownloadThumb,
          isFavorite: resultFavorite.length > 0 ? true : false,
          refThumbLoc: refThumbLoc,
          attachmentType: attachmentFlag,
          statusRead: statusMessage[0]["MIN(status)"] != null
              ? statusMessage[0]["MIN(status)"].toString()
              : resultQuery[i]["status"],
          fileName: fileName,
          thumbName: thumbName,
          date: dateTime,
          ack: resultQuery[i]["read_receipts"].toString(),
          progress: progress!,
          credential: resultQuery[i]["credential"],
          isOriginator: resultQuery[i]["f_pin"] == idMe);
      temporaryChat.add(message);
    }
    temporaryChat.sort((b, a) =>
        a.date.millisecondsSinceEpoch.compareTo(b.date.millisecondsSinceEpoch));
    if (fromMessageInfo) {
      listMessage.replaceChat = temporaryChat;
    } else {
      listMessage.addChatAll = temporaryChat;
    }
    attachment.loading = false;
    if (widget.refresh) {
      attachment.refresh = widget.refresh;
    }
  }

  void getFirstIndex() {
    int itemCount = widget.select.getChat.length;
    double scrollOffset = _scrollController.position.pixels;
    double viewportHeight = _scrollController.position.viewportDimension;
    double scrollRange = _scrollController.position.maxScrollExtent -
        _scrollController.position.minScrollExtent;
    int firstVisible =
        (scrollOffset / (scrollRange + viewportHeight) * itemCount).floor();
    if (firstVisible != 0) {
      if (!widget.attachment.isNotBottom &&
          !_scrollController.position.outOfRange) {
        widget.attachment.isNotBottom = true;
      }
    }
    if (firstVisible == 0) {
      if (widget.attachment.isNotBottom) {
        if (widget.select.getNewMessages.length != 0) {
          widget.select.insertAllChat = widget.select.getNewMessages;
          widget.select.clearNewMessages();
        }
        widget.attachment.isNotBottom = false;
      }
    }
  }

  void onDownload(ListMessage listMessage, Attachment attachment) {
    API.onDownloadProgress = (double progress, String fileName) {
      if (mounted) {
        int idx = widget.select.getChat.indexOf(Chat(fileName: fileName));
        if (idx != -1) {
          widget.select.updateProgress = {'index': idx, 'value': progress};
        } else {
          int idx2 =
              widget.select.getChat.indexOf(Chat(unDownloadThumb: fileName));
          if (idx2 != -1) {
            widget.select.updateThumbnail = {
              'index': idx2,
              'value': "${widget.attachment.path.path}/$fileName"
            };
          }
        }
      }
    };
  }

  void onUpload(ListMessage listMessage, Attachment attachment) {
    API.onUploadProgress = (double progress, String fileName) {
      if (mounted) {
        int idx = widget.select.getChat.indexOf(Chat(fileName: fileName));
        if (idx != -1) {
          widget.select.updateProgress = {'index': idx, 'value': progress};
        }
      }
    };
  }

  void onUploadFile(ListMessage listMessage, Attachment attachment) {
    API.onUploadFileProgress = (double progress, String fileName) {
      if (mounted) {
        int idx = widget.select.getChat.indexOf(Chat(fileName: fileName));
        if (idx != -1) {
          widget.select.updateProgress = {'index': idx, 'value': progress};
        }
      }
    };
  }

  void onTyping(ListMessage listMessage, Attachment attachment) {
    API.onTyping = (dynamic result) async {
      if (mounted) {
        String thisLpin =
            widget.identifier.isEmpty ? widget.idGroup : widget.identifier;
        SharedPreferences prefs = await SharedPreferences.getInstance();
        String idMe = prefs.getString("hasLogin")!;
        if (result["A00"] != idMe &&
            result["A01"] == thisLpin &&
            result["A76"] == "4") {
          var resultQueryBuddy = await DBHelper().queryCustom(
              "SELECT first_name FROM BUDDY WHERE f_pin=='${result["A00"]}'");
          attachment.isTyping =
              "${resultQueryBuddy[0]["first_name"]} sedang mengetik...";
          if (_timerReceiveTyping != null && _timerReceiveTyping!.isActive) {
            _timerReceiveTyping!.cancel();
          }
          _timerReceiveTyping = Timer(Duration(seconds: 3), () {
            attachment.isTyping = widget.nameTopic;
          });
        }
      }
    };
  }

  void event(ListMessage listMessage, Attachment attachment) {
    onMessageReceiver(listMessage, attachment);
    onMessage(listMessage, attachment);
    onDownload(listMessage, attachment);
    onTyping(listMessage, attachment);
    onUpload(listMessage, attachment);
    onUploadFile(listMessage, attachment);
  }

  scrollListener() {
    getFirstIndex();
    int currentIndex = _scrollController.currentTagIndexInViewport(
        preferPosition: AutoScrollPosition.begin)!;
    if (currentIndex == null) {
      return;
    }
    String dateIndex = DateTime(
                    widget.select.getChat[currentIndex].date.year,
                    widget.select.getChat[currentIndex].date.month,
                    widget.select.getChat[currentIndex].date.day)
                .difference(DateTime(DateTime.now().year, DateTime.now().month,
                    DateTime.now().day))
                .inDays ==
            0
        ? "Today"
        : DateTime(
                        widget.select.getChat[currentIndex].date.year,
                        widget.select.getChat[currentIndex].date.month,
                        widget.select.getChat[currentIndex].date.day)
                    .difference(DateTime(DateTime.now().year,
                        DateTime.now().month, DateTime.now().day))
                    .inDays ==
                -1
            ? "Yesterday"
            : DateTime(
                            widget.select.getChat[currentIndex].date.year,
                            widget.select.getChat[currentIndex].date.month,
                            widget.select.getChat[currentIndex].date.day)
                        .difference(DateTime(DateTime.now().year,
                            DateTime.now().month, DateTime.now().day))
                        .inDays ==
                    -2
                ? "2 days ago"
                : DateFormat('dd MMMM yyyy').format(
                    widget.select.getChat[widget.select.getChat.length - 1].date);
    if (_scrollController.position.outOfRange) {
      if (widget.attachment.isShowDate) {
        widget.attachment.isShowDate = false;
      }
    }
    if (widget.attachment.dateIndex == null &&
        _scrollController.offset != 0.0 &&
        !_scrollController.position.outOfRange &&
        dateIndex != "Today") {
      widget.attachment.dateIndex = dateIndex;
      widget.attachment.isShowDate = true;
    } else {
      if (_timer2 != null) {
        _timer2?.cancel();
      }
      if (widget.attachment.dateIndex != dateIndex &&
          _scrollController.offset != 0.0 &&
          !_scrollController.position.outOfRange &&
          dateIndex != "Today") {
        widget.attachment.dateIndex = dateIndex;
        widget.attachment.isShowDate = true;
        _timer2 = Timer(Duration(seconds: 1), () {
          widget.attachment.isShowDate = false;
        });
      } else {
        if (!widget.attachment.isShowDate &&
            _scrollController.offset != 0.0 &&
            !_scrollController.position.outOfRange &&
            dateIndex != "Today") {
          widget.attachment.isShowDate = true;
        }
        _timer2 = Timer(Duration(seconds: 1), () {
          widget.attachment.isShowDate = false;
        });
      }
    }
    // if (_scrollController.position.atEdge &&
    //     _scrollController.position.pixels != 0) {
    //   getChat(listMessage, widget.attachment, listMessage.getChat.length,
    //       end: 20);
    // }
  }

  textListener() {
    String text = _controllerEditor.text
        .substring(0, _controllerEditor.selection.baseOffset);
    String spaceSplit = text.split(" ")[text.split(" ").length - 1];
    String enterSplit = spaceSplit;
    if (spaceSplit.contains("\n")) {
      enterSplit = spaceSplit.split("\n")[spaceSplit.split("\n").length - 1];
    }
    if (_controllerEditor.text.trim().isNotEmpty &&
        (_timerSendTyping == null || !_timerSendTyping!.isActive)) {
      API.sendTyping(
          widget.identifier.isEmpty ? widget.idGroup : widget.identifier,
          "4",
          "4");
      _timerSendTyping = Timer(Duration(seconds: 2), () {});
    }
    if (text.split(" ").length - 2 != -1 || text.split("\n").length - 2 != -1) {
      if (widget.attachment.textLink != null) {
        return;
      }
    }
    if (enterSplit.contains("://")) {
      if (enterSplit.toLowerCase().substring(0, 7) == "http://" ||
          enterSplit.toLowerCase().substring(0, 8) == "https://") {
        widget.attachment.textLink = enterSplit;
        if (_timerLink != null && _timerLink!.isActive) {
          _timerLink!.cancel();
          if (widget.attachment.hitLink) {
            widget.attachment.hitLink = false;
          }
        }
        _timerLink = Timer(Duration(seconds: 1), () {
          widget.attachment.hitLink = true;
        });
      }
    } else {
      if (widget.attachment.textLink != null) {
        widget.attachment.textLink = null;
      }
    }
  }

  getLink(String text) {
    List<String> data = text.split(" ");
    String? link;
    for (int i = 0; i < data.length; i++) {
      if (data[i].toLowerCase().contains("http://")) {
        if (data[i].contains("\n")) {
          var shttp2 = data[i].split("\n");
          for (var j = 0; j < shttp2.length; j++) {
            if (shttp2[j].startsWith("http://")) {
              link = shttp2[j];
              break;
            }
          }
        } else {
          link = data[i];
          break;
        }
      } else if (data[i].toLowerCase().contains("https://")) {
        if (data[i].contains("\n")) {
          var shttp2 = data[i].split("\n");
          for (var j = 0; j < shttp2.length; j++) {
            if (shttp2[j].startsWith("https://")) {
              link = shttp2[j];
              break;
            }
          }
        } else {
          link = data[i];
          break;
        }
      }
    }
    return link;
  }

  void getMembers(Attachment attachment) async {
    Map<String, String> temp1 = Map();
    Map<String, String> temp2 = Map();
    var resultQueryMember = await DBHelper().queryCustom(
        "SELECT f_pin,position FROM GROUPZ_MEMBER WHERE group_id='${widget.idGroup}'");
    var resultQueryBuddyMembers = await DBHelper()
        .queryCustom("SELECT f_pin, first_name, last_name FROM BUDDY");
    for (var j = 0; j < resultQueryMember.length; j++) {
      var resultQueryBuddy = await DBHelper().queryCustom(
          "SELECT first_name, last_name FROM BUDDY WHERE f_pin='${resultQueryMember[j]["f_pin"]}'");
      if (resultQueryBuddy.length == 0) {
        continue;
      }
      temp1["@${resultQueryMember[j]["f_pin"]}"] = (resultQueryBuddy[0]
                  ["first_name"] +
              " " +
              resultQueryBuddy[0]["last_name"])
          .trim();
    }
    for (var j = 0; j < resultQueryBuddyMembers.length; j++) {
      temp2["@${resultQueryBuddyMembers[j]["f_pin"]}"] =
          (resultQueryBuddyMembers[j]["first_name"] +
                  " " +
                  resultQueryBuddyMembers[j]["last_name"])
              .trim();
    }
    attachment.addAllMembers = temp1;
    attachment.addAllContacts = temp2;
  }

  void checkMention(String text) async {
    text = _controllerEditor.text
        .substring(0, _controllerEditor.selection.baseOffset);
    List<String> split_space = text.split(" ");
    List<String> split_enter = text.split("\n");
    var lastEnter = split_enter[split_enter.length - 1];
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String idMe = prefs.getString("hasLogin")!;

    if (widget.attachment.isMention) {
      if (split_space[split_space.length - 1].contains("@")) {
        widget.attachment.clearTempMentions();
        List<String> split_mention = text.split("@");
        for (int i = 0; i < widget.attachment.getListMembers.length; i++) {
          if (widget.attachment.getListMembers.values
              .elementAt(i)
              .toLowerCase()
              .contains(
                  split_mention[split_mention.length - 1].toLowerCase())) {
            if (widget.attachment.getListMembers.keys
                    .elementAt(i)
                    .split("@")[1] !=
                idMe) {
              widget.attachment.addTempMentions = {
                "id": widget.attachment.getListMembers.keys.elementAt(i),
                "name": widget.attachment.getListMembers.values.elementAt(i)
              };
            }
          }
        }

        if (widget.attachment.getTempMentions.length == 0 &&
            widget.attachment.isMention) {
          widget.attachment.isMention = false;
        }
      } else {
        List<String> split_space_afterEnter = lastEnter.split(" ");
        if (split_space_afterEnter[split_space_afterEnter.length - 1].length >
            2) {
          widget.attachment.clearTempMentions();
          for (int i = 0; i < widget.attachment.getListMembers.length; i++) {
            if (widget.attachment.getListMembers.values
                .elementAt(i)
                .toLowerCase()
                .contains(
                    split_space_afterEnter[split_space_afterEnter.length - 1]
                        .toLowerCase())) {
              if (widget.attachment.getListMembers.keys
                      .elementAt(i)
                      .split("@")[1] !=
                  idMe) {
                widget.attachment.addTempMentions = {
                  "id": widget.attachment.getListMembers.keys.elementAt(i),
                  "name": widget.attachment.getListMembers.values.elementAt(i)
                };
              }
            }
          }
          if (widget.attachment.getTempMentions.length == 0 &&
              widget.attachment.isMention) {
            widget.attachment.isMention = false;
          }
        } else if (widget.attachment.isMention) {
          widget.attachment.clearTempMentions();
          widget.attachment.isMention = false;
        }
      }
    } else if (lastEnter.length > 2) {
      widget.attachment.clearTempMentions();
      if (split_space[split_space.length - 1].contains("@")) {
        widget.attachment.clearTempMentions();
        List<String> split_mention = text.split("@");
        for (int i = 0; i < widget.attachment.getListMembers.length; i++) {
          if (widget.attachment.getListMembers.values
              .elementAt(i)
              .toLowerCase()
              .contains(
                  split_mention[split_mention.length - 1].toLowerCase())) {
            if (widget.attachment.getListMembers.keys
                    .elementAt(i)
                    .split("@")[1] !=
                idMe) {
              widget.attachment.addTempMentions = {
                "id": widget.attachment.getListMembers.keys.elementAt(i),
                "name": widget.attachment.getListMembers.values.elementAt(i)
              };
              if (!widget.attachment.isMention) {
                widget.attachment.isMention = true;
              }
            }
          }
        }

        if (widget.attachment.getTempMentions.length == 0 &&
            widget.attachment.isMention) {
          widget.attachment.isMention = false;
        }
      } else {
        List<String> split_space_afterEnter = lastEnter.split(" ");
        for (int i = 0; i < widget.attachment.getListMembers.length; i++) {
          if (widget.attachment.getListMembers.values
                  .elementAt(i)
                  .toLowerCase()
                  .contains(
                      split_space_afterEnter[split_space_afterEnter.length - 1]
                          .toLowerCase()) &&
              split_space_afterEnter[split_space_afterEnter.length - 1]
                  .toLowerCase()
                  .isNotEmpty &&
              split_space_afterEnter[split_space_afterEnter.length - 1]
                      .toLowerCase()
                      .length >
                  2) {
            if (!text.contains(
                "@${widget.attachment.getListMembers.values.elementAt(i)}")) {
              if (widget.attachment.getListMembers.keys
                      .elementAt(i)
                      .split("@")[1] !=
                  idMe) {
                widget.attachment.addTempMentions = {
                  "id": widget.attachment.getListMembers.keys.elementAt(i),
                  "name": widget.attachment.getListMembers.values.elementAt(i)
                };
              }
              widget.attachment.isMention = true;
            }
          } else if (split_space_afterEnter[
                  split_space_afterEnter.length - 1] ==
              "@") {
            widget.attachment.clearTempMentions();
            for (int i = 0; i < widget.attachment.getListMembers.length; i++) {
              if (!text.contains(
                  "@${widget.attachment.getListMembers.values.elementAt(i)}")) {
                if (widget.attachment.getListMembers.keys
                        .elementAt(i)
                        .split("@")[1] !=
                    idMe) {
                  widget.attachment.addTempMentions = {
                    "id": widget.attachment.getListMembers.keys.elementAt(i),
                    "name": widget.attachment.getListMembers.values.elementAt(i)
                  };
                }
              }
            }
            widget.attachment.isMention = true;
          }
        }
      }
    } else if (split_enter.length > 1 &&
        lastEnter.length > 0 &&
        lastEnter[0] == "@") {
      widget.attachment.clearTempMentions();
      for (int i = 0; i < widget.attachment.getListMembers.length; i++) {
        if (widget.attachment.getListMembers.keys.elementAt(i).split("@")[1] !=
            idMe) {
          widget.attachment.addTempMentions = {
            "id": widget.attachment.getListMembers.keys.elementAt(i),
            "name": widget.attachment.getListMembers.values.elementAt(i)
          };
        }
      }
      widget.attachment.isMention = true;
    } else if (split_enter.length > 1 && lastEnter.length == 0) {
      for (int k = 0; k < split_enter.length; k++) {
        if (split_enter[k].contains("@")) {
          for (int i = 0; i < widget.attachment.getListMembers.length; i++) {
            if (widget.attachment.getListMembers.values
                .elementAt(i)
                .toLowerCase()
                .contains(split_enter[k].substring(1).toLowerCase())) {
              if (widget.attachment.getListMembers.keys
                      .elementAt(i)
                      .split("@")[1] !=
                  idMe) {
                widget.attachment.addTempMentions = {
                  "id": widget.attachment.getListMembers.keys.elementAt(i),
                  "name": widget.attachment.getListMembers.values.elementAt(i)
                };
              }
            }
          }
        }
      }
    } else if (text.length > 0 && text[0] == "@") {
      widget.attachment.clearTempMentions();
      for (int i = 0; i < widget.attachment.getListMembers.length; i++) {
        if (widget.attachment.getListMembers.keys.elementAt(i).split("@")[1] !=
            idMe) {
          widget.attachment.addTempMentions = {
            "id": widget.attachment.getListMembers.keys.elementAt(i),
            "name": widget.attachment.getListMembers.values.elementAt(i)
          };
        }
      }
      widget.attachment.isMention = true;
    }
  }

  @override
  void initState() {
    API.sendTyping(
        widget.identifier.isEmpty ? widget.idGroup : widget.identifier,
        "4",
        "1");
    final listMessage = Provider.of<ListMessage>(context, listen: false);
    getMembers(widget.attachment);
    getLanguage();
    getChat(listMessage, widget.attachment, 0);
    _scrollController.addListener(scrollListener);
    event(listMessage, widget.attachment);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: onWillPop,
      child: Scaffold(
        backgroundColor: Colors.grey[200],
        appBar: AppBar(
          automaticallyImplyLeading: true,
          backgroundColor: AppColor.primaryColor,
          titleSpacing: 0.0,
          title: widget.attachment.isSearch
              ? TextField(
                  autofocus: true,
                  controller: _controllerSearch,
                  cursorColor: AppColor.primaryColor,
                  style: TextStyle(color: Colors.grey[300]),
                  onChanged: (text) {
                    if (text.isNotEmpty && !widget.attachment.isSuffix) {
                      widget.attachment.isSuffix = true;
                    } else if (text.isEmpty && widget.attachment.isSuffix) {
                      widget.attachment.isSuffix = false;
                    }
                    widget.attachment.textSearch = text;
                  },
                  onSubmitted: (text) {
                    if (_controllerSearch.text.isNotEmpty) {
                      if (FocusScope.of(context).hasFocus) {
                        FocusScope.of(context).unfocus();
                      }
                      int currentIndex = widget.attachment.indexLastSearch;
                      if (currentIndex == null) {
                        currentIndex =
                            _scrollController.currentTagIndexInViewport(
                                preferPosition: AutoScrollPosition.begin)!;
                      } else {
                        if (currentIndex >
                            _scrollController.currentTagIndexInViewport(
                                preferPosition: AutoScrollPosition.begin)!) {
                          currentIndex =
                              _scrollController.currentTagIndexInViewport(
                                  preferPosition: AutoScrollPosition.begin)!;
                        } else {
                          currentIndex = widget.attachment.indexLastSearch - 1;
                        }
                      }

                      showDialog(
                          barrierDismissible: false,
                          context: context,
                          builder: (_) {
                            return CupertinoAlertDialog(
                              title: Row(
                                mainAxisAlignment: MainAxisAlignment.start,
                                children: <Widget>[
                                  Container(
                                      child: CupertinoActivityIndicator(),
                                      width: 20,
                                      height: 20),
                                  SizedBox(width: 20.0),
                                  Text("Searching...",
                                      style: TextStyle(
                                          fontWeight: FontWeight.normal))
                                ],
                              ),
                            );
                          });
                      if (currentIndex != null) {
                        for (var i = currentIndex;
                            i < widget.select.getChat.length;
                            i++) {
                          if (widget.select.getChat[i].text!
                              .toLowerCase()
                              .contains(_controllerSearch.text)) {
                            _scrollController.scrollToIndex(i,
                                duration: Duration(milliseconds: 500));
                            widget.select.trueSelected = i;
                            widget.attachment.indexLastSearch = i;
                            Navigator.of(context).pop();
                            Future.delayed(Duration(seconds: 1), () {
                              widget.select.falseSelected = i;
                            });
                            return;
                          }
                        }
                      }
                      Fluttertoast.showToast(
                          msg: "Search not found",
                          gravity: ToastGravity.CENTER,
                          toastLength: Toast.LENGTH_SHORT);
                      Navigator.of(context).pop();
                    }
                  },
                  decoration: InputDecoration(
                      border: InputBorder.none,
                      hintText: "Searching for... ",
                      hintStyle: TextStyle(color: Colors.grey[300]),
                      suffixIcon: widget.attachment.isSuffix
                          ? IconButton(
                              onPressed: () {
                                _controllerSearch.clear();
                                widget.attachment.isSuffix = false;
                                widget.attachment.textSearch = null;
                              },
                              icon: Icon(Icons.clear, color: Colors.grey[300]))
                          : null),
                )
              : Material(
                  color: Colors.transparent,
                  child: InkWell(
                    onTap: () async {
                      SharedPreferences prefs =
                          await SharedPreferences.getInstance();
                      String idMe = prefs.getString("hasLogin")!;
                      var resultQueryMember = await DBHelper().queryCustom(
                          "SELECT position FROM GROUPZ_MEMBER WHERE f_pin='$idMe' AND group_id='${widget.idGroup}'");
                      var getGroupType = await DBHelper().queryCustom(
                          "SELECT * FROM GROUPZ WHERE group_id='${widget.idGroup}'");
                      if (getGroupType[0]["group_type"] == 1) {
                        return;
                      } else if (resultQueryMember.length == 0) {
                        Fluttertoast.showToast(msg: "You're not in this group");
                        return;
                      }
                      Navigator.pop(context);
                      Navigator.push(
                          context,
                          PageTransition(
                              type: PageTransitionType.fade,
                              child: GroupDetail(
                                idGroup: widget.idGroup,
                                createGroup: false,
                                creator: resultQueryMember[0]["position"],
                                nameGroup: widget.nameGroup,
                              )));
                    },
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.start,
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        CircleAvatar(
                          backgroundColor: Colors.transparent,
                          child: Container(
                            decoration: BoxDecoration(
                                shape: BoxShape.circle,
                                color: Colors.grey,
                                image: DecorationImage(
                                    fit: BoxFit.cover,
                                    image: widget.image == ""
                                        ? AssetImage(
                                            "assets/images/groupicon.png",
                                            package: "palio_lite")
                                        : widget.image == "palio_logo_round.png"
                                            ? AssetImage(
                                                "assets/images/palio.png",
                                                package: "palio_lite")
                                            : LocalImage(widget.image)
                                                as ImageProvider)),
                          ),
                        ),
                        SizedBox(
                          width: 5.0,
                        ),
                        Expanded(
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: <Widget>[
                              Text(widget.nameGroup,
                                  overflow: TextOverflow.ellipsis,
                                  style: TextStyle(
                                      fontSize: 18, color: Colors.white)),
                              Text(
                                  widget.attachment.isTyping.isEmpty
                                      ? widget.nameTopic
                                      : widget.attachment.isTyping,
                                  overflow: TextOverflow.ellipsis,
                                  style: TextStyle(
                                      fontSize: 14, color: Colors.white))
                            ],
                          ),
                        )
                      ],
                    ),
                  ),
                ),
          actions: <Widget>[
            widget.attachment.isSearch && widget.attachment.justDelete == 0
                ? Row(
                    children: <Widget>[
                      IconButton(
                          icon: Icon(Icons.keyboard_arrow_up,
                              color: Colors.white),
                          iconSize: 25,
                          onPressed: () {
                            if (_controllerSearch.text.isNotEmpty) {
                              if (FocusScope.of(context).hasFocus) {
                                FocusScope.of(context).unfocus();
                              }
                              int currentIndex =
                                  widget.attachment.indexLastSearch;
                              if (currentIndex == null) {
                                currentIndex =
                                    _scrollController.currentTagIndexInViewport(
                                        preferPosition:
                                            AutoScrollPosition.begin)!;
                              } else {
                                if (currentIndex >
                                    _scrollController.currentTagIndexInViewport(
                                        preferPosition:
                                            AutoScrollPosition.begin)!) {
                                  currentIndex = _scrollController
                                      .currentTagIndexInViewport(
                                          preferPosition:
                                              AutoScrollPosition.begin)!;
                                } else {
                                  currentIndex =
                                      widget.attachment.indexLastSearch - 1;
                                }
                              }

                              showDialog(
                                  barrierDismissible: false,
                                  context: context,
                                  builder: (_) {
                                    return CupertinoAlertDialog(
                                      title: Row(
                                        mainAxisAlignment:
                                            MainAxisAlignment.start,
                                        children: <Widget>[
                                          Container(
                                              child:
                                                  CupertinoActivityIndicator(),
                                              width: 20,
                                              height: 20),
                                          SizedBox(width: 20.0),
                                          Text("Searching...",
                                              style: TextStyle(
                                                  fontWeight:
                                                      FontWeight.normal))
                                        ],
                                      ),
                                    );
                                  });
                              if (currentIndex != null) {
                                for (var i = currentIndex;
                                    i < widget.select.getChat.length;
                                    i++) {
                                  if (widget.select.getChat[i].text!
                                      .toLowerCase()
                                      .contains(_controllerSearch.text)) {
                                    _scrollController.scrollToIndex(i,
                                        duration: Duration(milliseconds: 500));
                                    widget.select.trueSelected = i;
                                    widget.attachment.indexLastSearch = i;
                                    Navigator.of(context).pop();
                                    Future.delayed(Duration(seconds: 1), () {
                                      widget.select.falseSelected = i;
                                    });
                                    return;
                                  }
                                }
                              }
                              Fluttertoast.showToast(
                                  msg: "Search not found",
                                  gravity: ToastGravity.CENTER,
                                  toastLength: Toast.LENGTH_SHORT);
                              Navigator.of(context).pop();
                            }
                          }),
                      IconButton(
                          icon: Icon(Icons.keyboard_arrow_down,
                              color: Colors.white),
                          iconSize: 25,
                          onPressed: () {
                            if (_controllerSearch.text.isNotEmpty) {
                              if (FocusScope.of(context).hasFocus) {
                                FocusScope.of(context).unfocus();
                              }
                              int currentIndex =
                                  widget.attachment.indexLastSearch;
                              if (currentIndex == null) {
                                currentIndex =
                                    _scrollController.currentTagIndexInViewport(
                                        preferPosition:
                                            AutoScrollPosition.begin)!;
                              } else {
                                if (currentIndex >
                                    _scrollController.currentTagIndexInViewport(
                                        preferPosition:
                                            AutoScrollPosition.begin)!) {
                                  currentIndex = _scrollController
                                      .currentTagIndexInViewport(
                                          preferPosition:
                                              AutoScrollPosition.begin)!;
                                } else {
                                  currentIndex =
                                      widget.attachment.indexLastSearch - 1;
                                }
                              }

                              showDialog(
                                  barrierDismissible: false,
                                  context: context,
                                  builder: (_) {
                                    return CupertinoAlertDialog(
                                      title: Row(
                                        mainAxisAlignment:
                                            MainAxisAlignment.start,
                                        children: <Widget>[
                                          Container(
                                              child:
                                                  CupertinoActivityIndicator(),
                                              width: 20,
                                              height: 20),
                                          SizedBox(width: 20.0),
                                          Text("Searching...",
                                              style: TextStyle(
                                                  fontWeight:
                                                      FontWeight.normal))
                                        ],
                                      ),
                                    );
                                  });
                              if (currentIndex != null) {
                                for (var i = currentIndex; i >= 0; i--) {
                                  if (widget.select.getChat[i].text!
                                      .toLowerCase()
                                      .contains(_controllerSearch.text)) {
                                    _scrollController.scrollToIndex(i,
                                        duration: Duration(milliseconds: 500));
                                    widget.select.trueSelected = i;
                                    widget.attachment.indexLastSearch = i;
                                    Navigator.of(context).pop();
                                    Future.delayed(Duration(seconds: 1), () {
                                      widget.select.falseSelected = i;
                                    });
                                    return;
                                  }
                                }
                              }
                              Fluttertoast.showToast(
                                  msg: "Search not found",
                                  gravity: ToastGravity.CENTER,
                                  toastLength: Toast.LENGTH_SHORT);
                              Navigator.of(context).pop();
                            }
                          }),
                    ],
                  )
                : widget.selectChat.getSelectedChat.length > 0
                    ? Row(
                        children: [
                          widget.selectChat.getSelectedChat.length == 1 &&
                                  widget.attachment.justDelete == 0
                              ? IconButton(
                                  icon: Icon(CupertinoIcons.reply_thick_solid,
                                      color: Colors.white),
                                  iconSize: 25,
                                  onPressed: () {
                                    widget.selectChat.getSelectedChat
                                        .forEach((k, v) {
                                      String? imageLocation;
                                      if (v.attachmentType == "3") {
                                        String textDecode =
                                            Uri.decodeQueryComponent(
                                                Uri.decodeFull(v.text!));
                                        int idx = textDecode.indexOf("(");
                                        var textLatLng = textDecode
                                            .substring(idx + 1,
                                                textDecode.trim().length - 1)
                                            .split(",");
                                        imageLocation =
                                            "https://maps.googleapis.com/maps/api/staticmap?center=" +
                                                textLatLng[0] +
                                                "," +
                                                textLatLng[1].trimLeft() +
                                                "&zoom=15&size=" +
                                                300.toString() +
                                                "x" +
                                                150.toString() +
                                                "&maptype=normal&markers=color:red|" +
                                                textLatLng[0] +
                                                "," +
                                                textLatLng[1].trimLeft() +
                                                "&key=AIzaSyC79oNmadezP9O_4Q_Ud2bbuJ_L7M--y1s";
                                      }
                                      widget.attachment.replyContent = {
                                        "message_id": v.messageId,
                                        "attachment_type": v.attachmentType,
                                        "fromName": v.isOriginator
                                            ? "You"
                                            : "${v.fName}",
                                        "text": v.attachmentType == "3" ||
                                                v.attachmentType == "-1"
                                            ? ""
                                            : v.text,
                                        "thumb_id": v.thumbName,
                                        "thumb_location": imageLocation
                                      };
                                      widget.select.falseSelected = k;
                                    });
                                    widget.attachment.cantForward = 0;
                                    widget.attachment.justDelete = 0;
                                    if (widget
                                        .attachment.disableFavoriteMessage) {
                                      widget.attachment.disableFavoriteMessage =
                                          false;
                                    }
                                    widget.selectChat.clearList = true;
                                  })
                              : Container(),
                          widget.selectChat.getSelectedChat.length == 1 &&
                                  widget.selectChat.getSelectedChat.values
                                          .elementAt(0)
                                          .ack ==
                                      "8" &&
                                  widget.selectChat.getSelectedChat.values
                                          .elementAt(0)
                                          .statusRead !=
                                      "8" &&
                                  !widget.selectChat.getSelectedChat.values
                                      .elementAt(0)
                                      .isOriginator
                              ? IconButton(
                                  icon: Icon(
                                    CupertinoIcons.check_mark_circled_solid,
                                    color: Colors.white,
                                  ),
                                  onPressed: () async {
                                    var resultQuery = await DBHelper().queryCustom(
                                        "SELECT f_pin FROM MESSAGE WHERE message_id='${widget.selectChat.getSelectedChat.values.elementAt(0).messageId}'");
                                    API.sendAckmessage(
                                        widget.identifier.isNotEmpty
                                            ? widget.identifier
                                            : widget.idGroup,
                                        resultQuery[0]["f_pin"],
                                        widget.selectChat.getSelectedChat.values
                                            .elementAt(0)
                                            .date
                                            .toString(),
                                        "4",
                                        widget.selectChat.getSelectedChat.values
                                            .elementAt(0)
                                            .messageId!,
                                        "",
                                        "",
                                        "");
                                    widget.selectChat.getSelectedChat
                                        .forEach((k, v) {
                                      int idx = widget.select.getChat
                                          .indexWhere((element) =>
                                              element.messageId == v.messageId);
                                      widget.select.falseSelected = idx;
                                      widget.select.updateStatus = {
                                        "index": idx,
                                        "value": "8"
                                      };
                                    });
                                    widget.attachment.justDelete = 0;
                                    widget.attachment.cantForward = 0;
                                    if (widget
                                        .attachment.disableFavoriteMessage) {
                                      widget.attachment.disableFavoriteMessage =
                                          false;
                                    }
                                    widget.selectChat.clearList = true;
                                  },
                                )
                              : Container(),
                          widget.attachment.justDelete != 0 ||
                                  widget.attachment.cantForward != 0
                              ? Container()
                              : IconButton(
                                  icon: Icon(Icons.content_copy,
                                      color: Colors.white),
                                  iconSize: 25,
                                  onPressed: () async {
                                    String text = "";
                                    List<Map<int, Chat>> temp = [];
                                    widget.selectChat.getSelectedChat.forEach(
                                        (key, value) => temp.add({key: value}));
                                    temp.sort((b, a) {
                                      int aValue = a.keys.first;
                                      int bValue = b.keys.first;
                                      return aValue > bValue ? 1 : 0;
                                    });
                                    Map<int, Chat> sortedMap = Map.fromIterable(
                                        temp,
                                        key: (e) => e.keys.first,
                                        value: (e) => e.values.first);
                                    var tempText = "";
                                    for (var i = 0; i < sortedMap.length; i++) {
                                      widget.select.falseSelected =
                                          sortedMap.keys.elementAt(i);
                                      String name = "";
                                      name =
                                          sortedMap.values.elementAt(i).fName!;
                                      tempText = sortedMap.values
                                          .elementAt(i)
                                          .text!
                                          .trim();
                                      if (tempText.contains("@")) {
                                        for (int j = 0;
                                            j <
                                                widget.attachment.getAllContacts
                                                    .length;
                                            j++) {
                                          if (tempText.contains(
                                              "@${widget.attachment.getAllContacts.values.elementAt(j)}")) {
                                            tempText = tempText.replaceAll(
                                                widget.attachment.getAllContacts
                                                    .keys
                                                    .elementAt(j),
                                                "@${widget.attachment.getAllContacts.values.elementAt(j)}");
                                          }
                                        }
                                      }
                                      text = text +
                                          "*[" +
                                          DateFormat('dd MMMM yyyy').format(
                                              sortedMap.values
                                                  .elementAt(i)
                                                  .date) +
                                          " " +
                                          sortedMap.values.elementAt(i).time! +
                                          "]" +
                                          " " +
                                          name +
                                          ":*" +
                                          "\n" +
                                          tempText +
                                          "\n";
                                      if (text.contains("@")) {
                                        var resultQuery = await DBHelper()
                                            .queryCustom(
                                                "SELECT f_pin, first_name, last_name FROM BUDDY");
                                        for (var i = 0;
                                            i < resultQuery.length;
                                            i++) {
                                          if (text.contains(
                                              "@${resultQuery[i]["f_pin"]}")) {
                                            if (resultQuery[i]["last_name"]
                                                .isNotEmpty) {
                                              text = text.replaceAll(
                                                  "@${resultQuery[i]["f_pin"]}",
                                                  "@${resultQuery[i]["first_name"]} ${resultQuery[i]["last_name"]} ");
                                            } else {
                                              text = text.replaceAll(
                                                  "@${resultQuery[i]["f_pin"]}",
                                                  "@${resultQuery[i]["first_name"]} ");
                                            }
                                          }
                                        }
                                      }
                                      if (i == sortedMap.length - 1) {
                                        text = text +
                                            "\n\n\n" +
                                            "chat powered by Palio Lite\n\n";
                                      }
                                    }
                                    widget.attachment.justDelete = 0;
                                    widget.attachment.cantForward = 0;
                                    if (widget
                                        .attachment.disableFavoriteMessage) {
                                      widget.attachment.disableFavoriteMessage =
                                          false;
                                    }
                                    widget.selectChat.clearList = true;
                                    if (tempText.isNotEmpty) {
                                      Clipboard.setData(
                                          new ClipboardData(text: text));
                                      Fluttertoast.showToast(
                                          msg: "Copied to clipboard",
                                          toastLength: Toast.LENGTH_SHORT,
                                          gravity: ToastGravity.CENTER);
                                    } else {
                                      Fluttertoast.showToast(
                                          msg: "Nothing to copy to clipboard",
                                          toastLength: Toast.LENGTH_SHORT,
                                          gravity: ToastGravity.CENTER);
                                    }
                                  }),
                          widget.attachment.cantForward > 0 ||
                                  widget.attachment.justDelete != 0
                              ? Container()
                              : IconButton(
                                  icon: Icon(CupertinoIcons.forward,
                                      color: Colors.white),
                                  iconSize: 25,
                                  onPressed: () {
                                    List<dynamic> data = [];
                                    for (var i = 0;
                                        i <
                                            widget.selectChat.getSelectedChat
                                                .length;
                                        i++) {
                                      data.add({
                                        "index": widget
                                            .selectChat.getSelectedChat.keys
                                            .elementAt(i),
                                        "data": widget
                                            .selectChat.getSelectedChat.values
                                            .elementAt(i)
                                      });
                                      widget.select.falseSelected = widget
                                          .selectChat.getSelectedChat.keys
                                          .elementAt(i);
                                    }
                                    widget.attachment.cantForward = 0;
                                    widget.attachment.justDelete = 0;
                                    if (widget
                                        .attachment.disableFavoriteMessage) {
                                      widget.attachment.disableFavoriteMessage =
                                          false;
                                    }
                                    widget.selectChat.clearList = true;
                                    if (widget.attachment.cantForward > 0) {
                                      Fluttertoast.showToast(
                                          msg: "Upload file or document first",
                                          toastLength: Toast.LENGTH_LONG);
                                      return null;
                                    }
                                    data.sort((b, a) =>
                                        a["index"].compareTo(b["index"]));
                                    showDialog(
                                        context: context,
                                        builder: (_) {
                                          return Dialog(
                                            child: Container(
                                              height: MediaQuery.of(context)
                                                  .size
                                                  .height,
                                              width: MediaQuery.of(context)
                                                  .size
                                                  .width,
                                              child: ForwardChat(data,
                                                  idGroup: widget.idGroup,
                                                  idTopic: widget.identifier),
                                            ),
                                          );
                                        });
                                  }),
                          widget.selectChat.getSelectedChat.length == 1 &&
                                  widget.selectChat.getSelectedChat.values
                                      .elementAt(0)
                                      .isOriginator &&
                                  widget.attachment.justDelete == 0
                              ? IconButton(
                                  icon: Icon(CupertinoIcons.info,
                                      color: Colors.white),
                                  iconSize: 25,
                                  onPressed: () async {
                                    var data = widget
                                        .selectChat.getSelectedChat.values
                                        .elementAt(0);
                                    widget.selectChat.getSelectedChat
                                        .forEach((k, v) {
                                      int idx = widget.select.getChat
                                          .indexWhere((element) =>
                                              element.messageId == v.messageId);
                                      widget.select.falseSelected = idx;
                                    });
                                    widget.attachment.cantForward = 0;
                                    widget.attachment.justDelete = 0;
                                    if (widget
                                        .attachment.disableFavoriteMessage) {
                                      widget.attachment.disableFavoriteMessage =
                                          false;
                                    }
                                    widget.selectChat.clearList = true;
                                    await Navigator.push(
                                        context,
                                        CupertinoPageRoute(
                                            builder: (_) => MessageInfo(
                                                data,
                                                widget.attachment
                                                    .getListMembers)));
                                    onMessage(widget.select, widget.attachment);
                                    getChat(widget.select, widget.attachment, 0,
                                        fromMessageInfo: true);
                                  })
                              : Container(),
                        ],
                      )
                    : widget.attachment.justDelete != 0
                        ? Container()
                        : IconButton(
                            icon: Icon(Icons.search, color: Colors.white),
                            iconSize: 25,
                            onPressed: () {
                              widget.attachment.isSearch =
                                  !widget.attachment.isSearch;
                              Future.delayed(const Duration(milliseconds: 500),
                                  () async {
                                _scrollController.scrollToIndex(0);
                              });
                            }),
            widget.attachment.justDelete != 0
                ? IconButton(
                    icon: Icon(CupertinoIcons.delete_solid),
                    onPressed: () async {
                      if (FocusScope.of(context).hasFocus) {
                        FocusScope.of(context).unfocus();
                        await Future.delayed(Duration(milliseconds: 100));
                      }
                      showDialog(
                          context: context,
                          builder: (context) {
                            return CupertinoAlertDialog(
                                insetAnimationCurve: Curves.easeInBack,
                                title: Text("Delete Message"),
                                content: Container(
                                  margin: EdgeInsets.only(top: 10),
                                  child: Text(
                                      "Are you sure to delete this message?"),
                                ),
                                actions: <Widget>[
                                  CupertinoDialogAction(
                                    onPressed: () {
                                      Navigator.of(context).pop();
                                      List<Chat> tempChat = [];
                                      tempChat.addAll(widget.select.getChat);
                                      for (var i = 0;
                                          i <
                                              widget.selectChat.getSelectedChat
                                                  .length;
                                          i++) {
                                        API.deleteMessage(
                                            widget.idGroup,
                                            widget.identifier,
                                            "1",
                                            "4",
                                            widget.selectChat.getSelectedChat
                                                .values
                                                .elementAt(i)
                                                .messageId!);
                                        tempChat[tempChat.indexWhere((item) =>
                                                item.messageId ==
                                                widget.selectChat
                                                    .getSelectedChat.values
                                                    .elementAt(i)
                                                    .messageId)]
                                            .isSelected = false;
                                        tempChat.removeAt(tempChat.indexWhere(
                                            (item) =>
                                                item.messageId ==
                                                widget.selectChat
                                                    .getSelectedChat.values
                                                    .elementAt(i)
                                                    .messageId));
                                      }
                                      widget.select.replaceChat = tempChat;
                                      widget.attachment.cantForward = 0;
                                      widget.attachment.justDelete = 0;
                                      if (widget
                                          .attachment.disableFavoriteMessage) {
                                        widget.attachment
                                            .disableFavoriteMessage = false;
                                      }
                                      widget.selectChat.clearList = true;
                                      if (!widget.attachment.refresh) {
                                        widget.attachment.refresh = true;
                                      }
                                    },
                                    child: Text("DELETE",
                                        style: TextStyle(
                                            color: AppColor.primaryColor,
                                            fontSize: 14)),
                                  ),
                                  CupertinoDialogAction(
                                    onPressed: () => Navigator.pop(context),
                                    child: Text("Cancel",
                                        style: TextStyle(
                                            color: AppColor.primaryColor,
                                            fontSize: 14)),
                                  ),
                                ]);
                          });
                    },
                  )
                : widget.attachment.isSearch
                    ? PopupMenuButton(
                        itemBuilder: (context) => [
                              PopupMenuItem(
                                value: 3,
                                child: Text("Delete conversation"),
                              ),
                            ],
                        onSelected: (select) async {
                          if (select == 3) {
                            if (FocusScope.of(context).hasFocus) {
                              FocusScope.of(context).unfocus();
                              await Future.delayed(Duration(milliseconds: 100));
                            }
                            showDialog(
                                context: context,
                                builder: (context) {
                                  return CupertinoAlertDialog(
                                      insetAnimationCurve: Curves.easeInBack,
                                      content: Container(
                                        margin: EdgeInsets.only(top: 10),
                                        child: Text(
                                            "Are you sure to delete all message in this conversation?"),
                                      ),
                                      actions: <Widget>[
                                        CupertinoDialogAction(
                                          onPressed: () =>
                                              Navigator.pop(context),
                                          child: Text("Cancel",
                                              style: TextStyle(
                                                  color: AppColor.primaryColor,
                                                  fontSize: 14)),
                                        ),
                                        CupertinoDialogAction(
                                          onPressed: () async {
                                            await API.clearChat(
                                                widget.idGroup,
                                                widget.identifier,
                                                "",
                                                widget.identifier.isNotEmpty
                                                    ? widget.identifier
                                                    : widget.idGroup,
                                                "4");
                                            Navigator.pop(context);
                                            Navigator.pop(context);
                                          },
                                          child: Text("Delete",
                                              style: TextStyle(
                                                  color: AppColor.primaryColor,
                                                  fontSize: 14)),
                                        )
                                      ]);
                                });
                          }
                        })
                    : widget.selectChat.getSelectedChat.length > 0
                        ? PopupMenuButton(
                            itemBuilder: (context) => [
                              widget.attachment.disableFavoriteMessage
                                  ? PopupMenuItem(child: Container())
                                  : PopupMenuItem(
                                      value: 4,
                                      child: Text(widget
                                              .selectChat.getSelectedChat.values
                                              .elementAt(0)
                                              .isFavorite
                                          ? "Delete from favourite message"
                                          : "Add to favourite message"),
                                    ),
                              PopupMenuItem(
                                value: 5,
                                child: Text("Delete Message"),
                              ),
                            ],
                            onSelected: (value) async {
                              if (value == 5) {
                                if (FocusScope.of(context).hasFocus) {
                                  FocusScope.of(context).unfocus();
                                  await Future.delayed(
                                      Duration(milliseconds: 100));
                                }
                                showDialog(
                                    context: context,
                                    builder: (context) {
                                      return CupertinoAlertDialog(
                                          insetAnimationCurve:
                                              Curves.easeInBack,
                                          title: Text("Delete Message"),
                                          content: Container(
                                            margin: EdgeInsets.only(top: 10),
                                            child: Text(
                                                "Are you sure to delete this message?"),
                                          ),
                                          actions: <Widget>[
                                            CupertinoDialogAction(
                                              onPressed: () {
                                                Navigator.of(context).pop();
                                                List<Chat> tempChat = [];
                                                tempChat.addAll(
                                                    widget.select.getChat);
                                                for (var i = 0;
                                                    i <
                                                        widget
                                                            .selectChat
                                                            .getSelectedChat
                                                            .length;
                                                    i++) {
                                                  API.deleteMessage(
                                                      widget.idGroup,
                                                      widget.identifier,
                                                      "1",
                                                      "4",
                                                      widget
                                                          .selectChat
                                                          .getSelectedChat
                                                          .values
                                                          .elementAt(i)
                                                          .messageId!);
                                                  tempChat[tempChat.indexWhere(
                                                          (item) =>
                                                              item.messageId ==
                                                              widget
                                                                  .selectChat
                                                                  .getSelectedChat
                                                                  .values
                                                                  .elementAt(i)
                                                                  .messageId)]
                                                      .isSelected = false;
                                                  tempChat.removeAt(tempChat
                                                      .indexWhere((item) =>
                                                          item.messageId ==
                                                          widget
                                                              .selectChat
                                                              .getSelectedChat
                                                              .values
                                                              .elementAt(i)
                                                              .messageId));
                                                }
                                                widget.select.replaceChat =
                                                    tempChat;
                                                widget.selectChat.clearList =
                                                    true;
                                                if (!widget
                                                    .attachment.refresh) {
                                                  widget.attachment.refresh =
                                                      true;
                                                }
                                              },
                                              child: Text("Delete",
                                                  style: TextStyle(
                                                      color:
                                                          AppColor.primaryColor,
                                                      fontSize: 14)),
                                            ),
                                            CupertinoDialogAction(
                                              onPressed: () =>
                                                  Navigator.pop(context),
                                              child: Text("Cancel",
                                                  style: TextStyle(
                                                      color:
                                                          AppColor.primaryColor,
                                                      fontSize: 14)),
                                            ),
                                            widget.attachment
                                                        .cantDeleteForEveryone >
                                                    0
                                                ? Container()
                                                : CupertinoDialogAction(
                                                    onPressed: () {
                                                      Navigator.of(context)
                                                          .pop();
                                                      for (var i = 0;
                                                          i <
                                                              widget
                                                                  .selectChat
                                                                  .getSelectedChat
                                                                  .length;
                                                          i++) {
                                                        API.deleteMessage(
                                                            widget.idGroup,
                                                            widget.identifier,
                                                            "2",
                                                            "4",
                                                            widget
                                                                .selectChat
                                                                .getSelectedChat
                                                                .values
                                                                .elementAt(i)
                                                                .messageId!);
                                                        widget.select
                                                                .falseSelected =
                                                            widget
                                                                .selectChat
                                                                .getSelectedChat
                                                                .keys
                                                                .elementAt(i);
                                                      }
                                                      widget.selectChat
                                                          .clearList = true;
                                                      if (!widget
                                                          .attachment.refresh) {
                                                        widget.attachment
                                                            .refresh = true;
                                                      }
                                                    },
                                                    child: Text(
                                                        "Delete for everyone",
                                                        style: TextStyle(
                                                            color: AppColor
                                                                .primaryColor,
                                                            fontSize: 14)),
                                                  )
                                          ]);
                                    });
                              } else {
                                widget.selectChat.getSelectedChat
                                    .forEach((k, v) {
                                  int idx = widget.select.getChat.indexWhere(
                                      (element) =>
                                          element.messageId == v.messageId);
                                  if (widget.select.getChat[idx].isFavorite) {
                                    API.deleteMessageFavorite(v.messageId!);
                                    widget.select.updateFavoriteMessage = {
                                      "index": idx,
                                      "value": false
                                    };
                                  } else {
                                    API.addFavoriteMessage(v.messageId!);
                                    widget.select.updateFavoriteMessage = {
                                      "index": idx,
                                      "value": true
                                    };
                                  }
                                  widget.select.falseSelected = idx;
                                });
                                widget.attachment.justDelete = 0;
                                widget.attachment.cantForward = 0;
                                if (widget.attachment.disableFavoriteMessage) {
                                  widget.attachment.disableFavoriteMessage =
                                      false;
                                }
                                widget.selectChat.clearList = true;
                                widget.attachment.refresh = true;
                              }
                            },
                          )
                        : PopupMenuButton(
                            itemBuilder: (context) => [
                                  PopupMenuItem(
                                    value: 3,
                                    child: Text("Delete conversation"),
                                  ),
                                ],
                            onSelected: (select) async {
                              if (select == 3) {
                                if (FocusScope.of(context).hasFocus) {
                                  FocusScope.of(context).unfocus();
                                  await Future.delayed(
                                      Duration(milliseconds: 100));
                                }
                                showDialog(
                                    context: context,
                                    builder: (context) {
                                      return CupertinoAlertDialog(
                                          insetAnimationCurve:
                                              Curves.easeInBack,
                                          content: Container(
                                            margin: EdgeInsets.only(top: 10),
                                            child: Text(
                                                "Are you sure to delete all message in this conversation?"),
                                          ),
                                          actions: <Widget>[
                                            CupertinoDialogAction(
                                              onPressed: () =>
                                                  Navigator.pop(context),
                                              child: Text("Cancel",
                                                  style: TextStyle(
                                                      color:
                                                          AppColor.primaryColor,
                                                      fontSize: 14)),
                                            ),
                                            CupertinoDialogAction(
                                              onPressed: () async {
                                                await API.clearChat(
                                                    widget.idGroup,
                                                    widget.identifier,
                                                    "",
                                                    widget.identifier.isNotEmpty
                                                        ? widget.identifier
                                                        : widget.idGroup,
                                                    "4");
                                                Navigator.pop(context);
                                                Navigator.pop(context);
                                              },
                                              child: Text("Delete",
                                                  style: TextStyle(
                                                      color:
                                                          AppColor.primaryColor,
                                                      fontSize: 14)),
                                            )
                                          ]);
                                    });
                              }
                            })
          ],
        ),
        body: widget.attachment.loading
            ? Container()
            : Column(
                crossAxisAlignment: CrossAxisAlignment.end,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  Flexible(
                    child: Stack(
                      children: <Widget>[
                        GestureDetector(
                          onTap: () async {
                            if (FocusScope.of(context).hasFocus) {
                              FocusScope.of(context).unfocus();
                              await Future.delayed(Duration(milliseconds: 100));
                            }
                          },
                          child: CustomScrollView(
                            controller: _scrollController,
                            reverse: true,
                            shrinkWrap: true,
                            physics: ClampingScrollPhysics(),
                            slivers: <Widget>[
                              SliverList(
                                delegate: SliverChildBuilderDelegate(
                                    (context, index) {
                                  bool isShowDate = false;
                                  var data = widget.select.messageChat;
                                  String date = DateTime(
                                                  data[index].date.year,
                                                  data[index].date.month,
                                                  data[index].date.day)
                                              .difference(DateTime(
                                                  DateTime.now().year,
                                                  DateTime.now().month,
                                                  DateTime.now().day))
                                              .inDays ==
                                          0
                                      ? "Today"
                                      : DateTime(data[index].date.year, data[index].date.month, data[index].date.day)
                                                  .difference(DateTime(
                                                      DateTime.now().year,
                                                      DateTime.now().month,
                                                      DateTime.now().day))
                                                  .inDays ==
                                              -1
                                          ? "Yesterday"
                                          : DateTime(
                                                          data[index].date.year,
                                                          data[index]
                                                              .date
                                                              .month,
                                                          data[index].date.day)
                                                      .difference(DateTime(
                                                          DateTime.now().year,
                                                          DateTime.now().month,
                                                          DateTime.now().day))
                                                      .inDays ==
                                                  -2
                                              ? "2 days ago"
                                              : DateFormat('dd MMMM yyyy')
                                                  .format(data[index].date);
                                  if (data.length == 1) {
                                    isShowDate = true;
                                  } else {
                                    if (index == (data.length - 1)) {
                                      isShowDate = true;
                                    }
                                    if (index + 1 != data.length) {
                                      DateTime date1 = DateTime(
                                          data[index].date.year,
                                          data[index].date.month,
                                          data[index].date.day);
                                      DateTime date2 = DateTime(
                                          data[index + 1].date.year,
                                          data[index + 1].date.month,
                                          data[index + 1].date.day);
                                      if (date1 != date2) {
                                        isShowDate = true;
                                      }
                                    }
                                  }
                                  return AutoScrollTag(
                                    controller: _scrollController,
                                    index: index,
                                    child: Column(
                                      crossAxisAlignment:
                                          CrossAxisAlignment.stretch,
                                      children: <Widget>[
                                        isShowDate
                                            ? Center(
                                                child: Container(
                                                    child: Text(
                                                      date,
                                                      style: TextStyle(
                                                          fontSize: 10,
                                                          fontWeight:
                                                              FontWeight.bold),
                                                    ),
                                                    height: 24,
                                                    margin:
                                                        EdgeInsets.all(10.0),
                                                    padding:
                                                        EdgeInsets.all(5.0),
                                                    decoration: BoxDecoration(
                                                        border: Border.all(
                                                            color: Colors
                                                                .blueAccent),
                                                        borderRadius:
                                                            BorderRadius
                                                                .circular(
                                                                    3.0))),
                                              )
                                            : Container(),
                                        BubbleChatGroup(
                                          isOriginator:
                                              data[index].isOriginator,
                                          text: data[index].text,
                                          fileName: data[index].fileName,
                                          thumbName: data[index].thumbName,
                                          attachmentType:
                                              data[index].attachmentType,
                                          credential: data[index].credential,
                                          messageId: data[index].messageId,
                                          time: data[index].time,
                                          refId: data[index].refId,
                                          refFrom: data[index].refFrom,
                                          refType: data[index].refType,
                                          fName: data[index].fName,
                                          fImage: data[index].fImage,
                                          ack: data[index].ack,
                                          isSearch: widget.attachment.isSearch,
                                          link: getLink(data[index].text!),
                                          isFavorite: data[index].isFavorite,
                                          listMentions:
                                              widget.attachment.getListMembers,
                                          allContacts:
                                              widget.attachment.getAllContacts,
                                          textSearch:
                                              widget.attachment.textSearch,
                                          refText: data[index].refText,
                                          progress: data[index].progress,
                                          refThumbLoc: data[index].refThumbLoc,
                                          refThumbId: data[index].refThumbId,
                                          statusRead: data[index].statusRead,
                                          isSelected: data[index].isSelected,
                                          onTapReply: () async {
                                            if (widget.selectChat
                                                    .getSelectedChat.length ==
                                                0) {
                                              int idx = widget.select.getChat
                                                  .indexOf(Chat(
                                                      messageId:
                                                          data[index].refId));
                                              if (idx != -1) {
                                                _scrollController.scrollToIndex(
                                                    idx,
                                                    duration: Duration(
                                                        milliseconds: 500));
                                                widget.select.trueSelected =
                                                    idx;
                                                Future.delayed(
                                                    Duration(seconds: 1), () {
                                                  widget.select.falseSelected =
                                                      idx;
                                                });
                                              } else {
                                                int? idxDatabase;
                                                List<dynamic> resultQuery;
                                                if (widget.identifier != "") {
                                                  resultQuery = await DBHelper()
                                                      .queryCustom(
                                                          "SELECT * FROM MESSAGE where chat_id='${widget.identifier}' order by server_date");
                                                } else {
                                                  resultQuery = await DBHelper()
                                                      .queryCustom(
                                                          "SELECT * FROM MESSAGE where chat_id = '' AND l_pin='${widget.idGroup}' order by server_date desc");
                                                }
                                                if (resultQuery.length != 0) {
                                                  resultQuery.sort((b, a) => a[
                                                          'server_date']
                                                      .compareTo(
                                                          b['server_date']));
                                                }
                                                for (var i = 0;
                                                    i < resultQuery.length;
                                                    i++) {
                                                  if (resultQuery[i]
                                                          ['message_id'] ==
                                                      data[index].refId) {
                                                    idxDatabase = i;
                                                    break;
                                                  }
                                                }
                                                await getChat(
                                                    widget.select,
                                                    widget.attachment,
                                                    (widget.select.getChat
                                                            .length -
                                                        1),
                                                    end: (idxDatabase! -
                                                            (widget
                                                                    .select
                                                                    .getChat
                                                                    .length -
                                                                1)) +
                                                        1);
                                                idx = widget.select.getChat
                                                    .indexOf(Chat(
                                                        messageId:
                                                            data[index].refId));
                                                _scrollController.scrollToIndex(
                                                    idx,
                                                    duration: Duration(
                                                        milliseconds: 500));
                                                widget.select.trueSelected =
                                                    idx;
                                                Future.delayed(
                                                    Duration(seconds: 1), () {
                                                  widget.select.falseSelected =
                                                      idx;
                                                });
                                              }
                                            } else {
                                              {
                                                if (!data[index].isSelected) {
                                                  if (widget
                                                          .selectChat
                                                          .getSelectedChat
                                                          .length >
                                                      0) {
                                                    bool different = widget
                                                        .selectChat
                                                        .getSelectedChat
                                                        .values
                                                        .elementAt(widget
                                                                .selectChat
                                                                .getSelectedChat
                                                                .length -
                                                            1)
                                                        .isFavorite;
                                                    if (different !=
                                                        data[index]
                                                            .isFavorite) {
                                                      if (!widget.attachment
                                                          .disableFavoriteMessage) {
                                                        widget.attachment
                                                                .disableFavoriteMessage =
                                                            true;
                                                      }
                                                    }
                                                  }

                                                  widget.select.trueSelected =
                                                      index;
                                                  widget.selectChat.addList = {
                                                    "index": index,
                                                    "data": data[index]
                                                  };
                                                  if (data[index]
                                                              .attachmentType ==
                                                          "11" ||
                                                      (data[index].attachmentType !=
                                                              "11" &&
                                                          data[index]
                                                                  .attachmentType !=
                                                              "0" &&
                                                          data[index]
                                                                  .attachmentType !=
                                                              "3" &&
                                                          !File(widget
                                                                      .attachment
                                                                      .path
                                                                      .path +
                                                                  "/" +
                                                                  data[index]
                                                                      .fileName!)
                                                              .existsSync())) {
                                                    widget.attachment
                                                        .cantForward = widget
                                                            .attachment
                                                            .cantForward +
                                                        1;
                                                  }
                                                  if (data[index]
                                                      .text!
                                                      .contains("🚫")) {
                                                    widget.attachment
                                                        .justDelete = widget
                                                            .attachment
                                                            .justDelete +
                                                        1;
                                                  }
                                                } else if (widget
                                                    .select
                                                    .messageChat[index]
                                                    .isSelected) {
                                                  widget.select.falseSelected =
                                                      index;
                                                  widget.selectChat.removeList =
                                                      index;
                                                  if (widget
                                                          .selectChat
                                                          .getSelectedChat
                                                          .length >
                                                      0) {
                                                    if (widget
                                                            .selectChat
                                                            .getSelectedChat
                                                            .length ==
                                                        1) {
                                                      if (widget.attachment
                                                          .disableFavoriteMessage) {
                                                        widget.attachment
                                                                .disableFavoriteMessage =
                                                            false;
                                                      }
                                                    } else {
                                                      bool different2 = widget
                                                          .selectChat
                                                          .getSelectedChat
                                                          .values
                                                          .elementAt(widget
                                                                  .selectChat
                                                                  .getSelectedChat
                                                                  .length -
                                                              1)
                                                          .isFavorite;
                                                      bool different1 = widget
                                                          .selectChat
                                                          .getSelectedChat
                                                          .values
                                                          .elementAt(widget
                                                                  .selectChat
                                                                  .getSelectedChat
                                                                  .length -
                                                              2)
                                                          .isFavorite;
                                                      if (different1 ==
                                                          different2) {
                                                        if (widget.attachment
                                                            .disableFavoriteMessage) {
                                                          widget.attachment
                                                                  .disableFavoriteMessage =
                                                              false;
                                                        }
                                                      }
                                                    }
                                                  }

                                                  if (data[index]
                                                              .attachmentType ==
                                                          "11" ||
                                                      (data[index].attachmentType !=
                                                              "11" &&
                                                          data[index]
                                                                  .attachmentType !=
                                                              "0" &&
                                                          data[index]
                                                                  .attachmentType !=
                                                              "3" &&
                                                          !File(widget
                                                                      .attachment
                                                                      .path
                                                                      .path +
                                                                  "/" +
                                                                  data[index]
                                                                      .fileName!)
                                                              .existsSync())) {
                                                    widget.attachment
                                                        .cantForward = widget
                                                            .attachment
                                                            .cantForward -
                                                        1;
                                                  }
                                                  if (data[index]
                                                          .text!
                                                          .contains("🚫") &&
                                                      widget.attachment
                                                              .justDelete !=
                                                          0) {
                                                    widget.attachment
                                                        .justDelete = widget
                                                            .attachment
                                                            .justDelete -
                                                        1;
                                                  }
                                                }
                                              }
                                            }
                                          },
                                          onLongPress: () {
                                            if (!data[index].isSelected) {
                                              if (widget.selectChat
                                                      .getSelectedChat.length >
                                                  0) {
                                                bool different = widget
                                                    .selectChat
                                                    .getSelectedChat
                                                    .values
                                                    .elementAt(widget
                                                            .selectChat
                                                            .getSelectedChat
                                                            .length -
                                                        1)
                                                    .isFavorite;
                                                if (different !=
                                                    data[index].isFavorite) {
                                                  if (!widget.attachment
                                                      .disableFavoriteMessage) {
                                                    widget.attachment
                                                            .disableFavoriteMessage =
                                                        true;
                                                  }
                                                }
                                              }
                                              widget.select.trueSelected =
                                                  index;
                                              widget.selectChat.addList = {
                                                "index": index,
                                                "data": data[index]
                                              };
                                              if (data[index]
                                                          .attachmentType ==
                                                      "11" ||
                                                  (data[index]
                                                              .attachmentType !=
                                                          "11" &&
                                                      data[index]
                                                              .attachmentType !=
                                                          "0" &&
                                                      data[index]
                                                              .attachmentType !=
                                                          "3" &&
                                                      !File(widget.attachment
                                                                  .path.path +
                                                              "/" +
                                                              data[index]
                                                                  .fileName!)
                                                          .existsSync())) {
                                                widget.attachment.cantForward =
                                                    widget.attachment
                                                            .cantForward +
                                                        1;
                                              }
                                              if (data[index]
                                                  .text!
                                                  .contains("🚫")) {
                                                widget.attachment.justDelete =
                                                    widget.attachment
                                                            .justDelete +
                                                        1;
                                              }
                                              if (!data[index].isOriginator) {
                                                widget.attachment
                                                    .cantDeleteForEveryone = widget
                                                        .attachment
                                                        .cantDeleteForEveryone +
                                                    1;
                                              }
                                            } else if (data[index].isSelected) {
                                              widget.select.falseSelected =
                                                  index;
                                              widget.selectChat.removeList =
                                                  index;
                                              if (widget.selectChat
                                                      .getSelectedChat.length >
                                                  0) {
                                                if (widget
                                                        .selectChat
                                                        .getSelectedChat
                                                        .length ==
                                                    1) {
                                                  if (widget.attachment
                                                      .disableFavoriteMessage) {
                                                    widget.attachment
                                                            .disableFavoriteMessage =
                                                        false;
                                                  }
                                                } else {
                                                  bool different2 = widget
                                                      .selectChat
                                                      .getSelectedChat
                                                      .values
                                                      .elementAt(widget
                                                              .selectChat
                                                              .getSelectedChat
                                                              .length -
                                                          1)
                                                      .isFavorite;
                                                  bool different1 = widget
                                                      .selectChat
                                                      .getSelectedChat
                                                      .values
                                                      .elementAt(widget
                                                              .selectChat
                                                              .getSelectedChat
                                                              .length -
                                                          2)
                                                      .isFavorite;
                                                  if (different1 ==
                                                      different2) {
                                                    if (widget.attachment
                                                        .disableFavoriteMessage) {
                                                      widget.attachment
                                                              .disableFavoriteMessage =
                                                          false;
                                                    }
                                                  }
                                                }
                                              }

                                              if (data[index]
                                                          .attachmentType ==
                                                      "11" ||
                                                  (data[index]
                                                              .attachmentType !=
                                                          "11" &&
                                                      data[index]
                                                              .attachmentType !=
                                                          "0" &&
                                                      data[index]
                                                              .attachmentType !=
                                                          "3" &&
                                                      !File(widget.attachment
                                                                  .path.path +
                                                              "/" +
                                                              data[index]
                                                                  .fileName!)
                                                          .existsSync())) {
                                                widget.attachment.cantForward =
                                                    widget.attachment
                                                            .cantForward -
                                                        1;
                                              }
                                              if (data[index]
                                                      .text!
                                                      .contains("🚫") &&
                                                  widget.attachment
                                                          .justDelete !=
                                                      0) {
                                                widget.attachment.justDelete =
                                                    widget.attachment
                                                            .justDelete -
                                                        1;
                                              }
                                              if (!data[index].isOriginator) {
                                                widget.attachment
                                                    .cantDeleteForEveryone = widget
                                                        .attachment
                                                        .cantDeleteForEveryone -
                                                    1;
                                              }
                                            }
                                          },
                                          onTapPress: () async {
                                            if (widget.selectChat
                                                    .getSelectedChat.length >
                                                0) {
                                              if (!data[index].isSelected) {
                                                print("masuk 1");
                                                if (widget
                                                        .selectChat
                                                        .getSelectedChat
                                                        .length >
                                                    0) {
                                                  bool different = widget
                                                      .selectChat
                                                      .getSelectedChat
                                                      .values
                                                      .elementAt(widget
                                                              .selectChat
                                                              .getSelectedChat
                                                              .length -
                                                          1)
                                                      .isFavorite;
                                                  if (different !=
                                                      data[index].isFavorite) {
                                                    if (!widget.attachment
                                                        .disableFavoriteMessage) {
                                                      widget.attachment
                                                              .disableFavoriteMessage =
                                                          true;
                                                    }
                                                  }
                                                }

                                                widget.select.trueSelected =
                                                    index;
                                                widget.selectChat.addList = {
                                                  "index": index,
                                                  "data": data[index]
                                                };
                                                if (data[index]
                                                            .attachmentType ==
                                                        "11" ||
                                                    (data[index]
                                                                .attachmentType !=
                                                            "11" &&
                                                        data[index]
                                                                .attachmentType !=
                                                            "0" &&
                                                        data[index]
                                                                .attachmentType !=
                                                            "3" &&
                                                        !File(widget.attachment
                                                                    .path.path +
                                                                "/" +
                                                                data[index]
                                                                    .fileName!)
                                                            .existsSync())) {
                                                  widget.attachment
                                                      .cantForward = widget
                                                          .attachment
                                                          .cantForward +
                                                      1;
                                                }
                                                if (data[index]
                                                    .text!
                                                    .contains("🚫")) {
                                                  widget.attachment.justDelete =
                                                      widget.attachment
                                                              .justDelete +
                                                          1;
                                                }
                                                if (!data[index].isOriginator) {
                                                  widget.attachment
                                                      .cantDeleteForEveryone = widget
                                                          .attachment
                                                          .cantDeleteForEveryone +
                                                      1;
                                                }
                                              } else if (widget
                                                  .select
                                                  .messageChat[index]
                                                  .isSelected) {
                                                print("masuk 2");
                                                widget.select.falseSelected =
                                                    index;
                                                widget.selectChat.removeList =
                                                    index;
                                                if (widget
                                                        .selectChat
                                                        .getSelectedChat
                                                        .length >
                                                    0) {
                                                  if (widget
                                                          .selectChat
                                                          .getSelectedChat
                                                          .length ==
                                                      1) {
                                                    if (widget.attachment
                                                        .disableFavoriteMessage) {
                                                      widget.attachment
                                                              .disableFavoriteMessage =
                                                          false;
                                                    }
                                                  } else {
                                                    bool different2 = widget
                                                        .selectChat
                                                        .getSelectedChat
                                                        .values
                                                        .elementAt(widget
                                                                .selectChat
                                                                .getSelectedChat
                                                                .length -
                                                            1)
                                                        .isFavorite;
                                                    bool different1 = widget
                                                        .selectChat
                                                        .getSelectedChat
                                                        .values
                                                        .elementAt(widget
                                                                .selectChat
                                                                .getSelectedChat
                                                                .length -
                                                            2)
                                                        .isFavorite;
                                                    if (different1 ==
                                                        different2) {
                                                      if (widget.attachment
                                                          .disableFavoriteMessage) {
                                                        widget.attachment
                                                                .disableFavoriteMessage =
                                                            false;
                                                      }
                                                    }
                                                  }
                                                }

                                                if (data[index]
                                                            .attachmentType ==
                                                        "11" ||
                                                    (data[index]
                                                                .attachmentType !=
                                                            "11" &&
                                                        data[index]
                                                                .attachmentType !=
                                                            "0" &&
                                                        data[index]
                                                                .attachmentType !=
                                                            "3" &&
                                                        !File(widget.attachment
                                                                    .path.path +
                                                                "/" +
                                                                data[index]
                                                                    .fileName!)
                                                            .existsSync())) {
                                                  widget.attachment
                                                      .cantForward = widget
                                                          .attachment
                                                          .cantForward -
                                                      1;
                                                }
                                                if (data[index]
                                                        .text!
                                                        .contains("🚫") &&
                                                    widget.attachment
                                                            .justDelete !=
                                                        0) {
                                                  widget.attachment.justDelete =
                                                      widget.attachment
                                                              .justDelete -
                                                          1;
                                                }
                                                if (!data[index].isOriginator) {
                                                  widget.attachment
                                                      .cantDeleteForEveryone = widget
                                                          .attachment
                                                          .cantDeleteForEveryone -
                                                      1;
                                                }
                                              }
                                            } else if (data[index].fileName !=
                                                    null &&
                                                data[index]
                                                    .fileName!
                                                    .isNotEmpty) {
                                              if (data[index].attachmentType == "2" ||
                                                  data[index].attachmentType ==
                                                      "6" ||
                                                  data[index].attachmentType ==
                                                      "-1") {
                                                if (!File(widget.attachment.path
                                                                .path +
                                                            "/" +
                                                            data[index]
                                                                .fileName!)
                                                        .existsSync() &&
                                                    data[index].progress ==
                                                        null) {
                                                  Fluttertoast.showToast(
                                                      msg: "Downloading...",
                                                      toastLength:
                                                          Toast.LENGTH_SHORT);
                                                  API.download(
                                                      data[index].fileName!);
                                                } else if (File(widget
                                                            .attachment
                                                            .path
                                                            .path +
                                                        "/" +
                                                        data[index].fileName!)
                                                    .existsSync()) {
                                                  OpenFile.open(widget
                                                          .attachment
                                                          .path
                                                          .path +
                                                      "/" +
                                                      data[index].fileName!);
                                                }
                                              } else if (data[index]
                                                      .attachmentType ==
                                                  "1") {
                                                if (!File(widget.attachment.path
                                                                .path +
                                                            "/" +
                                                            data[index]
                                                                .fileName!)
                                                        .existsSync() &&
                                                    data[index].progress ==
                                                        null) {
                                                  Fluttertoast.showToast(
                                                      msg: "Downloading...",
                                                      toastLength:
                                                          Toast.LENGTH_SHORT);
                                                  API.download(
                                                      data[index].fileName!);
                                                } else if (File(widget
                                                            .attachment
                                                            .path
                                                            .path +
                                                        "/" +
                                                        data[index].fileName!)
                                                    .existsSync()) {
                                                  Navigator.push(
                                                      context,
                                                      PageTransition(
                                                          type:
                                                              PageTransitionType
                                                                  .fade,
                                                          child: Stack(
                                                            children: <Widget>[
                                                              Center(
                                                                child: PhotoView(
                                                                    imageProvider: FileImage(File(widget
                                                                            .attachment
                                                                            .path
                                                                            .path +
                                                                        "/" +
                                                                        data[index]
                                                                            .fileName!)),
                                                                    backgroundDecoration:
                                                                        BoxDecoration(
                                                                            color:
                                                                                Colors.blueGrey)),
                                                              ),
                                                              Align(
                                                                alignment:
                                                                    Alignment
                                                                        .topLeft,
                                                                child:
                                                                    Container(
                                                                  child:
                                                                      Material(
                                                                    color: Colors
                                                                        .transparent,
                                                                    child:
                                                                        GestureDetector(
                                                                      child:
                                                                          Row(
                                                                        mainAxisAlignment:
                                                                            MainAxisAlignment.start,
                                                                        children: <
                                                                            Widget>[
                                                                          Icon(
                                                                              Icons.arrow_back_ios,
                                                                              color: Colors.white),
                                                                          Text(
                                                                              "Back",
                                                                              style: TextStyle(color: Colors.white, fontSize: 18))
                                                                        ],
                                                                      ),
                                                                      onTap: () =>
                                                                          Navigator.pop(
                                                                              context),
                                                                    ),
                                                                  ),
                                                                  margin: EdgeInsets.only(
                                                                      top: 40.0,
                                                                      left:
                                                                          5.0),
                                                                ),
                                                              )
                                                            ],
                                                          )));
                                                }
                                              }
                                            } else if (data[index]
                                                    .attachmentType ==
                                                "3") {
                                              String textDecode =
                                                  Uri.decodeQueryComponent(
                                                      Uri.decodeFull(
                                                          data[index].text!));
                                              int idx = textDecode.indexOf("(");
                                              var textLatLng = textDecode
                                                  .substring(
                                                      idx + 1,
                                                      textDecode.trim().length -
                                                          1)
                                                  .split(",");
                                              final url =
                                                  'https://www.google.com/maps/search/?api=1&query=${textLatLng[0]},${textLatLng[1].trimLeft()}';
                                              if (await canLaunch(url)) {
                                                await launch(url);
                                              }
                                            }
                                          },
                                          onSelectProfile: () async {
                                            if (data[index].fId != "-999") {
                                              List<dynamic> resultQuery =
                                                  await DBHelper().queryCustom(
                                                      "SELECT * FROM BUDDY where f_pin='${data[index].fId}'");
                                              Navigator.push(
                                                  context,
                                                  PageTransition(
                                                      type: PageTransitionType
                                                          .fade,
                                                      child: PAProfile(
                                                        profileDet: {
                                                          'f_pin':
                                                              data[index].fId,
                                                          'myProfile': false,
                                                          'name': resultQuery[0]
                                                              ["first_name"],
                                                          'follower': 0,
                                                          'official_flag': resultQuery[
                                                                          0][
                                                                      "official_account"] ==
                                                                  "1"
                                                              ? 1
                                                              : resultQuery[0][
                                                                          "official_account"] ==
                                                                      "2"
                                                                  ? 0
                                                                  : 2,
                                                          'isFriend': 1,
                                                          'post': 0,
                                                          'following': 0,
                                                          'isOffline':
                                                              resultQuery[0][
                                                                  "offline_mode"],
                                                          'isMuted':
                                                              resultQuery[0]
                                                                  ["muted"],
                                                          'industry': resultQuery[
                                                                  0][
                                                              "bidang_industri"],
                                                          'email_address':
                                                              resultQuery[0]
                                                                  ["email"],
                                                          'phone_number':
                                                              resultQuery[0]
                                                                  ["msisdn"],
                                                          'website':
                                                              resultQuery[0]
                                                                  ["web"],
                                                          'address':
                                                              resultQuery[0]
                                                                  ["address"],
                                                          'status':
                                                              resultQuery[0]
                                                                  ["quote"],
                                                          'isOnline': int.parse(
                                                              resultQuery[0][
                                                                  "connected"]),
                                                          'image':
                                                              resultQuery[0]
                                                                  ["image_id"],
                                                          'official_account':
                                                              resultQuery[0][
                                                                  "official_account"],
                                                          'user_type':
                                                              resultQuery[0]
                                                                  ["user_type"]
                                                        },
                                                      )));
                                            }
                                          },
                                        )
                                      ],
                                    ),
                                  );
                                },
                                    childCount:
                                        widget.select.messageChat.length),
                              )
                            ],
                          ),
                        ),
                        ShowDate(widget.attachment.isShowDate,
                            widget.attachment.dateIndex),
                        widget.attachment.isMention
                            ? Align(
                                alignment: Alignment.bottomLeft,
                                child: Container(
                                  margin: EdgeInsets.only(left: 5.0),
                                  constraints: BoxConstraints(
                                      maxHeight: 250, minHeight: 80),
                                  width: 250,
                                  child: ListView.builder(
                                      shrinkWrap: true,
                                      reverse: true,
                                      itemBuilder: (context, i) {
                                        return Container(
                                          alignment: Alignment.centerLeft,
                                          width: 250,
                                          height: 50,
                                          decoration: BoxDecoration(
                                              color: Colors.white,
                                              borderRadius:
                                                  BorderRadius.circular(4.0)),
                                          margin: EdgeInsets.only(top: 5.0),
                                          child: Material(
                                            color: Colors.transparent,
                                            child: InkWell(
                                              onTap: () {
                                                String text = _controllerEditor
                                                    .text
                                                    .substring(
                                                        0,
                                                        _controllerEditor
                                                            .selection
                                                            .baseOffset);
                                                String text2 = _controllerEditor
                                                    .text
                                                    .substring(_controllerEditor
                                                        .selection.baseOffset);
                                                List<String> temp_text = [];
                                                if (text.contains("\n")) {
                                                  temp_text = text.split("\n");

                                                  var enter_split = temp_text[
                                                          temp_text.length - 1]
                                                      .split(" ");
                                                  if (text2.isNotEmpty) {
                                                    enter_split[
                                                        enter_split.length -
                                                            1] = "@" +
                                                        widget
                                                            .attachment
                                                            .getTempMentions
                                                            .values
                                                            .elementAt(i);

                                                    temp_text[temp_text.length -
                                                            1] =
                                                        enter_split.join(" ");
                                                    temp_text[temp_text.length -
                                                            1] =
                                                        "@" +
                                                            widget
                                                                .attachment
                                                                .getTempMentions
                                                                .values
                                                                .elementAt(i);
                                                    temp_text.add(text2);
                                                  } else {
                                                    enter_split[
                                                        enter_split.length -
                                                            1] = "@" +
                                                        widget
                                                            .attachment
                                                            .getTempMentions
                                                            .values
                                                            .elementAt(i) +
                                                        " ";

                                                    temp_text[temp_text.length -
                                                            1] =
                                                        enter_split.join(" ");
                                                  }
                                                  text = temp_text.join("\n");
                                                } else {
                                                  temp_text = text.split(" ");
                                                  if (text2.isNotEmpty) {
                                                    temp_text[temp_text.length -
                                                            1] =
                                                        "@" +
                                                            widget
                                                                .attachment
                                                                .getTempMentions
                                                                .values
                                                                .elementAt(i);
                                                    temp_text.add(text2);
                                                  } else {
                                                    temp_text[temp_text.length -
                                                            1] =
                                                        "@" +
                                                            widget
                                                                .attachment
                                                                .getTempMentions
                                                                .values
                                                                .elementAt(i) +
                                                            " ";
                                                  }
                                                  text = temp_text.join(" ");
                                                }

                                                widget.attachment.isMention =
                                                    false;
                                                int newPosition = text.length;
                                                _controllerEditor.value =
                                                    TextEditingValue(
                                                  text: text,
                                                  selection:
                                                      TextSelection.collapsed(
                                                          offset: newPosition),
                                                );
                                              },
                                              child: Container(
                                                margin: EdgeInsets.all(10.0),
                                                child: Row(
                                                  crossAxisAlignment:
                                                      CrossAxisAlignment.center,
                                                  mainAxisAlignment:
                                                      MainAxisAlignment.start,
                                                  children: [
                                                    FutureBuilder(
                                                      future: DBHelper()
                                                          .queryCustom(
                                                              "SELECT image_id FROM BUDDY WHERE f_pin='${widget.attachment.getTempMentions.keys.elementAt(i).split("@")[1]}'"),
                                                      builder:
                                                          (context, snaps) {
                                                        if (snaps.hasData) {
                                                          var value = snaps.data
                                                              as List<dynamic>;
                                                          if (value[0]
                                                                  ["image_id"]
                                                              .trim()
                                                              .isNotEmpty) {
                                                            return Container(
                                                              width: 30,
                                                              height: 30,
                                                              decoration: BoxDecoration(
                                                                  shape: BoxShape
                                                                      .circle,
                                                                  color: Colors
                                                                      .grey,
                                                                  image: DecorationImage(
                                                                      fit: BoxFit
                                                                          .cover,
                                                                      image: value[0]["image_id"] ==
                                                                              "palio_logo_round.png"
                                                                          ? AssetImage(
                                                                              "assets/images/palio.png",
                                                                              package:
                                                                                  "palio_lite")
                                                                          : LocalImage(value[0]["image_id"])
                                                                              as ImageProvider)),
                                                            );
                                                          }
                                                        }
                                                        return Container(
                                                          width: 30,
                                                          height: 30,
                                                          decoration: BoxDecoration(
                                                              shape: BoxShape
                                                                  .circle,
                                                              color:
                                                                  Colors.grey,
                                                              image: DecorationImage(
                                                                  fit: BoxFit
                                                                      .cover,
                                                                  image: AssetImage(
                                                                      "assets/images/default_pp.png",
                                                                      package:
                                                                          "palio_lite"))),
                                                        );
                                                      },
                                                    ),
                                                    SizedBox(width: 5.0),
                                                    Text(widget.attachment
                                                        .getTempMentions.values
                                                        .elementAt(i))
                                                  ],
                                                ),
                                              ),
                                            ),
                                          ),
                                        );
                                      },
                                      itemCount: widget
                                          .attachment.getTempMentions.length),
                                ),
                              )
                            : Container(width: 0.0, height: 0.0),
                        TapToBottom(widget.attachment.isNotBottom, () {
                          _scrollController.scrollToIndex(0);
                        }, widget.select.getNewMessages.length)
                      ],
                    ),
                  ),
                  Column(
                    crossAxisAlignment: CrossAxisAlignment.end,
                    children: <Widget>[
                      widget.attachment.textLink != null
                          ? Container(
                              padding: EdgeInsets.only(top: 5.0, bottom: 5.0),
                              color: Colors.white,
                              child: Stack(
                                children: [
                                  widget.attachment.hitLink
                                      ? LinkPreviewer(
                                          widget.attachment.textLink, true)
                                      : Container(
                                          width: double.infinity,
                                          decoration: BoxDecoration(
                                              color: Colors.grey[100],
                                              borderRadius:
                                                  BorderRadius.circular(8.0)),
                                          padding: EdgeInsets.only(
                                              top: 10.0,
                                              bottom: 10.0,
                                              left: 5.0,
                                              right: 5.0),
                                          child:
                                              Text(widget.attachment.textLink),
                                        ),
                                  Align(
                                    alignment: Alignment.topRight,
                                    child: Material(
                                      color: Colors.transparent,
                                      child: InkWell(
                                        onTap: () {
                                          widget.attachment.textLink = null;
                                          widget.attachment.hitLink = false;
                                        },
                                        child: Icon(Icons.cancel,
                                            color:
                                                Colors.grey.withOpacity(0.5)),
                                      ),
                                    ),
                                  )
                                ],
                              ),
                            )
                          : Container(),
                      widget.attachment.getListImage.length > 0
                          ? screenImageList(widget.attachment)
                          : widget.attachment.getVideo != null
                              ? screenVideo(widget.attachment)
                              : Container(),
                      clip(widget.attachment),
                      _textComposerWidget(widget.attachment),
                      !widget.attachment.clickedSticker
                          ? Container(width: 0.0, height: 0.0)
                          : Container(
                              height: 250,
                              child: Sticker(
                                stickerClicked: (sticker) async {
                                  SharedPreferences prefs =
                                      await SharedPreferences.getInstance();
                                  String idMe = prefs.getString("hasLogin")!;
                                  widget.attachment.clickedSticker = false;
                                  var resultSend = await API.sendMessage(
                                      lPin: widget.idGroup,
                                      chatId: widget.identifier,
                                      messageScopeId: "4",
                                      status: "2",
                                      reffId:
                                          widget.attachment.getReplyContent !=
                                                  null
                                              ? widget.attachment
                                                  .getReplyContent["message_id"]
                                              : "",
                                      attachmentFlag: "11",
                                      readReceipts:
                                          widget.attachment.getAckActive
                                              ? "8"
                                              : "",
                                      messageText: sticker);
                                  var formatTime = new DateFormat('Hm');
                                  String? refId,
                                      refName,
                                      refText,
                                      refType,
                                      refThumbId,
                                      refThumbLoc;
                                  if (widget.attachment.getReplyContent !=
                                      null) {
                                    refId = widget.attachment
                                        .getReplyContent["message_id"];
                                    refName = widget
                                        .attachment.getReplyContent["fromName"];
                                    refText = widget.attachment.getReplyContent[
                                                    "attachment_type"] ==
                                                "3" ||
                                            widget.attachment.getReplyContent[
                                                    "attachment_type"] ==
                                                "-1"
                                        ? ""
                                        : widget
                                            .attachment.getReplyContent["text"];
                                    refType = widget.attachment
                                        .getReplyContent["attachment_type"];
                                    refThumbId = widget
                                        .attachment.getReplyContent["thumb_id"];
                                    refThumbLoc = widget.attachment
                                        .getReplyContent["thumb_location"];
                                  }
                                  var resultQueryBuddy = await DBHelper()
                                      .queryCustom(
                                          "SELECT first_name, last_name, image_id FROM BUDDY WHERE f_pin=='$idMe'");
                                  widget.select.addChat = Chat(
                                      messageId: resultSend,
                                      isOriginator: true,
                                      text: sticker,
                                      statusRead: "1",
                                      attachmentType: "11",
                                      refFrom: refName!,
                                      refType: refType!,
                                      refText: refText!,
                                      refThumbLoc: refThumbLoc!,
                                      refId: refId!,
                                      fName: resultQueryBuddy[0]["first_name"] +
                                          " " +
                                          resultQueryBuddy[0]["last_name"],
                                      fImage: resultQueryBuddy[0]["image_id"],
                                      refThumbId: refThumbId!,
                                      ack: widget.attachment.getAckActive
                                          ? "8"
                                          : "",
                                      time: formatTime.format(DateTime.now()),
                                      date: DateTime.now());
                                  _controllerEditor.clear();
                                  widget.attachment.refresh = true;
                                  widget.attachment.replyContent = null;
                                  Future.delayed(Duration(milliseconds: 50),
                                      () {
                                    _scrollController.animateTo(0.0,
                                        duration: Duration(milliseconds: 1),
                                        curve: Curves.easeOut);
                                  });
                                  if (widget.selectChat.getSelectedChat.length >
                                      0) {
                                    widget.selectChat.getSelectedChat
                                        .forEach((k, v) {
                                      int idx = widget.select.getChat
                                          .indexWhere((element) =>
                                              element.messageId == v.messageId);
                                      widget.select.falseSelected = idx;
                                    });
                                    widget.attachment.justDelete = 0;
                                    widget.attachment.cantForward = 0;
                                    if (widget
                                        .attachment.disableFavoriteMessage) {
                                      widget.attachment.disableFavoriteMessage =
                                          false;
                                    }
                                    widget.selectChat.clearList = true;
                                  }
                                },
                              ),
                            )
                    ],
                  )
                ],
              ),
      ),
    );
  }
}

class Chat {
  bool isOriginator, isSelected, isFavorite;
  String? text,
      fileName,
      thumbName,
      fName,
      fImage,
      fId,
      messageId,
      refId,
      refFrom,
      refText,
      refType,
      refThumbId,
      refThumbLoc,
      statusRead,
      time,
      attachmentType,
      credential,
      ack,
      message_cat,
      unDownloadThumb;

  DateTime date;
  double? progress;

  Chat(
      {this.isOriginator = false,
      this.text,
      this.fileName,
      this.fId,
      this.fName,
      this.fImage,
      this.thumbName,
      this.messageId,
      this.refId,
      this.refFrom,
      this.refText,
      this.refType,
      this.refThumbId,
      this.refThumbLoc,
      this.statusRead,
      this.time,
      date,
      this.progress,
      this.attachmentType,
      this.credential,
      this.ack,
      this.message_cat,
      this.unDownloadThumb,
      this.isSelected = false,
      this.isFavorite = false})
      : date = date ?? DateTime.now();

  @override
  bool operator ==(other) =>
      other is Chat &&
      (other.messageId == messageId ||
          (other.fileName == fileName &&
              other.fileName != '' &&
              other.fileName != null) ||
          (other.unDownloadThumb == unDownloadThumb &&
              other.unDownloadThumb != '' &&
              other.unDownloadThumb != null));

  @override
  int get hashCode => super.hashCode;
}

class ListMessage with ChangeNotifier {
  List<Chat> messageChat = [];
  List<Chat> newMessages = [];

  List<Chat> get getChat => messageChat;

  List<Chat> get getNewMessages => newMessages;

  set addChat(Chat value) {
    messageChat.insert(0, value);
    notifyListeners();
  }

  set insertAllChat(List<Chat> value) {
    messageChat.insertAll(0, value);
    notifyListeners();
  }

  set addChatAll(List<Chat> value) {
    messageChat.addAll(value);
    notifyListeners();
  }

  set addNewMessages(Chat value) {
    newMessages.insert(0, value);
    notifyListeners();
  }

  void clearNewMessages() {
    newMessages.clear();
    notifyListeners();
  }

  set replaceChat(List<Chat> value) {
    messageChat = value;
    notifyListeners();
  }

  set trueSelected(int i) {
    messageChat[i].isSelected = true;
    notifyListeners();
  }

  set updateProgress(var value) {
    messageChat[value["index"]].progress = value['value'];
    notifyListeners();
  }

  set updateThumbnail(var value) {
    messageChat[value["index"]].thumbName = value['value'];
    notifyListeners();
  }

  set updateStatus(var value) {
    messageChat[value["index"]].statusRead = value['value'];
    notifyListeners();
  }

  set updateFavoriteMessage(var value) {
    messageChat[value["index"]].isFavorite = value['value'];
    notifyListeners();
  }

  set updateStatusDelete(var value) {
    messageChat[value["index"]].text = value['value'];
    notifyListeners();
  }

  set falseSelected(int i) {
    messageChat[i].isSelected = false;
    notifyListeners();
  }
}

class SelectedChat with ChangeNotifier {
  Map<int, Chat> listSelectedChat = Map();

  Map<int, Chat> get getSelectedChat => listSelectedChat;

  set addList(var i) {
    listSelectedChat[i["index"]] = i["data"];
    notifyListeners();
  }

  set removeList(int i) {
    listSelectedChat.remove(i);
    notifyListeners();
  }

  set clearList(bool value) {
    listSelectedChat.clear();
    notifyListeners();
  }
}

class Attachment with ChangeNotifier {
  var _replyContent;
  bool _ackActive = false,
      _refresh = false,
      _loading = true,
      _isShowDate = false,
      _isSearch = false,
      _isSuffix = false,
      _clickedSticker = false,
      _isNotBottom = false,
      _hitLink = false,
      _isMention = false,
      _disableFavoriteMessage = false;
  int? _cantForward = 0,
      _indexLastSearch,
      _justDelete = 0,
      _cantDeleteForEveryone = 0;
  String? _dateIndex, _textSearch, _isTyping = "", _textLink;
  Directory? _path;
  List<File> _listImage = [];
  Map<String, String> _listMembers = Map(),
      _tempMentions = Map(),
      _allContacts = Map();
  File? _video, _thumbnailVideo, _file;

  get getReplyContent => _replyContent;

  bool get getAckActive => _ackActive;

  bool get refresh => _refresh;

  bool get loading => _loading;

  bool get isShowDate => _isShowDate;

  bool get isSearch => _isSearch;

  bool get isSuffix => _isSuffix;

  String get isTyping => _isTyping!;

  bool get isNotBottom => _isNotBottom;

  bool get disableFavoriteMessage => _disableFavoriteMessage;

  bool get hitLink => _hitLink;

  bool get clickedSticker => _clickedSticker;

  bool get isMention => _isMention;

  int get cantForward => _cantForward!;

  int get justDelete => _justDelete!;

  int get indexLastSearch => _indexLastSearch!;

  int get cantDeleteForEveryone => _cantDeleteForEveryone!;

  String get dateIndex => _dateIndex!;

  String get textSearch => _textSearch!;

  String get textLink => _textLink!;

  Directory get path => _path!;

  List<File> get getListImage => _listImage;

  Map<String, String> get getListMembers => _listMembers;

  Map<String, String> get getTempMentions => _tempMentions;

  Map<String, String> get getAllContacts => _allContacts;

  File get getVideo => _video!;

  File get getThumbnailVideo => _thumbnailVideo!;

  File get getFile => _file!;

  set replyContent(var value) {
    _replyContent = value;
    notifyListeners();
  }

  set ackActive(bool value) {
    _ackActive = value;
    notifyListeners();
  }

  set loading(bool value) {
    _loading = value;
    notifyListeners();
  }

  set disableFavoriteMessage(bool value) {
    _disableFavoriteMessage = value;
    notifyListeners();
  }

  set isTyping(String value) {
    _isTyping = value;
    notifyListeners();
  }

  set isShowDate(bool value) {
    _isShowDate = value;
    notifyListeners();
  }

  set isSearch(bool value) {
    _isSearch = value;
    notifyListeners();
  }

  set isMention(bool value) {
    _isMention = value;
    notifyListeners();
  }

  set isSuffix(bool value) {
    _isSuffix = value;
    notifyListeners();
  }

  set isNotBottom(bool value) {
    _isNotBottom = value;
    notifyListeners();
  }

  set clickedSticker(bool value) {
    _clickedSticker = value;
    notifyListeners();
  }

  set hitLink(bool value) {
    _hitLink = value;
    notifyListeners();
  }

  set cantForward(int value) {
    _cantForward = value;
    notifyListeners();
  }

  set justDelete(int value) {
    _justDelete = value;
    notifyListeners();
  }

  set indexLastSearch(int? value) {
    _indexLastSearch = value;
    notifyListeners();
  }

  set cantDeleteForEveryone(int value) {
    _cantDeleteForEveryone = value;
    notifyListeners();
  }

  set dateIndex(String value) {
    _dateIndex = value;
    notifyListeners();
  }

  set textLink(String? value) {
    _textLink = value;
    notifyListeners();
  }

  set textSearch(String? value) {
    _textSearch = value;
    notifyListeners();
  }

  set path(Directory value) {
    _path = value;
    notifyListeners();
  }

  set refresh(bool value) {
    _refresh = value;
    notifyListeners();
  }

  set listImage(List<File> value) {
    _listImage.addAll(value);
    notifyListeners();
  }

  set addListImage(File value) {
    _listImage.add(value);
    notifyListeners();
  }

  set removeListImage(int index) {
    _listImage.removeAt(index);
    notifyListeners();
  }

  set clearListImage(bool value) {
    _listImage.clear();
    notifyListeners();
  }

  set video(File? value) {
    _video = value;
    notifyListeners();
  }

  set thumbnailVideo(File? value) {
    _thumbnailVideo = value;
    notifyListeners();
  }

  set file(File? value) {
    _file = value;
    notifyListeners();
  }

  set addAllMembers(Map<String, String> value) {
    _listMembers.addAll(value);
    notifyListeners();
  }

  set addAllContacts(Map<String, String> value) {
    _allContacts.addAll(value);
    notifyListeners();
  }

  set addTempMentions(var value) {
    _tempMentions[value["id"]] = value["name"];
    Map<String, String> temp = _tempMentions;
    var sortedEntries = temp.entries.toList()
      ..sort((e1, e2) {
        var diff = e2.value.compareTo(e1.value);
        if (diff == 0) diff = e2.key.compareTo(e1.key);
        return diff;
      });
    Map<String, String> sorted = Map();
    for (var i = 0; i < sortedEntries.length; i++) {
      sorted[sortedEntries[i].key] = sortedEntries[i].value;
    }
    _tempMentions = sorted;
    notifyListeners();
  }

  void clearTempMentions() {
    _tempMentions.clear();
    notifyListeners();
  }
}
