import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_translate/flutter_translate.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'pa_profile.dart';

class ChangeState with ChangeNotifier {
  List<Map<dynamic, dynamic>> _dataContact = [], _srcContact = [];

  List<Map<dynamic, dynamic>> get getDataContact => _dataContact;

  List<Map<dynamic, dynamic>> get getSrcContact => _srcContact;

  set dataContact(Map<dynamic, dynamic> value) {
    _dataContact.add(value);
    notifyListeners();
  }

  set srcContact(List<Map<dynamic, dynamic>> value) {
    _srcContact.addAll(value);
    notifyListeners();
  }

  set addSrcContact(Map<dynamic, dynamic> value) {
    _srcContact.add(value);
    notifyListeners();
  }

  set clearSrcContact(bool value) {
    _srcContact.clear();
    notifyListeners();
  }
}

class FriendList extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
        create: (_) => ChangeState(),
        child: Consumer<ChangeState>(
            builder: (context, changeState, _) => FriendLists(changeState)));
  }
}

class FriendLists extends StatefulWidget {
  final ChangeState changeState;

  FriendLists(this.changeState);

  _FriendList createState() => _FriendList();
}

class _FriendList extends State<FriendLists> {
  TextEditingController _editingController = new TextEditingController();

  _searchPerson(name) async {
    if (name.toString().isNotEmpty) {
      widget.changeState.clearSrcContact = true;
      for (int i = 0; i < widget.changeState.getDataContact.length; i++) {
        if (widget.changeState.getDataContact[i]["name"]
            .toString()
            .toLowerCase()
            .contains(name.toString().toLowerCase())) {
          widget.changeState.addSrcContact =
              widget.changeState.getDataContact[i];
        }
      }
    } else {
      widget.changeState.clearSrcContact = true;
      widget.changeState.srcContact = widget.changeState.getDataContact;
    }
  }

  @override
  void initState() {
    getData();
    super.initState();
  }

  List<dynamic> dummyAccounts = [
    {
      "f_pin": "1",
      "first_name": "Phil",
      "image_id": "000001",
      "official_account": "0",
      "device_id": "000001"
    },
    {
      "f_pin": "2",
      "first_name": "Yan",
      "image_id": "000002",
      "official_account": "0",
      "device_id": "000002"
    },
    {
      "f_pin": "3",
      "first_name": "Crumbs",
      "image_id": "000003",
      "official_account": "0",
      "device_id": "000003"
    },
    {
      "f_pin": "4",
      "first_name": "Yums",
      "image_id": "000004",
      "official_account": "0",
      "device_id": "000004"
    },
    {
      "f_pin": "5",
      "first_name": "Dimitri",
      "image_id": "000005",
      "official_account": "0",
      "device_id": "000005"
    },
    {
      "f_pin": "6",
      "first_name": "Claude",
      "image_id": "000006",
      "official_account": "0",
      "device_id": "000006"
    }
  ];

  void getData() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String idMe = prefs.getString("hasLogin")!;
    print("IDME $idMe");
    var resultQuery = await DBHelper().queryCustom(
        "SELECT * FROM BUDDY where f_pin != '$idMe' AND f_pin != ''");
//    List<dynamic> resultQuery = dummyAccounts;
    print("Result query buddy: $resultQuery");
    for (var i = 0; i < resultQuery.length; i++) {
      String name = "";
      if (resultQuery[i]["first_name"].isNotEmpty) {
        name = resultQuery[i]["first_name"] +
            (resultQuery[i]["last_name"].isNotEmpty ? " " : "") +
            resultQuery[i]["last_name"];
      } else {
        name = resultQuery[i]["last_name"];
      }
      widget.changeState.dataContact = {
        "f_pin": resultQuery[i]["f_pin"],
        "name": name,
        "picture": resultQuery[i]["image_id"],
        "isVerified": resultQuery[i]["official_account"] == "2" ? "1" : "0",
        "isOfficial": resultQuery[i]["official_account"] == "1" ? "1" : "0",
        "deviceId": resultQuery[i]["device_id"],
        'isOffline': resultQuery[i]["offline_mode"],
        'isMuted': resultQuery[i]["muted"],
        'industry': resultQuery[i]["bidang_industri"],
        'email_address': resultQuery[i]["email"],
        'phone_number': resultQuery[i]["msisdn"],
        'website': resultQuery[i]["web"],
        'address': resultQuery[i]["address"],
        'status': resultQuery[i]["quote"],
        "myProfile": false,
        "official_account": resultQuery[i]["official_account"]
      };
    }
    widget.changeState.getDataContact
        .sort((a, b) => (a["name"]).compareTo(b["name"]));
    widget.changeState.srcContact = widget.changeState.getDataContact;
    print("src contact: ${widget.changeState.getSrcContact}");
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
          automaticallyImplyLeading: true,
          backgroundColor: AppColor.primaryColor,
          title: Text("Friend")),
      body: Container(
        child: Column(
          children: <Widget>[
            Container(
              decoration: BoxDecoration(
                  color: Colors.grey[100],
                  border: Border(
                      bottom:
                          BorderSide(color: Colors.grey[200]!, width: 2.0))),
              padding: EdgeInsets.all(2.0),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: <Widget>[
                  Container(
                    width: MediaQuery.of(context).size.width * 0.98,
                    child: Theme(
                      child: TextField(
                          controller: _editingController,
                          onChanged: (val) async {
                            await _searchPerson(val);
                          },
                          cursorColor: AppColor.primaryColor,
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: Colors.white,
                              contentPadding: EdgeInsets.all(5.0),
                              prefixIcon: Icon(Icons.search),
                              focusedBorder: OutlineInputBorder(
                                  borderRadius: BorderRadius.circular(3.0),
                                  borderSide: BorderSide(color: Colors.grey)),
                              border: OutlineInputBorder(
                                  borderRadius: BorderRadius.circular(3.0),
                                  borderSide: BorderSide(color: Colors.grey)))),
                      data:
                          Theme.of(context).copyWith(primaryColor: Colors.grey),
                    ),
                  )
                ],
              ),
            ),
            Flexible(
              child: ListView.builder(
                  itemCount: widget.changeState.getSrcContact.length,
                  itemBuilder: (builder, index) {
                    return ListTile(
                        leading: Container(
                          width: 45,
                          height: 45,
                          decoration: BoxDecoration(
                              shape: BoxShape.circle,
                              image: DecorationImage(
                                  fit: BoxFit.cover,
                                  // image: Image.asset(
                                  //   "assets/images/ic_person_boy.png",
                                  // ).image
                                  image: LocalImage(widget.changeState
                                      .getSrcContact[index]["picture"]))),
                        ),
                        title: Row(
                          children: <Widget>[
                            widget.changeState.getSrcContact[index]
                                        ["isVerified"] ==
                                    "1"
                                ? Image.asset(
                                    "assets/images/ic_verified_flag.png",
                                    width: 15,
                                    height: 15,
                                    package: "palio_lite")
                                : widget.changeState.getSrcContact[index]
                                            ["isOfficial"] ==
                                        "1"
                                    ? Image.asset(
                                        "assets/images/ic_official_flag.png",
                                        width: 15,
                                        height: 15,
                                        package: "palio_lite")
                                    : Container(),
                            Padding(
                              padding: EdgeInsets.only(
                                left: 8.0,
                              ),
                              child: Text(
                                widget.changeState.getSrcContact[index]["name"],
                              ),
                            )
                          ],
                        ),
                        onTap: () {
                          Navigator.pushReplacement(
                            context,
                            PageTransition(
                              type: PageTransitionType.fade,
                              child: PAProfile(
                                profileDet: {
                                  'f_pin': widget.changeState
                                      .getSrcContact[index]["f_pin"],
                                  'myProfile': false,
                                  'name': widget
                                      .changeState.getSrcContact[index]["name"],
                                  'follower': 0,
                                  'official_flag': widget.changeState
                                      .getSrcContact[index]["isOfficial"],
                                  'isFriend': 1,
                                  'post': 0,
                                  'following': 0,
                                  'image': widget.changeState
                                      .getSrcContact[index]["picture"],
                                  'isOffline': widget.changeState
                                      .getSrcContact[index]["isOffline"],
                                  'isMuted': widget.changeState
                                      .getSrcContact[index]["isMuted"],
                                  'industry': widget.changeState
                                      .getSrcContact[index]["industry"],
                                  'email_address': widget.changeState
                                      .getSrcContact[index]["email_address"],
                                  'phone_number': widget.changeState
                                      .getSrcContact[index]["phone_number"],
                                  'website': widget.changeState
                                      .getSrcContact[index]["website"],
                                  'address': widget.changeState
                                      .getSrcContact[index]["address"],
                                  'status': widget.changeState
                                      .getSrcContact[index]["status"],
                                  'official_account': widget.changeState
                                      .getSrcContact[index]["official_account"],
                                  'user_type': widget.changeState
                                      .getSrcContact[index]["user_type"]
                                },
                              ),
                            ),
                          );
                        });
                  }),
            )
          ],
        ),
      ),
    );
  }
}
