import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:image_picker/image_picker.dart';
import 'package:intl/intl.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/chat/add_member_group.dart';
import 'package:palio_lite/chat/chat_group_screen.dart';
import 'package:palio_lite/chat/pa_profile.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:palio_lite/ip_list.dart';
import 'package:palio_lite/palio_button.dart';
import 'package:path/path.dart' as path;
import 'package:path_provider/path_provider.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
// import 'package:palio_lite/home.dart';

class ChangeState with ChangeNotifier {
  bool _valSwitch = false,
      _loading = true,
      _isAdmin = false,
      _createSubGroup = false;
  String? _idMe;
  File? _imgFile;
  List<Map<String, dynamic>> _topicList = [];

  List<Map<String, dynamic>> _memberList = [];

  Map<String, dynamic> _groupData = Map<String, dynamic>();

  bool get valSwitch => _valSwitch;

  bool get createSubGroup => _createSubGroup;

  String get idMe => _idMe == null ? "" : _idMe!;

  bool get loading => _loading;

  bool get isAdmin => _isAdmin;

  File? get imgFile => _imgFile;

  List<Map<String, dynamic>> get getTopicList => _topicList;

  List<Map<String, dynamic>> get getMemberList => _memberList;

  Map<String, dynamic> get groupData => _groupData;

  set valSwitch(bool value) {
    _valSwitch = value;
    notifyListeners();
  }

  set loading(bool value) {
    _loading = value;
    notifyListeners();
  }

  set isAdmin(bool value) {
    _isAdmin = value;
    notifyListeners();
  }

  set createSubGroup(bool value) {
    _createSubGroup = value;
    notifyListeners();
  }

  set idMe(String value) {
    _idMe = value;
    notifyListeners();
  }

  set imgFile(File? value) {
    _imgFile = value;
    notifyListeners();
  }

  set topicList(Map<String, dynamic> value) {
    _topicList.add(value);
    notifyListeners();
  }

  set changeTopic(var value) {
    _topicList[value["index"]]["name"] = value["name"];
    notifyListeners();
  }

  set removeTopicList(int index) {
    _topicList.removeAt(index);
    notifyListeners();
  }

  set memberList(Map<String, dynamic> value) {
    _memberList.add(value);
    notifyListeners();
  }

  set removeMemberList(int index) {
    _memberList.removeAt(index);
    notifyListeners();
  }

  set changeAdminMember(var value) {
    _memberList[value["index"]]["isInitiator"] = value["admin"];
    notifyListeners();
  }

  set groupData(Map<String, dynamic> value) {
    _groupData = value;
    notifyListeners();
  }

  set updateGroupData(Map<String, dynamic> value) {
    _groupData[value.keys.elementAt(0)] = value.values.elementAt(0);
    notifyListeners();
  }
}

class GroupDetail extends StatelessWidget {
  final String idGroup;
  final String nameGroup;
  final String creator;
  final bool createGroup;
  final String createdDate;
  final String parent;

  GroupDetail(
      {required this.idGroup,
      required this.creator,
      required this.createGroup,
      required this.nameGroup,
      this.createdDate = "0",
      this.parent = "-1"});

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
      create: (_) => ChangeState(),
      child: Consumer<ChangeState>(
        builder: (context, changeState, _) => GroupDetails(
            idGroup, creator, createGroup, nameGroup, createdDate, changeState,
            parent: parent),
      ),
    );
  }
}

class GroupDetails extends StatefulWidget {
  final String idGroup;
  final String nameGroup;
  final String creator;
  final bool createGroup;
  final String createdDate;
  final ChangeState changeState;
  final String parent;

  GroupDetails(this.idGroup, this.creator, this.createGroup, this.nameGroup,
      this.createdDate, this.changeState,
      {this.parent = "-1"});

  @override
  State<StatefulWidget> createState() => _GroupDetailsState();
}

class _GroupDetailsState extends State<GroupDetails> {
  ScrollController? _scrollController;
  TextEditingController _textEditingControllerAddTopic =
          new TextEditingController(),
      _textEditingControllerChangeTopic = new TextEditingController(),
      _textEditingControllerChangeNameGroup = new TextEditingController(),
      _textEditingControllerChangeDescGroup = new TextEditingController(),
      _textEditingControllerAddSubGroup = new TextEditingController();
  FocusNode focusChangeTopic = FocusNode();
  String lang = "";

  @override
  void initState() {
    final changeState = Provider.of<ChangeState>(context, listen: false);
    getData(changeState);
    event(changeState);
    super.initState();
  }

  Future<String> getLanguageSetting() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("language")) {
      lang = prefs.getString("language") ?? "";
    }
    return lang;
  }

  void getData(ChangeState changeState) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    changeState.idMe = prefs.getString("hasLogin")!;
    if (widget.createGroup) {
      var resultQuery = await DBHelper().queryCustom(
          "SELECT first_name,last_name, image_id FROM BUDDY WHERE f_pin='${widget.creator}'");
      DateTime dateTime =
          DateTime.fromMillisecondsSinceEpoch(int.parse(widget.createdDate));
      var formatDate = new DateFormat('dd MMMM yyyy');
      // if (lang == "en_US") {
      //   changeState.topicList = {"id": "", "name": "Lounge"};
      // } else {
      //   changeState.topicList = {"id": "", "name": "Beranda"};
      // }
      changeState.topicList = {"id": "", "name": "Lounge"};
      changeState.memberList = {
        "id": widget.creator,
        "name":
            "${resultQuery[0]["first_name"]} ${resultQuery[0]["last_name"]}",
        "picture": resultQuery[0]["image_id"],
        "isInitiator": "1"
      };
      changeState.groupData = {
        "name": widget.nameGroup,
        "desc": "",
        "picture": "",
        "topics": changeState.getTopicList,
        "created_date": formatDate.format(dateTime),
        "memberCount": changeState.getMemberList.length,
        "memberList": changeState.getMemberList,
        "initiator": changeState.getMemberList[0],
        "level": "-1",
        "parent": widget.parent,
        "countAdmin": 1
      };
      changeState.loading = false;
    } else {
      var resultQuery = await DBHelper().queryCustom(
          "SELECT * FROM GROUPZ WHERE group_id='${widget.idGroup}'");
      var resultQueryTopic = await DBHelper().queryCustom(
          "SELECT title, chat_id FROM DISCUSSION_FORUM WHERE group_id='${widget.idGroup}' order by title");
      var resultQueryMember = await DBHelper().queryCustom(
          "SELECT f_pin,position FROM GROUPZ_MEMBER WHERE group_id='${widget.idGroup}' order by first_name");
      var countAdmin = await DBHelper().queryCustom(
          "SELECT COUNT(*) FROM GROUPZ_MEMBER WHERE group_id='${widget.idGroup}' AND position='1'");
      DateTime dateTime = DateTime.fromMillisecondsSinceEpoch(
          int.parse(resultQuery[0]["created_date"]));
      var formatDate = new DateFormat('dd MMMM yyyy');
      // if (lang == "en_US") {
      //   changeState.topicList = {"id": "", "name": "Lounge"};
      // } else {
      //   changeState.topicList = {"id": "", "name": "Beranda"};
      // }
      changeState.topicList = {"id": "", "name": "Lounge"};
      for (var i = 0; i < resultQueryTopic.length; i++) {
        changeState.topicList = {
          "name": resultQueryTopic[i]["title"],
          "id": resultQueryTopic[i]["chat_id"]
        };
      }
      int indexInitiator;
      for (var j = 0; j < resultQueryMember.length; j++) {
        String f_pin = resultQueryMember[j]["f_pin"];
        if (f_pin.isEmpty) {
          continue;
        }
        var resultQueryBuddy = await DBHelper().queryCustom(
            "SELECT first_name, image_id, official_account FROM BUDDY WHERE f_pin='$f_pin'");
        if (resultQueryBuddy.length == 0) {
          await API.addFriend(f_pin);
          resultQueryBuddy = await DBHelper().queryCustom(
              "SELECT first_name, image_id, official_account FROM BUDDY WHERE f_pin='$f_pin'");
        }
        changeState.memberList = {
          "id": f_pin,
          "name": resultQueryBuddy[0]["first_name"],
          "picture": resultQueryBuddy[0]["image_id"],
          "isInitiator": resultQueryMember[j]["position"]
        };
        widget.changeState.getMemberList.sort((a, b) =>
            (a["name"].toLowerCase()).compareTo(b["name"].toLowerCase()));
        if (widget.changeState.getMemberList.length == 2 &&
            widget.changeState.getMemberList[0]["id"] ==
                widget.changeState.getMemberList[1]["id"]) {
          widget.changeState.removeMemberList = 0;
        }
      }
      var initiatorData;
      var resultQueryBuddy = await DBHelper().queryCustom(
          "SELECT first_name, image_id FROM BUDDY WHERE f_pin='${resultQuery[0]["created_by"]}'");
      if (resultQueryBuddy == 0) {
        initiatorData = {
          "picture": "",
        };
      } else {
        initiatorData = {
          "id": resultQuery[0]["created_by"],
          "name": resultQueryBuddy[0]["first_name"],
          "picture": resultQueryBuddy[0]["image_id"],
          "isInitiator": "1"
        };
      }

      changeState.groupData = {
        "name": widget.nameGroup,
        "desc": resultQuery[0]["quote"],
        "picture": resultQuery[0]["image_id"],
        "topics": changeState.getTopicList,
        "created_date": formatDate.format(dateTime),
        "memberCount": changeState.getMemberList.length,
        "memberList": changeState.getMemberList,
        "initiator": initiatorData,
        "parent": resultQuery[0]["parent"],
        "level": resultQuery[0]["level"],
        "countAdmin": countAdmin[0]["COUNT(*)"],
        "official": resultQuery[0]["official"]
      };
      if (resultQuery[0]["is_open"] == 1) {
        widget.changeState.valSwitch = true;
      }
      if (widget.creator == "1") {
        changeState.isAdmin = true;
      }
      changeState.loading = false;
    }
  }

  void event(ChangeState changeState) {
    API.onGroup = (String code, String fPin, String groupId) async {
      if (mounted) {
        print("onGroup $code $groupId");
        if (code == "A010") {
          if (widget.idGroup == groupId) {
            var resultQuery = await DBHelper().queryCustom(
                "SELECT f_name,created_by, created_date, quote, is_open, image_id FROM GROUPZ WHERE group_id='${widget.idGroup}'");
            var resultQueryMember = await DBHelper().queryCustom(
                "SELECT f_pin,position FROM GROUPZ_MEMBER WHERE group_id='${widget.idGroup}' order by first_name");
            var countAdmin = await DBHelper().queryCustom(
                "SELECT COUNT(*) FROM GROUPZ_MEMBER WHERE group_id='${widget.idGroup}' AND position='1'");
            DateTime dateTime = DateTime.fromMillisecondsSinceEpoch(
                int.parse(resultQuery[0]["created_date"]));
            var formatDate = new DateFormat('dd MMMM yyyy');
            int indexInitiator = -1;
            for (var j = 0; j < resultQueryMember.length; j++) {
              if (resultQueryMember[j]["f_pin"] ==
                  resultQuery[0]["created_by"]) {
                indexInitiator = j;
              }
            }
            var initiatorData;
            if (indexInitiator != -1) {
              initiatorData = changeState.getMemberList[indexInitiator];
            } else {
              var resultQueryBuddy = await DBHelper().queryCustom(
                  "SELECT first_name, image_id FROM BUDDY WHERE f_pin='${resultQuery[0]["created_by"]}'");
              if (resultQueryBuddy == 0) {
                initiatorData = {
                  "picture": "",
                };
              } else {
                initiatorData = {
                  "id": resultQuery[0]["created_by"],
                  "name": resultQueryBuddy[0]["first_name"],
                  "picture": resultQueryBuddy[0]["image_id"],
                  "isInitiator": "1"
                };
              }
            }
            changeState.groupData = {
              "name": resultQuery[0]["f_name"],
              "desc": resultQuery[0]["quote"],
              "picture": resultQuery[0]["image_id"],
              "topics": changeState.getTopicList,
              "created_date": formatDate.format(dateTime),
              "memberCount": changeState.getMemberList.length,
              "memberList": changeState.getMemberList,
              "initiator": initiatorData,
              "countAdmin": countAdmin[0]["COUNT(*)"]
            };
            if (resultQuery[0]["is_open"] == 1 && !changeState.valSwitch) {
              widget.changeState.valSwitch = true;
            } else if (changeState.valSwitch) {
              widget.changeState.valSwitch = false;
            }
          }
        } else if (code == "A012" || code == "A008") {
          if (code == "A012" && fPin == "-1" && groupId != widget.idGroup) {
            var resultQuery = await DBHelper().queryCustom(
                "SELECT f_name,created_by, created_date, parent FROM GROUPZ WHERE group_id='$groupId'");
            SharedPreferences prefs = await SharedPreferences.getInstance();
            String idMe = prefs.getString("hasLogin")!;
            if (resultQuery[0]["created_by"] == idMe &&
                widget.changeState.createSubGroup) {
              Navigator.pushReplacement(
                  context,
                  PageTransition(
                      type: PageTransitionType.fade,
                      child: GroupDetail(
                        nameGroup: resultQuery[0]["f_name"],
                        idGroup: groupId,
                        creator: resultQuery[0]["created_by"],
                        createGroup: true,
                        createdDate: resultQuery[0]["created_date"],
                        parent: resultQuery[0]["parent"],
                      )));
            }
          } else if (widget.idGroup == groupId) {
            if (fPin == changeState.idMe) {
              Navigator.pop(context);
            }
          }
        }
      }
    };
    API.onMemberGroup =
        (String code, String fPin, String groupId, String member) async {
      if (mounted) {
        print("OnMemberGroup $code $groupId $member");
        if (code == "A009" && groupId == widget.idGroup) {
          var resultQuery = await DBHelper().queryCustom(
              "SELECT first_name,last_name, image_id FROM BUDDY WHERE f_pin='$member'");
          if (member != widget.creator) {
            changeState.memberList = {
              "id": member,
              "name":
                  "${resultQuery[0]["first_name"]} ${resultQuery[0]["last_name"]}",
              "picture": resultQuery[0]["image_id"],
              "isInitiator": "0"
            };
            Fluttertoast.showToast(
                msg: "${resultQuery[0]["first_name"]} " +
                    "added as a new member",
                toastLength: Toast.LENGTH_LONG);
          }
        } else if (code == "A008" && groupId == widget.idGroup) {
          for (var i = 0; i < changeState.getMemberList.length; i++) {
            if (changeState.getMemberList[i].containsValue(member)) {
              changeState.removeMemberList = i;
              break;
            }
          }
        } else if (code == "A091") {
          for (var i = 0; i < changeState.getMemberList.length; i++) {
            if (changeState.getMemberList[i].containsValue(member)) {
              String isAdmin = changeState.getMemberList[i]["isInitiator"];
              print("masuk $i $isAdmin");
              var countAdmin = await DBHelper().queryCustom(
                  "SELECT COUNT(*) FROM GROUPZ_MEMBER WHERE group_id='${widget.idGroup}' AND position='1'");
              changeState.updateGroupData = {
                "countAdmin": countAdmin[0]["COUNT(*)"]
              };
              if (isAdmin == "0") {
                changeState.changeAdminMember = {"index": i, "admin": "1"};
                if (changeState.idMe == member) {
                  changeState.isAdmin = true;
                }
              } else {
                print("masuk $i");
                changeState.changeAdminMember = {"index": i, "admin": "0"};
                if (changeState.idMe == member) {
                  changeState.isAdmin = false;
                }
              }
              break;
            }
          }
        }
      }
    };
    API.onTopic = (String code, String fPin, String topicId) async {
      if (mounted) {
        print("ONTOPIC $code $topicId");
        if (code == "BF") {
          if (fPin == changeState.idMe) {
            widget.changeState.topicList = {
              "id": topicId,
              "name": _textEditingControllerAddTopic.text
            };
            Fluttertoast.showToast(
                msg: "Succesfully changed", toastLength: Toast.LENGTH_LONG);
            _textEditingControllerAddTopic.clear();
          } else {
            var resultQueryTopic = await DBHelper().queryCustom(
                "SELECT title FROM DISCUSSION_FORUM WHERE chat_id='$topicId' order by title");
            widget.changeState.topicList = {
              "id": topicId,
              "name": resultQueryTopic[0]["title"]
            };
          }
        } else if (code == "BE") {
          for (var i = 0; i < changeState.getMemberList.length; i++) {
            if (changeState.getTopicList[i].containsValue(topicId)) {
              changeState.removeTopicList = i;
              break;
            }
          }
        } else if (code == "BD") {
          var resultQueryTopic = await DBHelper().queryCustom(
              "SELECT title FROM DISCUSSION_FORUM WHERE chat_id='$topicId' order by title");
          for (var i = 0; i < changeState.getMemberList.length; i++) {
            if (changeState.getTopicList[i].containsValue(topicId)) {
              changeState.changeTopic = {
                "index": i,
                "name": resultQueryTopic[0]["title"]
              };
              break;
            }
          }
        }
      }
    };
  }

  void _selected(val) async {
    if (val == 1) {
      showDialog(
          context: context,
          builder: (_) {
            return CupertinoAlertDialog(
              title: Text("Delete group",
                  style: TextStyle(color: AppColor.primaryColor)),
              content: Container(
                padding: EdgeInsets.only(top: 15.0),
                child: Text("Are you sure want to delete this group?",
                    style: TextStyle(color: AppColor.primaryColor)),
              ),
              actions: <Widget>[
                FlatButton(
                  child: Text(
                    "Cancel",
                    style: TextStyle(color: AppColor.primaryColor),
                  ),
                  onPressed: () {
                    Navigator.of(context).pop();
                  },
                ),
                FlatButton(
                  child: Text(
                    "DELETE",
                    style: TextStyle(color: AppColor.primaryColor),
                  ),
                  onPressed: () async {
                    Navigator.of(context).pop();
                    showDialog(
                        barrierDismissible: false,
                        context: context,
                        builder: (_) {
                          return CupertinoAlertDialog(
                            title: Row(
                              mainAxisAlignment: MainAxisAlignment.start,
                              children: <Widget>[
                                Container(
                                    child: CupertinoActivityIndicator(),
                                    width: 20,
                                    height: 20),
                                SizedBox(width: 20.0),
                                Text("Delete group",
                                    style: TextStyle(
                                        fontWeight: FontWeight.normal))
                              ],
                            ),
                          );
                        });
                    // for (var i = 0;
                    //     i < widget.changeState.getMemberList.length;
                    //     i++) {
                    //   await API.exitGroup(widget.idGroup,
                    //       widget.changeState.getMemberList[i]["id"]);
                    // }
                    var result = await API.exitGroup(widget.idGroup, "ALL");
                    if (result == "00") {
                      Navigator.of(context).pop();
                    }
                  },
                )
              ],
            );
          });
    } else if (val == 2) {
      var getGroupType = await DBHelper().queryCustom(
          "SELECT * FROM GROUPZ WHERE group_id='${widget.idGroup}'");
      bool isCC = false;
      if (getGroupType[0]["official"] == 1 &&
          getGroupType[0]["is_education"] == 2) {
        isCC = true;
      }
      showDialog(
          context: context,
          builder: (_) {
            return Dialog(
              child: Container(
                height: MediaQuery.of(context).size.height,
                width: MediaQuery.of(context).size.width,
                child: AddMemberGroup(
                  idGroup: widget.idGroup,
                  fPinAdd: widget.changeState.getMemberList,
                  isCC: isCC,
                ),
              ),
            );
          });
    } else if (val == 3) {
      showDialog(
          context: context,
          builder: (_) {
            return CupertinoAlertDialog(
              title: Text("Add topic",
                  style: TextStyle(color: AppColor.primaryColor)),
              content: Material(
                color: Colors.transparent,
                child: TextField(
                  cursorColor: AppColor.primaryColor,
                  controller: _textEditingControllerAddTopic,
                  decoration: InputDecoration(
                      fillColor: Colors.transparent,
                      focusedBorder: UnderlineInputBorder(
                          borderSide: BorderSide(color: AppColor.primaryColor)),
                      border: UnderlineInputBorder(
                          borderSide: BorderSide(color: AppColor.primaryColor)),
                      filled: true),
                ),
              ),
              actions: <Widget>[
                FlatButton(
                  child: Text(
                    "Cancel",
                    style: TextStyle(color: AppColor.primaryColor),
                  ),
                  onPressed: () {
                    Navigator.of(context).pop();
                  },
                ),
                FlatButton(
                  child: Text(
                    "NEXT",
                    style: TextStyle(color: AppColor.primaryColor),
                  ),
                  onPressed: () {
                    if (_textEditingControllerAddTopic.text.trim().isNotEmpty) {
                      bool same = false;
                      for (var i = 0;
                          i < widget.changeState.getTopicList.length;
                          i++) {
                        if (widget.changeState.getTopicList[i]["name"]
                            .toLowerCase()
                            .contains(_textEditingControllerAddTopic.text
                                .toLowerCase())) {
                          same = true;
                          break;
                        }
                      }
                      if (same) {
                        Fluttertoast.showToast(
                            msg: "Name already found",
                            toastLength: Toast.LENGTH_SHORT);
                      } else {
                        Navigator.of(context).pop();
                        API.addChat(widget.idGroup,
                            _textEditingControllerAddTopic.text);
                      }
                    } else {
                      Fluttertoast.showToast(
                          msg: "Name can't be empty",
                          toastLength: Toast.LENGTH_SHORT);
                    }
                  },
                )
              ],
            );
          });
    } else {
      //tambah sub group
      showDialog(
          context: context,
          builder: (_) {
            return CupertinoAlertDialog(
              title: Row(
                mainAxisAlignment: MainAxisAlignment.start,
                children: <Widget>[
                  Image.asset("assets/images/groupicon.png",
                      width: 25, height: 25, package: "palio_lite"),
                  Text("Create Sub Group")
                ],
              ),
              content: Material(
                color: Colors.transparent,
                child: TextField(
                  cursorColor: AppColor.primaryColor,
                  controller: _textEditingControllerAddSubGroup,
                  decoration: InputDecoration(
                      fillColor: Colors.transparent,
                      focusedBorder: UnderlineInputBorder(
                          borderSide: BorderSide(color: Color(0xFF364656))),
                      border: UnderlineInputBorder(
                          borderSide: BorderSide(color: Color(0xFF364656))),
                      hintText: "Input Group Name",
                      filled: true),
                ),
              ),
              actions: <Widget>[
                FlatButton(
                  child: Text(
                    "Cancel",
                  ),
                  onPressed: () {
                    Navigator.of(context).pop();
                  },
                ),
                FlatButton(
                  child: Text(
                    "CREATE",
                  ),
                  onPressed: () async {
                    Navigator.pop(context);
                    String level;
                    if (widget.changeState.groupData["level"] == null ||
                        widget.changeState.groupData["level"] == "-1") {
                      level = "2";
                    } else {
                      level =
                          "${int.parse(widget.changeState.groupData["level"]) + 1}";
                    }
                    widget.changeState.createSubGroup = true;
                    String result = await API.createSubGroup(
                        _textEditingControllerAddSubGroup.text,
                        widget.idGroup,
                        level);
                    if (result != "00") {
                      widget.changeState.createSubGroup = false;
                      Fluttertoast.showToast(
                          msg: "Failed created group",
                          toastLength: Toast.LENGTH_SHORT);
                    }
                  },
                )
              ],
            );
          });
    }
  }

  Future<bool> onWillPop() {
    PalioButtonState statePalio = PalioButtonState();
    statePalio.receiveMessage(context);
    statePalio.incomingCall(context);
    Navigator.pop(context);
    return Future.value(false);
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: onWillPop,
      child: Scaffold(
        appBar: AppBar(
          automaticallyImplyLeading: true,
          backgroundColor: AppColor.primaryColor,
          title: Text(widget.changeState.loading
              ? ""
              : widget.changeState.groupData["name"] != null
                  ? widget.changeState.groupData["name"]
                  : widget.nameGroup),
          actions: <Widget>[
            !widget.createGroup && !widget.changeState.isAdmin ||
                    widget.changeState.groupData["official"] == 1
                ? Container()
                : IconButton(
                    icon: Icon(
                      Icons.edit,
                      color: Colors.white,
                    ),
                    onPressed: () {
                      _textEditingControllerChangeNameGroup.text =
                          widget.changeState.groupData["name"];
                      showDialog(
                          context: context,
                          builder: (_) {
                            return CupertinoAlertDialog(
                              title: Row(
                                mainAxisAlignment: MainAxisAlignment.start,
                                children: <Widget>[
                                  Image.asset("assets/images/groupicon.png",
                                      width: 25,
                                      height: 25,
                                      package: "palio_lite"),
                                  Text("Input group name",
                                      style: TextStyle(
                                          color: AppColor.primaryColor))
                                ],
                              ),
                              content: Material(
                                color: Colors.transparent,
                                child: TextField(
                                  cursorColor: AppColor.primaryColor,
                                  controller:
                                      _textEditingControllerChangeNameGroup,
                                  decoration: InputDecoration(
                                      fillColor: Colors.transparent,
                                      focusedBorder: UnderlineInputBorder(
                                          borderSide: BorderSide(
                                              color: AppColor.primaryColor)),
                                      border: UnderlineInputBorder(
                                          borderSide: BorderSide(
                                              color: AppColor.primaryColor)),
                                      filled: true),
                                ),
                              ),
                              actions: <Widget>[
                                FlatButton(
                                  child: Text(
                                    "Cancel",
                                    style:
                                        TextStyle(color: AppColor.primaryColor),
                                  ),
                                  onPressed: () {
                                    Navigator.of(context).pop();
                                  },
                                ),
                                FlatButton(
                                  child: Text(
                                    "NEXT",
                                    style:
                                        TextStyle(color: AppColor.primaryColor),
                                  ),
                                  onPressed: () async {
                                    Navigator.pop(context);
                                    if (_textEditingControllerChangeNameGroup
                                        .text
                                        .trim()
                                        .isNotEmpty) {
                                      var result = await API.updateGroup(
                                          _textEditingControllerChangeNameGroup
                                              .text,
                                          widget.idGroup,
                                          "",
                                          "",
                                          "");
                                      if (result == "00") {
                                        Fluttertoast.showToast(
                                            msg: "Succesfully changed",
                                            toastLength: Toast.LENGTH_SHORT);
                                      } else {
                                        Fluttertoast.showToast(
                                            msg: "Failed to change",
                                            toastLength: Toast.LENGTH_SHORT);
                                      }
                                    } else {
                                      Fluttertoast.showToast(
                                          msg: "Name can't be empty",
                                          toastLength: Toast.LENGTH_SHORT);
                                    }
                                  },
                                )
                              ],
                            );
                          });
                    },
                  ),
            widget.changeState.groupData["countAdmin"] != null &&
                    widget.changeState.groupData["countAdmin"] > 1 &&
                    widget.changeState.isAdmin &&
                    widget.changeState.groupData["official"] != 1 &&
                    !widget.changeState.loading
                ? IconButton(
                    icon: Icon(Icons.exit_to_app),
                    onPressed: () async {
                      showDialog(
                          context: context,
                          builder: (_) {
                            return CupertinoAlertDialog(
                              title: Text("Leave Group",
                                  style:
                                      TextStyle(color: AppColor.primaryColor)),
                              content: Container(
                                padding: EdgeInsets.only(top: 15.0),
                                child: Text(
                                    "Are you sure want to leave from this group?",
                                    style: TextStyle(
                                        color: AppColor.primaryColor)),
                              ),
                              actions: <Widget>[
                                FlatButton(
                                  child: Text(
                                    "Cancel",
                                    style:
                                        TextStyle(color: AppColor.primaryColor),
                                  ),
                                  onPressed: () {
                                    Navigator.of(context).pop();
                                  },
                                ),
                                FlatButton(
                                  child: Text(
                                    "YES",
                                    style:
                                        TextStyle(color: AppColor.primaryColor),
                                  ),
                                  onPressed: () async {
                                    Navigator.pop(context);
                                    await API.exitGroup(widget.idGroup,
                                        widget.changeState.idMe);
                                  },
                                )
                              ],
                            );
                          });
                    },
                  )
                : Container(),
            widget.changeState.loading
                ? Container()
                : !widget.createGroup &&
                        !widget.changeState.isAdmin &&
                        widget.changeState.groupData["official"] != 1
                    ? IconButton(
                        icon: Icon(Icons.exit_to_app),
                        onPressed: () async {
                          showDialog(
                              context: context,
                              builder: (_) {
                                return CupertinoAlertDialog(
                                  title: Text("Leave Group",
                                      style: TextStyle(
                                          color: AppColor.primaryColor)),
                                  content: Container(
                                    padding: EdgeInsets.only(top: 15.0),
                                    child: Text(
                                        "Are you sure want to leave from this group?",
                                        style: TextStyle(
                                            color: AppColor.primaryColor)),
                                  ),
                                  actions: <Widget>[
                                    FlatButton(
                                      child: Text(
                                        "Cancel",
                                        style: TextStyle(
                                            color: AppColor.primaryColor),
                                      ),
                                      onPressed: () {
                                        Navigator.of(context).pop();
                                      },
                                    ),
                                    FlatButton(
                                      child: Text(
                                        "YES",
                                        style: TextStyle(
                                            color: AppColor.primaryColor),
                                      ),
                                      onPressed: () async {
                                        Navigator.pop(context);
                                        await API.exitGroup(widget.idGroup,
                                            widget.changeState.idMe);
                                      },
                                    )
                                  ],
                                );
                              });
                        },
                      )
                    : widget.changeState.isAdmin || widget.createGroup
                        ? PopupMenuButton(
                            onSelected: (val) {
                              _selected(val);
                            },
                            itemBuilder: (context) => [
                              widget.changeState.idMe !=
                                      widget.changeState.groupData["initiator"]
                                          ["id"]
                                  ? PopupMenuItem(child: Container())
                                  : PopupMenuItem(
                                      value: 1,
                                      child: Text("Delete group"),
                                    ),
                              PopupMenuItem(
                                value: 2,
                                child: Text("Add new member"),
                              ),
                              PopupMenuItem(
                                value: 3,
                                child: Text("Add topic"),
                              ),
                              widget.changeState.groupData["official"] == 1
                                  ? PopupMenuItem(child: Container())
                                  : PopupMenuItem(
                                      value: 4,
                                      child: Text("Add Sub Group"),
                                    )
                            ],
                          )
                        : Container()
          ],
        ),
        body: widget.changeState.loading
            ? Center(
                child: CupertinoActivityIndicator(),
              )
            : SingleChildScrollView(
                controller: _scrollController,
                child: Container(
                  color: Colors.black12,
                  child: Column(
                    children: <Widget>[
                      FutureBuilder(
                        future: _profilePic(
                            widget.changeState.groupData["picture"]),
                        builder: (_, snaps) {
                          if (snaps.hasData) {
                            if (widget.changeState.groupData["picture"] !=
                                    null &&
                                widget.changeState.groupData["picture"]
                                    .isNotEmpty) {
                              return snaps.data as Widget;
                            }
                          }
                          return Container(
                            height: 250,
                            width: MediaQuery.of(context).size.width,
                            child: Stack(
                              children: <Widget>[
                                Center(
                                  child: Image.asset(
                                      "assets/images/groupicon.png",
                                      height: 200,
                                      package: "palio_lite"),
                                ),
                                !widget.createGroup &&
                                        !widget.changeState.isAdmin
                                    ? Container()
                                    : Align(
                                        alignment: Alignment.bottomRight,
                                        child: Padding(
                                          padding: EdgeInsets.only(
                                              right: 10, bottom: 10),
                                          child: FloatingActionButton(
                                            backgroundColor: Colors.green,
                                            child: Icon(
                                              Icons.camera_alt,
                                            ),
                                            onPressed: () {
                                              showDialog(
                                                  barrierDismissible: true,
                                                  context: context,
                                                  builder:
                                                      (BuildContext context) {
                                                    return CupertinoAlertDialog(
                                                      content: Material(
                                                        color:
                                                            Colors.transparent,
                                                        child: Column(
                                                          crossAxisAlignment:
                                                              CrossAxisAlignment
                                                                  .start,
                                                          children: <Widget>[
                                                            ListTile(
                                                              title: Text(
                                                                  "Take Photo"),
                                                              onTap: () {
                                                                Navigator.pop(
                                                                    context);
                                                                takePhoto();
                                                              },
                                                            ),
                                                            ListTile(
                                                              title: Text(
                                                                  "Select from Gallery"),
                                                              onTap: () {
                                                                Navigator.pop(
                                                                    context);
                                                                pickImages();
                                                              },
                                                            )
                                                          ],
                                                        ),
                                                      ),
                                                    );
                                                  });
                                            },
                                          ),
                                        ),
                                      )
                              ],
                            ),
                          );
                        },
                      ),
                      _descGroup(widget.changeState.groupData["desc"]),
                      _topicList(widget.changeState.getTopicList),
                      _detailGroup(widget.changeState.groupData),
                      _members(widget.changeState.groupData["memberList"])
                    ],
                  ),
                ),
              ),
      ),
    );
  }

  void takePhoto() async {
    PickedFile? file = await ImagePicker.platform.pickImage(
      source: ImageSource.camera,
      imageQuality: 100,
      maxHeight: 720,
      maxWidth: 1280,
    );
    if (file == null) {
      return;
    } else {
      Directory directory = await getApplicationDocumentsDirectory();
      File(file.path).copy("${directory.path}/${path.basename(file.path)}");
      var result = await API.updateGroup(
          "", widget.idGroup, "", path.basename(file.path), "");
      if (result == "00") {
        widget.changeState.imgFile = File(file.path);
      }
    }
  }

  void pickImages() async {
    PickedFile? file = await ImagePicker.platform.pickImage(
      source: ImageSource.gallery,
      imageQuality: 100,
      maxHeight: 720,
      maxWidth: 1280,
    );
    if (file == null) {
      return;
    } else {
      Directory directory = await getApplicationDocumentsDirectory();
      File(file.path).copy("${directory.path}/${path.basename(file.path)}");
      var result = await API.updateGroup(
          "", widget.idGroup, "", path.basename(file.path), "");
      if (result == "00") {
        widget.changeState.imgFile = File(file.path);
      }
    }
  }

  Future<Widget> _profilePic(data) async {
    Directory directory = await getApplicationDocumentsDirectory();
    String picture = "${directory.path}/$data";
    if (!File(picture).existsSync()) {
      String newpic = path.basename(picture);
      picture = IPList.imagePost + newpic;
    }
    return Container(
      height: 250,
      decoration: BoxDecoration(
          image: DecorationImage(
              fit: BoxFit.cover,
              image: picture.contains(IPList.imagePost)
                  ? NetworkImage(picture)
                  : FileImage(File(picture)) as ImageProvider)),
      width: MediaQuery.of(context).size.width,
      child: Stack(
        children: <Widget>[
          !widget.createGroup && !widget.changeState.isAdmin
              ? Container()
              : Align(
                  alignment: Alignment.bottomRight,
                  child: Padding(
                    padding: EdgeInsets.only(right: 10, bottom: 10),
                    child: FloatingActionButton(
                      backgroundColor: Colors.green,
                      child: Icon(
                        Icons.camera_alt,
                      ),
                      onPressed: () {
                        showDialog(
                            barrierDismissible: true,
                            context: context,
                            builder: (BuildContext context) {
                              return CupertinoAlertDialog(
                                content: Material(
                                  color: Colors.transparent,
                                  child: Column(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.start,
                                    children: <Widget>[
                                      ListTile(
                                        title: Text("Take Photo"),
                                        onTap: () {
                                          Navigator.pop(context);
                                          takePhoto();
                                        },
                                      ),
                                      ListTile(
                                        title: Text("Select from Gallery"),
                                        onTap: () {
                                          Navigator.pop(context);
                                          pickImages();
                                        },
                                      )
                                    ],
                                  ),
                                ),
                              );
                            });
                      },
                    ),
                  ),
                )
        ],
      ),
    );
  }

  Widget _descGroup(data) {
    var desc = data;
    if (desc == null) {
      desc = "";
    }
    return Card(
      margin: EdgeInsets.symmetric(vertical: 2),
      child: Padding(
        padding: EdgeInsets.symmetric(horizontal: 10),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Text(
                  "Description",
                  style: TextStyle(color: Colors.green),
                ),
                !widget.createGroup && !widget.changeState.isAdmin
                    ? Container(height: 50.0)
                    : IconButton(
                        iconSize: 20,
                        icon: Icon(
                          Icons.edit,
                          color: Colors.green,
                        ),
                        onPressed: () {
                          _textEditingControllerChangeDescGroup.text =
                              widget.changeState.groupData["desc"];
                          showDialog(
                              context: context,
                              builder: (_) {
                                return CupertinoAlertDialog(
                                  title: Text("Description",
                                      style: TextStyle(
                                          color: AppColor.primaryColor)),
                                  content: Material(
                                    color: Colors.transparent,
                                    child: TextField(
                                      cursorColor: AppColor.primaryColor,
                                      controller:
                                          _textEditingControllerChangeDescGroup,
                                      decoration: InputDecoration(
                                          fillColor: Colors.transparent,
                                          focusedBorder: UnderlineInputBorder(
                                              borderSide: BorderSide(
                                                  color:
                                                      AppColor.primaryColor)),
                                          border: UnderlineInputBorder(
                                              borderSide: BorderSide(
                                                  color:
                                                      AppColor.primaryColor)),
                                          filled: true),
                                    ),
                                  ),
                                  actions: <Widget>[
                                    FlatButton(
                                      child: Text(
                                        "Cancel",
                                        style: TextStyle(
                                            color: AppColor.primaryColor),
                                      ),
                                      onPressed: () {
                                        Navigator.of(context).pop();
                                      },
                                    ),
                                    FlatButton(
                                      child: Text(
                                        "NEXT",
                                        style: TextStyle(
                                            color: AppColor.primaryColor),
                                      ),
                                      onPressed: () async {
                                        Navigator.pop(context);
                                        if (_textEditingControllerChangeDescGroup
                                            .text
                                            .trim()
                                            .isNotEmpty) {
                                          var result = await API.updateGroup(
                                              "",
                                              widget.idGroup,
                                              "",
                                              "",
                                              _textEditingControllerChangeDescGroup
                                                  .text);
                                          if (result == "00") {
                                            Fluttertoast.showToast(
                                                msg: "Succesfully changed",
                                                toastLength:
                                                    Toast.LENGTH_SHORT);
                                          } else {
                                            Fluttertoast.showToast(
                                                msg: "Failed to change",
                                                toastLength:
                                                    Toast.LENGTH_SHORT);
                                          }
                                        } else {
                                          Fluttertoast.showToast(
                                              msg: "Description can't be empty",
                                              toastLength: Toast.LENGTH_SHORT);
                                        }
                                      },
                                    )
                                  ],
                                );
                              });
                        },
                      )
              ],
            ),
            Divider(
              height: 1,
              thickness: 1,
              color: Colors.black12,
            ),
            Padding(
              padding: EdgeInsets.symmetric(vertical: 5),
              child: Text(
                desc,
              ),
            )
          ],
        ),
      ),
    );
  }

  Widget _topicList(data) {
    return Card(
      margin: EdgeInsets.symmetric(vertical: 2),
      child: Padding(
        padding: EdgeInsets.all(10.0),
        child: Column(
          children: <Widget>[
            (!widget.createGroup && !widget.changeState.isAdmin) ||
                    widget.changeState.groupData["official"] == 1
                ? Container()
                : Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: <Widget>[
                      Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          Text(
                            "Public Forum",
                            style: TextStyle(color: Colors.green),
                          ),
                          Text("Group can be accessed by all user")
                        ],
                      ),
                      Switch(
                        activeColor: AppColor.primaryColor,
                        value: widget.changeState.valSwitch,
                        onChanged: (val) async {
                          if (val) {
                            var result = await API.updateGroup(
                                "", widget.idGroup, "1", "", "");
                            if (result == "00") {
                              Fluttertoast.showToast(
                                  msg: "Succesfully changed",
                                  toastLength: Toast.LENGTH_SHORT);
                            }
                          } else {
                            var result = await API.updateGroup(
                                "", widget.idGroup, "0", "", "");
                            if (result == "00") {
                              Fluttertoast.showToast(
                                  msg: "Succesfully changed",
                                  toastLength: Toast.LENGTH_SHORT);
                            }
                          }
                        },
                      )
                    ],
                  ),
            (!widget.createGroup && !widget.changeState.isAdmin) ||
                    widget.changeState.groupData["official"] == 1
                ? Container()
                : Divider(
                    height: 5,
                    thickness: 1,
                    color: Colors.black12,
                  ),
            Align(
              alignment: Alignment.centerLeft,
              child: Padding(
                padding: EdgeInsets.symmetric(vertical: 5),
                child: Text(
                  "Topics",
                  style: TextStyle(color: Colors.green, fontSize: 15),
                ),
              ),
            ),
            Divider(
              height: 5,
              thickness: 1,
              color: Colors.black12,
            ),
            ListView.builder(
                shrinkWrap: true,
                physics: NeverScrollableScrollPhysics(),
                itemCount: data.length,
                itemBuilder: (builder, index) {
                  return ListTile(
                    contentPadding: EdgeInsets.symmetric(horizontal: 10),
                    title: Text(
                      data[index]["name"],
                      style: TextStyle(fontSize: 14),
                    ),
                    onTap: () {
                      Navigator.of(context).pop();
                      Navigator.push(
                          context,
                          CupertinoPageRoute(
                              builder: (_) => ChatGroupPage(
                                  widget.changeState.groupData["name"],
                                  data[index]["name"],
                                  widget.changeState.groupData["picture"],
                                  identifier: data[index]["id"],
                                  idGroup: widget.idGroup,
                                  refresh: true)));
                    },
                    onLongPress: () {
                      if (index != 0) {
                        showDialog(
                            context: context,
                            builder: (_) {
                              return CupertinoAlertDialog(
                                content: Material(
                                  color: Colors.transparent,
                                  child: Column(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.start,
                                    children: <Widget>[
                                      ListTile(
                                        title: Text("Change Topic Name"),
                                        contentPadding: EdgeInsets.all(0.0),
                                        onTap: () {
                                          Navigator.pop(context);
                                          _textEditingControllerChangeTopic
                                              .text = data[index]["name"];
                                          showDialog(
                                              context: context,
                                              builder: (_) {
                                                return CupertinoAlertDialog(
                                                  title: Text(
                                                      "Change Topic Name",
                                                      style: TextStyle(
                                                          color: Color(
                                                              0xFF364656))),
                                                  content: Material(
                                                    color: Colors.transparent,
                                                    child: TextField(
                                                      cursorColor:
                                                          AppColor.primaryColor,
                                                      controller:
                                                          _textEditingControllerChangeTopic,
                                                      focusNode:
                                                          focusChangeTopic,
                                                      decoration: InputDecoration(
                                                          fillColor: Colors
                                                              .transparent,
                                                          focusedBorder: UnderlineInputBorder(
                                                              borderSide: BorderSide(
                                                                  color: Color(
                                                                      0xFF364656))),
                                                          border: UnderlineInputBorder(
                                                              borderSide: BorderSide(
                                                                  color: Color(
                                                                      0xFF364656))),
                                                          filled: true),
                                                    ),
                                                  ),
                                                  actions: <Widget>[
                                                    FlatButton(
                                                      child: Text(
                                                        "Cancel",
                                                        style: TextStyle(
                                                            color: Color(
                                                                0xFF364656)),
                                                      ),
                                                      onPressed: () {
                                                        Navigator.of(context)
                                                            .pop();
                                                      },
                                                    ),
                                                    FlatButton(
                                                      child: Text(
                                                        "NEXT".toUpperCase(),
                                                        style: TextStyle(
                                                            color: Color(
                                                                0xFF364656)),
                                                      ),
                                                      onPressed: () async {
                                                        if (focusChangeTopic
                                                            .hasFocus) {
                                                          focusChangeTopic
                                                              .unfocus();
                                                          await Future.delayed(
                                                              Duration(
                                                                  milliseconds:
                                                                      100));
                                                        }
                                                        if (_textEditingControllerChangeTopic
                                                            .text
                                                            .trim()
                                                            .isNotEmpty) {
                                                          bool same = false;
                                                          for (var i = 0;
                                                              i <
                                                                  widget
                                                                      .changeState
                                                                      .getTopicList
                                                                      .length;
                                                              i++) {
                                                            if (widget
                                                                    .changeState
                                                                    .getTopicList[
                                                                        i]
                                                                        ["name"]
                                                                    .toLowerCase() ==
                                                                _textEditingControllerChangeTopic
                                                                    .text
                                                                    .toLowerCase()) {
                                                              same = true;
                                                              break;
                                                            }
                                                          }
                                                          if (same) {
                                                            Fluttertoast.showToast(
                                                                msg:
                                                                    "Name already found",
                                                                toastLength: Toast
                                                                    .LENGTH_SHORT);
                                                          } else {
                                                            Navigator.of(
                                                                    context)
                                                                .pop();
                                                            var result = await API
                                                                .updateChat(
                                                                    widget
                                                                        .idGroup,
                                                                    _textEditingControllerChangeTopic
                                                                        .text,
                                                                    data[index]
                                                                        ["id"]);
                                                            if (result ==
                                                                "00") {
                                                              for (var i = 0;
                                                                  i <
                                                                      widget
                                                                          .changeState
                                                                          .getTopicList
                                                                          .length;
                                                                  i++) {
                                                                if (widget
                                                                    .changeState
                                                                    .getTopicList[
                                                                        i]
                                                                    .containsValue(
                                                                        data[index]
                                                                            [
                                                                            "id"])) {
                                                                  widget
                                                                      .changeState
                                                                      .changeTopic = {
                                                                    "index": i,
                                                                    "name":
                                                                        _textEditingControllerChangeTopic
                                                                            .text
                                                                  };
                                                                }
                                                              }
                                                              Fluttertoast.showToast(
                                                                  msg:
                                                                      "Succesfully changed",
                                                                  toastLength: Toast
                                                                      .LENGTH_SHORT);
                                                              _textEditingControllerChangeTopic
                                                                  .clear();
                                                            }
                                                          }
                                                        } else {
                                                          Fluttertoast.showToast(
                                                              msg:
                                                                  "Name can't be empty",
                                                              toastLength: Toast
                                                                  .LENGTH_SHORT);
                                                        }
                                                      },
                                                    )
                                                  ],
                                                );
                                              });
                                        },
                                      ),
                                      ListTile(
                                        title: Text("Delete Topic"),
                                        contentPadding: EdgeInsets.all(0.0),
                                        onTap: () {
                                          Navigator.pop(context);
                                          showDialog(
                                              context: context,
                                              builder: (_) {
                                                return CupertinoAlertDialog(
                                                  title: Text(
                                                      data[index]["name"],
                                                      textAlign:
                                                          TextAlign.start,
                                                      style: TextStyle(
                                                          color: Color(
                                                              0xFF364656))),
                                                  content: Material(
                                                    color: Colors.transparent,
                                                    child: Text(
                                                        "Are you sure to delete this topic?"),
                                                  ),
                                                  actions: <Widget>[
                                                    FlatButton(
                                                      child: Text(
                                                        "Cancel",
                                                        style: TextStyle(
                                                            color: Color(
                                                                0xFF364656)),
                                                      ),
                                                      onPressed: () {
                                                        Navigator.of(context)
                                                            .pop();
                                                      },
                                                    ),
                                                    FlatButton(
                                                      child: Text(
                                                        "DELETE",
                                                        style: TextStyle(
                                                            color: Color(
                                                                0xFF364656)),
                                                      ),
                                                      onPressed: () async {
                                                        Navigator.of(context)
                                                            .pop();
                                                        String result =
                                                            await API.deleteChat(
                                                                widget.idGroup,
                                                                data[index]
                                                                    ["id"]);
                                                        if (result == "00") {
                                                          for (var i = 0;
                                                              i <
                                                                  widget
                                                                      .changeState
                                                                      .getTopicList
                                                                      .length;
                                                              i++) {
                                                            if (widget
                                                                .changeState
                                                                .getTopicList[i]
                                                                .containsValue(
                                                                    data[index][
                                                                        "id"])) {
                                                              widget.changeState
                                                                  .removeTopicList = i;
                                                              break;
                                                            }
                                                          }
                                                        }
                                                      },
                                                    )
                                                  ],
                                                );
                                              });
                                        },
                                      ),
                                    ],
                                  ),
                                ),
                              );
                            });
                      }
                    },
                  );
                })
          ],
        ),
      ),
    );
  }

  Widget _detailGroup(data) {
    return Card(
      margin: EdgeInsets.symmetric(vertical: 2),
      child: ExpansionTile(
        initiallyExpanded: true,
        title: Text(
          "Details",
          style: TextStyle(color: Colors.green),
        ),
        children: <Widget>[
          ListTile(
            title: Text(
              "Initiator",
              style: TextStyle(fontWeight: FontWeight.bold),
            ),
            trailing: data["initiator"]["picture"] == ""
                ? Container(
                    decoration: BoxDecoration(
                        color: Colors.grey, shape: BoxShape.circle),
                    child: Image.asset("assets/images/default_pp.png",
                        height: 30, width: 30, package: "palio_lite"),
                  )
                : Container(
                    width: 30,
                    height: 30,
                    decoration: BoxDecoration(
                        shape: BoxShape.circle,
                        image: DecorationImage(
                            fit: BoxFit.cover,
                            image: LocalImage(data["initiator"]["picture"]))),
                  ),
          ),
          ListTile(
            title: Text(
              "Date Created",
              style: TextStyle(fontWeight: FontWeight.bold),
            ),
            trailing: Text(data["created_date"]),
          ),
          ListTile(
            title: Text(
              "Members",
              style: TextStyle(fontWeight: FontWeight.bold),
            ),
            trailing: Text(widget.changeState.getMemberList.length.toString()),
          )
        ],
      ),
    );
  }

  Widget _members(data) {
    return Card(
      margin: EdgeInsets.symmetric(vertical: 2),
      child: ExpansionTile(
        initiallyExpanded: true,
        title: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            Text(
              "members",
              style: TextStyle(color: Colors.green),
            ),
            !widget.createGroup && !widget.changeState.isAdmin
                ? Container()
                : IconButton(
                    icon: Icon(
                      Icons.person_add,
                      color: Colors.grey,
                    ),
                    onPressed: () async {
                      var getGroupType = await DBHelper().queryCustom(
                          "SELECT * FROM GROUPZ WHERE group_id='${widget.idGroup}'");
                      bool isCC = false;
                      if (getGroupType[0]["official"] == 1 &&
                          getGroupType[0]["is_education"] == 2) {
                        isCC = true;
                      }
                      showDialog(
                          context: context,
                          builder: (_) {
                            return Dialog(
                              child: Container(
                                height: MediaQuery.of(context).size.height,
                                width: MediaQuery.of(context).size.width,
                                child: AddMemberGroup(
                                  idGroup: widget.idGroup,
                                  fPinAdd: widget.changeState.getMemberList,
                                  isCC: isCC,
                                ),
                              ),
                            );
                          });
                    },
                  )
          ],
        ),
        children: <Widget>[
          ListView.builder(
              shrinkWrap: true,
              physics: NeverScrollableScrollPhysics(),
              itemCount: data.length,
              itemBuilder: (builder, index) {
                var item = data[index];
                return ListTile(
                  leading: item["picture"] == ""
                      ? Container(
                          decoration: BoxDecoration(
                              color: Colors.grey, shape: BoxShape.circle),
                          child: Image.asset("assets/images/default_pp.png",
                              height: 30, width: 30, package: "palio_lite"),
                        )
                      : Container(
                          width: 30,
                          height: 30,
                          decoration: BoxDecoration(
                              shape: BoxShape.circle,
                              image: DecorationImage(
                                  fit: BoxFit.cover,
                                  image: LocalImage(item["picture"]))),
                        ),
                  title: Text(item["name"]),
                  trailing: item["isInitiator"] == "1"
                      ? Icon(
                          Icons.verified_user,
                          color: Colors.orange,
                        )
                      : Container(
                          height: 15,
                          width: 15,
                        ),
                  onTap: () async {
                    if (item["id"] != widget.changeState.idMe) {
                      if (widget.changeState.idMe ==
                              widget.changeState.groupData["initiator"]["id"] ||
                          widget.changeState.isAdmin) {
                        showDialog(
                            context: context,
                            builder: (_) {
                              return CupertinoAlertDialog(
                                title: Text(
                                  item["name"],
                                  textAlign: TextAlign.start,
                                ),
                                content: Material(
                                  color: Colors.transparent,
                                  child: Column(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.start,
                                    children: <Widget>[
                                      ListTile(
                                        title: Text("Contact Details"),
                                        contentPadding: EdgeInsets.all(0.0),
                                        onTap: () async {
                                          List<dynamic> resultQuery =
                                              await DBHelper().queryCustom(
                                                  "SELECT * FROM BUDDY where f_pin='${item["id"]}'");
                                          Navigator.push(
                                              context,
                                              PageTransition(
                                                  type: PageTransitionType.fade,
                                                  child: PAProfile(
                                                    profileDet: {
                                                      'f_pin': item["id"],
                                                      'myProfile': false,
                                                      'name': resultQuery[0]
                                                          ["first_name"],
                                                      'follower': 0,
                                                      'official_flag': resultQuery[
                                                                      0][
                                                                  "official_account"] ==
                                                              "1"
                                                          ? 1
                                                          : resultQuery[0][
                                                                      "official_account"] ==
                                                                  "2"
                                                              ? 0
                                                              : 2,
                                                      'isFriend': 1,
                                                      'post': 0,
                                                      'following': 0,
                                                      'isOffline':
                                                          resultQuery[0]
                                                              ["offline_mode"],
                                                      'isMuted': resultQuery[0]
                                                          ["muted"],
                                                      'industry': resultQuery[0]
                                                          ["bidang_industri"],
                                                      'email_address':
                                                          resultQuery[0]
                                                              ["email"],
                                                      'phone_number':
                                                          resultQuery[0]
                                                              ["msisdn"],
                                                      'website': resultQuery[0]
                                                          ["web"],
                                                      'address': resultQuery[0]
                                                          ["address"],
                                                      'status': resultQuery[0]
                                                          ["quote"],
                                                      'isOnline': int.parse(
                                                          resultQuery[0]
                                                              ["connected"]),
                                                      'image': resultQuery[0]
                                                          ["image_id"],
                                                      'official_account':
                                                          resultQuery[0][
                                                              "official_account"],
                                                      'user_type':
                                                          resultQuery[0]
                                                              ["user_type"]
                                                    },
                                                  )));
                                        },
                                      ),
                                      widget.changeState
                                                  .groupData["official"] ==
                                              1
                                          ? Container()
                                          : ListTile(
                                              title: Text(
                                                  item["isInitiator"] == "0"
                                                      ? "Assign as admin"
                                                      : "Remove as admin"),
                                              contentPadding:
                                                  EdgeInsets.all(0.0),
                                              onTap: () {
                                                Navigator.pop(context);
                                                if (item["isInitiator"] ==
                                                    "0") {
                                                  API.changePosition(
                                                      widget.idGroup,
                                                      item["id"],
                                                      "1");
                                                } else {
                                                  API.changePosition(
                                                      widget.idGroup,
                                                      item["id"],
                                                      "0");
                                                }
                                              },
                                            ),
                                      ListTile(
                                        title: Text("Delete member"),
                                        contentPadding: EdgeInsets.all(0.0),
                                        onTap: () {
                                          showDialog(
                                              context: context,
                                              builder: (_) {
                                                return CupertinoAlertDialog(
                                                  title: Text("Delete Member",
                                                      style: TextStyle(
                                                          color: AppColor
                                                              .primaryColor)),
                                                  content: Container(
                                                    padding: EdgeInsets.only(
                                                        top: 15.0),
                                                    child: Text(
                                                        "Are you sure want to delete ${item["name"]} from this group?",
                                                        style: TextStyle(
                                                            color: AppColor
                                                                .primaryColor)),
                                                  ),
                                                  actions: <Widget>[
                                                    FlatButton(
                                                      child: Text(
                                                        "Cancel",
                                                        style: TextStyle(
                                                            color: AppColor
                                                                .primaryColor),
                                                      ),
                                                      onPressed: () {
                                                        Navigator.of(context)
                                                            .pop();
                                                      },
                                                    ),
                                                    FlatButton(
                                                      child: Text(
                                                        "DELETE",
                                                        style: TextStyle(
                                                            color: AppColor
                                                                .primaryColor),
                                                      ),
                                                      onPressed: () async {
                                                        Navigator.pop(context);
                                                        Navigator.pop(context);
                                                        API.exitGroup(
                                                            widget.idGroup,
                                                            item["id"]);
                                                      },
                                                    )
                                                  ],
                                                );
                                              });
                                        },
                                      )
                                    ],
                                  ),
                                ),
                              );
                            });
                      } else {
                        List<dynamic> resultQuery = await DBHelper().queryCustom(
                            "SELECT * FROM BUDDY where f_pin='${item["id"]}'");
                        Navigator.push(
                            context,
                            PageTransition(
                                type: PageTransitionType.fade,
                                child: PAProfile(
                                  profileDet: {
                                    'f_pin': item["id"],
                                    'myProfile': false,
                                    'name': resultQuery[0]["first_name"],
                                    'follower': 0,
                                    'official_flag': resultQuery[0]
                                                ["official_account"] ==
                                            "1"
                                        ? 1
                                        : resultQuery[0]["official_account"] ==
                                                "2"
                                            ? 0
                                            : 2,
                                    'isFriend': 1,
                                    'post': 0,
                                    'following': 0,
                                    'isOffline': resultQuery[0]["offline_mode"],
                                    'isMuted': resultQuery[0]["muted"],
                                    'industry': resultQuery[0]
                                        ["bidang_industri"],
                                    'email_address': resultQuery[0]["email"],
                                    'phone_number': resultQuery[0]["msisdn"],
                                    'website': resultQuery[0]["web"],
                                    'address': resultQuery[0]["address"],
                                    'status': resultQuery[0]["quote"],
                                    'isOnline':
                                        int.parse(resultQuery[0]["connected"]),
                                    'image': resultQuery[0]["image_id"],
                                    'official_account': resultQuery[0]
                                        ["official_account"],
                                    'user_type': resultQuery[0]["user_type"]
                                  },
                                )));
                      }
                    } else {
                      var resultQuery = await DBHelper().queryCustom(
                          "SELECT * FROM BUDDY where f_pin='${widget.changeState.idMe}'");
                      List<dynamic> resultQueryFriend = await DBHelper()
                          .queryCustom(
                              "SELECT f_pin, first_name,last_name,official_account, image_id FROM BUDDY where f_pin <> '${widget.changeState.idMe}'");
                      Navigator.push(
                          context,
                          PageTransition(
                              type: PageTransitionType.fade,
                              child: PAProfile(
                                profileDet: {
                                  'f_pin': widget.changeState.idMe,
                                  'myProfile': true,
                                  'name': resultQuery[0]["first_name"] +
                                      " " +
                                      resultQuery[0]["last_name"],
                                  'follower': 0,
                                  'official_account': resultQuery[0]
                                      ["official_account"],
                                  'isFriend': 0,
                                  'post': 0,
                                  'following': 0,
                                  'friend': resultQueryFriend.length,
                                  'isOffline': resultQuery[0]["offline_mode"],
                                  'isMuted': resultQuery[0]["muted"],
                                  'industry': resultQuery[0]["bidang_industri"],
                                  'email_address': resultQuery[0]["email"],
                                  'phone_number': resultQuery[0]["msisdn"],
                                  'website': resultQuery[0]["web"],
                                  'address': resultQuery[0]["address"],
                                  'status': resultQuery[0]["quote"],
                                  'isOnline':
                                      int.parse(resultQuery[0]["connected"]),
                                  'image': resultQuery[0]["image_id"],
                                  'user_type': resultQuery[0]["user_type"]
                                },
                              )));
                    }
                  },
                );
              })
        ],
      ),
    );
  }
}
