import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';
import 'package:shared_preferences/shared_preferences.dart';

class LoginAdminInternal extends StatefulWidget {
  final bool isAdmin;

  LoginAdminInternal(this.isAdmin);

  @override
  _LoginAdminInternalState createState() => _LoginAdminInternalState();
}

class _LoginAdminInternalState extends State<LoginAdminInternal> {
  Color fillColorTextFieldNickName = Colors.white,
      fillColorTextFieldPassword = Colors.white;
  TextEditingController _controllerPassword = TextEditingController();
  bool visibilityPassword = false;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
          backgroundColor: Colors.transparent,
          elevation: 0.0,
          iconTheme: IconThemeData(color: Colors.black)),
      body: Container(
        width: double.infinity,
        child: SingleChildScrollView(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              SizedBox(height: 50),
              Container(
                padding: EdgeInsets.only(bottom: 20.0, right: 20.0, left: 20.0),
                decoration: BoxDecoration(
                    border: Border(
                        bottom: BorderSide(color: Colors.black, width: 1.0))),
                alignment: Alignment.center,
                width: 200,
                height: 200,
                child: Image.asset(
                  "assets/images/admin.png",
                  package: "palio_lite",
                ),
              ),
              SizedBox(height: 20),
              Text(widget.isAdmin ? "Login as Admin" : "Login as Internal Team",
                  style: TextStyle(
                      fontFamily: "SansMSBerlin",
                      package: "palio_lite",
                      fontSize: 16.0)),
              SizedBox(height: 20),
              Container(
                padding: EdgeInsets.only(left: 80.0, right: 80.0),
                child: Stack(
                  alignment: Alignment.centerRight,
                  children: <Widget>[
                    TextField(
                      textAlign: TextAlign.center,
                      onTap: () {
                        fillColorTextFieldNickName = Colors.white;
                        fillColorTextFieldPassword = Colors.grey;
                        setState(() {});
                      },
                      cursorColor: AppColor.primaryColor,
                      controller: _controllerPassword,
                      obscureText: visibilityPassword ? false : true,
                      decoration: InputDecoration(
                          filled: true,
                          fillColor: fillColorTextFieldPassword,
                          hintText: "Password",
                          contentPadding: EdgeInsets.all(5.0),
                          focusedBorder: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(3.0),
                              borderSide: BorderSide(color: Colors.grey)),
                          border: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(3.0),
                              borderSide: BorderSide(color: Colors.grey))),
                    ),
                    Padding(
                      padding: EdgeInsets.only(right: 10.0),
                      child: InkWell(
                          child: visibilityPassword
                              ? Icon(Icons.visibility, color: Colors.black)
                              : Icon(Icons.visibility_off, color: Colors.black),
                          onTap: () {
                            visibilityPassword = !visibilityPassword;
                            setState(() {});
                          }),
                    )
                  ],
                ),
              ),
              SizedBox(height: 20),
              Container(
                padding: EdgeInsets.only(left: 80.0, right: 80.0),
                child: RawMaterialButton(
                  onPressed: () async {
                    if (_controllerPassword.text.trim().isNotEmpty) {
                      if (FocusScope.of(context).hasFocus) {
                        FocusScope.of(context).unfocus();
                        await Future.delayed(Duration(milliseconds: 100));
                      }
                      SharedPreferences prefs =
                          await SharedPreferences.getInstance();
                      String idMe = prefs.getString("hasLogin") ?? "";
                      String response;
                      showDialog(
                          barrierDismissible: false,
                          context: context,
                          builder: (_) {
                            return CupertinoAlertDialog(
                              title: Row(
                                mainAxisAlignment: MainAxisAlignment.start,
                                children: <Widget>[
                                  Container(
                                      child: CupertinoActivityIndicator(),
                                      width: 20,
                                      height: 20),
                                  SizedBox(width: 20.0),
                                  Text("Updating...",
                                      style: TextStyle(
                                          fontWeight: FontWeight.normal))
                                ],
                              ),
                            );
                          });
                      if (widget.isAdmin) {
                        response = await API.signInAdmin(
                            idMe, _controllerPassword.text);
                      } else {
                        response = await API.signInInternal(
                            idMe, _controllerPassword.text);
                      }
                      if (response == null) {
                        Navigator.pop(context);
                        Fluttertoast.showToast(
                            msg: "Unable to access servers.");
                      } else {
                        if (response == "11") {
                          Navigator.pop(context);
                          Fluttertoast.showToast(
                              msg: "Username or password does not match");
                        } else if (response == "20") {
                          Navigator.pop(context);
                          Fluttertoast.showToast(msg: "Invalid Password");
                        } else {
                          if (widget.isAdmin) {
                            await API.postRegistration();
                            Navigator.pop(context);
                            Fluttertoast.showToast(
                                msg: "Successfully login Admin");
                          } else {
                            await API.postRegistration();
                            Navigator.pop(context);
                            Fluttertoast.showToast(
                                msg: "Successfully login Internal Team");
                          }
                          Navigator.pop(context);
                        }
                      }
                    } else {
                      Fluttertoast.showToast(
                          msg: "Password's field can't be empty");
                    }
                  },
                  fillColor: Color(0xFF213458),
                  constraints: BoxConstraints.tight(Size(double.infinity, 50)),
                  shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(8.0),
                  ),
                  child: Text(
                    "SIGN-IN",
                    textAlign: TextAlign.center,
                    style: TextStyle(
                        fontWeight: FontWeight.bold,
                        fontFamily: "SansMSBerlin",
                        color: Colors.white),
                  ),
                ),
              )
            ],
          ),
        ),
      ),
    );
  }
}
