import 'dart:async';
import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/chat/add_friend_screen.dart';
import 'package:palio_lite/chat/chat_group_screen.dart';
import 'package:palio_lite/chat/chat_screen.dart';
import 'package:palio_lite/chat/group_detail.dart';
import 'package:palio_lite/chat/initiate_broadcast.dart';
import 'package:palio_lite/chat/settings_screen.dart';
import 'package:palio_lite/chat/tab_3.dart';
import 'package:palio_lite/chat/widgets/custom_expansion_tile.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ChangeState with ChangeNotifier {
  List<Map<dynamic, dynamic>> _groupData = [],
      _dataContact = [],
      _srcContact = [],
      _srcGrup = [];

  List<Map<dynamic, dynamic>> get getGroupData => _groupData;

  List<Map<dynamic, dynamic>> get getDataContact => _dataContact;

  List<Map<dynamic, dynamic>> get getSrcContact => _srcContact;

  List<Map<dynamic, dynamic>> get getSrcGrup => _srcGrup;

  void clearData() {
    _groupData.clear();
    _dataContact.clear();
    notifyListeners();
  }

  set groupData(List<Map<dynamic, dynamic>> value) {
    _groupData.addAll(value);
    notifyListeners();
  }

  set replaceGroupData(List<Map<dynamic, dynamic>> value) {
    _groupData = value;
    notifyListeners();
  }

  set dataContact(Map<dynamic, dynamic> value) {
    _dataContact.add(value);
    notifyListeners();
  }

  set srcContact(List<Map<dynamic, dynamic>> value) {
    _srcContact.addAll(value);
    notifyListeners();
  }

  set replaceSrcContact(List<Map<dynamic, dynamic>> value) {
    _srcContact = value;
    notifyListeners();
  }

  set addSrcContact(Map<dynamic, dynamic> value) {
    _srcContact.add(value);
    notifyListeners();
  }

  set clearSrcContact(bool value) {
    _srcContact.clear();
    notifyListeners();
  }

  set srcGrup(List<Map<dynamic, dynamic>> value) {
    _srcGrup.addAll(value);
    notifyListeners();
  }

  set replaceSrcGrup(List<Map<dynamic, dynamic>> value) {
    _srcGrup = value;
    notifyListeners();
  }

  set addSrcGrup(Map<dynamic, dynamic> value) {
    _srcGrup.add(value);
    notifyListeners();
  }

  set clearSrcGrup(bool value) {
    _srcGrup.clear();
    notifyListeners();
  }

  void clearChatDataGroup() {
    _srcGrup.clear();
    _groupData.clear();
    notifyListeners();
  }

// set hideTopic(var value) {
//   if (value["indexSubGroup"] == -1) {
//     _srcGrup[value["index"]]["saveSubGroup"] =
//         _srcGrup[value["index"]]["subGroup"];
//     _srcGrup[value["index"]]["subGroup"] = [];
//   } else {
//     _srcGrup[value["index"]]["saveTopic"] = _srcGrup[value["index"]]["topic"];
//     List<Map<dynamic, dynamic>> listSubGroup =
//         _srcGrup[value["index"]]["subGroup"];
//     List<Map<dynamic, dynamic>> tempListSubGroup = List();
//     tempListSubGroup.addAll(listSubGroup);
//     _srcGrup[value["index"]]["saveSubGroup"] = tempListSubGroup;
//     listSubGroup.removeWhere((element) =>
//         element["id"] != listSubGroup[value["indexSubGroup"]]["id"]);
//   }
//   notifyListeners();
// }
//
// set showTopic(var value) {
//   _srcGrup[value["index"]]["subGroup"] =
//       _srcGrup[value["index"]]["saveSubGroup"];
//   if (value["indexSubGroup"] == -1) {
//     _srcGrup[value["index"]]["saveSubGroup"] = null;
//   } else {
//     _srcGrup[value["index"]]["saveTopic"] = null;
//     _srcGrup[value["index"]]["saveSubGroup"] = null;
//   }
//   notifyListeners();
// }
}

class NewChat extends StatefulWidget {
  @override
  _NewChatState createState() => _NewChatState();
}

class _NewChatState extends State<NewChat> {
  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
        create: (_) => ChangeState(),
        child: Consumer<ChangeState>(
            builder: (context, changeState, _) => NewChats(changeState)));
  }
}

class NewChats extends StatefulWidget {
  final ChangeState changeState;

  NewChats(this.changeState);

  _NewChats createState() => _NewChats();
}

class _NewChats extends State<NewChats> with SingleTickerProviderStateMixin {
  TextEditingController _editingController = new TextEditingController();
  TextEditingController _textEditingController = new TextEditingController();
  FocusNode focusTextGroup = FocusNode();
  String lang = "";
  GlobalKey<ThirdTabsState> tab3Key = GlobalKey();

  static List<Widget> _tabs = <Widget>[
    Tab(text: "Chat"),
    Tab(text: "Contact"),
    Tab(text: "Group")
  ];

  TabController? _tabController;
  String idMe = "";

  Future<String> getLogin() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("hasLogin")) {
      idMe = prefs.getString("hasLogin") ?? "";
      API.pullBatchBuddyList(idMe);
    }
    return idMe;
  }

  void onGroup(BuildContext context) {
    API.onGroup = (String code, String fPin, String groupId) async {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      String idMe = prefs.getString("hasLogin") ?? "";
      if (code == "A012" && fPin == "-1") {
        var resultQuery = await DBHelper().queryCustom(
            "SELECT f_name,created_by, created_date FROM GROUPZ WHERE group_id='$groupId'");
        if (resultQuery[0]["created_by"] == idMe) {
          Fluttertoast.showToast(
              msg: "Successfully created a group",
              toastLength: Toast.LENGTH_SHORT);
          Navigator.pushReplacement(
              context,
              PageTransition(
                  type: PageTransitionType.fade,
                  child: GroupDetail(
                    nameGroup: resultQuery[0]["f_name"],
                    idGroup: groupId,
                    creator: resultQuery[0]["created_by"],
                    createGroup: true,
                    createdDate: resultQuery[0]["created_date"],
                  )));
        } else {
          if (mounted) {
            // bool contain = false;
            // for (var map in widget.changeState.getGroupData) {
            //   if (map.containsKey("id")) {
            //     if (map["id"] == groupId) {
            //       contain = true;
            //       break;
            //     }
            //   }
            // }
            // if(!contain) {
            //   widget.changeState.clearChatDataGroup();
            //   getGroup();
            // }
            getGroup(replace: true);
          }
        }
      } else if (code == "A008" || code == "A010") {
        if (mounted) {
          getGroup(replace: true);
        }
      }
    };
  }

  @override
  void initState() {
    super.initState();
    onUpdateStatus();
    getLogin();
    _tabController = TabController(
      length: _tabs.length,
      vsync: this,
    );
    _tabController?.addListener(() {
      onUpdateStatus();
      if (_editingController.text.isNotEmpty) {
        _editingController.clear();
        if (_tabController?.previousIndex == 0) {
          tab3Key.currentState?.searchChat("");
        } else if (_tabController?.previousIndex == 1) {
          widget.changeState.clearSrcContact = true;
          widget.changeState.srcContact = widget.changeState.getDataContact;
        } else {
          widget.changeState.clearSrcGrup = true;
          widget.changeState.srcGrup = widget.changeState.getGroupData;
        }
      }
    });
    onGroup(this.context);
    getData();
  }

  void onUpdateStatus() {
    API.onStatus = (int state, String status) {
      if (state == 99) {
        getData(replace: true);
      }
    };
  }

  Future<String> getLanguageSetting() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("language")) {
      lang = prefs.getString("language") ?? "";
    }
    return lang;
  }

  void getData({bool replace = false}) async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    String idMe = prefs.getString("hasLogin") ?? "";
    List<dynamic> resultQuery = await DBHelper().queryCustom(
        "SELECT f_pin, first_name, last_name, official_account, image_id, user_type FROM BUDDY where f_pin <> '$idMe'");
    if (replace) {
      if (mounted) {
        widget.changeState.clearData();
      }
    }
    for (var i = 0; i < resultQuery.length; i++) {
      String name;
      if (resultQuery[i]["last_name"].trim().isNotEmpty) {
        name = resultQuery[i]["first_name"] + " " + resultQuery[i]["last_name"];
      } else {
        name = resultQuery[i]["first_name"];
      }
      if (idMe.isNotEmpty && idMe != resultQuery[i]["f_pin"]) {
        widget.changeState.dataContact = {
          "f_pin": resultQuery[i]["f_pin"],
          "name": resultQuery[i]["first_name"],
          "picture": resultQuery[i]["image_id"],
          "isVerified": resultQuery[i]["official_account"] == "2" ? "1" : "0",
          "isOfficial": resultQuery[i]["official_account"] == "1" ? "1" : "0",
          "deviceId": resultQuery[i]["device_id"],
          "user_type": resultQuery[i]["user_type"]
        };
      }
    }
    widget.changeState.getDataContact
        .sort((a, b) => (a["name"]).compareTo(b["name"]));
    if (mounted) {
      if (replace) {
        widget.changeState.replaceSrcContact =
            widget.changeState.getDataContact;
      } else {
        widget.changeState.srcContact = widget.changeState.getDataContact;
      }
    }
    if (!replace) {
      getGroup();
    }
  }

  getGroup(
      {bool replace = false,
      bool fromSearch = false,
      String textSearch = ""}) async {
    List<dynamic> resultQueryGroup;
    if (fromSearch) {
      List<dynamic> tempGroupFromTopic = [];
      List<dynamic> tempGroup = await DBHelper().queryCustom(
          "SELECT * FROM GROUPZ WHERE f_name LIKE '%$textSearch%' AND parent = ''");
      List<dynamic> resultQueryTopicTemp = await DBHelper().queryCustom(
          "SELECT DISTINCT group_id FROM DISCUSSION_FORUM WHERE title LIKE '%$textSearch%'");
      for (var i = 0; i < resultQueryTopicTemp.length; i++) {
        resultQueryGroup = await DBHelper().queryCustom(
            "SELECT * FROM GROUPZ WHERE group_id='${resultQueryTopicTemp[i]["group_id"]}' AND parent = '' AND is_education != 1");
        tempGroupFromTopic.add(resultQueryGroup[0]);
      }
      List<dynamic> combinedTemp = [...tempGroup, ...tempGroupFromTopic];
      List<dynamic> newCombinedList(items) {
        Map<String, dynamic> uniqueItems = {};
        for (Map item in items) {
          final key = item["group_id"];
          if (uniqueItems[key] == null) {
            uniqueItems[key] = item;
          }
        }

        return uniqueItems.values.toList();
      }

      final finalCombinedList = newCombinedList(combinedTemp);
      print(finalCombinedList);
      resultQueryGroup = finalCombinedList;
    } else {
      resultQueryGroup = await DBHelper().queryCustom(
          "SELECT * FROM GROUPZ WHERE (parent = '' OR parent IS NULL) AND is_education != 1");
    }
    List<Map<dynamic, dynamic>> temporaryGroupData = [];
    for (var j = 0; j < resultQueryGroup.length; j++) {
      List<dynamic> resultQueryTopic = await DBHelper().queryCustom(
          "SELECT chat_id, title FROM DISCUSSION_FORUM WHERE group_id='${resultQueryGroup[j]["group_id"]}'");
      List<dynamic> temporaryTopic = [];
      // if (lang == "en_US") {
      //   temporaryTopic.add({"name": "Lounge", "id": ""});
      // } else {
      //   temporaryTopic.add({"name": "Beranda", "id": ""});
      // }
      temporaryTopic.add({"name": "Lounge", "id": ""});
      if (resultQueryTopic.length > 0) {
        for (var j = 0; j < resultQueryTopic.length; j++) {
          temporaryTopic.add({
            "name": resultQueryTopic[j]["title"],
            "id": resultQueryTopic[j]["chat_id"]
          });
        }
        temporaryTopic.sort((a, b) => (a["name"]).compareTo(b["name"]));
      }

      final Map myVariableDynamic = {};

      Future addSubGroup(String groupId, int groupType, int level) async {
        myVariableDynamic["dataSubGroup$level"] = [];
        myVariableDynamic["resultQuerySubGroup$level"] = await DBHelper()
            .queryCustom(
                "SELECT group_id, f_name, image_id, official, is_open, level_edu, materi_edu, is_education FROM GROUPZ WHERE parent='$groupId' AND is_education != 1");
        for (var j = 0;
            j < myVariableDynamic["resultQuerySubGroup$level"].length;
            j++) {
          List<dynamic> resultQueryTopic = await DBHelper().queryCustom(
              "SELECT chat_id, title FROM DISCUSSION_FORUM WHERE group_id='${myVariableDynamic["resultQuerySubGroup$level"][j]["group_id"]}'");
          List<dynamic> temporaryTopic = [];
          // if (lang == "en_US") {
          //   temporaryTopic.add({"name": "Lounge", "id": ""});
          // } else {
          //   temporaryTopic.add({"name": "Beranda", "id": ""});
          // }
          temporaryTopic.add({"name": "Lounge", "id": ""});
          if (resultQueryTopic.length > 0) {
            for (var j = 0; j < resultQueryTopic.length; j++) {
              temporaryTopic.add({
                "name": resultQueryTopic[j]["title"],
                "id": resultQueryTopic[j]["chat_id"]
              });
            }
            temporaryTopic.sort((a, b) => (a["name"]).compareTo(b["name"]));
          }

          myVariableDynamic["listSubGroup${level + 1}"] = await addSubGroup(
              myVariableDynamic["resultQuerySubGroup$level"][j]["group_id"],
              groupType,
              (level + 1));
          myVariableDynamic["dataSubGroup$level"].add({
            "id": myVariableDynamic["resultQuerySubGroup$level"][j]["group_id"],
            "name": myVariableDynamic["resultQuerySubGroup$level"][j]["f_name"],
            "picture": myVariableDynamic["resultQuerySubGroup$level"][j]
                ["image_id"],
            "topic": temporaryTopic,
            "subGroup": myVariableDynamic["listSubGroup${level + 1}"],
            "saveSubGroup": null,
            "saveTopic": null,
            "isVerified": myVariableDynamic["resultQuerySubGroup$level"][j]
                        ["official"] ==
                    "2"
                ? "1"
                : "0",
            "isOfficial": myVariableDynamic["resultQuerySubGroup$level"][j]
                            ["official"] ==
                        "1" ||
                    groupType == 1
                ? "1"
                : "0",
            "isOpen": myVariableDynamic["resultQuerySubGroup$level"][j]
                        ["is_education"] ==
                    1
                ? "2"
                : myVariableDynamic["resultQuerySubGroup$level"][j]["is_open"]
                    .toString()
          });
        }
        return myVariableDynamic["dataSubGroup$level"];
      }

      var listSubGroup;

      var resultQueryBuddy = await DBHelper()
          .queryCustom("SELECT user_type FROM BUDDY where f_pin='$idMe'");

      if (resultQueryBuddy[0]["user_type"] != "23" &&
          resultQueryBuddy[0]["user_type"] != "24" &&
          resultQueryGroup[j]["group_type"] == 1) {
        listSubGroup = [];
      } else {
        listSubGroup = await addSubGroup(resultQueryGroup[j]["group_id"],
            resultQueryGroup[j]["group_type"], 1);
      }
      temporaryGroupData.add({
        "id": resultQueryGroup[j]["group_id"],
        "name": resultQueryGroup[j]["f_name"],
        "picture": resultQueryGroup[j]["image_id"],
        "topic": temporaryTopic,
        "subGroup": listSubGroup,
        "saveSubGroup": null,
        "saveTopic": null,
        "isVerified": resultQueryGroup[j]["official"] == "2" ? "1" : "0",
        "isOfficial": resultQueryGroup[j]["official"] == "1" ||
                resultQueryGroup[j]["group_type"] == 1
            ? "1"
            : "0",
        "isOpen": resultQueryGroup[j]["is_education"] == 1
            ? "2"
            : resultQueryGroup[j]["is_open"].toString()
      });
    }
    if (replace) {
      widget.changeState.clearChatDataGroup();
    }
    widget.changeState.groupData = temporaryGroupData;
    widget.changeState.getGroupData
        .sort((b, a) => (a["isOfficial"]).compareTo(b["isOfficial"]));
    temporaryGroupData.clear();
    var openGroups = await API.getOpenGroups(
        "1,2,3,5,6,7", "0", fromSearch ? textSearch : "");
    openGroups = jsonDecode(openGroups);
    for (var j = 0; j < openGroups.length; j++) {
      bool ada = false;
      for (var k = 0; k < widget.changeState.getGroupData.length; k++) {
        if (widget.changeState.getGroupData[k]["id"] == openGroups[j]["A04"]) {
          ada = true;
          break;
        }
      }
      if (ada) {
        continue;
      }
      temporaryGroupData.add({
        "id": openGroups[j]["A04"],
        "name": openGroups[j]["A05"],
        "picture": openGroups[j]["A74"],
        "topic": null,
        "isVerified": "0",
        "isOfficial": "0",
        "isOpen": openGroups[j]["IOP"],
        "description": openGroups[j]["A16"],
        "members": openGroups[j]["A43"]
      });
    }
    widget.changeState.groupData = temporaryGroupData;
    if (replace) {
      widget.changeState.replaceSrcGrup = widget.changeState.getGroupData;
    } else {
      widget.changeState.srcGrup = widget.changeState.getGroupData;
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  _search(name) async {
    if (_tabController?.index == 0) {
      tab3Key.currentState?.searchChat(name);
    } else if (_tabController?.index == 1) {
      if (name.toString().isNotEmpty) {
        widget.changeState.clearSrcContact = true;
        for (int i = 0; i < widget.changeState.getDataContact.length; i++) {
          if (widget.changeState.getDataContact[i]["name"]
              .toLowerCase()
              .contains(name.toLowerCase())) {
            widget.changeState.addSrcContact =
                widget.changeState.getDataContact[i];
          }
        }
      } else {
        widget.changeState.clearSrcContact = true;
        widget.changeState.srcContact = widget.changeState.getDataContact;
      }
    } else {
      if (name.toString().isNotEmpty) {
        widget.changeState.clearSrcGrup = true;
        for (int i = 0; i < widget.changeState.getGroupData.length; i++) {
          if (widget.changeState.getGroupData[i]["name"]
              .toLowerCase()
              .contains(name.toLowerCase())) {
            widget.changeState.addSrcGrup = widget.changeState.getGroupData[i];
          }
        }
      } else {
        widget.changeState.clearSrcGrup = true;
        widget.changeState.srcGrup = widget.changeState.getGroupData;
      }
    }
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        automaticallyImplyLeading: true,
        backgroundColor: AppColor.primaryColor,
        title: Text("Start Conversation"),
        actions: [
          IconButton(
              icon: Image.asset("assets/images/ic_broadcast.png",
                  package: 'palio_lite'),
              onPressed: () async {
                SharedPreferences prefs = await SharedPreferences.getInstance();
                var resultQueryBuddy = await DBHelper().queryCustom(
                    "SELECT user_type FROM BUDDY where f_pin='$idMe'");
                if (resultQueryBuddy[0]["user_type"] == "23" ||
                    resultQueryBuddy[0]["user_type"] == "24") {
                  await Navigator.push(
                      context,
                      PageTransition(
                          type: PageTransitionType.fade,
                          child: BroadcastPage()));
                } else {
                  Fluttertoast.showToast(msg: "Feature Disabled...");
                }
              }),
          PopupMenuButton(
            icon: Icon(CupertinoIcons.add),
            itemBuilder: (context) => [
              PopupMenuItem(
                  child: ListTile(
                leading: Icon(Icons.group_add, color: Colors.black),
                title: Text("Create Group"),
                contentPadding: EdgeInsets.all(0.0),
                dense: true,
                onTap: () {
                  Navigator.pop(context);
                  showDialog(
                      context: context,
                      builder: (_) {
                        return CupertinoAlertDialog(
                          title: Row(
                            mainAxisAlignment: MainAxisAlignment.start,
                            children: <Widget>[
                              Image.asset(
                                "assets/images/groupicon.png",
                                width: 25,
                                height: 25,
                                package: "palio_lite",
                              ),
                              Text("Create Group",
                                  style:
                                      TextStyle(color: AppColor.primaryColor))
                            ],
                          ),
                          content: Material(
                            color: Colors.transparent,
                            child: TextField(
                              cursorColor: AppColor.primaryColor,
                              controller: _textEditingController,
                              focusNode: focusTextGroup,
                              decoration: InputDecoration(
                                  fillColor: Colors.transparent,
                                  focusedBorder: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFF364656))),
                                  border: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFF364656))),
                                  hintText: "Set Group Name",
                                  filled: true),
                            ),
                          ),
                          actions: <Widget>[
                            FlatButton(
                              child: Text(
                                "Cancel",
                                style: TextStyle(
                                    color: AppColor.primaryColor,
                                    fontSize: 12.0),
                              ),
                              onPressed: () {
                                Navigator.of(context).pop();
                              },
                            ),
                            FlatButton(
                              child: Text(
                                "CREATE GROUP",
                                style: TextStyle(
                                    color: AppColor.primaryColor,
                                    fontSize: 12.0),
                                textAlign: TextAlign.center,
                              ),
                              onPressed: () async {
                                focusTextGroup.unfocus();
                                if (_textEditingController.text
                                    .trim()
                                    .isEmpty) {
                                  Fluttertoast.showToast(
                                      msg: "That field can't be empty",
                                      toastLength: Toast.LENGTH_SHORT);
                                  return;
                                }
                                if (RegExp(r'[!@#<>?":_`~;[\]\\|=+)(*&^%-]')
                                    .hasMatch(_textEditingController.text)) {
                                  Fluttertoast.showToast(
                                      msg:
                                          "Input can only be alphabet and numbers",
                                      toastLength: Toast.LENGTH_SHORT);
                                  return;
                                }
                                Navigator.pop(context);
                                String result = await API
                                    .createGroup(_textEditingController.text);
                                if (result != "00") {
                                  Fluttertoast.showToast(
                                      msg: "CREATE GROUP FAILED",
                                      toastLength: Toast.LENGTH_SHORT);
                                }
                              },
                            )
                          ],
                        );
                      });
                },
              )),
              PopupMenuItem(
                  child: ListTile(
                leading:
                    Icon(CupertinoIcons.person_add_solid, color: Colors.black),
                title: Text("Add Friend"),
                contentPadding: EdgeInsets.all(0.0),
                dense: true,
                onTap: () async {
                  Navigator.pop(context);
                  Navigator.pushReplacement(
                      context,
                      PageTransition(
                          type: PageTransitionType.fade,
                          child: AddFriendScreen(afterLogin: true)));
                },
              )),
              PopupMenuItem(
                  child: ListTile(
                leading: Icon(CupertinoIcons.mail_solid, color: Colors.black),
                title: Text("Email Configuration"),
                contentPadding: EdgeInsets.all(0.0),
                dense: true,
                onTap: () {
                  Navigator.of(context).pop();
                  showDialog(
                      context: context,
                      builder: (_) {
                        return CupertinoAlertDialog(
                          title: Text("Email account is not active"),
                          content: Text(
                              "Connect your email to the application via the settings page"),
                          actions: <Widget>[
                            FlatButton(
                              child: Text(
                                "Cancel",
                                style: TextStyle(
                                    color: AppColor.primaryColor,
                                    fontSize: 12.0),
                              ),
                              onPressed: () {
                                Navigator.of(context).pop();
                              },
                            ),
                            FlatButton(
                              child: Text(
                                "SETTINGS",
                                style: TextStyle(
                                    color: AppColor.primaryColor,
                                    fontSize: 12.0),
                                textAlign: TextAlign.center,
                              ),
                              onPressed: () {
                                Navigator.of(context).pop();
                                Navigator.pushReplacement(
                                    context,
                                    CupertinoPageRoute(
                                        builder: (_) => SettingsView()));
                              },
                            )
                          ],
                        );
                      });
                },
              ))
            ],
          ),
        ],
      ),
      body: Container(
        child: Column(
          children: <Widget>[
            Container(
              decoration: BoxDecoration(
                  color: Colors.grey[100],
                  border: Border(
                      bottom:
                          BorderSide(color: Colors.grey[200]!, width: 2.0))),
              padding: EdgeInsets.all(2.0),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: <Widget>[
                  Container(
                    width: MediaQuery.of(context).size.width * 0.98,
                    child: Theme(
                      child: TextField(
                          controller: _editingController,
                          onSubmitted: (val) async {
                            await _search(val);
                          },
                          cursorColor: AppColor.primaryColor,
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: Colors.white,
                              contentPadding: EdgeInsets.all(5.0),
                              prefixIcon: Icon(Icons.search),
                              focusedBorder: OutlineInputBorder(
                                  borderRadius: BorderRadius.circular(3.0),
                                  borderSide: BorderSide(color: Colors.grey)),
                              border: OutlineInputBorder(
                                  borderRadius: BorderRadius.circular(3.0),
                                  borderSide: BorderSide(color: Colors.grey)))),
                      data:
                          Theme.of(context).copyWith(primaryColor: Colors.grey),
                    ),
                  )
                ],
              ),
            ),
            Container(
              child: TabBar(
                labelColor: Color(0xFF213458),
                indicatorColor: Color(0xFF213458),
                controller: _tabController,
                tabs: _tabs,
              ),
            ),
            Expanded(
              child: TabBarView(
                controller: _tabController,
                children: <Widget>[
                  ThirdTab(keyState: tab3Key),
                  _contactList(widget.changeState.getSrcContact),
                  _groupList(widget.changeState.getSrcGrup)
                ],
              ),
            )
          ],
        ),
      ),
    );
  }

  Widget _contactList(data) {
    return ListView.builder(
        shrinkWrap: true,
        itemCount: data.length,
        itemBuilder: (builder, index) {
          return ListTile(
            leading: Container(
              width: 45,
              height: 45,
              decoration: BoxDecoration(
                  shape: BoxShape.circle,
                  color: Colors.grey,
                  image: DecorationImage(
                      fit: BoxFit.cover,
                      image: data[index]["picture"].trim().isEmpty
                          ? AssetImage("assets/images/default_pp.png",
                              package: "palio_lite")
                          : data[index]["picture"] == "palio_logo_round.png"
                              ? AssetImage("assets/images/palio.png",
                                  package: "palio_lite")
                              : LocalImage(data[index]["picture"])
                                  as ImageProvider)),
            ),
            title: Row(
              children: <Widget>[
                data[index]["isVerified"] == "1"
                    ? Image.asset("assets/images/ic_verified_flag.png",
                        width: 15, height: 15, package: "palio_lite")
                    : data[index]["isOfficial"] == "1"
                        ? Image.asset("assets/images/ic_official_flag.png",
                            width: 15, height: 15, package: "palio_lite")
                        : data[index]["user_type"] == "23"
                            ? Image.asset("assets/images/ic_internal.png",
                                width: 15, height: 15, package: "palio_lite")
                            : data[index]["user_type"] == "24"
                                ? Image.asset("assets/images/call_center.png",
                                    width: 15,
                                    height: 15,
                                    package: "palio_lite")
                                : Container(),
                Padding(
                  padding: EdgeInsets.only(
                    left: 8.0,
                  ),
                  child: Text(
                    data[index]["name"],
                  ),
                )
              ],
            ),
            onTap: () async {
              Navigator.pushReplacement(
                  context,
                  PageTransition(
                      type: PageTransitionType.fade,
                      child: ChatPage(
                        fullName: data[index]["name"],
                        image: data[index]["picture"],
                        identifier: data[index]["f_pin"],
                        refresh: true,
                      )));
            },
          );
        });
  }

  Widget _groupList(data) {
    return ListView.builder(
        shrinkWrap: true,
        itemCount: data.length,
        itemBuilder: (builder, index) {
          var topic = data[index]["topic"];
          var subGroup = data[index]["subGroup"];
          print(data[index]["picture"]);
          return CustomExpansionTile(
            onExpansionChanged: (cond) {
              // if (!cond) {
              //   widget.changeState.clearChatDataGroup();
              //   getGroup();
              // }
            },
            leading: Container(
              width: 45,
              height: 45,
              decoration: BoxDecoration(
                  shape: BoxShape.circle,
                  color: Colors.grey,
                  image: DecorationImage(
                      fit: BoxFit.cover,
                      image: data[index]["picture"] == null ||
                              data[index]["picture"].trim().isEmpty
                          ? AssetImage("assets/images/groupicon.png",
                              package: "palio_lite")
                          : data[index]["picture"] == "palio_logo_round.png"
                              ? AssetImage("assets/images/palio.png",
                                  package: "palio_lite")
                              : LocalImage(data[index]["picture"])
                                  as ImageProvider)),
            ),
            title: Row(
              children: <Widget>[
                data[index]["isVerified"] == "1"
                    ? Image.asset("assets/images/ic_verified_flag.png",
                        width: 20, height: 20, package: "palio_lite")
                    : data[index]["isOfficial"] == "1"
                        ? Image.asset("assets/images/ic_official_flag.png",
                            width: 20, height: 20, package: "palio_lite")
                        : data[index]["isOpen"] == "1"
                            ? Icon(Icons.public, size: 20)
                            : data[index]["isOpen"] == "2"
                                ? Image.asset(
                                    "assets/images/ic_educational.png",
                                    width: 20,
                                    height: 20,
                                    package: "palio_lite",
                                    color: Colors.black)
                                : Icon(Icons.lock, size: 20),
                Padding(
                  padding: EdgeInsets.only(
                    left: 8.0,
                  ),
                  child: Text("${data[index]["name"]}"),
                )
              ],
            ),
            children: <Widget>[
              data[index]["saveTopic"] != null
                  ? Container()
                  : Container(
                      margin: EdgeInsets.only(left: 28.0),
                      child: CustomExpansionTile(
                        onExpansionChanged: (cond) {
                          // if (cond) {
                          //   widget.changeState.hideTopic = {
                          //     "index": index,
                          //     "indexSubGroup": -1
                          //   };
                          // } else {
                          //   widget.changeState.showTopic = {
                          //     "index": index,
                          //     "indexSubGroup": -1
                          //   };
                          // }
                        },
                        leading: Container(
                          width: 45,
                          height: 45,
                          decoration: BoxDecoration(
                              shape: BoxShape.circle, color: Colors.grey),
                          child: Image.asset("assets/images/forum.png",
                              package: "palio_lite"),
                        ),
                        title: Text(
                          "Forum",
                          style: TextStyle(fontSize: 15, color: Colors.black),
                        ),
                        children: [
                          Container(
                            margin: EdgeInsets.only(left: 28.0),
                            child: ListView.builder(
                              shrinkWrap: true,
                              physics: NeverScrollableScrollPhysics(),
                              itemCount: topic.length,
                              itemBuilder: (context, indexTopic) {
                                return ListTile(
                                  contentPadding: EdgeInsets.only(left: 5.0),
                                  leading: Container(
                                    width: 30,
                                    height: 30,
                                    decoration: BoxDecoration(
                                        image: DecorationImage(
                                            fit: BoxFit.cover,
                                            image: AssetImage(
                                                "assets/images/tab_menu_uc.png",
                                                package: "palio_lite"))),
                                  ),
                                  title: Text(
                                    "${topic[indexTopic]["name"]}",
                                    style: TextStyle(
                                        fontSize: 15, color: Colors.black),
                                  ),
                                  onTap: () async {
                                    Navigator.pushReplacement(context,
                                        CupertinoPageRoute(builder: (context) {
                                      return ChatGroupPage(
                                          data[index]["name"],
                                          topic[indexTopic]["name"],
                                          data[index]["picture"],
                                          identifier: topic[indexTopic]["id"],
                                          idGroup: data[index]["id"]);
                                    }));
                                  },
                                );
                              },
                            ),
                          )
                        ],
                      ),
                    ),
              ListView.builder(
                shrinkWrap: true,
                physics: NeverScrollableScrollPhysics(),
                itemCount: subGroup.length,
                itemBuilder: (context, indexSubGroub) {
                  var topicSubGroup1 = subGroup[indexSubGroub]["topic"];
                  var subGroup1 = subGroup[indexSubGroub]["subGroup"];
                  return Container(
                    margin: EdgeInsets.only(left: 28.0),
                    child: CustomExpansionTile(
                      onExpansionChanged: (cond) {
                        // if (cond) {
                        //   if (data[index]["saveSubGroup"] == null) {
                        //     widget.changeState.hideTopic = {
                        //       "index": index,
                        //       "indexSubGroup": indexSubGroub
                        //     };
                        //   }
                        // } else {
                        //   widget.changeState.showTopic = {
                        //     "index": index,
                        //     "indexSubGroup": indexSubGroub
                        //   };
                        // }
                      },
                      leading: Container(
                          width: 45,
                          height: 45,
                          decoration: BoxDecoration(
                              shape: BoxShape.circle,
                              color: Colors.grey,
                              image: DecorationImage(
                                  fit: BoxFit.cover,
                                  image: subGroup[indexSubGroub]["picture"] ==
                                              null ||
                                          subGroup[indexSubGroub]["picture"]
                                              .trim()
                                              .isEmpty
                                      ? AssetImage(
                                          "assets/images/groupicon.png",
                                          package: "palio_lite")
                                      : LocalImage(subGroup[indexSubGroub]
                                          ["picture"]) as ImageProvider))),
                      title: Row(
                        mainAxisSize: MainAxisSize.max,
                        children: <Widget>[
                          subGroup[indexSubGroub]["isVerified"] == "1"
                              ? Image.asset(
                                  "assets/images/ic_verified_flag.png",
                                  width: 20,
                                  height: 20,
                                  package: "palio_lite")
                              : subGroup[indexSubGroub]["isOfficial"] == "1"
                                  ? Image.asset(
                                      "assets/images/ic_official_flag.png",
                                      width: 20,
                                      height: 20,
                                      package: "palio_lite")
                                  : subGroup[indexSubGroub]["isOpen"] == "1"
                                      ? Icon(Icons.public, size: 20)
                                      : subGroup[indexSubGroub]["isOpen"] == "2"
                                          ? Image.asset(
                                              "assets/images/ic_educational.png",
                                              width: 20,
                                              height: 20,
                                              package: "palio_lite",
                                              color: Colors.black,
                                            )
                                          : Icon(Icons.lock, size: 20),
                          Container(
                            padding: EdgeInsets.only(
                              left: 8.0,
                            ),
                            child: Text("${subGroup[indexSubGroub]["name"]}",
                                overflow: TextOverflow.ellipsis),
                            constraints: BoxConstraints(maxWidth: 210),
                          )
                        ],
                      ),
                      children: [
                        Container(
                          margin: EdgeInsets.only(left: 28.0),
                          child: CustomExpansionTile(
                            leading: Container(
                              width: 45,
                              height: 45,
                              decoration: BoxDecoration(
                                  shape: BoxShape.circle, color: Colors.grey),
                              child: Image.asset("assets/images/forum.png",
                                  package: "palio_lite"),
                            ),
                            title: Text(
                              "Forum",
                              style:
                                  TextStyle(fontSize: 15, color: Colors.black),
                            ),
                            children: [
                              Container(
                                margin: EdgeInsets.only(left: 28.0),
                                child: ListView.builder(
                                  shrinkWrap: true,
                                  physics: NeverScrollableScrollPhysics(),
                                  itemCount: topicSubGroup1.length,
                                  itemBuilder: (context, indexTopicSubgroup1) {
                                    return ListTile(
                                      title: Text(
                                        "${topicSubGroup1[indexTopicSubgroup1]["name"]}",
                                        style: TextStyle(
                                            fontSize: 15, color: Colors.black),
                                      ),
                                      leading: Container(
                                          height: 30,
                                          width: 30,
                                          decoration: BoxDecoration(
                                              image: DecorationImage(
                                                  fit: BoxFit.cover,
                                                  image: AssetImage(
                                                      "assets/images/tab_menu_uc.png",
                                                      package: "palio_lite")))),
                                      onTap: () async {
                                        Navigator.pushReplacement(context,
                                            CupertinoPageRoute(
                                                builder: (context) {
                                          return ChatGroupPage(
                                              subGroup[indexSubGroub]["name"],
                                              topicSubGroup1[
                                                  indexTopicSubgroup1]["name"],
                                              subGroup[indexSubGroub]
                                                  ["picture"],
                                              identifier: topicSubGroup1[
                                                  indexTopicSubgroup1]["id"],
                                              idGroup: subGroup[indexSubGroub]
                                                  ["id"]);
                                        }));
                                      },
                                    );
                                  },
                                ),
                              )
                            ],
                          ),
                        ),
                        ListView.builder(
                          shrinkWrap: true,
                          physics: NeverScrollableScrollPhysics(),
                          itemCount: subGroup1.length,
                          itemBuilder: (context, indexSubGroub1) {
                            var topicSubGroup2 =
                                subGroup1[indexSubGroub1]["topic"];
                            var subGroup2 =
                                subGroup1[indexSubGroub1]["subGroup"];
                            return Container(
                              margin: EdgeInsets.only(left: 28.0),
                              child: CustomExpansionTile(
                                onExpansionChanged: (cond) {
                                  // if (cond) {
                                  //   if (widget.changeState
                                  //       .getChatDataGroup[
                                  //   index]["saveSubGroup"] ==
                                  //       null) {
                                  //     widget.changeState.hideTopic = {
                                  //       "index": index,
                                  //       "indexSubGroup": indexSubGroub1
                                  //     };
                                  //   }
                                  // } else {
                                  //   widget.changeState.showTopic = {
                                  //     "index": index,
                                  //     "indexSubGroup": indexSubGroub
                                  //   };
                                  // }
                                },
                                leading: Container(
                                    width: 45,
                                    height: 45,
                                    decoration: BoxDecoration(
                                        shape: BoxShape.circle,
                                        color: Colors.grey,
                                        image: DecorationImage(
                                            fit: BoxFit.cover,
                                            image: subGroup1[indexSubGroub1]
                                                            ["picture"] ==
                                                        null ||
                                                    subGroup1[indexSubGroub1]
                                                            ["picture"]
                                                        .trim()
                                                        .isEmpty
                                                ? AssetImage(
                                                    "assets/images/groupicon.png",
                                                    package: "palio_lite")
                                                : LocalImage(subGroup1[
                                                            indexSubGroub1]
                                                        ["picture"])
                                                    as ImageProvider))),
                                title: Row(
                                  mainAxisSize: MainAxisSize.max,
                                  children: <Widget>[
                                    subGroup1[indexSubGroub1]["isVerified"] ==
                                            "1"
                                        ? Image.asset(
                                            "assets/images/ic_verified_flag.png",
                                            width: 20,
                                            height: 20,
                                            package: "palio_lite")
                                        : subGroup1[indexSubGroub1]
                                                    ["isOfficial"] ==
                                                "1"
                                            ? Image.asset(
                                                "assets/images/ic_official_flag.png",
                                                width: 20,
                                                height: 20,
                                                package: "palio_lite")
                                            : subGroup1[indexSubGroub1]
                                                        ["isOpen"] ==
                                                    "1"
                                                ? Icon(Icons.public, size: 20)
                                                : subGroup1[indexSubGroub1]
                                                            ["isOpen"] ==
                                                        "2"
                                                    ? Image.asset(
                                                        "assets/images/ic_educational.png",
                                                        width: 20,
                                                        height: 20,
                                                        color: Colors.black,
                                                        package: "palio_lite")
                                                    : Icon(Icons.lock,
                                                        size: 20),
                                    Container(
                                      padding: EdgeInsets.only(
                                        left: 8.0,
                                      ),
                                      child: Text(
                                          "${subGroup1[indexSubGroub1]["name"]}",
                                          overflow: TextOverflow.ellipsis),
                                      constraints:
                                          BoxConstraints(maxWidth: 210),
                                    )
                                  ],
                                ),
                                children: [
                                  Container(
                                    margin: EdgeInsets.only(left: 28.0),
                                    child: CustomExpansionTile(
                                      leading: Container(
                                        width: 45,
                                        height: 45,
                                        decoration: BoxDecoration(
                                            shape: BoxShape.circle,
                                            color: Colors.grey),
                                        child: Image.asset(
                                            "assets/images/forum.png",
                                            package: "palio_lite"),
                                      ),
                                      title: Text(
                                        "Forum",
                                        style: TextStyle(
                                            fontSize: 15, color: Colors.black),
                                      ),
                                      children: [
                                        Container(
                                          margin: EdgeInsets.only(left: 28.0),
                                          child: ListView.builder(
                                            shrinkWrap: true,
                                            physics:
                                                NeverScrollableScrollPhysics(),
                                            itemCount: topicSubGroup2.length,
                                            itemBuilder:
                                                (context, indexTopicSubgroup2) {
                                              return ListTile(
                                                title: Text(
                                                  "${topicSubGroup2[indexTopicSubgroup2]["name"]}",
                                                  style: TextStyle(
                                                      fontSize: 15,
                                                      color: Colors.black),
                                                ),
                                                leading: Container(
                                                    height: 30,
                                                    width: 30,
                                                    decoration: BoxDecoration(
                                                        image: DecorationImage(
                                                            fit: BoxFit.cover,
                                                            image: AssetImage(
                                                                "assets/images/tab_menu_uc.png",
                                                                package:
                                                                    "palio_lite")))),
                                                onTap: () async {
                                                  Navigator.pushReplacement(
                                                      context,
                                                      CupertinoPageRoute(
                                                          builder: (context) {
                                                    return ChatGroupPage(
                                                        subGroup1[
                                                                indexSubGroub1]
                                                            ["name"],
                                                        topicSubGroup2[
                                                                indexTopicSubgroup2]
                                                            ["name"],
                                                        subGroup1[
                                                                indexSubGroub1]
                                                            ["picture"],
                                                        identifier: topicSubGroup2[
                                                                indexTopicSubgroup2]
                                                            ["id"],
                                                        idGroup: subGroup1[
                                                                indexSubGroub1]
                                                            ["id"]);
                                                  }));
                                                },
                                              );
                                            },
                                          ),
                                        )
                                      ],
                                    ),
                                  ),
                                  ListView.builder(
                                    shrinkWrap: true,
                                    physics: NeverScrollableScrollPhysics(),
                                    itemCount: subGroup2.length,
                                    itemBuilder: (context, indexSubGroub2) {
                                      var topicSubGroup3 =
                                          subGroup2[indexSubGroub2]["topic"];
                                      var subGroup3 =
                                          subGroup2[indexSubGroub2]["subGroup"];
                                      return Container(
                                        margin: EdgeInsets.only(left: 28.0),
                                        child: CustomExpansionTile(
                                          onExpansionChanged: (cond) {
                                            // if (cond) {
                                            //   if (widget.changeState
                                            //       .getChatDataGroup[
                                            //   index]["saveSubGroup"] ==
                                            //       null) {
                                            //     widget.changeState.hideTopic = {
                                            //       "index": index,
                                            //       "indexSubGroup": indexSubGroub1
                                            //     };
                                            //   }
                                            // } else {
                                            //   widget.changeState.showTopic = {
                                            //     "index": index,
                                            //     "indexSubGroup": indexSubGroub
                                            //   };
                                            // }
                                          },
                                          leading: Container(
                                              width: 45,
                                              height: 45,
                                              decoration: BoxDecoration(
                                                  shape: BoxShape.circle,
                                                  color: Colors.grey,
                                                  image: DecorationImage(
                                                      fit: BoxFit.cover,
                                                      image: subGroup2[indexSubGroub2][
                                                                      "picture"] ==
                                                                  null ||
                                                              subGroup2[indexSubGroub2]
                                                                      [
                                                                      "picture"]
                                                                  .trim()
                                                                  .isEmpty
                                                          ? AssetImage(
                                                              "assets/images/groupicon.png",
                                                              package:
                                                                  "palio_lite")
                                                          : LocalImage(
                                                                  subGroup2[indexSubGroub2]
                                                                      ["picture"])
                                                              as ImageProvider))),
                                          title: Row(
                                            mainAxisSize: MainAxisSize.max,
                                            children: <Widget>[
                                              subGroup2[indexSubGroub2]
                                                          ["isVerified"] ==
                                                      "1"
                                                  ? Image.asset(
                                                      "assets/images/ic_verified_flag.png",
                                                      width: 20,
                                                      height: 20,
                                                      package: "palio_lite")
                                                  : subGroup2[indexSubGroub2]
                                                              ["isOfficial"] ==
                                                          "1"
                                                      ? Image.asset(
                                                          "assets/images/ic_official_flag.png",
                                                          width: 20,
                                                          height: 20,
                                                          package: "palio_lite")
                                                      : subGroup2[indexSubGroub2]
                                                                  ["isOpen"] ==
                                                              "1"
                                                          ? Icon(Icons.public,
                                                              size: 20)
                                                          : subGroup2[indexSubGroub2]["isOpen"] ==
                                                                  "2"
                                                              ? Image.asset(
                                                                  "assets/images/ic_educational.png",
                                                                  width: 20,
                                                                  height: 20,
                                                                  color: Colors
                                                                      .black,
                                                                  package:
                                                                      "palio_lite")
                                                              : Icon(Icons.lock, size: 20),
                                              Container(
                                                padding: EdgeInsets.only(
                                                  left: 8.0,
                                                ),
                                                child: Text(
                                                    "${subGroup2[indexSubGroub2]["name"]}",
                                                    overflow:
                                                        TextOverflow.ellipsis),
                                                constraints: BoxConstraints(
                                                    maxWidth: 210),
                                              )
                                            ],
                                          ),
                                          children: [
                                            Container(
                                              margin:
                                                  EdgeInsets.only(left: 28.0),
                                              child: CustomExpansionTile(
                                                leading: Container(
                                                  width: 45,
                                                  height: 45,
                                                  decoration: BoxDecoration(
                                                      shape: BoxShape.circle,
                                                      color: Colors.grey),
                                                  child: Image.asset(
                                                      "assets/images/forum.png",
                                                      package: "palio_lite"),
                                                ),
                                                title: Text(
                                                  "Forum",
                                                  style: TextStyle(
                                                      fontSize: 15,
                                                      color: Colors.black),
                                                ),
                                                children: [
                                                  Container(
                                                    margin: EdgeInsets.only(
                                                        left: 28.0),
                                                    child: ListView.builder(
                                                      shrinkWrap: true,
                                                      physics:
                                                          NeverScrollableScrollPhysics(),
                                                      itemCount:
                                                          topicSubGroup3.length,
                                                      itemBuilder: (context,
                                                          indexTopicSubgroup3) {
                                                        return ListTile(
                                                          title: Text(
                                                            "${topicSubGroup3[indexTopicSubgroup3]["name"]}",
                                                            style: TextStyle(
                                                                fontSize: 15,
                                                                color: Colors
                                                                    .black),
                                                          ),
                                                          leading: Container(
                                                              height: 30,
                                                              width: 30,
                                                              decoration: BoxDecoration(
                                                                  image: DecorationImage(
                                                                      fit: BoxFit
                                                                          .cover,
                                                                      image: AssetImage(
                                                                          "assets/images/tab_menu_uc.png",
                                                                          package:
                                                                              "palio_lite")))),
                                                          onTap: () async {
                                                            Navigator.pushReplacement(
                                                                context,
                                                                CupertinoPageRoute(
                                                                    builder:
                                                                        (context) {
                                                              return ChatGroupPage(
                                                                  subGroup2[
                                                                          indexSubGroub2]
                                                                      ["name"],
                                                                  topicSubGroup3[
                                                                          indexTopicSubgroup3]
                                                                      ["name"],
                                                                  subGroup2[
                                                                          indexSubGroub2]
                                                                      [
                                                                      "picture"],
                                                                  identifier:
                                                                      topicSubGroup3[
                                                                              indexTopicSubgroup3]
                                                                          [
                                                                          "id"],
                                                                  idGroup:
                                                                      subGroup2[
                                                                              indexSubGroub2]
                                                                          [
                                                                          "id"]);
                                                            }));
                                                          },
                                                        );
                                                      },
                                                    ),
                                                  )
                                                ],
                                              ),
                                            ),
                                            ListView.builder(
                                              shrinkWrap: true,
                                              physics:
                                                  NeverScrollableScrollPhysics(),
                                              itemCount: subGroup3.length,
                                              itemBuilder:
                                                  (context, indexSubGroub3) {
                                                var topicSubGroup4 =
                                                    subGroup3[indexSubGroub3]
                                                        ["topic"];
                                                // var subGroup3 = subGroup2[indexSubGroub2]["subGroup"];
                                                return Container(
                                                  margin: EdgeInsets.only(
                                                      left: 28.0),
                                                  child: CustomExpansionTile(
                                                    onExpansionChanged: (cond) {
                                                      // if (cond) {
                                                      //   if (widget.changeState
                                                      //       .getChatDataGroup[
                                                      //   index]["saveSubGroup"] ==
                                                      //       null) {
                                                      //     widget.changeState.hideTopic = {
                                                      //       "index": index,
                                                      //       "indexSubGroup": indexSubGroub1
                                                      //     };
                                                      //   }
                                                      // } else {
                                                      //   widget.changeState.showTopic = {
                                                      //     "index": index,
                                                      //     "indexSubGroup": indexSubGroub
                                                      //   };
                                                      // }
                                                    },
                                                    leading: Container(
                                                        width: 45,
                                                        height: 45,
                                                        decoration: BoxDecoration(
                                                            shape:
                                                                BoxShape.circle,
                                                            color: Colors.grey,
                                                            image: DecorationImage(
                                                                fit: BoxFit
                                                                    .cover,
                                                                image: subGroup3[indexSubGroub3]["picture"] ==
                                                                            null ||
                                                                        subGroup3[indexSubGroub3]["picture"]
                                                                            .trim()
                                                                            .isEmpty
                                                                    ? AssetImage(
                                                                        "assets/images/groupicon.png")
                                                                    : LocalImage(subGroup3[indexSubGroub3]
                                                                            [
                                                                            "picture"])
                                                                        as ImageProvider))),
                                                    title: Row(
                                                      mainAxisSize:
                                                          MainAxisSize.max,
                                                      children: <Widget>[
                                                        subGroup3[indexSubGroub3][
                                                                    "isVerified"] ==
                                                                "1"
                                                            ? Image.asset(
                                                                "assets/images/ic_verified_flag.png",
                                                                width: 20,
                                                                height: 20,
                                                                package:
                                                                    "palio_lite")
                                                            : subGroup3[indexSubGroub3]
                                                                        [
                                                                        "isOfficial"] ==
                                                                    "1"
                                                                ? Image.asset(
                                                                    "assets/images/ic_official_flag.png",
                                                                    width: 20,
                                                                    height: 20,
                                                                    package:
                                                                        "palio_lite")
                                                                : subGroup3[indexSubGroub3]
                                                                            [
                                                                            "isOpen"] ==
                                                                        "1"
                                                                    ? Icon(Icons.public,
                                                                        size:
                                                                            20)
                                                                    : subGroup3[indexSubGroub3]["isOpen"] ==
                                                                            "2"
                                                                        ? Image.asset(
                                                                            "assets/images/ic_educational.png",
                                                                            width: 20,
                                                                            height: 20,
                                                                            color: Colors.black,
                                                                            package: "palio_lite")
                                                                        : Icon(Icons.lock, size: 20),
                                                        Container(
                                                          padding:
                                                              EdgeInsets.only(
                                                            left: 8.0,
                                                          ),
                                                          child: Text(
                                                              "${subGroup3[indexSubGroub3]["name"]}",
                                                              overflow:
                                                                  TextOverflow
                                                                      .ellipsis),
                                                          constraints:
                                                              BoxConstraints(
                                                                  maxWidth:
                                                                      210),
                                                        )
                                                      ],
                                                    ),
                                                    children: [
                                                      Container(
                                                        margin: EdgeInsets.only(
                                                            left: 28.0),
                                                        child:
                                                            CustomExpansionTile(
                                                          leading: Container(
                                                            width: 45,
                                                            height: 45,
                                                            decoration:
                                                                BoxDecoration(
                                                                    shape: BoxShape
                                                                        .circle,
                                                                    color: Colors
                                                                        .grey),
                                                            child: Image.asset(
                                                                "assets/images/forum.png",
                                                                package:
                                                                    "palio_lite"),
                                                          ),
                                                          title: Text(
                                                            "Forum",
                                                            style: TextStyle(
                                                                fontSize: 15,
                                                                color: Colors
                                                                    .black),
                                                          ),
                                                          children: [
                                                            Container(
                                                              margin: EdgeInsets
                                                                  .only(
                                                                      left:
                                                                          28.0),
                                                              child: ListView
                                                                  .builder(
                                                                shrinkWrap:
                                                                    true,
                                                                physics:
                                                                    NeverScrollableScrollPhysics(),
                                                                itemCount:
                                                                    topicSubGroup4
                                                                        .length,
                                                                itemBuilder:
                                                                    (context,
                                                                        indexTopicSubgroup4) {
                                                                  return ListTile(
                                                                    title: Text(
                                                                      "${topicSubGroup4[indexTopicSubgroup4]["name"]}",
                                                                      style: TextStyle(
                                                                          fontSize:
                                                                              15,
                                                                          color:
                                                                              Colors.black),
                                                                    ),
                                                                    leading: Container(
                                                                        height:
                                                                            30,
                                                                        width:
                                                                            30,
                                                                        decoration:
                                                                            BoxDecoration(image: DecorationImage(fit: BoxFit.cover, image: AssetImage("assets/images/tab_menu_uc.png", package: "palio_lite")))),
                                                                    onTap:
                                                                        () async {
                                                                      Navigator.pushReplacement(
                                                                          context,
                                                                          CupertinoPageRoute(builder:
                                                                              (context) {
                                                                        return ChatGroupPage(
                                                                            subGroup3[indexSubGroub3][
                                                                                "name"],
                                                                            topicSubGroup4[indexTopicSubgroup4][
                                                                                "name"],
                                                                            subGroup3[indexSubGroub3][
                                                                                "picture"],
                                                                            identifier:
                                                                                topicSubGroup4[indexTopicSubgroup4]["id"],
                                                                            idGroup: subGroup3[indexSubGroub3]["id"]);
                                                                      }));
                                                                    },
                                                                  );
                                                                },
                                                              ),
                                                            )
                                                          ],
                                                        ),
                                                      )
                                                    ],
                                                  ),
                                                );
                                              },
                                            )
                                          ],
                                        ),
                                      );
                                    },
                                  )
                                ],
                              ),
                            );
                          },
                        )
                      ],
                    ),
                  );
                },
              )
            ],
          );
        });
  }
}
