import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/chat/login_admin.dart';
import 'package:palio_lite/chat/pa_profile.dart';
import 'package:palio_lite/chat/settings_email.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:shared_preferences/shared_preferences.dart';

class SettingsView extends StatefulWidget {
  @override
  _SettingsViewState createState() => _SettingsViewState();
}

class _SettingsViewState extends State<SettingsView> {
  bool? isVibrate = false;
  bool? isSave = false;
  bool? isSaveAuto = false;
  bool isPlay = false;
  bool? isRandom = false;
  bool isBackgroundTheme = true;
  bool isEmailActive = false;
  bool isEducational = false;
  bool isVerifiedEducational = false;
  bool isSubAccount = false;
  bool isLoginAdminInternal = false;
  String? phoneNumber = "";
  String image_id = "";
  List<dynamic> resultQuery = [];

  Future<bool> setLanguage(String lang) async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.setString("language", lang);
  }

  Future<bool> setVibration(bool vibrate) async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.setBool("vibrate", vibrate);
  }

  Future<bool> setAutoDownload(bool autoDownload) async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.setBool("autoDownload", autoDownload);
  }

  setShuffleImage(bool request) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setBool("shuffleImage", request);
  }

  Future<bool?> getShuffleImage() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("shuffleImage")) {
      isRandom = await prefs.getBool("shuffleImage");
    }
    return isRandom;
  }

  Future<String?> getPhoneNumber() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("phoneNumber")) {
      phoneNumber = await prefs.getString("phoneNumber");
    }
    return phoneNumber;
  }

  setSaveImage(bool save) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setBool("saveToGallery", save);
  }

  Future<bool?> getSaveImage() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("saveToGallery")) {
      isSave = await prefs.getBool("saveToGallery");
    }
    return isSave;
  }

  Future<bool?> getVibrate() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("vibrate")) {
      isVibrate = await prefs.getBool("vibrate");
    }
    return isVibrate;
  }

  Future<bool?> getAutoDownload() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("autoDownload")) {
      isSaveAuto = await prefs.getBool("autoDownload");
    }
    return isSaveAuto;
  }

  Future<String> getPhotos() async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    String f_pin = prefs.getString("hasLogin") ?? "";
    List<dynamic> persons = await DBHelper()
        .queryCustom("SELECT image_id FROM BUDDY where f_pin='$f_pin'");
    if (persons.isNotEmpty) {
      image_id = persons[0]["image_id"] == null ? "" : persons[0]["image_id"];
    }
    return image_id;
  }

  Future<dynamic> getResultQueryMe() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String idMe = prefs.getString("hasLogin") ?? "";
    resultQuery =
        await DBHelper().queryCustom("SELECT * FROM BUDDY where f_pin='$idMe'");
    return resultQuery;
  }

  @override
  void initState() {
    super.initState();
    getResultQueryMe().then((value) {
      if (value != null) {
        isEducational = (value[0]["official_account"] == "5");
        isVerifiedEducational = (value[0]["official_account"] == "11");
        isSubAccount = (value[0]["is_sub_account"] == "1");
        String userType = value[0]["user_type"];
        if (userType == "23" || userType == "24") {
          isLoginAdminInternal = true;
        }
        setState(() {});
      }
    });
    getVibrate().then((value) {
      if (value != null) {
        isVibrate = value;
      }
    });
    getAutoDownload().then((value) {
      if (value != null) {
        isSaveAuto = value;
      }
    });
    getShuffleImage().then((value) {
      if (value != null) {
        isRandom = value;
      }
    });
    getPhotos().then((value) {
      if (value != null) {
        image_id = value;
        setState(() {});
      }
    });
    getSaveImage().then((value) {
      if (value != null) {
        isSave = value;
      }
    });
    getPhoneNumber().then((value) {
      if (value != null) {
        phoneNumber = value;
      }
    });
    setState(() {});
  }

  Widget bottom() {
    return Container(
      alignment: Alignment.bottomCenter,
      padding: EdgeInsets.symmetric(vertical: 18),
      // child: _toolbar(),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Align(
            alignment: Alignment.bottomRight,
            child: _poweredBy(),
          ),
        ],
      ),
    );
  }

  Widget _poweredBy() {
    return Container(
      alignment: Alignment.center,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Container(
            height: 25,
            alignment: Alignment.centerRight,
            child: Text(
              "Powered by Palio.io",
              style: TextStyle(fontSize: 15),
            ),
          ),
          SizedBox(
            width: 5,
          ),
          Container(
            width: 25,
            height: 25,
//              color: Colors.grey,
            decoration: BoxDecoration(
                image: DecorationImage(
                    image: AssetImage("assets/images/palio.png",
                        package: "palio_lite"))),
          )
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        backgroundColor: AppColor.primaryColor,
        automaticallyImplyLeading: true,
        title: Text("Settings"),
      ),
      body: SingleChildScrollView(
        child: Container(
          child: Column(
            children: <Widget>[
              Card(
                margin: EdgeInsets.all(0),
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(5),
                      topRight: Radius.circular(5),
                      bottomRight: Radius.circular(5),
                      bottomLeft: Radius.circular(5)),
                ),
                elevation: 1,
                child: Column(
                  children: <Widget>[
//                    ListTile(
//                      onTap: () {
//                        showGeneralDialog(
//                            context: context,
//                            barrierDismissible: true,
//                            barrierLabel: MaterialLocalizations.of(context)
//                                .modalBarrierDismissLabel,
//                            barrierColor: Colors.black45,
//                            transitionDuration:
//                                const Duration(milliseconds: 200),
//                            pageBuilder: (BuildContext buildContext,
//                                Animation animation,
//                                Animation secondaryAnimation) {
//                              return Center(
//                                child: Container(
//                                  width:
//                                      MediaQuery.of(context).size.width * 0.95,
//                                  height: 330,
//                                  padding: EdgeInsets.fromLTRB(25, 25, 25, 10),
//                                  decoration: BoxDecoration(
//                                      color: Colors.grey,
//                                      borderRadius:
//                                          BorderRadius.circular(30.0)),
//                                  child: Column(
//                                    crossAxisAlignment:
//                                        CrossAxisAlignment.start,
//                                    children: [
//                                      Row(
//                                        mainAxisAlignment:
//                                            MainAxisAlignment.start,
//                                        children: <Widget>[
//                                          Icon(
//                                            Icons.gps_fixed,
//                                            size: 40,
//                                          ),
//                                          SizedBox(width: 10.0),
//                                          Material(
//                                            color: Colors.transparent,
//                                            child: Text(translate("covid"),
//                                                style: TextStyle(fontSize: 20)),
//                                          )
//                                        ],
//                                      ),
//                                      Material(
//                                        color: Colors.transparent,
//                                        child: Text(
//                                          translate("titleKuisioner1"),
//                                          style: TextStyle(
//                                              fontStyle: FontStyle.italic,
//                                              color: Colors.grey[800],
//                                              fontWeight: FontWeight.w600),
//                                        ),
//                                      ),
//                                      SizedBox(height: 10.0),
//                                      Material(
//                                        color: Colors.transparent,
//                                        child: Text(
//                                          translate("titleKuisioner2"),
//                                          style: TextStyle(
//                                              fontStyle: FontStyle.italic,
//                                              color: Colors.grey[800],
//                                              fontWeight: FontWeight.w600),
//                                        ),
//                                      ),
//                                      SizedBox(height: 10.0),
//                                      Material(
//                                        color: Colors.transparent,
//                                        child: Text(
//                                          translate("titleKuisioner3"),
//                                          style: TextStyle(
//                                              fontStyle: FontStyle.italic,
//                                              color: Colors.grey[800],
//                                              fontWeight: FontWeight.w600),
//                                        ),
//                                      ),
//                                      SizedBox(height: 30.0),
//                                      Container(
//                                        alignment: Alignment.centerRight,
//                                        child: RawMaterialButton(
//                                          padding: EdgeInsets.all(0.0),
//                                          constraints: BoxConstraints(
//                                              minWidth: 40.0, minHeight: 40),
//                                          onPressed: () {
//                                            Navigator.pop(context);
//                                            Navigator.push(
//                                                context,
//                                                PageTransition(
//                                                    type:
//                                                        PageTransitionType.fade,
//                                                    child: Kuesioner()));
//                                          },
//                                          child: Material(
//                                            color: Colors.transparent,
//                                            child: Text(
//                                              translate("ok"),
//                                              style: TextStyle(
//                                                  color: Colors.grey[800],
//                                                  fontWeight: FontWeight.w600),
//                                            ),
//                                          ),
//                                        ),
//                                      )
//                                    ],
//                                  ),
//                                ),
//                              );
//                            });
//                      },
//                      leading: Icon(
//                        Icons.my_location,
//                        size: 30,
//                        color: Colors.black,
//                      ),
//                      title: Text(
//                        translate('questioner'),
//                        style: TextStyle(fontSize: 20),
//                      ),
//                      trailing: Icon(CupertinoIcons.right_chevron),
//                    ),
                    ListTile(
                      onTap: () async {
                        SharedPreferences prefs =
                            await SharedPreferences.getInstance();
                        String idMe = prefs.getString("hasLogin") ?? "";
                        List<dynamic> resultQueryFriend = await DBHelper()
                            .queryCustom(
                                "SELECT f_pin, first_name,last_name,official_account, image_id FROM BUDDY where f_pin <> '$idMe'");
                        print(resultQuery);
                        Navigator.pushReplacement(
                            context,
                            PageTransition(
                                type: PageTransitionType.fade,
                                child: PAProfile(
                                  profileDet: {
                                    'f_pin': idMe,
                                    'myProfile': true,
                                    'name': resultQuery[0]["first_name"] +
                                        " " +
                                        resultQuery[0]["last_name"],
                                    'follower': 0,
                                    'official_account': resultQuery[0]
                                        ["official_account"],
                                    'isFriend': 0,
                                    'post': 0,
                                    'following': 0,
                                    'friend': resultQueryFriend.length,
                                    'isOffline': resultQuery[0]["offline_mode"],
                                    'isMuted': resultQuery[0]["muted"],
                                    'industry': resultQuery[0]
                                        ["bidang_industri"],
                                    'email_address': resultQuery[0]["email"],
                                    'phone_number': resultQuery[0]["msisdn"],
                                    'website': resultQuery[0]["web"],
                                    'address': resultQuery[0]["address"],
                                    'status': resultQuery[0]["quote"],
                                    'isOnline':
                                        int.parse(resultQuery[0]["connected"]),
                                    'image': resultQuery[0]["image_id"],
                                    'user_type': resultQuery[0]["user_type"]
                                  },
//                                    {
//                                      'f_pin': "0000a",
//                                      'myProfile': false,
//                                      'name': "Test 1",
//                                      'follower': 0,
//                                      'official_flag': 1,
//                                      'isFriend': 0,
//                                      'post': 0,
//                                      'following': 0,
//                                      'isOffline': 0,
//                                      'isMuted': 0,
//                                      'industry': "Makanan",
//                                      'email_address': "test@test.com",
//                                      'phone_number': "0231414151",
//                                      'website': "test.com",
//                                      'address': "Jakarta",
//                                      'status': "Baik",
//                                      'isOnline': 0,
//                                      'image': "https://cdn.pixabay.com/photo/2012/04/26/19/47/man-42934_1280.png",
//                                      'isFriend' : 1
//                                    },
                                )));
                      },
                      leading: Container(
                          width: 35.0,
                          height: 35.0,
                          decoration: BoxDecoration(
                              shape: BoxShape.circle,
                              image: DecorationImage(
                                  fit: BoxFit.fill,
                                  image: image_id != null && image_id.length > 0
                                      ? LocalImage(image_id)
                                      : AssetImage(
                                              'assets/images/default_pp.png',
                                              package: "palio_lite")
                                          as ImageProvider))),
                      // FutureBuilder(
                      //     future: DBHelper().queryCustom(
                      //         "SELECT image_id FROM BUDDY where type='1'"),
                      //     builder: (context, snapshot) {
                      //       if (snapshot.hasData) {
                      //         return Container(
                      //             width: 35.0,
                      //             height: 35.0,
                      //             decoration: BoxDecoration(
                      //                 shape: BoxShape.circle,
                      //                 image: DecorationImage(
                      //                     fit: BoxFit.fill,
                      //                     image: LocalImage(
                      //                         snapshot.data[0]["image_id"]))));
                      //       }
                      //       return Container(
                      //           width: 35.0,
                      //           height: 35.0,
                      //           decoration: BoxDecoration(
                      //               shape: BoxShape.circle,
                      //               image: DecorationImage(
                      //                   fit: BoxFit.fill,
                      //                   image: AssetImage(
                      //                       'assets/images/default_pp.png'))));
                      //     }),
                      title: Text(
                        "Personal information",
                        style: TextStyle(fontSize: 20),
                      ),
                      trailing: Icon(CupertinoIcons.right_chevron),
                    ),
                    isLoginAdminInternal
                        ? Container()
                        : ListTile(
                            onTap: () {
                              showDialog(
                                  context: context,
                                  builder: (BuildContext context) =>
                                      SimpleDialog(
                                        children: <Widget>[
                                          ListTile(
                                            title: Text("Login as Admin"),
                                            onTap: () {
                                              Navigator.pop(context);
                                              Navigator.pushReplacement(
                                                  context,
                                                  PageTransition(
                                                      type: PageTransitionType
                                                          .fade,
                                                      child: LoginAdminInternal(
                                                          true)));
                                            },
                                          ),
                                          ListTile(
                                            title:
                                                Text("Login as Internal Team"),
                                            onTap: () {
                                              Navigator.pop(context);
                                              Navigator.pushReplacement(
                                                  context,
                                                  PageTransition(
                                                      type: PageTransitionType
                                                          .fade,
                                                      child: LoginAdminInternal(
                                                          false)));
                                            },
                                          )
                                        ],
                                      ));
                            },
                            leading: Container(
                                width: 35.0,
                                height: 35.0,
                                child: Image.asset(
                                  'assets/images/twsn_group_admin_11.png',
                                  package: "palio_lite",
                                  color: Colors.red,
                                )),
                            title: Text(
                              "Login as Admin / Internal",
                              style: TextStyle(fontSize: 20),
                            ),
                            trailing: Icon(CupertinoIcons.right_chevron),
                          ),
                    // ListTile(
                    //   onTap: () {
                    //     Navigator.push(
                    //         context,
                    //         PageTransition(
                    //             type: PageTransitionType.fade,
                    //             child: UpdateMSISDNScreen()));
                    //   },
                    //   leading: Icon(
                    //     Icons.contact_phone,
                    //     size: 30,
                    //     color: AppColor.primaryColor,
                    //   ),
                    //   title: Text(
                    //     translate("changePhoneTitle"),
                    //     style: TextStyle(fontSize: 20),
                    //   ),
                    //   trailing: Icon(CupertinoIcons.right_chevron),
                    // ),
                    // Visibility(
                    //   visible: !isVerifiedEducational && !isEducational && !isSubAccount,
                    //   child: ListTile(
                    //     onTap: () {
                    //       Navigator.push(
                    //           context,
                    //           PageTransition(
                    //               type: PageTransitionType.fade,
                    //               child: VirtualSchoolRegistration(msisdn: phoneNumber,)));
                    //     },
                    //     leading: Icon(
                    //       Icons.school,
                    //       size: 30,
                    //       color: Colors.blueAccent,
                    //     ),
                    //     title: Text(
                    //       translate("eduAccountRegistration"),
                    //       style: TextStyle(fontSize: 20),
                    //     ),
                    //     trailing: Icon(CupertinoIcons.right_chevron),
                    //   ),
                    // ),
                    // Visibility(
                    //   visible: !isVerifiedEducational && !isEducational && !isSubAccount,
                    //   child: ListTile(
                    //     onTap: () {
                    //       Navigator.push(
                    //           context,
                    //           PageTransition(
                    //               type: PageTransitionType.fade,
                    //               child: SubAccountPage()));
                    //     },
                    //     leading: Icon(
                    //       Icons.person,
                    //       size: 30,
                    //       color: Colors.grey,
                    //     ),
                    //     title: Text(
                    //       translate("registerSA"),
                    //       style: TextStyle(fontSize: 20),
                    //     ),
                    //     trailing: Icon(CupertinoIcons.right_chevron),
                    //   ),
                    // ),
                    // ListTile(
                    //   onTap: () {
                    //     showDialog(
                    //         context: context,
                    //         builder: (context) {
                    //           return CupertinoAlertDialog(
                    //             title: Text(translate("chooseLang")),
                    //             content: Material(
                    //               color: Colors.transparent,
                    //               child: Column(
                    //                 crossAxisAlignment:
                    //                     CrossAxisAlignment.start,
                    //                 children: <Widget>[
                    //                   ListTile(
                    //                     title: Text(translate("indonesia")),
                    //                     onTap: () {
                    //                       Navigator.pop(context);
                    //                       changeLocale(context, 'id');
                    //                       setLanguage("id");
                    //                       setState(() {});
                    //                     },
                    //                   ),
                    //                   ListTile(
                    //                     title: Text(translate("english")),
                    //                     onTap: () {
                    //                       Navigator.pop(context);
                    //                       changeLocale(context, 'en_US');
                    //                       setLanguage("en_US");
                    //                       setState(() {});
                    //                     },
                    //                   )
                    //                 ],
                    //               ),
                    //             ),
                    //           );
                    //         });
                    //   },
                    //   leading: Icon(
                    //     Icons.g_translate,
                    //     size: 30,
                    //     color: AppColor.primaryColor,
                    //   ),
                    //   title: Text(
                    //     translate("chooseLang"),
                    //     style: TextStyle(fontSize: 20),
                    //   ),
                    //   trailing: Icon(CupertinoIcons.right_chevron),
                    // ),

                    // ListTile(
                    //   onTap: () {
                    //     Navigator.push(
                    //         context,
                    //         PageTransition(
                    //             type: PageTransitionType.fade,
                    //             child: RegisterMedicVerified()));
                    //   },
                    //   leading: Icon(
                    //     Icons.person,
                    //     size: 30,
                    //     color: Colors.grey,
                    //   ),
                    //   title: Text(
                    //     "Pendaftaran Akun Medis",
                    //     style: TextStyle(fontSize: 20),
                    //   ),
                    //   trailing: Icon(CupertinoIcons.right_chevron),
                    // )
                  ],
                ),
              ),
              Card(
                margin: EdgeInsets.only(top: 15),
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(5),
                      topRight: Radius.circular(5),
                      bottomRight: Radius.circular(5),
                      bottomLeft: Radius.circular(5)),
                ),
                elevation: 1,
                child: Column(
                  children: <Widget>[
                    ListTile(
                      leading: Icon(
                        CupertinoIcons.volume_up,
                        size: 30,
                        color: Colors.red,
                      ),
                      title: Text(
                        "Incoming messages",
                        style: TextStyle(fontSize: 20),
                      ),
                      subtitle: Text("Default"),
                      trailing: Icon(CupertinoIcons.right_chevron),
                    ),
                    ListTile(
                      leading: Icon(
                        CupertinoIcons.volume_up,
                        size: 30,
                        color: Colors.blueAccent,
                      ),
                      title: Text(
                        "Incoming call",
                        style: TextStyle(fontSize: 20),
                      ),
                      subtitle: Text("Default"),
                      trailing: Icon(CupertinoIcons.right_chevron),
                    ),
                    ListTile(
                      leading: Icon(
                        Icons.vibration,
                        size: 30,
                        color: Colors.purple[300],
                      ),
                      title: Text(
                        "Vibrate mode",
                        style: TextStyle(fontSize: 20),
                      ),
                      trailing: Switch(
                          activeColor: AppColor.primaryColor,
                          value: isVibrate ?? false,
                          onChanged: (bool val) {
                            setState(() {
                              isVibrate = val;
                              print("value vibrate $isVibrate");
                              API
                                  .setVibrate(isVibrate ?? false)
                                  .then((value) async {
                                print("value vibrate save : $value");
                                setVibration(value);
                              });
                            });
                          }),
                    )
                  ],
                ),
              ),
              Card(
                margin: EdgeInsets.only(top: 15),
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(5),
                      topRight: Radius.circular(5),
                      bottomRight: Radius.circular(5),
                      bottomLeft: Radius.circular(5)),
                ),
                elevation: 1,
                child: Column(
                  children: <Widget>[
                    ListTile(
                      leading: Icon(
                        Icons.image_aspect_ratio,
                        size: 30,
                        color: Colors.indigo,
                      ),
                      title: Text(
                        "Save to gallery",
                        style: TextStyle(fontSize: 20),
                      ),
                      trailing: Switch(
                          activeColor: AppColor.primaryColor,
                          value: isSave ?? false,
                          onChanged: (bool val) {
                            setState(() {
                              isSave = val;
                              print("value save to gallery $isSave");
                              API.saveToGallery(isSave ?? false);
                              setSaveImage(isSave ?? false);
                            });
                          }),
                    ),
                    ListTile(
                      leading: Icon(
                        Icons.save,
                        size: 30,
                        color: Colors.red,
                      ),
                      title: Text(
                        "Automatic download",
                        style: TextStyle(fontSize: 20),
                      ),
                      trailing: Switch(
                          activeColor: AppColor.primaryColor,
                          value: isSaveAuto ?? false,
                          onChanged: (bool val) {
                            setState(() {
                              isSaveAuto = val;
                              setAutoDownload(val);
                            });
                          }),
                    ),
                    // ListTile(
                    //   leading: Icon(
                    //     CupertinoIcons.video_camera_solid,
                    //     size: 30,
                    //     color: Colors.blueAccent,
                    //   ),
                    //   title: Text(
                    //     translate('autoPlayVideo'),
                    //     style: TextStyle(fontSize: 20),
                    //   ),
                    //   trailing: Switch(
                    //       activeColor: AppColor.primaryColor,
                    //       value: isPlay,
                    //       onChanged: (bool val) {
                    //         setState(() => isPlay = val);
                    //       }),
                    // ),
                    ListTile(
                      leading: Icon(
                        CupertinoIcons.shuffle_thick,
                        size: 30,
                        color: Colors.deepOrangeAccent,
                      ),
                      title: Text(
                        "Random image display",
                        style: TextStyle(fontSize: 20),
                      ),
                      trailing: Switch(
                          activeColor: AppColor.primaryColor,
                          value: isRandom ?? false,
                          onChanged: (bool val) {
                            setState(() {
                              isRandom = val;
                              setShuffleImage(isRandom ?? false);
                            });
                          }),
                    ),
                    // ListTile(
                    //   leading: Icon(
                    //     Icons.image,
                    //     size: 30,
                    //     color: Colors.purple[300],
                    //   ),
                    //   title: Text(
                    //     "Background theme",
                    //     style: TextStyle(fontSize: 20),
                    //   ),
                    //   trailing: Switch(
                    //       activeColor: AppColor.primaryColor,
                    //       value: isBackgroundTheme,
                    //       onChanged: (bool val) {
                    //         setState(() => isBackgroundTheme = val);
                    //       }),
                    // ),
                    ListTile(
                      leading: Icon(
                        CupertinoIcons.mail_solid,
                        size: 30,
                        color: Colors.red,
                      ),
                      title: Text(
                        "Email account",
                        style: TextStyle(fontSize: 20),
                      ),
                      trailing: Switch(
                          activeColor: AppColor.primaryColor,
                          value: isEmailActive,
                          onChanged: (bool val) async {
                            if (isEmailActive) {
                              setState(() => isEmailActive = val);
                            } else {
                              var setted = await Navigator.push(
                                  context,
                                  PageTransition(
                                      type: PageTransitionType.fade,
                                      child: SettingEmail()));
                              if (setted) {
                                isEmailActive = setted;
                              }
                              setState(() {});
                            }
                          }),
                    )
                  ],
                ),
              ),
              // Card(
              //   margin: EdgeInsets.only(top: 15),
              //   shape: RoundedRectangleBorder(
              //     borderRadius: BorderRadius.only(
              //         topLeft: Radius.circular(5),
              //         topRight: Radius.circular(5),
              //         bottomRight: Radius.circular(5),
              //         bottomLeft: Radius.circular(5)),
              //   ),
              //   elevation: 1,
              //   child: Column(
              //     children: <Widget>[
              //       ListTile(
              //         leading: Icon(
              //           CupertinoIcons.battery_charging,
              //           size: 30,
              //           color: Colors.blueAccent,
              //         ),
              //         title: Text(
              //           translate('batteryOptimization'),
              //           style: TextStyle(fontSize: 20),
              //         ),
              //         subtitle: Text(translate('notOptimized')),
              //         trailing: Icon(CupertinoIcons.right_chevron),
              //       ),
              //     ],
              //   ),
              // ),
              Card(
                margin: EdgeInsets.only(top: 15),
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(5),
                      topRight: Radius.circular(5),
                      bottomRight: Radius.circular(5),
                      bottomLeft: Radius.circular(5)),
                ),
                elevation: 1,
                child: Column(
                  children: <Widget>[
                    ListTile(
                      leading: Icon(
                        Icons.devices,
                        size: 30,
                        color: Colors.indigo,
                      ),
                      title: Text(
                        "Version",
                        style: TextStyle(fontSize: 20),
                      ),
                      subtitle: Text('0.0.1'),
                    ),
                    // Container(
                    //   alignment: Alignment.center,
                    //   padding: EdgeInsets.symmetric(vertical: 18),
                    //   // child: _toolbar(),
                    //   child: Column(
                    //     mainAxisAlignment: MainAxisAlignment.center,
                    //     crossAxisAlignment: CrossAxisAlignment.center,
                    //     children: [
                    //       Align(
                    //         alignment: Alignment.center,
                    //         child: _poweredBy(),
                    //       ),
                    //     ],
                    //   ),
                    // )
                  ],
                ),
              )
            ],
          ),
        ),
      ),
    );
  }
}
