import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:intl/intl.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/chat/widgets/notifications.dart';
import 'package:palio_lite/chat/widgets/uc_list.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:palio_lite/shared_obj.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ListUcList {
  String lPin,
      fPin,
      myPin,
      fName,
      fPicture,
      messageId,
      message,
      contentType,
      type,
      isVerified,
      credential,
      isOfficial,
      counter,
      time,
      statusRead;
  int sortTime, countCredential;
  List<ListTopic> topicList;

  ListUcList(
      {required this.lPin,
      this.fPin = "",
      this.myPin = "",
      this.fName = "",
      this.fPicture = "",
      this.message = "",
      required this.messageId,
      this.type = "",
      this.credential = "",
      this.contentType = "",
      this.isVerified = "0",
      this.isOfficial = "0",
      this.counter = "0",
      this.time = "0",
      this.statusRead = "0",
      this.sortTime = 0,
      this.countCredential = 0,
      this.topicList = const []});

  @override
  bool operator ==(other) =>
      other is ListUcList &&
      (other.lPin == lPin || other.messageId == messageId);

  @override
  int get hashCode => super.hashCode;
}

class ListTopic {
  String lPin,
      fName,
      myPin,
      fPin,
      idGroup,
      messageId,
      message,
      contentType,
      counter,
      time,
      statusRead;
  int sortTime;
  Map<String, String> listMembers = Map();

  ListTopic(
      {this.lPin = "",
      this.fPin = "",
      this.myPin = "",
      this.idGroup = "",
      this.fName = "",
      this.contentType = "",
      this.message = "",
      required this.messageId,
      this.counter = "0",
      this.time = "",
      this.sortTime = 0,
      this.listMembers = const {},
      this.statusRead = "0"});

  @override
  bool operator ==(other) => other is ListTopic && other.messageId == messageId;

  @override
  int get hashCode => super.hashCode;
}

class ChangeState with ChangeNotifier {
  bool _isVisible = false,
      _hasFavorite = false,
      _checkSocial = true,
      _checkEducation = true;
  int _selectedRadioTile = 1;
  List<ListTopic> _topicData = [];
  List<ListUcList> _chatData = [];
  List<Map<dynamic, dynamic>> _chatDataGroup = [];

  bool get isVisible => _isVisible;

  bool get hasFavorite => _hasFavorite;

  bool get checkSocial => _checkSocial;

  bool get checkEducation => _checkEducation;

  int get selectedRadioTile => _selectedRadioTile;

  List<ListTopic> get getTopicData => _topicData;

  List<ListUcList> get getChatData => _chatData;

  List<Map<dynamic, dynamic>> get getChatDataGroup => _chatDataGroup;

  set isVisible(bool value) {
    _isVisible = value;
    notifyListeners();
  }

  set hasFavorite(bool value) {
    _hasFavorite = value;
    notifyListeners();
  }

  set checkSocial(bool value) {
    _checkSocial = value;
    notifyListeners();
  }

  set checkEducation(bool value) {
    _checkEducation = value;
    notifyListeners();
  }

  set selectedRadioTile(int value) {
    _selectedRadioTile = value;
    notifyListeners();
  }

  set topicData(ListTopic value) {
    _topicData.add(value);
    notifyListeners();
  }

  set chatData(ListUcList value) {
    _chatData.insert(0, value);
    notifyListeners();
  }

  set updateImageChatData(var value) {
    _chatData[value["index"]].fPicture = value["image"];
    notifyListeners();
  }

  set updateStatusChatData(var value) {
    _chatData[value["index"]].statusRead = value["status"];
    notifyListeners();
  }

  set updateStatusDelete(var value) {
    _chatData[value["index"]].message = value["message"];
    _chatData[value["index"]].contentType = value["flag"];
    notifyListeners();
  }

  set updateStatusChatDataGroup(var value) {
    _chatData[value["idxGroup"]].topicList[value["idxTopic"]].statusRead =
        value["status"];
    notifyListeners();
  }

  set updateStatusDeleteGroup(var value) {
    _chatData[value["idxGroup"]].topicList[value["idxTopic"]].message =
        value["message"];
    _chatData[value["idxGroup"]].topicList[value["idxTopic"]].contentType =
        value["flag"];
    notifyListeners();
  }

  set chatDataAll(List<ListUcList> value) {
    _chatData.addAll(value);
    notifyListeners();
  }

  set chatDataReplace(List<ListUcList> value) {
    _chatData = value;
    notifyListeners();
  }

  set removeChatData(bool value) {
    _chatData.clear();
    notifyListeners();
  }

  void clearChatDataGroup() {
    _chatDataGroup.clear();
    notifyListeners();
  }

  set chatDataGroup(Map<dynamic, dynamic> value) {
    _chatDataGroup.add(value);
    notifyListeners();
  }

  set hideTopic(var value) {
    if (value["indexSubGroup"] == -1) {
      _chatDataGroup[value["index"]]["saveSubGroup"] =
          _chatDataGroup[value["index"]]["subGroup"];
      _chatDataGroup[value["index"]]["subGroup"] = [];
    } else {
      _chatDataGroup[value["index"]]["saveTopic"] =
          _chatDataGroup[value["index"]]["topic"];
      List<Map<dynamic, dynamic>> listSubGroup =
          _chatDataGroup[value["index"]]["subGroup"];
      List<Map<dynamic, dynamic>> tempListSubGroup = [];
      tempListSubGroup.addAll(listSubGroup);
      _chatDataGroup[value["index"]]["saveSubGroup"] = tempListSubGroup;
      listSubGroup.removeWhere((element) =>
          element["id"] != listSubGroup[value["indexSubGroup"]]["id"]);
    }
    notifyListeners();
  }

  set showTopic(var value) {
    _chatDataGroup[value["index"]]["subGroup"] =
        _chatDataGroup[value["index"]]["saveSubGroup"];
    if (value["indexSubGroup"] == -1) {
      _chatDataGroup[value["index"]]["saveSubGroup"] = null;
    } else {
      _chatDataGroup[value["index"]]["saveTopic"] = null;
      _chatDataGroup[value["index"]]["saveSubGroup"] = null;
    }
    notifyListeners();
  }
}

class ThirdTab extends StatelessWidget {
  final keyState;

  ThirdTab({this.keyState});

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
      create: (_) => ChangeState(),
      child: Consumer<ChangeState>(
        builder: (context, changeState, _) =>
            ThirdTabs(changeState: changeState, key: keyState),
      ),
    );
  }
}

class ThirdTabs extends StatefulWidget {
  final ChangeState? changeState;

  ThirdTabs({Key? key, this.changeState}) : super(key: key);

  @override
  ThirdTabsState createState() => ThirdTabsState();
}

class ThirdTabsState extends State<ThirdTabs> {
  FocusNode searchFocus = FocusNode();
  String lang = "";

  ChangeState? getState() {
    return widget.changeState;
  }

  void searchChat(String text) {
    if (text.trim().isNotEmpty) {
      getUcList(
          changeState: widget.changeState, fromSearch: true, textSearch: text);
    } else {
      getUcList(changeState: widget.changeState, fromReceiver: true);
    }
  }

  static const List<IconData> icons = const [
    // Icons.mail,
    Icons.person_add,
    Icons.group_add,
  ];

  Future<String> getLanguageSetting() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("language")) {
      lang = prefs.getString("language") ?? "";
    }
    return lang;
  }

  void getUcList(
      {ChangeState? changeState,
      bool fromReceiver = false,
      bool fromSearch = false,
      String textSearch = ""}) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String idMe = prefs.getString("hasLogin") ?? "";
    void process(List<dynamic> resultQuery) async {
      List<ListUcList> temporaryUcList = [];
      for (var i = 0; i < resultQuery.length; i++) {
        var resultQueryMessage = await DBHelper().queryCustom(
            "SELECT * FROM MESSAGE WHERE message_id='${resultQuery[i]["message_id"]}' AND is_call_center = 0");
        var resultQueryMessageStatus = await DBHelper().queryCustom(
            "SELECT f_pin, status FROM MESSAGE_STATUS WHERE message_id='${resultQuery[i]["message_id"]}'");
        String contentType;
        if (resultQueryMessage.isEmpty) {
          continue;
        }
        if (resultQueryMessage[0]["image_id"].trim().isNotEmpty) {
          contentType = "1";
        } else if (resultQueryMessage[0]["video_id"].trim().isNotEmpty) {
          contentType = "2";
        } else if (resultQueryMessage[0]["file_id"].trim().isNotEmpty) {
          contentType = "4";
        } else if (resultQueryMessage[0]["audio_id"].trim().isNotEmpty) {
          contentType = "-1";
        } else if (resultQueryMessage[0]["message_text"]
            .contains("lat/lng: (")) {
          contentType = "3";
        } else if (resultQueryMessage[0]["attachment_flag"].toString() ==
            "11") {
          contentType = "11";
        } else if (resultQueryMessage[0]["attachment_flag"].toString() ==
                "25" ||
            resultQueryMessage[0]["attachment_flag"].toString() == "27") {
          contentType = resultQueryMessage[0]["attachment_flag"].toString();
        } else if (resultQueryMessage[0]["attachment_flag"].toString() ==
            "30") {
          contentType = "30";
        } else if (resultQueryMessage[0]["attachment_flag"].toString() ==
                "12" ||
            resultQueryMessage[0]["attachment_flag"].toString() == "19" ||
            resultQueryMessage[0]["attachment_flag"].toString() == "24") {
          contentType = resultQueryMessage[0]["attachment_flag"].toString();
        } else {
          contentType = "0";
        }
        DateTime dateTime = DateTime.fromMillisecondsSinceEpoch(
            resultQueryMessage[0]["server_date"]);
        var formatTime = new DateFormat('Hm');
        String showTime = DateTime(dateTime.year, dateTime.month, dateTime.day)
                    .difference(DateTime(DateTime.now().year,
                        DateTime.now().month, DateTime.now().day))
                    .inDays ==
                0
            ? formatTime.format(dateTime)
            : DateTime(dateTime.year, dateTime.month, dateTime.day)
                        .difference(DateTime(DateTime.now().year,
                            DateTime.now().month, DateTime.now().day))
                        .inDays ==
                    -1
                ? "Yesterday"
                : DateFormat('dd MMMM yyyy').format(dateTime);
        var resultQueryBuddy, resultQueryTopic, firstName, officialAccount;
        List<ListTopic> temporaryTopic = [];
        String idGroup = "";
        var statusMessage;
        var counterFromSearch;
        if (fromSearch) {
          counterFromSearch = await DBHelper().queryCustom(
              "SELECT counter FROM MESSAGE_SUMMARY where message_id = '${resultQuery[i]["message_id"]}'");
          if (counterFromSearch.length == 0) {
            counterFromSearch = "0";
          } else {
            counterFromSearch = counterFromSearch[0]["counter"].toString();
          }
        }
        if (resultQueryMessage[0]["message_scope_id"] == "3") {
          if (fromSearch) {
            String from;
            if (resultQueryMessage[0]["f_pin"] == idMe) {
              from = resultQueryMessage[0]["l_pin"];
            } else {
              from = resultQueryMessage[0]["f_pin"];
            }
            resultQueryBuddy = await DBHelper().queryCustom(
                "SELECT first_name, last_name, image_id, official_account FROM BUDDY WHERE f_pin='$from'");
          } else {
            resultQueryBuddy = await DBHelper().queryCustom(
                "SELECT first_name, last_name, image_id, official_account FROM BUDDY WHERE f_pin='${resultQuery[i]["l_pin"]}'");
          }
          if (resultQueryBuddy.isNotEmpty) {
            String name;
            if (resultQueryBuddy[0]["last_name"].trim().isNotEmpty) {
              name = resultQueryBuddy[0]["first_name"] +
                  " " +
                  resultQueryBuddy[0]["last_name"];
            } else {
              name = resultQueryBuddy[0]["first_name"];
            }
            firstName = name;
            officialAccount = resultQueryBuddy[0]["official_account"];
          } else {
            firstName = "Bot";
            officialAccount = "0";
          }
        } else {
          statusMessage =
              await DBHelper().minStatusGroup(resultQuery[i]["message_id"]);
          if (fromSearch) {
            resultQueryBuddy = await DBHelper().queryCustom(
                "SELECT f_name, image_id, official FROM GROUPZ WHERE group_id='${resultQueryMessage[0]["l_pin"]}'");
          } else {
            resultQueryBuddy = await DBHelper().queryCustom(
                "SELECT f_name, image_id, official FROM GROUPZ WHERE group_id='${resultQuery[i]["l_pin"]}'");
          }
          Map<String, String> temp = Map();
          var resultQueryBuddyMembers = await DBHelper()
              .queryCustom("SELECT f_pin, first_name, last_name FROM BUDDY");
          for (var j = 0; j < resultQueryBuddyMembers.length; j++) {
            temp["@${resultQueryBuddyMembers[j]["f_pin"]}"] =
                (resultQueryBuddyMembers[j]["first_name"] +
                        " " +
                        resultQueryBuddyMembers[j]["last_name"])
                    .trim();
          }
          if (resultQueryBuddy.length != 0) {
            idGroup = resultQuery[i]["l_pin"];
            firstName = resultQueryBuddy[0]["f_name"];
            officialAccount = resultQueryBuddy[0]["official"];
            temporaryTopic.add(ListTopic(
                fName: "Lounge",
                fPin: resultQueryMessage[0]["f_pin"],
                myPin: idMe,
                contentType: contentType,
                lPin: "",
                listMembers: temp,
                idGroup: resultQuery[i]["l_pin"],
                statusRead: statusMessage[0]["MIN(status)"].toString(),
                counter: fromSearch
                    ? counterFromSearch
                    : resultQuery[i]["counter"].toString(),
                message: resultQuery[i]["message_id"].contains("-")
                    ? Uri.decodeFull(resultQueryMessage[0]["message_text"])
                    : resultQueryMessage[0]["message_text"],
                messageId: resultQuery[i]["message_id"],
                sortTime: resultQueryMessage[0]["server_date"],
                time: showTime));
          } else {
            if (fromSearch) {
              resultQueryTopic = await DBHelper().queryCustom(
                  "SELECT group_id, chat_id, title, scope_id FROM DISCUSSION_FORUM WHERE chat_id='${resultQueryMessage[0]["chat_id"]}'");
            } else {
              resultQueryTopic = await DBHelper().queryCustom(
                  "SELECT group_id, chat_id, title, scope_id FROM DISCUSSION_FORUM WHERE chat_id='${resultQuery[i]["l_pin"]}'");
            }
            if (resultQueryTopic.length == 0) {
              continue;
            }
            resultQueryBuddy = await DBHelper().queryCustom(
                "SELECT f_name, image_id, official FROM GROUPZ WHERE group_id='${resultQueryTopic[0]["group_id"]}'");
            if (resultQueryBuddy.length == 0) {
              continue;
            }
            idGroup = resultQueryTopic[0]["group_id"];
            firstName = resultQueryBuddy[0]["f_name"];
            officialAccount = resultQueryBuddy[0]["official"];
            temporaryTopic.add(ListTopic(
                fName: resultQueryTopic[0]["title"],
                counter: fromSearch
                    ? counterFromSearch
                    : resultQuery[i]["counter"].toString(),
                idGroup: resultQueryTopic[0]["group_id"],
                messageId: resultQuery[i]["message_id"],
                statusRead: statusMessage[0]["MIN(status)"].toString(),
                contentType: contentType,
                listMembers: temp,
                sortTime: resultQueryMessage[0]["server_date"],
                time: showTime,
                lPin: resultQueryTopic[0]["chat_id"],
                fPin: resultQueryMessage[0]["f_pin"],
                myPin: idMe,
                message: resultQuery[i]["message_id"].contains("-")
                    ? Uri.decodeFull(resultQueryMessage[0]["message_text"])
                    : resultQueryMessage[0]["message_text"]));
          }
        }
        var text;
        if (!resultQueryMessage[0]["message_text"].contains("🚫") &&
            resultQueryMessage[0]["message_text"].contains('"')) {
          text =
              Uri.decodeQueryComponent(resultQueryMessage[0]["message_text"]);
        } else {
          text = resultQueryMessage[0]["message_text"];
        }
        int countCredential = 0;
        if (resultQueryMessage[0]["credential"] == "1") {
          SharedPreferences prefs = await SharedPreferences.getInstance();
          if (prefs.getInt(resultQuery[i]["message_id"]) != null) {
            int timeCount = prefs.getInt(resultQuery[i]["message_id"]) ?? 0;
            int difSecond = DateTime.now()
                .difference(DateTime.fromMillisecondsSinceEpoch(timeCount))
                .inSeconds;
            if (60 - difSecond > 0) {
              countCredential = 60 - difSecond;
            } else {
              countCredential = 0;
            }
          }
        }
        // if (resultQueryBuddy.isNotEmpty &&
        //     resultQueryBuddy[0]["image_id"] != "") {
        //   checkImage(ip + resultQueryBuddy[0]["image_id"]);
        // }
        var listUcList = ListUcList(
            time: showTime,
            statusRead: resultQueryMessage[0]["message_scope_id"] == "3" &&
                    resultQueryMessageStatus.length != 0
                ? resultQueryMessageStatus[0]["status"].toString()
                : resultQueryMessage[0]["status"],
            sortTime: resultQueryMessage[0]["server_date"],
            messageId: resultQuery[i]["message_id"],
            type: resultQueryMessage[0]["message_scope_id"] == "3" ? "1" : "2",
            counter:
                fromSearch ? counterFromSearch : "${resultQuery[i]["counter"]}",
            fName: firstName,
            contentType: contentType,
            fPicture: resultQueryBuddy.isEmpty
                ? ""
                : resultQueryBuddy[0]["image_id"] == ""
                    ? ""
                    : resultQueryBuddy[0]["image_id"],
            lPin: resultQueryMessage[0]["message_scope_id"] == "3"
                ? fromSearch
                    ? resultQueryMessage[0]["f_pin"] == idMe
                        ? resultQueryMessage[0]["l_pin"]
                        : resultQueryMessage[0]["f_pin"]
                    : resultQuery[i]["l_pin"]
                : idGroup,
            myPin: idMe,
            fPin: resultQueryMessage[0]["f_pin"],
            credential: resultQueryMessage[0]["credential"],
            countCredential: countCredential,
            isOfficial: officialAccount == "1" ? "1" : "0",
            isVerified: officialAccount == "2" ? "1" : "0",
            topicList: temporaryTopic,
            message: text);
        if (firstName != "Bot") {
          if (resultQueryMessage[0]["message_scope_id"] == "3") {
            temporaryUcList.add(listUcList);
          } else {
            var idx = temporaryUcList.indexOf(listUcList);
            if (idx == -1) {
              temporaryUcList.add(listUcList);
            } else {
              var temp = temporaryUcList[idx].topicList;
              temp.add(temporaryTopic[0]);
              temp.sort((b, a) => a.sortTime.compareTo(b.sortTime));
              temporaryUcList[idx].topicList = temp;
              temporaryUcList[idx].sortTime = temporaryTopic[0].sortTime;
              temporaryUcList[idx].time = temporaryTopic[0].time;
            }
          }
        } else {
          if (resultQueryMessage[0]["message_scope_id"] == "3") {
            temporaryUcList.add(listUcList);
          }
        }
      }
      if (!fromSearch) {
        temporaryUcList.sort((b, a) => (a.sortTime).compareTo(b.sortTime));
      }
      if (fromReceiver || fromSearch) {
        changeState?.chatDataReplace = temporaryUcList;
      } else {
        changeState?.chatDataAll = temporaryUcList;
      }
    }

    List<dynamic> resultQuery;
    if (fromSearch) {
      List<dynamic> resultQueryJoinPersonal = await DBHelper().queryCustom(
          "SELECT MESSAGE_SUMMARY.message_id FROM MESSAGE_SUMMARY INNER JOIN BUDDY ON MESSAGE_SUMMARY.l_pin = BUDDY.f_pin AND (BUDDY.first_name LIKE '%$textSearch%' OR BUDDY.last_name LIKE '%$textSearch%')");
      List<dynamic> resultQueryJoinGroup = await DBHelper().queryCustom(
          "SELECT MESSAGE_SUMMARY.message_id FROM MESSAGE_SUMMARY INNER JOIN GROUPZ ON MESSAGE_SUMMARY.l_pin = GROUPZ.group_id AND GROUPZ.f_name LIKE '%$textSearch%'");
      List<dynamic> resultQueryJoinTopic = await DBHelper().queryCustom(
          "SELECT MESSAGE_SUMMARY.message_id FROM MESSAGE_SUMMARY INNER JOIN DISCUSSION_FORUM ON MESSAGE_SUMMARY.l_pin = DISCUSSION_FORUM.chat_id AND DISCUSSION_FORUM.title LIKE '%$textSearch%'");
      resultQuery = await DBHelper().queryCustom(
          "SELECT message_id FROM MESSAGE WHERE message_text LIKE '%$textSearch%' and is_call_center = 0");
      List<dynamic> combinedList = [
        ...resultQueryJoinPersonal,
        ...resultQueryJoinGroup,
        ...resultQueryJoinTopic,
        ...resultQuery
      ];
      process(combinedList);
    } else {
      resultQuery =
          await DBHelper().queryCustom("SELECT * FROM MESSAGE_SUMMARY");
      process(resultQuery);
    }
  }

  void onMessageReceiver(ChangeState? changeState) {
    API.onReceiveMessage = (dynamic result) async {
      if (mounted) {
        getUcList(changeState: changeState, fromReceiver: true);
      }
      if (result.containsKey('flag')) {
        final SharedPreferences prefs = await SharedPreferences.getInstance();
        String idMe = prefs.getString("hasLogin")!;
        if (result["flag"] == "PUSH_CALL_CENTER") {
          Notifications.showNotifications(result, 2, context);
          //   var jo = {
          //     'f_pin' : result["f_pin"],
          //     'status' : result["status"]
          //   };
          //   var jos = jsonEncode(jo);
          //   API.setCallCenterData(result["f_pin"], jos);
          //   SharedObj.CALL_CENTER_SESSION_F_PIN = result["f_pin"];
          //   Navigator.push(context, PageTransition(type: PageTransitionType.fade,
          //       child: RequestCallCenter(
          //         name: result["real_name"],
          //         image: result["image_id"],
          //         deviceId: result["f_pin"],
          //         channel: result["channel"],
          //       )
          //   ));
        } else if (result["flag"] == "ACCEPT_CALL_CENTER") {
          SharedObj.CALL_CENTER_SESSION_COMPLAINT_ID = result["complaint_id"];
          var resultQueryMe = await DBHelper()
              .queryCustom("SELECT * FROM BUDDY where f_pin='$idMe'");
          if (resultQueryMe[0]["user_type"] == '0') {
            Notifications.showNotifications(result, 2, context);
          }
          // var jo = {
          //   'f_pin' : result["f_pin"],
          //   'status' : result["status"]
          // };
          // var jos = jsonEncode(jo);
          // API.setCallCenterData(result["f_pin"], jos);
          // var resultQuery = await DBHelper().queryCustom(
          //     "SELECT * from BUDDY WHERE f_pin = '${result["f_pin"]}'");
          // if (resultQuery.isNotEmpty) {
          //   Navigator.push(
          //       context,
          //       PageTransition(
          //         type: PageTransitionType.fade,
          //         child: ChatPage(
          //           result["real_name"],
          //           resultQuery[0]["image_id"],
          //           identifier: result["f_pin"],
          //           refresh: true,
          //           isContactCenter: true,
          //           data: result,
          //           channel: result["channel"],
          //         ),
          //       ));
          // }
        } else if (result["flag"] == "END_CALL_CENTER") {
          SharedObj.deleteCacheCallCenter();
        }
        return;
      } else {
        Notifications.showNotifications(result, 0, context, level: 2);
      }
      // widget.refreshCounter();
    };
  }

  void onMessage(ChangeState? changeState) {
    API.onMessage = (dynamic result) async {
      if (mounted) {
        if (result["A06"] != "4") {
          if (!result["A18"].contains("-2,")) {
            int idx = changeState!.getChatData.indexOf(
                ListUcList(lPin: result["A01"], messageId: result["A18"]));
            if (idx != -1) {
              if (result["D01"] == "1") {
                changeState.updateStatusDelete = {
                  "index": idx,
                  "message": "🚫 _This message was deleted_",
                  "flag": "0"
                };
              } else {
                changeState.updateStatusChatData = {
                  "index": idx,
                  "status": result["A15"]
                };
              }
            }
          } else {
            int idx = changeState!.getChatData.indexOf(ListUcList(
                lPin: result["A01"], messageId: result["A18"].split(",")[1]));
            if (idx != -1) {
              changeState.updateStatusChatData = {
                "index": idx,
                "status": result["A15"]
              };
            }
          }
        } else {
          for (var i = 0; i < changeState!.getChatData.length; i++) {
            if (changeState.getChatData[i].topicList.length == 0) {
              continue;
            }
            List<ListTopic> temp = changeState.getChatData[i].topicList;
            if (!result["A18"].contains("-2,")) {
              int idx = temp.indexOf(ListTopic(messageId: result["A18"]));
              var statusMessage =
                  await DBHelper().minStatusGroup(result["A18"]);
              if (idx == -1) {
                continue;
              } else {
                if (result["D01"] == "1") {
                  changeState.updateStatusDeleteGroup = {
                    "idxGroup": i,
                    "idxTopic": idx,
                    "message": "🚫 _This message was deleted_",
                    "flag": "0"
                  };
                } else {
                  changeState.updateStatusChatDataGroup = {
                    "idxGroup": i,
                    "idxTopic": idx,
                    "status": statusMessage[0]["MIN(status)"].toString()
                  };
                }
                break;
              }
            } else {
              int idx = temp
                  .indexOf(ListTopic(messageId: result["A18"].split(",")[1]));
              if (idx == -1) {
                continue;
              } else {
                var statusMessage = await DBHelper()
                    .minStatusGroup(result["A18"].split(",")[1]);
                changeState.updateStatusChatDataGroup = {
                  "idxGroup": i,
                  "idxTopic": idx,
                  "status": statusMessage[0]["MIN(status)"].toString()
                };
                break;
              }
            }
          }
        }
      }
    };
  }

  @override
  void initState() {
    // onGroup(this.context);
    getUcList(changeState: widget.changeState);
    onMessageReceiver(widget.changeState);
    onMessage(widget.changeState);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Stack(
        children: <Widget>[
          SingleChildScrollView(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                widget.changeState!.getChatData.length == 0
                    ? Container(
                        alignment: Alignment.topCenter,
                        margin: EdgeInsets.only(top: 15.0),
                        child: Text(
                          "There are no conversations",
                          textAlign: TextAlign.center,
                          style: TextStyle(fontSize: 12),
                        ))
                    : ListView.builder(
                        shrinkWrap: true,
                        physics: NeverScrollableScrollPhysics(),
                        itemCount: widget.changeState!.getChatData.length,
                        itemBuilder: (context, index) {
                          return Container(
                            child: UcList(
                              data: widget.changeState!.getChatData[index],
                              refresh: () {
                                onMessageReceiver(widget.changeState);
                                onMessage(widget.changeState);
                              },
                            ),
                          );
                        },
                      ),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
