import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_translate/flutter_translate.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/chat/widgets/link_previewer.dart';
import 'package:palio_lite/chat/widgets/rtf_text_view.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:path/path.dart' as path;
import 'package:percent_indicator/circular_percent_indicator.dart';
import 'package:provider/provider.dart';

class BubbleChatGroup extends StatelessWidget {
  final bool isOriginator, isSelected, isSearch, isFavorite;
  final String? text,
      fileName,
      thumbName,
      messageId,
      refId,
      refFrom,
      refText,
      refType,
      refThumbId,
      refThumbLoc,
      fName,
      fImage,
      statusRead,
      time,
      attachmentType,
      credential,
      ack,
      textSearch,
      link;
  final double? progress;
  final VoidCallback onLongPress, onTapReply, onTapPress, onSelectProfile;
  final Map<String, String> listMentions;
  final Map<String, String> allContacts;

  BubbleChatGroup(
      {required this.isOriginator,
      this.text,
      this.fileName,
      this.fImage,
      this.thumbName,
      this.messageId,
      this.refId,
      this.refFrom,
      this.refText,
      this.refType,
      this.refThumbId,
      this.refThumbLoc,
      required this.isSearch,
      this.textSearch,
      this.fName,
      this.statusRead,
      this.time,
      this.attachmentType,
      this.credential,
      this.ack,
      this.progress,
      required this.onLongPress,
      required this.onTapPress,
      required this.onTapReply,
      required this.onSelectProfile,
      this.link,
      required this.isSelected,
      required this.listMentions,
      required this.allContacts,
      required this.isFavorite});

  @override
  Widget build(BuildContext context) {
    String textPlace = "";
    String imageLocation = "";
    if (attachmentType == "3") {
      String textDecode = text!;
      if (!isOriginator) {
        textDecode = Uri.decodeQueryComponent(Uri.decodeFull(text!));
      }
      int idx = textDecode.indexOf("(");
      int idx2 = textDecode.indexOf("lat/lng: ");
      var textLatLng = textDecode
          .substring(idx + 1, textDecode.trim().length - 1)
          .split(",");
      imageLocation = "https://maps.googleapis.com/maps/api/staticmap?center=" +
          textLatLng[0] +
          "," +
          textLatLng[1].trimLeft() +
          "&zoom=15&size=" +
          300.toString() +
          "x" +
          150.toString() +
          "&maptype=normal&markers=color:red|" +
          textLatLng[0] +
          "," +
          textLatLng[1].trimLeft() +
          "&key=AIzaSyC79oNmadezP9O_4Q_Ud2bbuJ_L7M--y1s";
      textPlace = textDecode.substring(15, idx2);
    }
    return MultiProvider(
      providers: [
        ChangeNotifierProvider(create: (_) => ChangeColorOriginator()),
        ChangeNotifierProvider(create: (_) => ChangeColor())
      ],
      child: isOriginator
          ? Column(
              crossAxisAlignment: CrossAxisAlignment.end,
              children: <Widget>[
                Consumer<ChangeColor>(
                  builder: (context, changeColor, _) => GestureDetector(
                    onTapDown: (onTapDown) {
                      changeColor.selectProfile = true;
                    },
                    onTapUp: (onTapUP) {
                      changeColor.selectProfile = false;
                    },
                    onLongPress: () {
                      changeColor.selectProfile = false;
                    },
                    onTap: () {
                      onSelectProfile();
                    },
                    child: Container(
                      margin: EdgeInsets.only(right: 15.0),
                      alignment: Alignment.bottomRight,
                      color: Colors.transparent,
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.end,
                        children: <Widget>[
                          Text(fName!),
                          SizedBox(width: 5.0),
                          Container(
                              width: 30,
                              height: 30,
                              decoration: BoxDecoration(
                                  shape: BoxShape.circle,
                                  color: Colors.grey,
                                  image: DecorationImage(
                                      fit: BoxFit.cover,
                                      image: fImage == ""
                                          ? AssetImage(
                                              "assets/images/default_pp.png",
                                              package: "palio_lite")
                                          : LocalImage(fImage)
                                              as ImageProvider)))
                        ],
                      ),
                    ),
                  ),
                ),
                SizedBox(height: 2.0),
                Container(
                  color: isSelected ? Color(0xFFFFE399) : Colors.transparent,
                  alignment: Alignment.topRight,
                  padding: EdgeInsets.only(top: 5.0, right: 5.0, bottom: 25.0),
                  margin: EdgeInsets.only(left: 10.0, right: 10.0),
                  child: IntrinsicWidth(
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.stretch,
                      mainAxisAlignment: MainAxisAlignment.start,
                      children: [
                        Stack(
                          children: <Widget>[
                            Container(
                              child: Row(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                mainAxisSize: MainAxisSize.min,
                                children: <Widget>[
                                  Container(
                                    child: Column(
                                      crossAxisAlignment:
                                          CrossAxisAlignment.end,
                                      mainAxisAlignment:
                                          MainAxisAlignment.start,
                                      children: <Widget>[
                                        Container(
                                            margin: EdgeInsets.only(top: 8.0),
                                            child: Text(time!,
                                                style:
                                                    TextStyle(fontSize: 10))),
                                        text!.contains("🚫")
                                            ? Container()
                                            : statusRead == "8"
                                                ? Container(
                                                    width: 14.0,
                                                    height: 14.0,
                                                    child: Image.asset(
                                                        "assets/images/message_status_ack.png",
                                                        package: "palio_lite"),
                                                  )
                                                : Icon(
                                                    statusRead == "1" ||
                                                            statusRead == "2"
                                                        ? Icons.check
                                                        : Icons.done_all,
                                                    size: 14.0,
                                                    color: statusRead == "4"
                                                        ? Colors.blue
                                                        : Colors.grey)
                                      ],
                                    ),
                                    margin: EdgeInsets.only(right: 10.0),
                                  ),
                                  Consumer<ChangeColorOriginator>(
                                    builder: (context, changeColor, _) =>
                                        GestureDetector(
                                      onTapDown: (onTapDown) {
                                        changeColor.isColorBlue = false;
                                      },
                                      onTapUp: (onTapUp) {
                                        changeColor.isColorBlue = true;
                                      },
                                      onLongPress: () {
                                        onLongPress();
                                      },
                                      onTap: () {
                                        onTapPress();
                                      },
                                      onTapCancel: () {
                                        changeColor.isColorBlue = true;
                                      },
                                      child: Container(
                                          constraints: BoxConstraints(
                                              minWidth: MediaQuery.of(context)
                                                      .size
                                                      .width *
                                                  0.1,
                                              maxWidth: MediaQuery.of(context)
                                                      .size
                                                      .width *
                                                  0.8),
                                          decoration: BoxDecoration(
                                              boxShadow:
                                                  attachmentType == "11" &&
                                                          !text!.contains("🚫")
                                                      ? null
                                                      : [
                                                          BoxShadow(
                                                            color: Colors.grey,
                                                            blurRadius: 3.0,
                                                          )
                                                        ],
                                              color: attachmentType == "11" &&
                                                      !text!.contains("🚫")
                                                  ? Colors.transparent
                                                  : changeColor.color,
                                              borderRadius:
                                                  BorderRadius.circular(10.0)),
                                          child: IntrinsicWidth(
                                            child: Column(
                                              crossAxisAlignment:
                                                  CrossAxisAlignment.stretch,
                                              children: <Widget>[
                                                link != null
                                                    ? LinkPreviewer(
                                                        link!, false)
                                                    : Container(
                                                        height: 0.0,
                                                        width: 0.0),
                                                refId != null
                                                    ? GestureDetector(
                                                        child: Container(
                                                          color: AppColor
                                                              .secondaryColor,
                                                          constraints:
                                                              BoxConstraints(
                                                                  minHeight: 30,
                                                                  maxHeight:
                                                                      80),
                                                          padding: EdgeInsets
                                                              .fromLTRB(
                                                                  25.0,
                                                                  8.0,
                                                                  15.0,
                                                                  8.0),
                                                          child: Row(
                                                            mainAxisAlignment:
                                                                MainAxisAlignment
                                                                    .spaceBetween,
                                                            children: <Widget>[
                                                              Expanded(
                                                                child: Column(
                                                                  crossAxisAlignment:
                                                                      CrossAxisAlignment
                                                                          .start,
                                                                  mainAxisSize:
                                                                      MainAxisSize
                                                                          .min,
                                                                  children: <
                                                                      Widget>[
                                                                    Text(
                                                                        refFrom!,
                                                                        style: TextStyle(
                                                                            color:
                                                                                Colors.black54,
                                                                            fontWeight: FontWeight.bold)),
                                                                    Wrap(
                                                                      children: <
                                                                          Widget>[
                                                                        RTFText(
                                                                          refType == "11"
                                                                              ? "Attachment sticker"
                                                                              : refType == "1" && refText!.isEmpty
                                                                                  ? "Attachment picture"
                                                                                  : refType == "2" && refText!.isEmpty
                                                                                      ? "Attachment video"
                                                                                      : refType == "6" && refText!.isEmpty
                                                                                          ? "Attachment document"
                                                                                          : refType == "-1" && refText!.isEmpty
                                                                                              ? "Attachment audio"
                                                                                              : refType == "3" && refText!.isEmpty
                                                                                                  ? "Attachment location"
                                                                                                  : refText!,
                                                                          maxLines:
                                                                              2,
                                                                          overflow:
                                                                              TextOverflow.ellipsis,
                                                                          textColor:
                                                                              Colors.black54,
                                                                          isGroupBubble:
                                                                              true,
                                                                          mentions:
                                                                              allContacts,
                                                                          currentMembers:
                                                                              listMentions,
                                                                          bLink: link != null
                                                                              ? true
                                                                              : false,
                                                                        )
                                                                      ],
                                                                    )
                                                                  ],
                                                                ),
                                                              ),
                                                              refType != "0" &&
                                                                      refType !=
                                                                          "11"
                                                                  ? Container(
                                                                      margin: EdgeInsets.only(
                                                                          left:
                                                                              10.0),
                                                                      width: 30,
                                                                      height:
                                                                          30,
                                                                      decoration: BoxDecoration(
                                                                          image: refType == "-1" || refType == "6"
                                                                              ? null
                                                                              : DecorationImage(fit: BoxFit.cover, image: refType == "3" ? NetworkImage(refThumbLoc!) : FileImage(File(refThumbId!)) as ImageProvider)),
                                                                      child: refType == "-1"
                                                                          ? Icon(
                                                                              Icons.volume_up,
                                                                              color: Colors.grey,
                                                                              size: 30,
                                                                            )
                                                                          : refType == "6"
                                                                              ? Icon(Icons.insert_drive_file, color: Colors.green, size: 30)
                                                                              : Container())
                                                                  : refType == "11"
                                                                      ? Container(
                                                                          margin:
                                                                              EdgeInsets.only(left: 10.0),
                                                                          width:
                                                                              30,
                                                                          height:
                                                                              30,
                                                                          child: Image.asset(
                                                                              "assets/stickers/$refText",
                                                                              package: "palio_lite"),
                                                                        )
                                                                      : Container(height: 0.0, width: 0.0)
                                                            ],
                                                          ),
                                                        ),
                                                        onTap: () {
                                                          onTapReply();
                                                        },
                                                      )
                                                    : Container(
                                                        height: 0.0,
                                                        width: 0.0),
                                                Container(
                                                  padding: EdgeInsets.fromLTRB(
                                                      25, 15, 15, 15),
                                                  child: Row(
                                                    crossAxisAlignment:
                                                        CrossAxisAlignment
                                                            .start,
                                                    mainAxisSize:
                                                        MainAxisSize.min,
                                                    children: <Widget>[
                                                      (fileName != null ||
                                                                  attachmentType ==
                                                                      "3") &&
                                                              !text!.contains(
                                                                  "🚫")
                                                          ? attachmentType ==
                                                                  "6"
                                                              ? Container(
                                                                  width: 80,
                                                                  height: 80,
                                                                  child: Icon(
                                                                    Icons
                                                                        .insert_drive_file,
                                                                    color: Colors
                                                                        .green,
                                                                    size: 80,
                                                                  ),
                                                                )
                                                              : Container(
                                                                  margin: EdgeInsets
                                                                      .only(
                                                                          right:
                                                                              10.0),
                                                                  width: 80,
                                                                  height: 80,
                                                                  decoration: BoxDecoration(
                                                                      image: DecorationImage(
                                                                          fit: BoxFit.cover,
                                                                          image: attachmentType == "3"
                                                                              ? NetworkImage(imageLocation)
                                                                              : thumbName!.contains("assets/images/")
                                                                                  ? AssetImage(thumbName!, package: "palio_lite")
                                                                                  : FileImage(File(thumbName!)) as ImageProvider)),
                                                                  child: attachmentType ==
                                                                          "2"
                                                                      ? Icon(
                                                                          CupertinoIcons
                                                                              .play_arrow_solid,
                                                                          color: Colors
                                                                              .white
                                                                              .withOpacity(0.5),
                                                                          size:
                                                                              30,
                                                                        )
                                                                      : Container(),
                                                                )
                                                          : Container(),
                                                      attachmentType == "11" &&
                                                              !text!.contains(
                                                                  "🚫")
                                                          ? Container(
                                                              height: 72,
                                                              width: 72,
                                                              child: Image.asset(
                                                                  "assets/stickers/$text",
                                                                  package:
                                                                      "palio_lite"),
                                                            )
                                                          : Container(
                                                              constraints: BoxConstraints(
                                                                  maxWidth: MediaQuery.of(
                                                                              context)
                                                                          .size
                                                                          .width *
                                                                      (fileName != null ||
                                                                              attachmentType == "3"
                                                                          ? 0.45
                                                                          : 0.68)),
                                                              child: Wrap(
                                                                children: <
                                                                    Widget>[
                                                                  RTFText(
                                                                    attachmentType ==
                                                                            "3"
                                                                        ? textPlace
                                                                        : attachmentType == "6" &&
                                                                                text == ""
                                                                            ? path.basename(fileName!)
                                                                            : text!,
                                                                    isSearch:
                                                                        isSearch,
                                                                    textSearch:
                                                                        textSearch,
                                                                    isGroupBubble:
                                                                        true,
                                                                    mentions:
                                                                        allContacts,
                                                                    currentMembers:
                                                                        listMentions,
                                                                    bLink: link !=
                                                                            null
                                                                        ? true
                                                                        : false,
                                                                  )
                                                                ],
                                                              ),
                                                            ),
                                                    ],
                                                  ),
                                                ),
                                              ],
                                            ),
                                          )),
                                    ),
                                  ),
                                ],
                              ),
                            ),
                            Container(
                              margin: EdgeInsets.only(
                                  top: link == null ? 8.0 : 110.0, left: 30),
                              width: 20,
                              height: 20,
                              decoration: BoxDecoration(
                                  color: Colors.white.withOpacity(0.95),
                                  boxShadow: [
                                    BoxShadow(
                                      color: Colors.grey,
                                      blurRadius: 3.0,
                                    )
                                  ],
                                  borderRadius: BorderRadius.only(
                                      topRight: Radius.circular(8.0),
                                      bottomRight: Radius.circular(8.0))),
                              child: Image.asset(
                                  "assets/images/cuc_scope_chat.png",
                                  package: "palio_lite"),
                            ),
                          ],
                        ),
                        Container(
                          margin: EdgeInsets.only(left: 35.0),
                          child: IntrinsicHeight(
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.start,
                              crossAxisAlignment: CrossAxisAlignment.stretch,
                              children: [
                                ack == "8" && !text!.contains("🚫")
                                    ? Container(
                                        margin: EdgeInsets.only(top: 5.0),
                                        width: 25,
                                        height: 25,
                                        child: Image.asset(
                                            statusRead == "8"
                                                ? "assets/images/twsn_menulist_ack_12.png"
                                                : "assets/images/twsn_menulist_ack_11.png",
                                            package: "palio_lite"),
                                      )
                                    : Container(),
                                isFavorite
                                    ? Container(
                                        margin: EdgeInsets.only(top: 5.0),
                                        width: 25,
                                        height: 25,
                                        child: Image.asset(
                                            "assets/images/od_favorite.png",
                                            package: "palio_lite"),
                                      )
                                    : Container(),
                              ],
                            ),
                          ),
                        )
                      ],
                    ),
                  ),
                ),
              ],
            )
          : Consumer<ChangeColor>(
              builder: (context, changeColor, _) => Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  GestureDetector(
                    onTapDown: (onTapDown) {
                      changeColor.selectProfile = true;
                    },
                    onTapUp: (onTapUP) {
                      changeColor.selectProfile = false;
                    },
                    onLongPress: () {
                      changeColor.selectProfile = false;
                    },
                    onTap: () {
                      onSelectProfile();
                    },
                    child: Container(
                      margin: EdgeInsets.only(left: 15.0),
                      alignment: Alignment.bottomLeft,
                      color: changeColor.colorProfile,
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.start,
                        children: <Widget>[
                          Container(
                              width: 30,
                              height: 30,
                              decoration: BoxDecoration(
                                  shape: BoxShape.circle,
                                  color: Colors.grey,
                                  image: DecorationImage(
                                      fit: BoxFit.cover,
                                      image: fImage == ""
                                          ? AssetImage(
                                              "assets/images/default_pp.png",
                                              package: "palio_lite")
                                          : LocalImage(fImage)
                                              as ImageProvider))),
                          SizedBox(width: 5.0),
                          Text(fName!)
                        ],
                      ),
                    ),
                  ),
                  SizedBox(height: 2.0),
                  Container(
                    color: isSelected ? Color(0xFFFFE399) : Colors.transparent,
                    alignment: Alignment.topLeft,
                    padding: EdgeInsets.only(top: 5.0, left: 5.0, bottom: 25.0),
                    margin: EdgeInsets.only(left: 10.0, right: 10.0),
                    child: IntrinsicWidth(
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.stretch,
                        mainAxisAlignment: MainAxisAlignment.start,
                        children: [
                          Stack(
                            alignment: Alignment.topRight,
                            children: <Widget>[
                              Container(
                                child: Row(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  mainAxisSize: MainAxisSize.min,
                                  children: <Widget>[
                                    GestureDetector(
                                      onLongPress: () {
                                        onLongPress();
                                      },
                                      onTap: () {
                                        onTapPress();
                                      },
                                      onTapDown: (onTapDown) {
                                        changeColor.isColorWhite = false;
                                      },
                                      onTapUp: (onTapUP) {
                                        changeColor.isColorWhite = true;
                                      },
                                      onTapCancel: () {
                                        changeColor.isColorWhite = true;
                                      },
                                      child: Container(
                                          constraints: BoxConstraints(
                                              minWidth: MediaQuery.of(context)
                                                      .size
                                                      .width *
                                                  0.1,
                                              maxWidth: MediaQuery.of(context)
                                                      .size
                                                      .width *
                                                  0.8),
                                          decoration: BoxDecoration(
                                              boxShadow:
                                                  attachmentType == "11" &&
                                                          !text!.contains("🚫")
                                                      ? null
                                                      : [
                                                          BoxShadow(
                                                            color: Colors.grey,
                                                            blurRadius: 3.0,
                                                          )
                                                        ],
                                              color: attachmentType == "11" &&
                                                      !text!.contains("🚫")
                                                  ? Colors.transparent
                                                  : changeColor.color,
                                              borderRadius:
                                                  BorderRadius.circular(10.0)),
                                          child: IntrinsicWidth(
                                            child: Column(
                                              crossAxisAlignment:
                                                  CrossAxisAlignment.stretch,
                                              children: <Widget>[
                                                link != null
                                                    ? LinkPreviewer(
                                                        link!, false)
                                                    : Container(
                                                        height: 0.0,
                                                        width: 0.0),
                                                refId != null
                                                    ? GestureDetector(
                                                        child: Container(
                                                          color:
                                                              Color(0xFFc4d1e1),
                                                          constraints:
                                                              BoxConstraints(
                                                                  minHeight: 30,
                                                                  maxHeight:
                                                                      80),
                                                          padding: EdgeInsets
                                                              .fromLTRB(
                                                                  15.0,
                                                                  8.0,
                                                                  25.0,
                                                                  8.0),
                                                          child: Row(
                                                            mainAxisAlignment:
                                                                MainAxisAlignment
                                                                    .spaceBetween,
                                                            children: <Widget>[
                                                              Expanded(
                                                                child: Column(
                                                                  crossAxisAlignment:
                                                                      CrossAxisAlignment
                                                                          .start,
                                                                  mainAxisSize:
                                                                      MainAxisSize
                                                                          .min,
                                                                  children: <
                                                                      Widget>[
                                                                    Text(
                                                                        refFrom!,
                                                                        style: TextStyle(
                                                                            color:
                                                                                Colors.black54,
                                                                            fontWeight: FontWeight.bold)),
                                                                    Wrap(
                                                                      children: <
                                                                          Widget>[
                                                                        RTFText(
                                                                          refType == "11"
                                                                              ? "Attachment sticker"
                                                                              : refType == "1" && refText!.isEmpty
                                                                                  ? "Attachment picture"
                                                                                  : refType == "2" && refText!.isEmpty
                                                                                      ? "Attachment video"
                                                                                      : refType == "6" && refText!.isEmpty
                                                                                          ? "Attachment document"
                                                                                          : refType == "-1" && refText!.isEmpty
                                                                                              ? "Attachment audio"
                                                                                              : refType == "3" && refText!.isEmpty
                                                                                                  ? "Attachment location"
                                                                                                  : refText!,
                                                                          maxLines:
                                                                              2,
                                                                          overflow:
                                                                              TextOverflow.ellipsis,
                                                                          textColor:
                                                                              Colors.black54,
                                                                          isGroupBubble:
                                                                              true,
                                                                          mentions:
                                                                              allContacts,
                                                                          currentMembers:
                                                                              listMentions,
                                                                          bLink: link != null
                                                                              ? true
                                                                              : false,
                                                                        )
                                                                      ],
                                                                    )
                                                                  ],
                                                                ),
                                                              ),
                                                              refType != "0" &&
                                                                      refType !=
                                                                          "11"
                                                                  ? Container(
                                                                      margin: EdgeInsets.only(
                                                                          left:
                                                                              10.0),
                                                                      width: 30,
                                                                      height:
                                                                          30,
                                                                      decoration: BoxDecoration(
                                                                          image: refType == "-1" || refType == "6"
                                                                              ? null
                                                                              : DecorationImage(fit: BoxFit.cover, image: refType == "3" ? NetworkImage(refThumbLoc!) : FileImage(File(refThumbId!)) as ImageProvider)),
                                                                      child: refType == "-1"
                                                                          ? Icon(
                                                                              Icons.volume_up,
                                                                              color: Colors.grey,
                                                                              size: 30,
                                                                            )
                                                                          : refType == "6"
                                                                              ? Icon(Icons.insert_drive_file, color: Colors.green, size: 30)
                                                                              : Container())
                                                                  : refType == "11"
                                                                      ? Container(
                                                                          margin:
                                                                              EdgeInsets.only(left: 10.0),
                                                                          width:
                                                                              30,
                                                                          height:
                                                                              30,
                                                                          child: Image.asset(
                                                                              "assets/stickers/$refText",
                                                                              package: "palio_lite"),
                                                                        )
                                                                      : Container(height: 0.0, width: 0.0)
                                                            ],
                                                          ),
                                                        ),
                                                        onTap: () {
                                                          onTapReply();
                                                        },
                                                      )
                                                    : Container(
                                                        height: 0.0,
                                                        width: 0.0),
                                                Container(
                                                  padding: EdgeInsets.fromLTRB(
                                                      15, 15, 25, 15),
                                                  child: Row(
                                                    crossAxisAlignment:
                                                        CrossAxisAlignment
                                                            .start,
                                                    mainAxisSize:
                                                        MainAxisSize.min,
                                                    children: <Widget>[
                                                      (fileName != null ||
                                                                  attachmentType ==
                                                                      "3") &&
                                                              !text!.contains(
                                                                  "🚫")
                                                          ? attachmentType ==
                                                                  "-1"
                                                              ? Container(
                                                                  width: 80,
                                                                  height: 80,
                                                                  child: Icon(
                                                                    Icons
                                                                        .volume_up,
                                                                    color: Colors
                                                                        .grey,
                                                                    size: 80,
                                                                  ),
                                                                )
                                                              : attachmentType ==
                                                                      "6"
                                                                  ? Container(
                                                                      width: 80,
                                                                      height:
                                                                          80,
                                                                      child:
                                                                          Icon(
                                                                        Icons
                                                                            .insert_drive_file,
                                                                        color: Colors
                                                                            .green,
                                                                        size:
                                                                            80,
                                                                      ),
                                                                    )
                                                                  : Container(
                                                                      margin: EdgeInsets.only(
                                                                          right:
                                                                              10.0),
                                                                      width: 80,
                                                                      height:
                                                                          80,
                                                                      decoration: BoxDecoration(
                                                                          image: DecorationImage(
                                                                              fit: BoxFit.cover,
                                                                              image: attachmentType == "3"
                                                                                  ? NetworkImage(imageLocation)
                                                                                  : thumbName!.contains("assets/images/")
                                                                                      ? AssetImage(thumbName!, package: "palio_lite")
                                                                                      : FileImage(File(thumbName!)) as ImageProvider)),
                                                                      child: attachmentType ==
                                                                              "2"
                                                                          ? Icon(
                                                                              CupertinoIcons.play_arrow_solid,
                                                                              color: Colors.white.withOpacity(0.5),
                                                                              size: 30,
                                                                            )
                                                                          : attachmentType == "1" && progress != 100.0
                                                                              ? Icon(
                                                                                  Icons.file_download,
                                                                                  color: Colors.white,
                                                                                )
                                                                              : Container(),
                                                                    )
                                                          : Container(),
                                                      attachmentType == "11" &&
                                                              !text!.contains(
                                                                  "🚫")
                                                          ? Container(
                                                              height: 72,
                                                              width: 72,
                                                              child: Image.asset(
                                                                  "assets/stickers/$text",
                                                                  package:
                                                                      "palio_lite"),
                                                            )
                                                          : Container(
                                                              constraints: BoxConstraints(
                                                                  maxWidth: MediaQuery.of(
                                                                              context)
                                                                          .size
                                                                          .width *
                                                                      (fileName != null ||
                                                                              attachmentType == "3"
                                                                          ? 0.45
                                                                          : 0.68)),
                                                              child: Wrap(
                                                                children: <
                                                                    Widget>[
                                                                  RTFText(
                                                                      attachmentType ==
                                                                              "-1"
                                                                          ? Uri.decodeQueryComponent(Uri.decodeFull(
                                                                              text!))
                                                                          : attachmentType ==
                                                                                  "3"
                                                                              ? textPlace
                                                                              : attachmentType == "6" &&
                                                                                      text ==
                                                                                          ""
                                                                                  ? path.basename(
                                                                                      fileName!)
                                                                                  : text!,
                                                                      isSearch:
                                                                          isSearch,
                                                                      mentions:
                                                                          allContacts,
                                                                      currentMembers:
                                                                          listMentions,
                                                                      isGroupBubble:
                                                                          true,
                                                                      bLink: link !=
                                                                              null
                                                                          ? true
                                                                          : false,
                                                                      textSearch:
                                                                          textSearch)
                                                                ],
                                                              ),
                                                            ),
                                                    ],
                                                  ),
                                                ),
                                              ],
                                            ),
                                          )),
                                    ),
                                    Container(
                                      child: Column(
                                        crossAxisAlignment:
                                            CrossAxisAlignment.start,
                                        mainAxisAlignment:
                                            MainAxisAlignment.start,
                                        children: <Widget>[
                                          Container(
                                              margin: EdgeInsets.only(top: 8.0),
                                              child: Text(time!,
                                                  style:
                                                      TextStyle(fontSize: 10))),
                                          progress != null && progress != 100.0
                                              ? CircularPercentIndicator(
                                                  radius: 25.0,
                                                  lineWidth: 4.0,
                                                  percent: progress! / 100,
                                                  center: Icon(
                                                    Icons.file_download,
                                                    size: 15,
                                                    color:
                                                        AppColor.primaryColor,
                                                  ),
                                                  progressColor:
                                                      AppColor.primaryColor,
                                                )
                                              : Container()
                                        ],
                                      ),
                                      margin: EdgeInsets.only(left: 10.0),
                                    ),
                                  ],
                                ),
                              ),
                              Container(
                                margin: EdgeInsets.only(
                                    top: link == null ? 8.0 : 110.0, right: 30),
                                width: 20,
                                height: 20,
                                decoration: BoxDecoration(
                                    color: Color(0xFFc4d1e1).withOpacity(0.95),
                                    borderRadius: BorderRadius.only(
                                        topLeft: Radius.circular(8.0),
                                        bottomLeft: Radius.circular(8.0))),
                                child: Image.asset(
                                    "assets/images/cuc_scope_chat.png",
                                    package: "palio_lite"),
                              ),
                            ],
                          ),
                          Container(
                            margin: EdgeInsets.only(right: 35.0),
                            child: IntrinsicHeight(
                              child: Row(
                                mainAxisAlignment: MainAxisAlignment.end,
                                crossAxisAlignment: CrossAxisAlignment.stretch,
                                children: [
                                  isFavorite
                                      ? Container(
                                          margin: EdgeInsets.only(top: 5.0),
                                          width: 25,
                                          height: 25,
                                          child: Image.asset(
                                              "assets/images/od_favorite.png",
                                              package: "palio_lite"),
                                        )
                                      : Container(),
                                  ack == "8" && !text!.contains("🚫")
                                      ? Container(
                                          margin: EdgeInsets.only(top: 5.0),
                                          width: 25,
                                          height: 25,
                                          child: Image.asset(
                                              statusRead == "8"
                                                  ? "assets/images/twsn_menulist_ack_12.png"
                                                  : "assets/images/twsn_menulist_ack_11.png",
                                              package: "palio_lite"),
                                        )
                                      : Container(),
                                ],
                              ),
                            ),
                          )
                        ],
                      ),
                    ),
                  ),
                ],
              ),
            ),
    );
  }
}

class ChangeColorOriginator with ChangeNotifier {
  bool _isColorBlue = true;

  bool get isColorBlue => _isColorBlue;

  set isColorBlue(bool value) {
    _isColorBlue = value;
    notifyListeners();
  }

  Color get color => (_isColorBlue) ? Color(0xFFc4d1e1) : Colors.white;
}

class ChangeColor with ChangeNotifier {
  bool _isColorWhite = true;
  bool _selectProfile = false;

  bool get isColorWhite => _isColorWhite;

  bool get selectProfile => _selectProfile;

  set isColorWhite(bool value) {
    _isColorWhite = value;
    notifyListeners();
  }

  set selectProfile(bool value) {
    _selectProfile = value;
    notifyListeners();
  }

  Color get color => (_isColorWhite) ? Colors.white : Color(0xFFc4d1e1);

  Color get colorProfile =>
      (selectProfile) ? Colors.blueAccent : Colors.transparent;
}
