import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:intl/intl.dart';

class CalendarForm extends StatefulWidget {
  final Function(String) date;
  final int times;
  final int? dateStart;

  CalendarForm({required this.date, required this.times, this.dateStart});

  @override
  _CalendarFormState createState() => _CalendarFormState();
}

class _CalendarFormState extends State<CalendarForm> {
  int time = 0;
  String date = DateFormat('dd MMM yyyy').format(DateTime.now());
  Color color = Colors.white;

  @override
  void initState() {
    super.initState();
    time = widget.times;
    if (time > 0) {
      date = DateFormat('dd MMM yyyy')
          .format(DateTime.fromMillisecondsSinceEpoch(time));
    }
  }

  @override
  void didUpdateWidget(covariant CalendarForm oldWidget) {
    if (widget.dateStart != null) {
      if (time < widget.times) {
        setState(() {
          time = widget.times;
          date = DateFormat('dd-MM-yyyy')
              .format(DateTime.fromMillisecondsSinceEpoch(time));
        });
      }
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        showDatePicker(
                context: context,
                initialDate: time == 0
                    ? DateTime.now()
                    : DateTime.fromMillisecondsSinceEpoch(time),
                firstDate: DateTime.now(),
                currentDate: time == 0
                    ? DateTime.now()
                    : DateTime.fromMillisecondsSinceEpoch(time),
                lastDate: DateTime(9999))
            .then((value) {
          if (value != null) {
            if (widget.dateStart != null) {
              if (value.millisecondsSinceEpoch >= widget.dateStart!) {
                widget.date(DateFormat('dd-MM-yyyy').format(value));
                setState(() {
                  date = DateFormat('dd-MM-yyyy').format(value);
                  time = value.millisecondsSinceEpoch;
                });
              } else {
                DateTime dateStart =
                    DateTime.fromMillisecondsSinceEpoch(widget.dateStart!);
                widget.date(DateFormat('dd-MM-yyyy').format(dateStart));
                setState(() {
                  date = DateFormat('dd-MM-yyyy').format(dateStart);
                  time = widget.dateStart!;
                });
              }
            } else {
              widget.date(DateFormat('dd-MM-yyyy').format(value));
              setState(() {
                date = DateFormat('dd-MM-yyyy').format(value);
                time = value.millisecondsSinceEpoch;
              });
            }
          }
        });
      },
      onTapDown: (tap) {
        setState(() {
          color = Colors.grey;
        });
      },
      onTapUp: (tap) {
        setState(() {
          color = Colors.white;
        });
      },
      onTapCancel: () {
        setState(() {
          color = Colors.white;
        });
      },
      child: Container(
        padding: EdgeInsets.only(left: 10.0, right: 10.0),
        height: 40,
        decoration: BoxDecoration(
            color: color,
            border: Border.all(color: Colors.black),
            borderRadius: BorderRadius.circular(3.0)),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            Icon(
              Icons.calendar_today,
              color: Colors.grey[600],
              size: 14,
            ),
            SizedBox(width: 10.0),
            Text(date)
          ],
        ),
      ),
    );
  }
}
