import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/chat/chat_group_screen.dart';
import 'package:palio_lite/chat/chat_screen.dart';
import 'package:palio_lite/chat/widgets/custom_expansion_tile.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:path/path.dart' as path;
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ChangeState with ChangeNotifier {
  List<Map<dynamic, dynamic>> _groupData = [],
      _dataContact = [],
      _srcContact = [],
      _srcGrup = [];
  int _tabFlag = 0;

  List<Map<dynamic, dynamic>> get getGroupData => _groupData;

  List<Map<dynamic, dynamic>> get getDataContact => _dataContact;

  List<Map<dynamic, dynamic>> get getSrcContact => _srcContact;

  List<Map<dynamic, dynamic>> get getSrcGrup => _srcGrup;

  int get tabFlag => _tabFlag;

  set groupData(Map<dynamic, dynamic> value) {
    _groupData.add(value);
    notifyListeners();
  }

  set dataContact(Map<dynamic, dynamic> value) {
    _dataContact.add(value);
    notifyListeners();
  }

  set srcContact(List<Map<dynamic, dynamic>> value) {
    _srcContact.addAll(value);
    notifyListeners();
  }

  set addSrcContact(Map<dynamic, dynamic> value) {
    _srcContact.add(value);
    notifyListeners();
  }

  set clearSrcContact(bool value) {
    _srcContact.clear();
    notifyListeners();
  }

  set srcGrup(List<Map<dynamic, dynamic>> value) {
    _srcGrup.addAll(value);
    notifyListeners();
  }

  set addSrcGrup(Map<dynamic, dynamic> value) {
    _srcGrup.add(value);
    notifyListeners();
  }

  set clearSrcGrup(bool value) {
    _srcGrup.clear();
    notifyListeners();
  }

  set tabFlag(int value) {
    _tabFlag = value;
    notifyListeners();
  }

  void clearChatDataGroup() {
    _srcGrup.clear();
    _groupData.clear();
    notifyListeners();
  }

  set hideTopic(var value) {
    if (value["indexSubGroup"] == -1) {
      _srcGrup[value["index"]]["saveSubGroup"] =
          _srcGrup[value["index"]]["subGroup"];
      _srcGrup[value["index"]]["subGroup"] = [];
    } else {
      _srcGrup[value["index"]]["saveTopic"] = _srcGrup[value["index"]]["topic"];
      List<Map<dynamic, dynamic>> listSubGroup =
          _srcGrup[value["index"]]["subGroup"];
      List<Map<dynamic, dynamic>> tempListSubGroup = [];
      tempListSubGroup.addAll(listSubGroup);
      _srcGrup[value["index"]]["saveSubGroup"] = tempListSubGroup;
      listSubGroup.removeWhere((element) =>
          element["id"] != listSubGroup[value["indexSubGroup"]]["id"]);
    }
    notifyListeners();
  }

  set showTopic(var value) {
    _srcGrup[value["index"]]["subGroup"] =
        _srcGrup[value["index"]]["saveSubGroup"];
    if (value["indexSubGroup"] == -1) {
      _srcGrup[value["index"]]["saveSubGroup"] = null;
    } else {
      _srcGrup[value["index"]]["saveTopic"] = null;
      _srcGrup[value["index"]]["saveSubGroup"] = null;
    }
    notifyListeners();
  }
}

class ForwardChat extends StatelessWidget {
  final List<dynamic> data;
  final bool isContactCenter;
  final String? idGroup;
  final String? idTopic;

  ForwardChat(this.data,
      {this.isContactCenter = false,
      this.idGroup,
      this.idTopic});

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
        create: (_) => ChangeState(),
        child: Consumer<ChangeState>(
            builder: (context, changeState, _) => ForwardChats(
                changeState, data, isContactCenter, idGroup, idTopic)));
  }
}

class ForwardChats extends StatefulWidget {
  final ChangeState changeState;
  final List<dynamic> data;
  final bool isContactCenter;
  final String? idGroup;
  final String? idTopic;

  ForwardChats(this.changeState, this.data, this.isContactCenter, this.idGroup,
      this.idTopic);

  _NewChats createState() => _NewChats();
}

class _NewChats extends State<ForwardChats>
    with SingleTickerProviderStateMixin {
  TextEditingController _editingController = new TextEditingController();
  String lang = "";

  static List<Widget> _tabs = <Widget>[
    Tab(text: "Contacts"),
    Tab(text: "Groups")
  ];

  TabController? _tabController;

  @override
  void initState() {
    super.initState();
    _tabController = TabController(
      length: _tabs.length,
      vsync: this,
    );
    getData();
    getLanguageSetting();
  }

  Future<String> getLanguageSetting() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey("language")) {
      lang = prefs.getString("language") ?? "";
    }
    return lang;
  }

  void getData() async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    String idMe = prefs.getString("hasLogin")!;
    List<dynamic> resultQuery = await DBHelper().queryCustom(
        "SELECT f_pin, first_name, last_name, official_account, image_id FROM BUDDY where f_pin <> '024ef50e8c' and f_pin <> '$idMe'");
    List<dynamic> resultQueryGroup = await DBHelper().queryCustom(
        "SELECT group_id, f_name, image_id, official, is_open, level_edu, materi_edu, is_education FROM GROUPZ WHERE (parent = '' OR parent IS NULL)");

    for (var i = 0; i < resultQuery.length; i++) {
      widget.changeState.dataContact = {
        "f_pin": resultQuery[i]["f_pin"],
        "name":
            resultQuery[i]["first_name"] + " " + resultQuery[i]["last_name"],
        "picture": resultQuery[i]["image_id"],
        "isVerified": resultQuery[i]["official_account"] == "2" ? "1" : "0",
        "isOfficial": resultQuery[i]["official_account"] == "1" ? "1" : "0"
      };
    }
    widget.changeState.getDataContact
        .sort((a, b) => (a["name"]).compareTo(b["name"]));
    widget.changeState.srcContact = widget.changeState.getDataContact;
    for (var j = 0; j < resultQueryGroup.length; j++) {
      List<dynamic> resultQueryTopic = await DBHelper().queryCustom(
          "SELECT chat_id, title FROM DISCUSSION_FORUM WHERE group_id='${resultQueryGroup[j]["group_id"]}'");
      List<dynamic> temporaryTopic = [];
      // if (lang == "en_US") {
      //   temporaryTopic.add({"name": "Lounge", "id": ""});
      // } else {
      //   temporaryTopic.add({"name": "Beranda", "id": ""});
      // }
      temporaryTopic.add({"name": "Lounge", "id": ""});
      if (resultQueryTopic.length > 0) {
        for (var j = 0; j < resultQueryTopic.length; j++) {
          temporaryTopic.add({
            "name": resultQueryTopic[j]["title"],
            "id": resultQueryTopic[j]["chat_id"]
          });
        }
        temporaryTopic.sort((a, b) => (a["name"]).compareTo(b["name"]));
      }

      final Map myVariableDynamic = {};

      Future addSubGroup(String groupId, int groupType, int level) async {
        myVariableDynamic["dataSubGroup$level"] = [];
        myVariableDynamic["resultQuerySubGroup$level"] = await DBHelper()
            .queryCustom(
                "SELECT group_id, f_name, image_id, official, is_open, level_edu, materi_edu, is_education FROM GROUPZ WHERE parent='$groupId' AND is_education != 1");
        for (var j = 0;
            j < myVariableDynamic["resultQuerySubGroup$level"].length;
            j++) {
          List<dynamic> resultQueryTopic = await DBHelper().queryCustom(
              "SELECT chat_id, title FROM DISCUSSION_FORUM WHERE group_id='${myVariableDynamic["resultQuerySubGroup$level"][j]["group_id"]}'");
          List<dynamic> temporaryTopic = [];
          // if (lang == "en_US") {
          //   temporaryTopic.add({"name": "Lounge", "id": ""});
          // } else {
          //   temporaryTopic.add({"name": "Beranda", "id": ""});
          // }
          temporaryTopic.add({"name": "Lounge", "id": ""});
          if (resultQueryTopic.length > 0) {
            for (var j = 0; j < resultQueryTopic.length; j++) {
              temporaryTopic.add({
                "name": resultQueryTopic[j]["title"],
                "id": resultQueryTopic[j]["chat_id"]
              });
            }
            temporaryTopic.sort((a, b) => (a["name"]).compareTo(b["name"]));
          }

          myVariableDynamic["listSubGroup${level + 1}"] = await addSubGroup(
              myVariableDynamic["resultQuerySubGroup$level"][j]["group_id"],
              groupType,
              (level + 1));
          myVariableDynamic["dataSubGroup$level"].add({
            "id": myVariableDynamic["resultQuerySubGroup$level"][j]["group_id"],
            "name": myVariableDynamic["resultQuerySubGroup$level"][j]["f_name"],
            "picture": myVariableDynamic["resultQuerySubGroup$level"][j]
                ["image_id"],
            "topic": temporaryTopic,
            "subGroup": myVariableDynamic["listSubGroup${level + 1}"],
            "saveSubGroup": null,
            "saveTopic": null,
            "isVerified": myVariableDynamic["resultQuerySubGroup$level"][j]
                        ["official"] ==
                    "2"
                ? "1"
                : "0",
            "isOfficial": myVariableDynamic["resultQuerySubGroup$level"][j]
                            ["official"] ==
                        "1" ||
                    groupType == 1
                ? "1"
                : "0",
            "isOpen": myVariableDynamic["resultQuerySubGroup$level"][j]
                        ["is_education"] ==
                    1
                ? "2"
                : myVariableDynamic["resultQuerySubGroup$level"][j]["is_open"]
                    .toString()
          });
        }
        return myVariableDynamic["dataSubGroup$level"];
      }

      var listSubGroup;

      var resultQueryBuddy = await DBHelper()
          .queryCustom("SELECT user_type FROM BUDDY where f_pin='$idMe'");

      if (resultQueryBuddy[0]["user_type"] != "23" &&
          resultQueryBuddy[0]["user_type"] != "24" &&
          resultQueryGroup[j]["group_type"] == 1) {
        listSubGroup = [];
      } else {
        listSubGroup = await addSubGroup(resultQueryGroup[j]["group_id"],
            resultQueryGroup[j]["group_type"], 1);
      }
      widget.changeState.groupData = {
        "id": resultQueryGroup[j]["group_id"],
        "name": resultQueryGroup[j]["f_name"],
        "picture": resultQueryGroup[j]["image_id"],
        "topic": temporaryTopic,
        "subGroup": listSubGroup,
        "saveSubGroup": null,
        "saveTopic": null,
        "isVerified": resultQueryGroup[j]["official"] == "2" ? "1" : "0",
        "isOfficial": resultQueryGroup[j]["official"] == "1" ? "1" : "0",
        "isOpen": resultQueryGroup[j]["is_education"] == 1
            ? "2"
            : resultQueryGroup[j]["is_open"].toString()
      };
    }
    widget.changeState.srcGrup = widget.changeState.getGroupData;
  }

  @override
  void dispose() {
    super.dispose();
  }

  onForward(var data, int type, {var dataTopic}) async {
    if (type == 0) {
      var resultQueryBuddy = await DBHelper().queryCustom(
          "SELECT ex_block FROM BUDDY WHERE f_pin=='${data["f_pin"]}'");
      if (resultQueryBuddy[0]["ex_block"] == "1" ||
          resultQueryBuddy[0]["ex_block"] == "-1") {
        if (resultQueryBuddy[0]["ex_block"] == "-1") {
          Fluttertoast.showToast(msg: "You are blocked by this user");
        } else {
          Fluttertoast.showToast(msg: "You block this user");
        }
        return;
      }
      for (var i = 0; i < widget.data.length; i++) {
        var dataChat = widget.data[i]["data"];
        if (dataChat.text.contains("@")) {
          var resultQuery = await DBHelper()
              .queryCustom("SELECT f_pin, first_name, last_name FROM BUDDY");
          for (var i = 0; i < resultQuery.length; i++) {
            if (dataChat.text.contains("@${resultQuery[i]["f_pin"]}")) {
              if (resultQuery[i]["last_name"].isNotEmpty) {
                dataChat.text = dataChat.text.replaceAll(
                    "@${resultQuery[i]["f_pin"]}",
                    "@${resultQuery[i]["first_name"]} ${resultQuery[i]["last_name"]} ");
              } else {
                dataChat.text = dataChat.text.replaceAll(
                    "@${resultQuery[i]["f_pin"]}",
                    "@${resultQuery[i]["first_name"]} ");
              }
            }
          }
        }
        if (dataChat.attachmentType == "1") {
          API.sendMessage(
              lPin: data["f_pin"],
              messageScopeId: "3",
              status: "2",
              messageText: dataChat.text,
              imageId: path.basename(dataChat.fileName),
              thumbId: path.basename(dataChat.thumbName),
              attachmentFlag: "1");
        } else if (dataChat.attachmentType == "2") {
          API.sendMessage(
              lPin: data["f_pin"],
              messageScopeId: "3",
              status: "2",
              messageText: dataChat.text,
              videoId: path.basename(dataChat.fileName),
              thumbId: path.basename(dataChat.thumbName),
              attachmentFlag: "2");
        } else if (dataChat.attachmentType == "6") {
          API.sendMessage(
              lPin: data["f_pin"],
              messageScopeId: "3",
              status: "2",
              messageText: dataChat.text,
              fileId: path.basename(dataChat.fileName),
              attachmentFlag: "6");
        } else if (dataChat.attachmentType == "-1") {
          API.sendMessage(
              lPin: data["f_pin"],
              messageScopeId: "3",
              status: "2",
              messageText: dataChat.text,
              audioId: path.basename(dataChat.fileName),
              attachmentFlag: "5");
        } else if (dataChat.attachmentType == "3") {
          API.sendMessage(
              lPin: data["f_pin"],
              messageScopeId: "3",
              status: "2",
              messageText:
                  Uri.decodeQueryComponent(Uri.decodeFull(dataChat.text)),
              attachmentFlag: "3");
        } else {
          API.sendMessage(
              lPin: data["f_pin"],
              messageScopeId: "3",
              status: "2",
              messageText: dataChat.text);
        }
      }
      Navigator.pop(context);
      if (!widget.isContactCenter) {
        Navigator.pushReplacement(
            context,
            PageTransition(
                type: PageTransitionType.fade,
                child: ChatPage(
                  fullName: data["name"],
                  image: data["picture"],
                  identifier: data["f_pin"],
                  refresh: true,
                )));
      }
    } else {
      for (var i = 0; i < widget.data.length; i++) {
        var dataChat = widget.data[i]["data"];
        if (!(widget.idGroup == data["id"] &&
            widget.idTopic == dataTopic["id"])) {
          if (dataChat.text.contains("@")) {
            var resultQuery = await DBHelper()
                .queryCustom("SELECT f_pin, first_name, last_name FROM BUDDY");
            for (var i = 0; i < resultQuery.length; i++) {
              if (dataChat.text.contains("@${resultQuery[i]["f_pin"]}")) {
                if (resultQuery[i]["last_name"].isNotEmpty) {
                  dataChat.text = dataChat.text.replaceAll(
                      "@${resultQuery[i]["f_pin"]}",
                      "@${resultQuery[i]["first_name"]} ${resultQuery[i]["last_name"]} ");
                } else {
                  dataChat.text = dataChat.text.replaceAll(
                      "@${resultQuery[i]["f_pin"]}",
                      "@${resultQuery[i]["first_name"]} ");
                }
              }
            }
          }
        }
        if (dataChat.attachmentType == "1") {
          API.sendMessage(
              lPin: data["id"],
              chatId: dataTopic["id"],
              messageScopeId: "4",
              status: "2",
              messageText: dataChat.text,
              imageId: path.basename(dataChat.fileName),
              thumbId: path.basename(dataChat.thumbName),
              attachmentFlag: "1");
        } else if (dataChat.attachmentType == "2") {
          API.sendMessage(
              lPin: data["id"],
              chatId: dataTopic["id"],
              messageScopeId: "4",
              status: "2",
              messageText: dataChat.text,
              videoId: path.basename(dataChat.fileName),
              thumbId: path.basename(dataChat.thumbName),
              attachmentFlag: "2");
        } else if (dataChat.attachmentType == "6") {
          API.sendMessage(
              lPin: data["id"],
              chatId: dataTopic["id"],
              messageScopeId: "4",
              status: "2",
              messageText: dataChat.text,
              fileId: path.basename(dataChat.fileName),
              attachmentFlag: "6");
        } else if (dataChat.attachmentType == "-1") {
          API.sendMessage(
              lPin: data["id"],
              chatId: dataTopic["id"],
              messageScopeId: "4",
              status: "2",
              messageText: dataChat.text,
              audioId: path.basename(dataChat.fileName),
              attachmentFlag: "5");
        } else if (dataChat.attachmentType == "3") {
          API.sendMessage(
              lPin: data["id"],
              chatId: dataTopic["id"],
              messageScopeId: "4",
              status: "2",
              messageText:
                  Uri.decodeQueryComponent(Uri.decodeFull(dataChat.text)),
              attachmentFlag: "4");
        } else {
          API.sendMessage(
              lPin: data["id"],
              chatId: dataTopic["id"],
              messageScopeId: "4",
              status: "2",
              messageText: dataChat.text);
        }
      }
      Navigator.pop(context);
      if (!widget.isContactCenter) {
        Navigator.pushReplacement(
            context,
            PageTransition(
                type: PageTransitionType.fade,
                child: ChatGroupPage(
                  data["name"],
                  dataTopic["name"],
                  data["picture"],
                  identifier: dataTopic["id"],
                  idGroup: data["id"],
                  refresh: true,
                )));
      }
    }
  }

  _search(name) async {
    if (widget.changeState.tabFlag == 0) {
      if (name.toString().isNotEmpty) {
        widget.changeState.clearSrcContact = true;
        for (int i = 0; i < widget.changeState.getDataContact.length; i++) {
          if (widget.changeState.getDataContact[i]["name"].contains(name)) {
            widget.changeState.addSrcContact =
                widget.changeState.getDataContact[i];
          }
        }
      } else {
        widget.changeState.clearSrcContact = true;
        widget.changeState.srcContact = widget.changeState.getDataContact;
      }
    } else {
      if (name.toString().isNotEmpty) {
        widget.changeState.clearSrcGrup = true;
        for (int i = 0; i < widget.changeState.getGroupData.length; i++) {
          if (widget.changeState.getGroupData[i]["name"]
              .toLowerCase()
              .contains(name)) {
            widget.changeState.addSrcGrup = widget.changeState.getGroupData[i];
          }
        }
      } else {
        widget.changeState.clearSrcGrup = true;
        widget.changeState.srcGrup = widget.changeState.getGroupData;
      }
    }
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      child: Container(
        child: Column(
          children: <Widget>[
            Padding(
              padding: EdgeInsets.all(5),
              child: Align(
                alignment: Alignment.centerLeft,
                child: Text(
                  "Forward to",
                  style: TextStyle(fontSize: 18),
                ),
              ),
            ),
            Container(
              decoration: BoxDecoration(
                  color: Colors.grey[100],
                  border: Border(
                      bottom:
                          BorderSide(color: Colors.grey[200]!, width: 2.0))),
              padding: EdgeInsets.all(2.0),
              child: Theme(
                child: TextField(
                    controller: _editingController,
                    onChanged: (val) async {
                      await _search(val);
                    },
                    cursorColor: AppColor.primaryColor,
                    decoration: InputDecoration(
                        filled: true,
                        fillColor: Colors.white,
                        contentPadding: EdgeInsets.all(5.0),
                        prefixIcon: Icon(Icons.search),
                        focusedBorder: OutlineInputBorder(
                            borderRadius: BorderRadius.circular(3.0),
                            borderSide: BorderSide(color: Colors.grey)),
                        border: OutlineInputBorder(
                            borderRadius: BorderRadius.circular(3.0),
                            borderSide: BorderSide(color: Colors.grey)))),
                data: Theme.of(context).copyWith(primaryColor: Colors.grey),
              ),
            ),
            Container(
              child: TabBar(
                labelColor: Color(0xFF213458),
                indicatorColor: Color(0xFF213458),
                controller: _tabController,
                tabs: _tabs,
                onTap: (valIndex) {
                  widget.changeState.tabFlag = valIndex;
                  _search(_editingController.text);
                },
              ),
            ),
            Expanded(
              child: TabBarView(
                controller: _tabController,
                children: <Widget>[
                  _contactList(widget.changeState.getSrcContact),
                  _groupList(widget.changeState.getSrcGrup)
                ],
              ),
            )
          ],
        ),
      ),
    );
  }

  Widget _contactList(data) {
    return ListView.builder(
        shrinkWrap: true,
        itemCount: data.length,
        itemBuilder: (builder, index) {
          return ListTile(
            leading: Container(
              width: 45,
              height: 45,
              decoration: BoxDecoration(
                  shape: BoxShape.circle,
                  color: Colors.grey,
                  image: DecorationImage(
                      fit: BoxFit.cover,
                      image: LocalImage(data[index]["picture"]))),
            ),
            title: Row(
              children: <Widget>[
                data[index]["isVerified"] == "1"
                    ? Image.asset("assets/images/ic_verified_flag.png",
                        width: 15, height: 15, package: "palio_lite")
                    : data[index]["isOfficial"] == "1"
                        ? Image.asset("assets/images/ic_official_flag.png",
                            width: 15, height: 15, package: "palio_lite")
                        : Container(),
                Padding(
                  padding: EdgeInsets.only(
                    left: 8.0,
                  ),
                  child: Text(
                    data[index]["name"],
                  ),
                )
              ],
            ),
            onTap: () {
              onForward(data[index], 0);
            },
          );
        });
  }

  Widget _groupList(data) {
    return ListView.builder(
        shrinkWrap: true,
        itemCount: data.length,
        itemBuilder: (builder, index) {
          var topic = data[index]["topic"];
          var subGroup = data[index]["subGroup"];
          return CustomExpansionTile(
            onExpansionChanged: (cond) {
              if (!cond) {
                widget.changeState.clearChatDataGroup();
                getData();
              }
            },
            leading: Container(
              width: 45,
              height: 45,
              decoration: BoxDecoration(
                  shape: BoxShape.circle,
                  color: Colors.grey,
                  image: DecorationImage(
                      fit: BoxFit.cover,
                      image: data[index]["picture"] == null ||
                              data[index]["picture"].trim().isEmpty
                          ? AssetImage("assets/images/groupicon.png",
                              package: "palio_lite")
                          : LocalImage(data[index]["picture"])
                              as ImageProvider)),
            ),
            title: Row(
              children: <Widget>[
                data[index]["isVerified"] == "1"
                    ? Image.asset("assets/images/ic_verified_flag.png",
                        width: 20, height: 20, package: "palio_lite")
                    : data[index]["isOfficial"] == "1"
                        ? Image.asset("assets/images/ic_official_flag.png",
                            width: 20, height: 20, package: "palio_lite")
                        : data[index]["isOpen"] == "1"
                            ? Icon(Icons.public, size: 20)
                            : data[index]["isOpen"] == "2"
                                ? Image.asset(
                                    "assets/images/ic_educational.png",
                                    width: 20,
                                    height: 20,
                                    package: "palio_lite",
                                    color: Colors.black)
                                : Icon(Icons.lock, size: 20),
                Padding(
                  padding: EdgeInsets.only(
                    left: 8.0,
                  ),
                  child: Text("${data[index]["name"]}"),
                )
              ],
            ),
            children: <Widget>[
              data[index]["saveTopic"] != null
                  ? Container()
                  : Container(
                      margin: EdgeInsets.only(left: 28.0),
                      child: CustomExpansionTile(
                        onExpansionChanged: (cond) {
                          if (cond) {
                            widget.changeState.hideTopic = {
                              "index": index,
                              "indexSubGroup": -1
                            };
                          } else {
                            widget.changeState.showTopic = {
                              "index": index,
                              "indexSubGroup": -1
                            };
                          }
                        },
                        leading: Container(
                          width: 45,
                          height: 45,
                          decoration: BoxDecoration(
                              shape: BoxShape.circle, color: Colors.grey),
                          child: Image.asset("assets/images/forum.png",
                              package: "palio_lite"),
                        ),
                        title: Text(
                          "Forum",
                          style: TextStyle(fontSize: 15, color: Colors.black),
                        ),
                        children: [
                          ListView.builder(
                            shrinkWrap: true,
                            physics: NeverScrollableScrollPhysics(),
                            itemCount: topic.length,
                            itemBuilder: (context, indexTopic) {
                              return ListTile(
                                contentPadding: EdgeInsets.only(left: 5.0),
                                leading: Container(
                                  width: 45,
                                  height: 45,
                                  decoration:
                                      BoxDecoration(shape: BoxShape.circle),
                                ),
                                title: Text(
                                  "${topic[indexTopic]["name"]}",
                                  style: TextStyle(
                                      fontSize: 15, color: Colors.black),
                                ),
                                onTap: () {
                                  onForward(data[index], 1,
                                      dataTopic: topic[indexTopic]);
                                },
                              );
                            },
                          )
                        ],
                      ),
                    ),
              ListView.builder(
                shrinkWrap: true,
                physics: NeverScrollableScrollPhysics(),
                itemCount: subGroup.length,
                itemBuilder: (context, indexSubGroub) {
                  var topicSubGroup1 = subGroup[indexSubGroub]["topic"];
                  var subGroup1 = subGroup[indexSubGroub]["subGroup"];
                  return Container(
                    margin: EdgeInsets.only(left: 28.0),
                    child: CustomExpansionTile(
                      onExpansionChanged: (cond) {
                        if (cond) {
                          if (data[index]["saveSubGroup"] == null) {
                            widget.changeState.hideTopic = {
                              "index": index,
                              "indexSubGroup": indexSubGroub
                            };
                          }
                        } else {
                          widget.changeState.showTopic = {
                            "index": index,
                            "indexSubGroup": indexSubGroub
                          };
                        }
                      },
                      leading: Container(
                          width: 45,
                          height: 45,
                          decoration: BoxDecoration(
                              shape: BoxShape.circle,
                              color: Colors.grey,
                              image: DecorationImage(
                                  fit: BoxFit.cover,
                                  image: subGroup[indexSubGroub]["picture"] ==
                                              null ||
                                          subGroup[indexSubGroub]["picture"]
                                              .trim()
                                              .isEmpty
                                      ? AssetImage(
                                          "assets/images/groupicon.png",
                                          package: "palio_lite")
                                      : LocalImage(subGroup[indexSubGroub]
                                          ["picture"]) as ImageProvider))),
                      title: Row(
                        mainAxisSize: MainAxisSize.max,
                        children: <Widget>[
                          subGroup[indexSubGroub]["isVerified"] == "1"
                              ? Image.asset(
                                  "assets/images/ic_verified_flag.png",
                                  width: 20,
                                  height: 20,
                                  package: "palio_lite")
                              : subGroup[indexSubGroub]["isOfficial"] == "1"
                                  ? Image.asset(
                                      "assets/images/ic_official_flag.png",
                                      width: 20,
                                      height: 20,
                                      package: "palio_lite")
                                  : subGroup[indexSubGroub]["isOpen"] == "1"
                                      ? Icon(Icons.public, size: 20)
                                      : subGroup[indexSubGroub]["isOpen"] == "2"
                                          ? Image.asset(
                                              "assets/images/ic_educational.png",
                                              width: 20,
                                              height: 20,
                                              package: "palio_lite",
                                              color: Colors.black,
                                            )
                                          : Icon(Icons.lock, size: 20),
                          Container(
                            padding: EdgeInsets.only(
                              left: 8.0,
                            ),
                            child: Text("${subGroup[indexSubGroub]["name"]}",
                                overflow: TextOverflow.ellipsis),
                            constraints: BoxConstraints(maxWidth: 210),
                          )
                        ],
                      ),
                      children: [
                        Container(
                          margin: EdgeInsets.only(left: 28.0),
                          child: CustomExpansionTile(
                            leading: Container(
                              width: 45,
                              height: 45,
                              decoration: BoxDecoration(
                                  shape: BoxShape.circle, color: Colors.grey),
                              child: Image.asset("assets/images/forum.png",
                                  package: "palio_lite"),
                            ),
                            title: Text(
                              "Forum",
                              style:
                                  TextStyle(fontSize: 15, color: Colors.black),
                            ),
                            children: [
                              ListView.builder(
                                shrinkWrap: true,
                                physics: NeverScrollableScrollPhysics(),
                                itemCount: topicSubGroup1.length,
                                itemBuilder: (context, indexTopicSubgroup1) {
                                  return ListTile(
                                    title: Text(
                                      "${topicSubGroup1[indexTopicSubgroup1]["name"]}",
                                      style: TextStyle(
                                          fontSize: 15, color: Colors.black),
                                    ),
                                    leading: Container(
                                      height: 45,
                                      width: 45,
                                    ),
                                    onTap: () {
                                      onForward(data[index], 1,
                                          dataTopic:
                                              topic[indexTopicSubgroup1]);
                                    },
                                  );
                                },
                              )
                            ],
                          ),
                        ),
                        ListView.builder(
                          shrinkWrap: true,
                          physics: NeverScrollableScrollPhysics(),
                          itemCount: subGroup1.length,
                          itemBuilder: (context, indexSubGroub1) {
                            var topicSubGroup2 =
                                subGroup1[indexSubGroub1]["topic"];
                            var subGroup2 =
                                subGroup1[indexSubGroub1]["subGroup"];
                            return Container(
                              margin: EdgeInsets.only(left: 28.0),
                              child: CustomExpansionTile(
                                onExpansionChanged: (cond) {
                                  // if (cond) {
                                  //   if (widget.changeState
                                  //       .getChatDataGroup[
                                  //   index]["saveSubGroup"] ==
                                  //       null) {
                                  //     widget.changeState.hideTopic = {
                                  //       "index": index,
                                  //       "indexSubGroup": indexSubGroub1
                                  //     };
                                  //   }
                                  // } else {
                                  //   widget.changeState.showTopic = {
                                  //     "index": index,
                                  //     "indexSubGroup": indexSubGroub
                                  //   };
                                  // }
                                },
                                leading: Container(
                                    width: 45,
                                    height: 45,
                                    decoration: BoxDecoration(
                                        shape: BoxShape.circle,
                                        color: Colors.grey,
                                        image: DecorationImage(
                                            fit: BoxFit.cover,
                                            image: subGroup1[indexSubGroub1]
                                                            ["picture"] ==
                                                        null ||
                                                    subGroup1[indexSubGroub1]
                                                            ["picture"]
                                                        .trim()
                                                        .isEmpty
                                                ? AssetImage(
                                                    "assets/images/groupicon.png",
                                                    package: "palio_lite")
                                                : LocalImage(subGroup1[
                                                            indexSubGroub1]
                                                        ["picture"])
                                                    as ImageProvider))),
                                title: Row(
                                  mainAxisSize: MainAxisSize.max,
                                  children: <Widget>[
                                    subGroup1[indexSubGroub1]["isVerified"] ==
                                            "1"
                                        ? Image.asset(
                                            "assets/images/ic_verified_flag.png",
                                            width: 20,
                                            height: 20,
                                            package: "palio_lite")
                                        : subGroup1[indexSubGroub1]
                                                    ["isOfficial"] ==
                                                "1"
                                            ? Image.asset(
                                                "assets/images/ic_official_flag.png",
                                                width: 20,
                                                height: 20,
                                                package: "palio_lite")
                                            : subGroup1[indexSubGroub1]
                                                        ["isOpen"] ==
                                                    "1"
                                                ? Icon(Icons.public, size: 20)
                                                : subGroup1[indexSubGroub1]
                                                            ["isOpen"] ==
                                                        "2"
                                                    ? Image.asset(
                                                        "assets/images/ic_educational.png",
                                                        width: 20,
                                                        height: 20,
                                                        color: Colors.black,
                                                        package: "palio_lite")
                                                    : Icon(Icons.lock,
                                                        size: 20),
                                    Container(
                                      padding: EdgeInsets.only(
                                        left: 8.0,
                                      ),
                                      child: Text(
                                          "${subGroup1[indexSubGroub1]["name"]}",
                                          overflow: TextOverflow.ellipsis),
                                      constraints:
                                          BoxConstraints(maxWidth: 210),
                                    )
                                  ],
                                ),
                                children: [
                                  Container(
                                    margin: EdgeInsets.only(left: 28.0),
                                    child: CustomExpansionTile(
                                      leading: Container(
                                        width: 45,
                                        height: 45,
                                        decoration: BoxDecoration(
                                            shape: BoxShape.circle,
                                            color: Colors.grey),
                                        child: Image.asset(
                                            "assets/images/forum.png",
                                            package: "palio_lite"),
                                      ),
                                      title: Text(
                                        "Forum",
                                        style: TextStyle(
                                            fontSize: 15, color: Colors.black),
                                      ),
                                      children: [
                                        ListView.builder(
                                          shrinkWrap: true,
                                          physics:
                                              NeverScrollableScrollPhysics(),
                                          itemCount: topicSubGroup2.length,
                                          itemBuilder:
                                              (context, indexTopicSubgroup2) {
                                            return ListTile(
                                              title: Text(
                                                "${topicSubGroup2[indexTopicSubgroup2]["name"]}",
                                                style: TextStyle(
                                                    fontSize: 15,
                                                    color: Colors.black),
                                              ),
                                              leading: Container(
                                                height: 45,
                                                width: 45,
                                              ),
                                              onTap: () {
                                                onForward(data[index], 1,
                                                    dataTopic: topic[
                                                        indexTopicSubgroup2]);
                                              },
                                            );
                                          },
                                        )
                                      ],
                                    ),
                                  ),
                                  ListView.builder(
                                    shrinkWrap: true,
                                    physics: NeverScrollableScrollPhysics(),
                                    itemCount: subGroup2.length,
                                    itemBuilder: (context, indexSubGroub2) {
                                      var topicSubGroup3 =
                                          subGroup2[indexSubGroub2]["topic"];
                                      var subGroup3 =
                                          subGroup2[indexSubGroub2]["subGroup"];
                                      return Container(
                                        margin: EdgeInsets.only(left: 28.0),
                                        child: CustomExpansionTile(
                                          onExpansionChanged: (cond) {
                                            // if (cond) {
                                            //   if (widget.changeState
                                            //       .getChatDataGroup[
                                            //   index]["saveSubGroup"] ==
                                            //       null) {
                                            //     widget.changeState.hideTopic = {
                                            //       "index": index,
                                            //       "indexSubGroup": indexSubGroub1
                                            //     };
                                            //   }
                                            // } else {
                                            //   widget.changeState.showTopic = {
                                            //     "index": index,
                                            //     "indexSubGroup": indexSubGroub
                                            //   };
                                            // }
                                          },
                                          leading: Container(
                                              width: 45,
                                              height: 45,
                                              decoration: BoxDecoration(
                                                  shape: BoxShape.circle,
                                                  color: Colors.grey,
                                                  image: DecorationImage(
                                                      fit: BoxFit.cover,
                                                      image: subGroup2[indexSubGroub2][
                                                                      "picture"] ==
                                                                  null ||
                                                              subGroup2[indexSubGroub2]
                                                                      [
                                                                      "picture"]
                                                                  .trim()
                                                                  .isEmpty
                                                          ? AssetImage(
                                                              "assets/images/groupicon.png",
                                                              package:
                                                                  "palio_lite")
                                                          : LocalImage(
                                                                  subGroup2[indexSubGroub2]
                                                                      ["picture"])
                                                              as ImageProvider))),
                                          title: Row(
                                            mainAxisSize: MainAxisSize.max,
                                            children: <Widget>[
                                              subGroup2[indexSubGroub2]
                                                          ["isVerified"] ==
                                                      "1"
                                                  ? Image.asset(
                                                      "assets/images/ic_verified_flag.png",
                                                      width: 20,
                                                      height: 20,
                                                      package: "palio_lite")
                                                  : subGroup2[indexSubGroub2]
                                                              ["isOfficial"] ==
                                                          "1"
                                                      ? Image.asset(
                                                          "assets/images/ic_official_flag.png",
                                                          width: 20,
                                                          height: 20,
                                                          package: "palio_lite")
                                                      : subGroup2[indexSubGroub2]
                                                                  ["isOpen"] ==
                                                              "1"
                                                          ? Icon(Icons.public,
                                                              size: 20)
                                                          : subGroup2[indexSubGroub2]["isOpen"] ==
                                                                  "2"
                                                              ? Image.asset(
                                                                  "assets/images/ic_educational.png",
                                                                  width: 20,
                                                                  height: 20,
                                                                  color: Colors
                                                                      .black,
                                                                  package:
                                                                      "palio_lite")
                                                              : Icon(Icons.lock, size: 20),
                                              Container(
                                                padding: EdgeInsets.only(
                                                  left: 8.0,
                                                ),
                                                child: Text(
                                                    "${subGroup2[indexSubGroub2]["name"]}",
                                                    overflow:
                                                        TextOverflow.ellipsis),
                                                constraints: BoxConstraints(
                                                    maxWidth: 210),
                                              )
                                            ],
                                          ),
                                          children: [
                                            Container(
                                              margin:
                                                  EdgeInsets.only(left: 28.0),
                                              child: CustomExpansionTile(
                                                leading: Container(
                                                  width: 45,
                                                  height: 45,
                                                  decoration: BoxDecoration(
                                                      shape: BoxShape.circle,
                                                      color: Colors.grey),
                                                  child: Image.asset(
                                                      "assets/images/forum.png",
                                                      package: "palio_lite"),
                                                ),
                                                title: Text(
                                                  "Forum",
                                                  style: TextStyle(
                                                      fontSize: 15,
                                                      color: Colors.black),
                                                ),
                                                children: [
                                                  ListView.builder(
                                                    shrinkWrap: true,
                                                    physics:
                                                        NeverScrollableScrollPhysics(),
                                                    itemCount:
                                                        topicSubGroup3.length,
                                                    itemBuilder: (context,
                                                        indexTopicSubgroup3) {
                                                      return ListTile(
                                                        title: Text(
                                                          "${topicSubGroup3[indexTopicSubgroup3]["name"]}",
                                                          style: TextStyle(
                                                              fontSize: 15,
                                                              color:
                                                                  Colors.black),
                                                        ),
                                                        leading: Container(
                                                          height: 45,
                                                          width: 45,
                                                        ),
                                                        onTap: () {
                                                          onForward(
                                                              data[index], 1,
                                                              dataTopic: topic[
                                                                  indexTopicSubgroup3]);
                                                        },
                                                      );
                                                    },
                                                  )
                                                ],
                                              ),
                                            ),
                                            ListView.builder(
                                              shrinkWrap: true,
                                              physics:
                                                  NeverScrollableScrollPhysics(),
                                              itemCount: subGroup3.length,
                                              itemBuilder:
                                                  (context, indexSubGroub3) {
                                                var topicSubGroup4 =
                                                    subGroup3[indexSubGroub3]
                                                        ["topic"];
                                                // var subGroup3 = subGroup2[indexSubGroub2]["subGroup"];
                                                return Container(
                                                  margin: EdgeInsets.only(
                                                      left: 28.0),
                                                  child: CustomExpansionTile(
                                                    onExpansionChanged: (cond) {
                                                      // if (cond) {
                                                      //   if (widget.changeState
                                                      //       .getChatDataGroup[
                                                      //   index]["saveSubGroup"] ==
                                                      //       null) {
                                                      //     widget.changeState.hideTopic = {
                                                      //       "index": index,
                                                      //       "indexSubGroup": indexSubGroub1
                                                      //     };
                                                      //   }
                                                      // } else {
                                                      //   widget.changeState.showTopic = {
                                                      //     "index": index,
                                                      //     "indexSubGroup": indexSubGroub
                                                      //   };
                                                      // }
                                                    },
                                                    leading: Container(
                                                        width: 45,
                                                        height: 45,
                                                        decoration: BoxDecoration(
                                                            shape:
                                                                BoxShape.circle,
                                                            color: Colors.grey,
                                                            image: DecorationImage(
                                                                fit: BoxFit
                                                                    .cover,
                                                                image: subGroup3[indexSubGroub3]["picture"] ==
                                                                            null ||
                                                                        subGroup3[indexSubGroub3]["picture"]
                                                                            .trim()
                                                                            .isEmpty
                                                                    ? AssetImage(
                                                                        "assets/images/groupicon.png")
                                                                    : LocalImage(subGroup3[indexSubGroub3]
                                                                            [
                                                                            "picture"])
                                                                        as ImageProvider))),
                                                    title: Row(
                                                      mainAxisSize:
                                                          MainAxisSize.max,
                                                      children: <Widget>[
                                                        subGroup3[indexSubGroub3][
                                                                    "isVerified"] ==
                                                                "1"
                                                            ? Image.asset(
                                                                "assets/images/ic_verified_flag.png",
                                                                width: 20,
                                                                height: 20,
                                                                package:
                                                                    "palio_lite")
                                                            : subGroup3[indexSubGroub3]
                                                                        [
                                                                        "isOfficial"] ==
                                                                    "1"
                                                                ? Image.asset(
                                                                    "assets/images/ic_official_flag.png",
                                                                    width: 20,
                                                                    height: 20,
                                                                    package:
                                                                        "palio_lite")
                                                                : subGroup3[indexSubGroub3]
                                                                            [
                                                                            "isOpen"] ==
                                                                        "1"
                                                                    ? Icon(Icons.public,
                                                                        size:
                                                                            20)
                                                                    : subGroup3[indexSubGroub3]["isOpen"] ==
                                                                            "2"
                                                                        ? Image.asset(
                                                                            "assets/images/ic_educational.png",
                                                                            width: 20,
                                                                            height: 20,
                                                                            color: Colors.black,
                                                                            package: "palio_lite")
                                                                        : Icon(Icons.lock, size: 20),
                                                        Container(
                                                          padding:
                                                              EdgeInsets.only(
                                                            left: 8.0,
                                                          ),
                                                          child: Text(
                                                              "${subGroup3[indexSubGroub3]["name"]}",
                                                              overflow:
                                                                  TextOverflow
                                                                      .ellipsis),
                                                          constraints:
                                                              BoxConstraints(
                                                                  maxWidth:
                                                                      210),
                                                        )
                                                      ],
                                                    ),
                                                    children: [
                                                      Container(
                                                        margin: EdgeInsets.only(
                                                            left: 28.0),
                                                        child:
                                                            CustomExpansionTile(
                                                          leading: Container(
                                                            width: 45,
                                                            height: 45,
                                                            decoration:
                                                                BoxDecoration(
                                                                    shape: BoxShape
                                                                        .circle,
                                                                    color: Colors
                                                                        .grey),
                                                            child: Image.asset(
                                                                "assets/images/forum.png",
                                                                package:
                                                                    "palio_lite"),
                                                          ),
                                                          title: Text(
                                                            "Forum",
                                                            style: TextStyle(
                                                                fontSize: 15,
                                                                color: Colors
                                                                    .black),
                                                          ),
                                                          children: [
                                                            ListView.builder(
                                                              shrinkWrap: true,
                                                              physics:
                                                                  NeverScrollableScrollPhysics(),
                                                              itemCount:
                                                                  topicSubGroup4
                                                                      .length,
                                                              itemBuilder: (context,
                                                                  indexTopicSubgroup4) {
                                                                return ListTile(
                                                                  title: Text(
                                                                    "${topicSubGroup4[indexTopicSubgroup4]["name"]}",
                                                                    style: TextStyle(
                                                                        fontSize:
                                                                            15,
                                                                        color: Colors
                                                                            .black),
                                                                  ),
                                                                  leading:
                                                                      Container(
                                                                    height: 45,
                                                                    width: 45,
                                                                  ),
                                                                  onTap: () {
                                                                    onForward(
                                                                        data[
                                                                            index],
                                                                        1,
                                                                        dataTopic:
                                                                            topic[indexTopicSubgroup4]);
                                                                  },
                                                                );
                                                              },
                                                            )
                                                          ],
                                                        ),
                                                      )
                                                    ],
                                                  ),
                                                );
                                              },
                                            )
                                          ],
                                        ),
                                      );
                                    },
                                  )
                                ],
                              ),
                            );
                          },
                        )
                      ],
                    ),
                  );
                },
              )
            ],
          );
        });
  }
}
