import 'package:cached_network_image/cached_network_image.dart';
import 'package:carousel_slider/carousel_slider.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_translate/flutter_translate.dart';
import 'package:http/http.dart' as http;
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/chat/widgets/vertical_personal_timeline.dart';
import 'package:palio_lite/ip_list.dart';
import 'package:video_player/video_player.dart';

class TimelineGrid extends StatefulWidget {
  final data;
  final openFrom;
  final int lengthTimeline;
  final List<Map<dynamic, dynamic>> dataTimeline;
  final ValueChanged<bool> isPlay;

  TimelineGrid(
      {this.data,
      this.openFrom,
      required this.lengthTimeline,
      required this.dataTimeline,
      required this.isPlay});

  @override
  State<StatefulWidget> createState() {
    return TimelineGridView();
  }
}

class TimelineGridView extends State<TimelineGrid> {
  bool isSpeakerOn = false;
  var sumTap = 0;
  bool isPlay = false, isMute = true;
  VideoPlayerController? _controller;

  String httpImage = IPList.imagePost;

  bool loadImage = true;

  Widget videoWidget(String url, String thumb) {
    if (isPlay) {
      _controller?.play();
      if (isMute) {
        _controller?.setVolume(0.0);
      } else {
        _controller?.setVolume(1.0);
      }
    }
    return Container(
      decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(10.0),
          image: !isPlay
              ? DecorationImage(
                  fit: BoxFit.cover,
                  image: loadImage
                      ? NetworkImage(thumb)
                      : AssetImage('assets/images/cuc_scope_image.png',
                          package: "palio_lite") as ImageProvider)
              : DecorationImage(
                  fit: BoxFit.cover,
                  image: AssetImage('assets/images/cuc_scope_image.png',
                      package: "palio_lite"))),
      child: Stack(
        children: <Widget>[
          isPlay ? VideoPlayer(_controller!) : Container(),
          Column(
            mainAxisSize: MainAxisSize.min,
            mainAxisAlignment: MainAxisAlignment.end,
            crossAxisAlignment: CrossAxisAlignment.end,
            children: <Widget>[
              Container(
                margin: EdgeInsets.all(5.0),
                child: Icon(
                  CupertinoIcons.video_camera_solid,
                  color: Colors.white,
                  size: 20.0,
                ),
              ),
              widget.data["is_big"] == 0
                  ? Container()
                  : Expanded(
                      child: Row(
                      mainAxisSize: MainAxisSize.max,
                      mainAxisAlignment: MainAxisAlignment.end,
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: <Widget>[
                        Container(
                          margin: EdgeInsets.all(5.0),
                          child: GestureDetector(
                              onTap: () {
                                setState(() {
                                  isMute = !isMute;
                                  _controller?.setVolume(isMute ? 0.0 : 1.0);
                                });
                              },
                              child: Icon(
                                isMute
                                    ? CupertinoIcons.volume_off
                                    : CupertinoIcons.volume_up,
                                color: Colors.white,
                                size: 20.0,
                              )),
                        )
                      ],
                    ))
            ],
          ),
        ],
      ),
    );
  }

  @override
  void dispose() {
    super.dispose();
    if (_controller != null) _controller?.dispose();
  }

  Future<dynamic> getLoadImage() async {
    var response;
    try {
      response = await http.get(widget.data['thumbnail']);
    } catch (exception) {
      loadImage = false;
    }
    if (response != null) {
      if (response.statusCode != 200) {
        loadImage = false;
      }
    } else {
      loadImage = false;
    }
    return loadImage;
  }

  @override
  void initState() {
    super.initState();
    getLoadImage();
    if (mounted) {
      setState(() {});
    }
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        if (sumTap == 0 && widget.data["media_type"] == 2) {
          if (widget.openFrom == "TIMELINE") {
            _controller =
                VideoPlayerController.network(widget.data['media'][0]);
            isPlay = true;
            _controller?.setLooping(false);
            _controller?.setVolume(0.0);
            _controller?.initialize();
            sumTap++;
            setState(() {});
          } else if (widget.openFrom == "ACCOUNT") {
            widget.isPlay(true);
            sumTap++;
          }
        } else {
          Navigator.push(
              context,
              PageTransition(
                  type: PageTransitionType.fade,
                  child: VerticalPersonalTimeLine(
                    f_pin: widget.data["user_id"],
                    data: widget.data["username"],
                    timeline: widget.dataTimeline,
                    post_id: [],
                    post_id_chosen: widget.data["timeline_id"],
                    openFrom: "GRID",
                  )));
          sumTap = 0;
        }
      },
      child: widget.data["media_type"] == 2
          ? videoWidget(widget.data["media"][0], widget.data["thumbnail"])
          : widget.data["media_type"] == 1
              ? CarouselSlider.builder(
                  options: CarouselOptions(
                    autoPlay: widget.data["image"].length > 1
                        ? widget.data["media_type"] == 1
                            ? true
                            : false
                        : false,
                    height: MediaQuery.of(context).size.height,
                    viewportFraction: 1.0,
                    scrollPhysics: NeverScrollableScrollPhysics(),
                    autoPlayInterval: Duration(seconds: 5),
                  ),
                  itemCount: widget.data["image"].length,
                  itemBuilder: (context, itemIndex, realIdx) => Container(
                    child: Stack(
                      children: <Widget>[
                        Container(
                          // decoration: BoxDecoration(
                          //     // borderRadius: BorderRadius.circular(10.0),
                          //     image: DecorationImage(
                          //         fit: BoxFit.cover,
                          //         image: NetworkImage(
                          //             widget.data["image"][itemIndex]))),
                          child: CachedNetworkImage(
                            imageUrl: widget.data["image"][itemIndex],
                            imageBuilder: (_, imageProvider) => Container(
                              margin: EdgeInsets.symmetric(horizontal: 5.0),
                              decoration: BoxDecoration(
                                  borderRadius: BorderRadius.circular(10.0),
                                  image: DecorationImage(
                                      fit: BoxFit.cover, image: imageProvider)),
                            ),
                            placeholder: (context, url) => Container(
                              margin: EdgeInsets.symmetric(horizontal: 5.0),
                              decoration: BoxDecoration(
                                  borderRadius: BorderRadius.circular(10.0),
                                  image: DecorationImage(
                                      fit: BoxFit.cover,
                                      image: AssetImage(
                                          'assets/images/cuc_scope_image.png',
                                          package: "palio_lite"))),
                            ),
                            errorWidget: (context, url, error) => Container(
                              margin: EdgeInsets.symmetric(horizontal: 5.0),
                              decoration: BoxDecoration(
                                  borderRadius: BorderRadius.circular(10.0),
                                  image: DecorationImage(
                                      fit: BoxFit.cover,
                                      image: AssetImage(
                                          'assets/images/cuc_scope_image.png',
                                          package: "palio_lite"))),
                            ),
                          ),
                        ),
                        Row(
                          mainAxisSize: MainAxisSize.max,
                          mainAxisAlignment: MainAxisAlignment.end,
                          crossAxisAlignment: CrossAxisAlignment.end,
                          children: <Widget>[
                            Container(
                              margin: EdgeInsets.all(5.0),
                              child: widget.data["image"].length > 1
                                  ? Icon(
                                      Icons.photo_library,
                                      color: Colors.white,
                                      size: 20.0,
                                    )
                                  : Container(),
                            )
                          ],
                        ),
                      ],
                    ),
                  ),
                )
              : widget.data["media_type"] == 3
                  ? Container(
                      margin: EdgeInsets.symmetric(horizontal: 5.0),
                      child: widget.data["thumbnail"] != httpImage &&
                              widget.data["thumbnail"].isNotEmpty
                          ? Container(
                              child: Stack(
                                children: <Widget>[
                                  Container(
                                    // decoration: BoxDecoration(
                                    //     // borderRadius: BorderRadius.circular(10.0),
                                    //     image: DecorationImage(
                                    //         fit: BoxFit.cover,
                                    //         image: NetworkImage(
                                    //             widget.data["image"][itemIndex]))),
                                    child: CachedNetworkImage(
                                      imageUrl: widget.data["thumbnail"],
                                      imageBuilder: (_, imageProvider) =>
                                          Container(
                                        margin: EdgeInsets.symmetric(
                                            horizontal: 5.0),
                                        decoration: BoxDecoration(
                                            borderRadius:
                                                BorderRadius.circular(10.0),
                                            image: DecorationImage(
                                                fit: BoxFit.cover,
                                                image: imageProvider)),
                                      ),
                                      placeholder: (context, url) => Container(
                                        margin: EdgeInsets.symmetric(
                                            horizontal: 5.0),
                                        decoration: BoxDecoration(
                                            borderRadius:
                                                BorderRadius.circular(10.0),
                                            image: DecorationImage(
                                                fit: BoxFit.cover,
                                                image: AssetImage(
                                                    'assets/images/cuc_scope_image.png',
                                                    package: "palio_lite"))),
                                      ),
                                      errorWidget: (context, url, error) =>
                                          Container(
                                        margin: EdgeInsets.symmetric(
                                            horizontal: 5.0),
                                        decoration: BoxDecoration(
                                            borderRadius:
                                                BorderRadius.circular(10.0),
                                            image: DecorationImage(
                                                fit: BoxFit.cover,
                                                image: AssetImage(
                                                    'assets/images/cuc_scope_image.png',
                                                    package: "palio_lite"))),
                                      ),
                                    ),
                                  )
                                ],
                              ),
                            )
                          : Stack(
                              alignment: Alignment.center,
                              children: <Widget>[
                                Row(
                                  crossAxisAlignment: CrossAxisAlignment.center,
                                  mainAxisAlignment: MainAxisAlignment.start,
                                  children: <Widget>[
                                    SizedBox(
                                        height: 80.0,
                                        width: 80.0,
                                        child: new IconButton(
                                          padding: EdgeInsets.only(right: 10.0),
                                          icon: new Icon(
                                            Icons.insert_drive_file,
                                            size: 80.0,
                                            color: AppColor.primaryColor,
                                          ),
                                          onPressed: null,
                                        )),
                                    Flexible(
                                      child: Text(
                                        widget.data["document"].length > 0
                                            ? widget.data["document"][0]
                                            : "File",
                                        softWrap: true,
                                      ),
                                    )
                                  ],
                                ),
                              ],
                            ),
                    )
                  : Container(
                      child: Stack(
                        children: <Widget>[
                          Container(
                            child: Container(
                              margin: EdgeInsets.symmetric(horizontal: 5.0),
                              decoration: BoxDecoration(
                                  borderRadius: BorderRadius.circular(10.0),
                                  image: DecorationImage(
                                      fit: BoxFit.cover,
                                      image: AssetImage(
                                          "assets/images/img_quiz_portrait.jpg"))),
                            ),
                          ),
                        ],
                      ),
                    ),
    );
  }
}
