import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:http/http.dart';
import 'package:path/path.dart' as pathS;
import 'package:palio_lite/chat/widgets/get_data_link.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:path_provider/path_provider.dart';

class LinkPreviewer extends StatefulWidget {
  final String link;
  final bool isPreview;

  LinkPreviewer(this.link, this.isPreview);

  @override
  _LinkPreviewerState createState() => _LinkPreviewerState();
}

class _LinkPreviewerState extends State<LinkPreviewer> {
  String pathLocal = "";

  _downloadImage(Uri uri, String path, String imageName) async {
    var response = await get(uri);
    File file = new File(pathS.join(path, imageName));

    file.writeAsBytesSync(response.bodyBytes);
  }

  bool isValidUrl(link) {
    String regexSource =
        "^(https?)://[-a-zA-Z0-9+&@#/%?=~_|!:,.;]*[-a-zA-Z0-9+&@#/%=~_|]";
    final regex = RegExp(regexSource);
    final matches = regex.allMatches(link);
    for (Match match in matches) {
      if (match.start == 0 && match.end == link.length) {
        return true;
      }
    }
    return false;
  }

  void _launchURL(url) async {
    if (url.toLowerCase().contains("www.")) {
      url = url.substring(0, 3).toLowerCase() + url.substring(3, url.length);
    } else if (url.contains("://")) {
      List<String> urll = url.split("://");
      url = urll[0].toLowerCase() + "://" + urll[1];
    }
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      throw 'Could not launch $url';
    }
  }

  Future<Map<dynamic, dynamic>?> _parserWeb(
      String link, int retry, int? idLink) async {
    var web = WebPageParser.getData(link);
    var linkId = idLink;
    if (idLink == null) {
      linkId = DateTime.now().millisecondsSinceEpoch;
    }
    web.whenComplete(() {}).then((value) async {
      if (value != null && value.isNotEmpty) {
        var dataLinkImage = pathS.basename(value["image"]);
        var nameImg;

        if (dataLinkImage.contains(".png")) {
          nameImg = dataLinkImage.split(".png")[0] + ".png";
        } else if (dataLinkImage.contains(".jpg")) {
          nameImg = dataLinkImage.split(".jpg")[0] + ".jpg";
        } else if (dataLinkImage.contains(".jpeg")) {
          nameImg = dataLinkImage.split(".jpeg")[0] + ".jpeg";
        } else if (dataLinkImage.contains(".svg")) {
          nameImg = dataLinkImage.split(".svg")[0] + ".svg";
        }
        String path = pathLocal;
        API.addLink(linkId.toString(), link, jsonEncode(value), 0);
        if (pathLocal.isEmpty) {
          var myPath = await getApplicationDocumentsDirectory();
          path = myPath.path;
        }
        if (nameImg != null) {
          if (!File("$path/$nameImg").existsSync()) {
            print("TIDAK ADA");
            await _downloadImage(value["image"], path, nameImg);
          }
        }
      }
    });
    return web;
  }

  getPath() async {
    var myPath = await getApplicationDocumentsDirectory();
    setState(() {
      pathLocal = myPath.path;
    });
  }

  Widget onPreview() {
    return FutureBuilder(
        future: DBHelper().queryCustom(
            "SELECT * FROM LINK_PREVIEW WHERE link='${widget.link}'"),
        builder: (context, snapsData) {
          if (snapsData.hasData) {
            var data = snapsData.data as List<dynamic>;
            if (data.length != 0) {
              var value = jsonDecode(data[0]["data_link"]);
              var nameImg = pathS.basename(value["image"]);
              if (File("$pathLocal/$nameImg").existsSync()) {
                return Container(
                  decoration: BoxDecoration(
                      color: Colors.grey[300]!.withOpacity(0.7),
                      borderRadius: BorderRadius.circular(8.0)),
                  padding: EdgeInsets.all(5.0),
                  margin: EdgeInsets.all(widget.isPreview ? 0.0 : 5.0),
                  child: Material(
                    color: Colors.transparent,
                    child: InkWell(
                      onTap: () {
                        _launchURL(widget.link);
                      },
                      child: IntrinsicHeight(
                        child: Row(
                          crossAxisAlignment: CrossAxisAlignment.stretch,
                          mainAxisAlignment: MainAxisAlignment.start,
                          children: [
                            Container(
                              width: 80,
                              height: 80,
                              decoration: BoxDecoration(
                                  image: DecorationImage(
                                      fit: BoxFit.cover,
                                      image: FileImage(
                                          File("$pathLocal/$nameImg")))),
                            ),
                            SizedBox(width: 5.0),
                            Expanded(
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                mainAxisAlignment: MainAxisAlignment.start,
                                children: [
                                  Text(
                                    value["title"],
                                    overflow: TextOverflow.ellipsis,
                                    maxLines: 1,
                                    style: TextStyle(color: Colors.black87),
                                  ),
                                  SizedBox(height: 5.0),
                                  Wrap(
                                    children: [
                                      Text(
                                        value["description"],
                                        overflow: TextOverflow.ellipsis,
                                        maxLines: 3,
                                        style: TextStyle(color: Colors.grey),
                                      ),
                                    ],
                                  )
                                ],
                              ),
                            )
                          ],
                        ),
                      ),
                    ),
                  ),
                );
              } else {
                return Container(
                  decoration: BoxDecoration(
                      color: Colors.grey[300]!.withOpacity(0.7),
                      borderRadius: BorderRadius.circular(8.0)),
                  padding: EdgeInsets.all(5.0),
                  margin: EdgeInsets.all(widget.isPreview ? 0.0 : 5.0),
                  child: Material(
                    color: Colors.transparent,
                    child: InkWell(
                      onTap: () {
                        _launchURL(widget.link);
                      },
                      child: IntrinsicHeight(
                        child: Row(
                          crossAxisAlignment: CrossAxisAlignment.stretch,
                          mainAxisAlignment: MainAxisAlignment.start,
                          children: [
                            Container(
                              width: 80,
                              height: 80,
                              decoration: BoxDecoration(
                                  image: DecorationImage(
                                      fit: BoxFit.cover,
                                      image: NetworkImage(value["image"]))),
                            ),
                            SizedBox(width: 5.0),
                            Expanded(
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                mainAxisAlignment: MainAxisAlignment.start,
                                children: [
                                  Text(
                                    value["title"],
                                    overflow: TextOverflow.ellipsis,
                                    maxLines: 1,
                                    style: TextStyle(color: Colors.black87),
                                  ),
                                  SizedBox(height: 5.0),
                                  Wrap(
                                    children: [
                                      Text(
                                        value["description"],
                                        overflow: TextOverflow.ellipsis,
                                        maxLines: 3,
                                        style: TextStyle(color: Colors.grey),
                                      ),
                                    ],
                                  )
                                ],
                              ),
                            )
                          ],
                        ),
                      ),
                    ),
                  ),
                );
              }
            } else {
              return FutureBuilder(
                future: _parserWeb(widget.link, 0, null),
                builder: (context, snaps) {
                  if (snaps.hasData) {
                    var value = snaps.data as Map<dynamic, dynamic>?;
                    if (value != null && value.isNotEmpty) {
                      var dataLinkImage = pathS.basename(value["image"]);
                      var nameImg;

                      if (dataLinkImage.contains(".png")) {
                        nameImg = dataLinkImage.split(".png")[0] + ".png";
                      } else if (dataLinkImage.contains(".jpg")) {
                        nameImg = dataLinkImage.split(".jpg")[0] + ".jpg";
                      } else if (dataLinkImage.contains(".jpeg")) {
                        nameImg = dataLinkImage.split(".jpeg")[0] + ".jpeg";
                      } else if (dataLinkImage.contains(".svg")) {
                        nameImg = dataLinkImage.split(".svg")[0] + ".svg";
                      }

                      if (nameImg != null) {
                        if (File("$pathLocal/$nameImg").existsSync()) {
                          return Container(
                            decoration: BoxDecoration(
                                color: Colors.grey[300]!.withOpacity(0.7),
                                borderRadius: BorderRadius.circular(8.0)),
                            padding: EdgeInsets.all(5.0),
                            margin:
                                EdgeInsets.all(widget.isPreview ? 0.0 : 5.0),
                            child: Material(
                              color: Colors.transparent,
                              child: InkWell(
                                onTap: () {
                                  _launchURL(widget.link);
                                },
                                child: IntrinsicHeight(
                                  child: Row(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.stretch,
                                    mainAxisAlignment: MainAxisAlignment.start,
                                    children: [
                                      Container(
                                        width: 80,
                                        height: 80,
                                        decoration: BoxDecoration(
                                            image: DecorationImage(
                                                fit: BoxFit.cover,
                                                image: FileImage(File(
                                                    "$pathLocal/$nameImg")))),
                                      ),
                                      SizedBox(width: 5.0),
                                      Expanded(
                                        child: Column(
                                          crossAxisAlignment:
                                              CrossAxisAlignment.start,
                                          mainAxisAlignment:
                                              MainAxisAlignment.start,
                                          children: [
                                            Text(
                                              value["title"],
                                              overflow: TextOverflow.ellipsis,
                                              maxLines: 1,
                                              style: TextStyle(
                                                  color: Colors.black87),
                                            ),
                                            SizedBox(height: 5.0),
                                            Wrap(
                                              children: [
                                                Text(
                                                  value["description"],
                                                  overflow:
                                                      TextOverflow.ellipsis,
                                                  maxLines: 3,
                                                  style: TextStyle(
                                                      color: Colors.grey),
                                                ),
                                              ],
                                            )
                                          ],
                                        ),
                                      )
                                    ],
                                  ),
                                ),
                              ),
                            ),
                          );
                        } else {
                          return Container(
                            decoration: BoxDecoration(
                                color: Colors.grey[300]!.withOpacity(0.7),
                                borderRadius: BorderRadius.circular(8.0)),
                            padding: EdgeInsets.all(5.0),
                            margin:
                                EdgeInsets.all(widget.isPreview ? 0.0 : 5.0),
                            child: Material(
                              color: Colors.transparent,
                              child: InkWell(
                                onTap: () {
                                  _launchURL(widget.link);
                                },
                                child: IntrinsicHeight(
                                  child: Row(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.stretch,
                                    mainAxisAlignment: MainAxisAlignment.start,
                                    children: [
                                      Container(
                                        width: 80,
                                        height: 80,
                                        decoration: BoxDecoration(
                                            image: DecorationImage(
                                                fit: BoxFit.cover,
                                                image: NetworkImage(
                                                    value["image"]))),
                                      ),
                                      SizedBox(width: 5.0),
                                      Expanded(
                                        child: Column(
                                          crossAxisAlignment:
                                              CrossAxisAlignment.start,
                                          mainAxisAlignment:
                                              MainAxisAlignment.start,
                                          children: [
                                            Text(
                                              value["title"],
                                              overflow: TextOverflow.ellipsis,
                                              maxLines: 1,
                                              style: TextStyle(
                                                  color: Colors.black87),
                                            ),
                                            SizedBox(height: 5.0),
                                            Wrap(
                                              children: [
                                                Text(
                                                  value["description"],
                                                  overflow:
                                                      TextOverflow.ellipsis,
                                                  maxLines: 3,
                                                  style: TextStyle(
                                                      color: Colors.grey),
                                                ),
                                              ],
                                            )
                                          ],
                                        ),
                                      )
                                    ],
                                  ),
                                ),
                              ),
                            ),
                          );
                        }
                      }
                    }
                  }
                  return widget.isPreview
                      ? Container(
                          width: double.infinity,
                          decoration: BoxDecoration(
                              color: Colors.grey[100],
                              borderRadius: BorderRadius.circular(8.0)),
                          padding: EdgeInsets.only(
                              top: 10.0, bottom: 10.0, left: 5.0, right: 5.0),
                          child: Text(widget.link),
                        )
                      : Container(
                          width: double.infinity,
                          decoration: BoxDecoration(
                              color: Colors.grey[300]!.withOpacity(0.7),
                              borderRadius: BorderRadius.circular(8.0)),
                          padding: EdgeInsets.all(5.0),
                          margin: EdgeInsets.all(5.0),
                          child: IntrinsicHeight(
                            child: Row(
                              crossAxisAlignment: CrossAxisAlignment.stretch,
                              mainAxisAlignment: MainAxisAlignment.start,
                              children: [
                                Container(
                                  width: 80,
                                  height: 80,
                                  decoration: BoxDecoration(
                                      image: DecorationImage(
                                          fit: BoxFit.cover,
                                          image: AssetImage(
                                              "assets/images/cuc_scope_image.png",
                                              package: "palio_lite"))),
                                ),
                                SizedBox(width: 5.0),
                                Expanded(
                                  child: Column(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.center,
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: [
                                      Text(
                                        "Link cannot be loaded...",
                                        overflow: TextOverflow.ellipsis,
                                        maxLines: 1,
                                        style: TextStyle(color: Colors.black87),
                                      )
                                    ],
                                  ),
                                )
                              ],
                            ),
                          ),
                        );
                },
              );
            }
          }
          return widget.isPreview
              ? Container(
                  width: double.infinity,
                  decoration: BoxDecoration(
                      color: Colors.grey[100],
                      borderRadius: BorderRadius.circular(8.0)),
                  padding: EdgeInsets.only(
                      top: 10.0, bottom: 10.0, left: 5.0, right: 5.0),
                  child: Text(widget.link),
                )
              : Container(
                  width: double.infinity,
                  decoration: BoxDecoration(
                      color: Colors.grey[300]!.withOpacity(0.7),
                      borderRadius: BorderRadius.circular(8.0)),
                  padding: EdgeInsets.all(5.0),
                  margin: EdgeInsets.all(5.0),
                  child: IntrinsicHeight(
                    child: Row(
                      crossAxisAlignment: CrossAxisAlignment.stretch,
                      mainAxisAlignment: MainAxisAlignment.start,
                      children: [
                        Container(
                          width: 80,
                          height: 80,
                          decoration: BoxDecoration(
                              image: DecorationImage(
                                  fit: BoxFit.cover,
                                  image: AssetImage(
                                      "assets/images/cuc_scope_image.png",
                                      package: "palio_lite"))),
                        ),
                        SizedBox(width: 5.0),
                        Expanded(
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Text(
                                "Sedang memuat...",
                                overflow: TextOverflow.ellipsis,
                                maxLines: 1,
                                style: TextStyle(color: Colors.black87),
                              )
                            ],
                          ),
                        )
                      ],
                    ),
                  ),
                );
        });
  }

  @override
  void initState() {
    getPath();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return onPreview();
  }
}
