import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_translate/flutter_translate.dart';
import 'package:intl/intl.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/chat/chat_group_screen.dart';
import 'package:palio_lite/chat/widgets/rtf_text_view.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:path/path.dart' as path;
import 'package:shared_preferences/shared_preferences.dart';

class MessageInfo extends StatefulWidget {
  final Chat data;
  final Map<String, String> currentMembers;

  MessageInfo(this.data, this.currentMembers);

  @override
  _MessageInfoState createState() => _MessageInfoState();
}

class _MessageInfoState extends State<MessageInfo> {
  var dataMembers;
  String status = "";

  void getDataMembers() async {
    dataMembers = await DBHelper().queryCustom(
        "SELECT f_pin, status, last_update FROM MESSAGE_STATUS WHERE message_id='${widget.data.messageId}'");
    setState(() {});
  }

  void onMessage() {
    API.onMessage = (dynamic result) async {
      if (mounted) {
        SharedPreferences prefs = await SharedPreferences.getInstance();
        String idMe = prefs.getString("hasLogin")!;
        if (result["A00"] == idMe || result["A01"] == idMe) {
          if (!result["A18"].contains("-2,")) {
            var statusMessage = await DBHelper().minStatusGroup(result["A18"]);
            if (result["A18"] == widget.data.messageId) {
              if (result["D01"] == "1") {
                return;
              } else {
                status = statusMessage[0]["MIN(status)"].toString();
                getDataMembers();
              }
            }
          } else {
            var statusMessage =
                await DBHelper().minStatusGroup(result["A18"].split(",")[1]);
            if (result["A18"].split(",")[1] == widget.data.messageId) {
              status = statusMessage[0]["MIN(status)"].toString();
              getDataMembers();
            }
          }
        }
      }
    };
  }

  @override
  void initState() {
    getDataMembers();
    onMessage();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    Chat chat = widget.data;
    String textPlace = "";
    String imageLocation = "";
    String date = DateTime(chat.date.year, chat.date.month, chat.date.day)
                .difference(DateTime(DateTime.now().year, DateTime.now().month,
                    DateTime.now().day))
                .inDays ==
            0
        ? chat.time!
        : DateFormat('dd MMM yyyy').format(chat.date);
    if (chat.attachmentType == "3") {
      String textDecode = chat.text!;
      if (!chat.isOriginator) {
        textDecode = Uri.decodeQueryComponent(Uri.decodeFull(chat.text!));
      }
      int idx = textDecode.indexOf("(");
      int idx2 = textDecode.indexOf("lat/lng: ");
      var textLatLng = textDecode
          .substring(idx + 1, textDecode.trim().length - 1)
          .split(",");
      imageLocation = "https://maps.googleapis.com/maps/api/staticmap?center=" +
          textLatLng[0] +
          "," +
          textLatLng[1].trimLeft() +
          "&zoom=15&size=" +
          300.toString() +
          "x" +
          150.toString() +
          "&maptype=normal&markers=color:red|" +
          textLatLng[0] +
          "," +
          textLatLng[1].trimLeft() +
          "&key=AIzaSyC79oNmadezP9O_4Q_Ud2bbuJ_L7M--y1s";
      textPlace = textDecode.substring(15, idx2);
    }
    return Scaffold(
      appBar: AppBar(
        title: Text("Message Info", style: TextStyle(color: Colors.white)),
        elevation: 0.0,
        backgroundColor: AppColor.primaryColor,
        automaticallyImplyLeading: true,
      ),
      body: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          Align(
            alignment: Alignment.topRight,
            child: Container(
              margin: EdgeInsets.all(5.0),
              constraints: BoxConstraints(
                  minWidth: MediaQuery.of(context).size.width * 0.1,
                  maxWidth: MediaQuery.of(context).size.width * 0.8),
              decoration: BoxDecoration(
                  boxShadow: widget.data.attachmentType == "11" &&
                          !widget.data.text!.contains("🚫")
                      ? null
                      : [
                          BoxShadow(
                            color: Colors.grey,
                            blurRadius: 3.0,
                          )
                        ],
                  color:
                      chat.attachmentType == "11" && !chat.text!.contains("🚫")
                          ? Colors.transparent
                          : Color(0xFFc4d1e1),
                  borderRadius: BorderRadius.only(
                      bottomLeft: Radius.circular(10.0),
                      bottomRight: Radius.circular(10.0),
                      topRight: Radius.circular(10.0),
                      topLeft: Radius.circular(10.0))),
              child: Container(
                padding: EdgeInsets.all(15),
                child: Row(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    (chat.fileName != null || chat.attachmentType == "3") &&
                            !chat.text!.contains("🚫")
                        ? chat.attachmentType == "6"
                            ? Container(
                                width: 80,
                                height: 80,
                                child: Icon(
                                  Icons.insert_drive_file,
                                  color: Colors.green,
                                  size: 80,
                                ),
                              )
                            : Container(
                                margin: EdgeInsets.only(right: 10.0),
                                width: 80,
                                height: 80,
                                decoration: BoxDecoration(
                                    image: DecorationImage(
                                        fit: BoxFit.cover,
                                        image: chat.attachmentType == "3"
                                            ? NetworkImage(imageLocation)
                                            : chat.attachmentType == "25" ||
                                                    chat.attachmentType == "27"
                                                ? AssetImage(chat.thumbName!,
                                                    package: "palio_lite")
                                                : FileImage(
                                                        File(chat.thumbName!))
                                                    as ImageProvider)),
                                child: chat.attachmentType == "2"
                                    ? Icon(
                                        CupertinoIcons.play_arrow_solid,
                                        color: Colors.white.withOpacity(0.5),
                                        size: 30,
                                      )
                                    : Container(),
                              )
                        : Container(),
                    chat.attachmentType == "11" && !chat.text!.contains("🚫")
                        ? Container(
                            height: 72,
                            width: 72,
                            child: Image.asset("assets/stickers/$chat.text"),
                          )
                        : Container(
                            constraints: BoxConstraints(
                                maxWidth: MediaQuery.of(context).size.width *
                                    (chat.fileName != null ||
                                            chat.attachmentType == "3"
                                        ? 0.45
                                        : 0.68)),
                            child: Wrap(
                              crossAxisAlignment: WrapCrossAlignment.start,
                              children: <Widget>[
                                RTFText(
                                    chat.attachmentType == "3"
                                        ? textPlace
                                        : chat.attachmentType == "6" &&
                                                chat.text == ""
                                            ? path.basename(chat.fileName!)
                                            : chat.text!,
                                    bLink: true)
                              ],
                            ),
                          ),
                  ],
                ),
              ),
            ),
          ),
          Container(
            margin: EdgeInsets.only(right: 5.0),
            child: IntrinsicHeight(
              child: Row(
                crossAxisAlignment: CrossAxisAlignment.stretch,
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  Text(date,
                      style: TextStyle(color: Colors.grey, fontSize: 12.0)),
                  status != null
                      ? status == "8"
                          ? Container(
                              width: 16.0,
                              height: 16.0,
                              child: Image.asset(
                                  "assets/images/message_status_ack.png"),
                            )
                          : Icon(
                              status == "1" || status == "2"
                                  ? Icons.check
                                  : Icons.done_all,
                              size: 16.0,
                              color: status == "4" ? Colors.blue : Colors.grey)
                      : chat.statusRead == "8"
                          ? Container(
                              width: 16.0,
                              height: 16.0,
                              child: Image.asset(
                                  "assets/images/message_status_ack.png"),
                            )
                          : Icon(
                              chat.statusRead == "1" || chat.statusRead == "2"
                                  ? Icons.check
                                  : Icons.done_all,
                              size: 16.0,
                              color: chat.statusRead == "4"
                                  ? Colors.blue
                                  : Colors.grey),
                ],
              ),
            ),
          ),
          Container(
            margin: EdgeInsets.all(5.0),
            child: ListView.builder(
              shrinkWrap: true,
              itemBuilder: (context, i) {
                if (dataMembers != null &&
                    widget.currentMembers
                        .containsKey("@${dataMembers[i]["f_pin"]}")) {
                  return FutureBuilder(
                    future: DBHelper().queryCustom(
                        "SELECT first_name, last_name, image_id FROM BUDDY WHERE f_pin='${dataMembers[i]["f_pin"]}'"),
                    builder: (context, snaps) {
                      if (snaps.hasData) {
                        var value = snaps.data as List<dynamic>;
                        Widget image = Container(
                          width: 40,
                          height: 40,
                          decoration: BoxDecoration(
                              shape: BoxShape.circle,
                              color: Colors.grey,
                              image: DecorationImage(
                                  fit: BoxFit.cover,
                                  image: AssetImage(
                                      "assets/images/default_pp.png",
                                      package: "palio_lite"))),
                        );
                        if (value[0]["image_id"].trim().isNotEmpty) {
                          image = Container(
                            width: 40,
                            height: 40,
                            decoration: BoxDecoration(
                                shape: BoxShape.circle,
                                color: Colors.grey,
                                image: DecorationImage(
                                    fit: BoxFit.cover,
                                    image: LocalImage(value[0]["image_id"]))),
                          );
                        }
                        var formatTime = new DateFormat('Hm');
                        DateTime dateTime = DateTime.fromMillisecondsSinceEpoch(
                            dataMembers[i]["last_update"]);
                        String dateMember = DateTime(dateTime.year,
                                        dateTime.month, dateTime.day)
                                    .difference(DateTime(
                                        DateTime.now().year,
                                        DateTime.now().month,
                                        DateTime.now().day))
                                    .inDays ==
                                0
                            ? formatTime.format(dateTime)
                            : DateFormat('dd MMM yyyy').format(dateTime);
                        return ListTile(
                          contentPadding: EdgeInsets.all(0.0),
                          leading: image,
                          title: Text((value[0]["first_name"] +
                                  " " +
                                  value[0]["last_name"])
                              .trim()),
                          subtitle: Text(dateMember,
                              style: TextStyle(fontSize: 12.0)),
                          trailing: dataMembers[i]["status"].toString() == "8"
                              ? Container(
                                  width: 16.0,
                                  height: 16.0,
                                  child: Image.asset(
                                      "assets/images/message_status_ack.png"),
                                )
                              : Icon(
                                  dataMembers[i]["status"].toString() == "1" ||
                                          dataMembers[i]["status"].toString() ==
                                              "2"
                                      ? Icons.check
                                      : Icons.done_all,
                                  size: 16.0,
                                  color:
                                      dataMembers[i]["status"].toString() == "4"
                                          ? Colors.blue
                                          : Colors.grey),
                        );
                      }
                      return Container();
                    },
                  );
                }
                return Container();
              },
              itemCount: widget.currentMembers.length - 1,
            ),
          )
        ],
      ),
    );
  }
}
