import 'dart:math' as math;
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

///
///  create by zmtzawqlp on 2019/8/3
///
const double _kHandleSize = 22.0;

// Padding between the toolbar and the anchor.
const double _kToolbarContentDistanceBelow = _kHandleSize - 2.0;
const double _kToolbarContentDistance = 8.0;

/// Android Material styled text selection controls.
class MyTextSelectionControls extends TextSelectionControls {
  /// Returns the size of the Material handle.
  @override
  Size getHandleSize(double textLineHeight) =>
      const Size(_kHandleSize, _kHandleSize);

  /// Builder for material-style copy/paste text selection toolbar.
  @override
  Widget buildToolbar(
    BuildContext context,
    Rect globalEditableRegion,
    double textLineHeight,
    Offset selectionMidpoint,
    List<TextSelectionPoint> endpoints,
    TextSelectionDelegate delegate,
    ClipboardStatusNotifier clipboardStatus,
    Offset? lastSecondaryTapDownPosition,
  ) {
    return _TextSelectionControlsToolbar(
      globalEditableRegion: globalEditableRegion,
      textLineHeight: textLineHeight,
      selectionMidpoint: selectionMidpoint,
      endpoints: endpoints,
      delegate: delegate,
      clipboardStatus: clipboardStatus,
      handleCut: canCut(delegate) ? () => handleCut(delegate) : null,
      handleCopy: canCopy(delegate)
          ? () => handleCopy(delegate, clipboardStatus)
          : null,
      handlePaste: canPaste(delegate) ? () => handlePaste(delegate) : null,
      handleSelectAll:
          canSelectAll(delegate) ? () => handleSelectAll(delegate) : null,
      handleBold: canCopy(delegate)
          ? () {
              final TextEditingValue value = delegate.textEditingValue;
              String textInSelection = value.selection.textInside(value.text);
              String textAfterSelection = value.selection.textAfter(value.text);
              String textBeforeSelection =
                  value.selection.textBefore(value.text);
              String starBefore = "";
              String starAfter = "";
              if (textAfterSelection.isEmpty) {
                starAfter = "* ";
              } else if (textAfterSelection.isNotEmpty) {
                if (textAfterSelection[0] == " " ||
                    textAfterSelection[0] == "\n" ||
                    textAfterSelection[0] == "*" ||
                    textAfterSelection[0] == "^" ||
                    textAfterSelection[0] == "_" ||
                    textAfterSelection[0] == "~") {
                  starAfter = "*";
                } else if (textAfterSelection[0].isNotEmpty) {
                  starAfter = "* ";
                }
              }
              if (textBeforeSelection.isEmpty) {
                starBefore = "*";
              } else if (textBeforeSelection.isNotEmpty) {
                if (textBeforeSelection[textBeforeSelection.length - 1] ==
                        " " ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "\n" ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "*" ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "^" ||
                    textBeforeSelection[textBeforeSelection.length - 1] ==
                        "_" ||
                    textBeforeSelection[textBeforeSelection.length - 1] ==
                        "~") {
                  starBefore = "*";
                } else if (textBeforeSelection[textBeforeSelection.length - 1]
                    .isNotEmpty) {
                  starBefore = " *";
                }
              }
              String textBold = starBefore + textInSelection + starAfter;
              delegate.textEditingValue = TextEditingValue(
                text: textBeforeSelection + textBold + textAfterSelection,
                selection: TextSelection.collapsed(
                    offset: (value.selection.end +
                        starBefore.length +
                        starAfter.length)),
              );
              delegate
                  .bringIntoView(delegate.textEditingValue.selection.extent);
              delegate.hideToolbar();
            }
          : null,
      handleItalic: canCopy(delegate)
          ? () {
              final TextEditingValue value = delegate.textEditingValue;
              String textInSelection = value.selection.textInside(value.text);
              String textAfterSelection = value.selection.textAfter(value.text);
              String textBeforeSelection =
                  value.selection.textBefore(value.text);
              String starBefore = "";
              String starAfter = "";
              if (textAfterSelection.isEmpty) {
                starAfter = "_ ";
              } else if (textAfterSelection.isNotEmpty) {
                if (textAfterSelection[0] == " " ||
                    textAfterSelection[0] == "\n" ||
                    textAfterSelection[0] == "*" ||
                    textAfterSelection[0] == "^" ||
                    textAfterSelection[0] == "_" ||
                    textAfterSelection[0] == "~") {
                  starAfter = "_";
                } else if (textAfterSelection[0].isNotEmpty) {
                  starAfter = "_ ";
                }
              }
              if (textBeforeSelection.isEmpty) {
                starBefore = "_";
              } else if (textBeforeSelection.isNotEmpty) {
                if (textBeforeSelection[textBeforeSelection.length - 1] ==
                        " " ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "\n" ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "*" ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "^" ||
                    textBeforeSelection[textBeforeSelection.length - 1] ==
                        "_" ||
                    textBeforeSelection[textBeforeSelection.length - 1] ==
                        "~") {
                  starBefore = "_";
                } else if (textBeforeSelection[textBeforeSelection.length - 1]
                    .isNotEmpty) {
                  starBefore = " _";
                }
              }
              String textItalic = starBefore + textInSelection + starAfter;
              delegate.textEditingValue = TextEditingValue(
                text: textBeforeSelection + textItalic + textAfterSelection,
                selection: TextSelection.collapsed(
                    offset: (value.selection.end +
                        starBefore.length +
                        starAfter.length)),
              );
              delegate
                  .bringIntoView(delegate.textEditingValue.selection.extent);
              delegate.hideToolbar();
            }
          : null,
      handleUnderline: canCopy(delegate)
          ? () {
              final TextEditingValue value = delegate.textEditingValue;
              String textInSelection = value.selection.textInside(value.text);
              String textAfterSelection = value.selection.textAfter(value.text);
              String textBeforeSelection =
                  value.selection.textBefore(value.text);
              String starBefore = "";
              String starAfter = "";
              if (textAfterSelection.isEmpty) {
                starAfter = "^ ";
              } else if (textAfterSelection.isNotEmpty) {
                if (textAfterSelection[0] == " " ||
                    textAfterSelection[0] == "\n" ||
                    textAfterSelection[0] == "*" ||
                    textAfterSelection[0] == "^" ||
                    textAfterSelection[0] == "_" ||
                    textAfterSelection[0] == "~") {
                  starAfter = "^";
                } else if (textAfterSelection[0].isNotEmpty) {
                  starAfter = "^ ";
                }
              }
              if (textBeforeSelection.isEmpty) {
                starBefore = "^";
              } else if (textBeforeSelection.isNotEmpty) {
                if (textBeforeSelection[textBeforeSelection.length - 1] ==
                        " " ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "\n" ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "*" ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "^" ||
                    textBeforeSelection[textBeforeSelection.length - 1] ==
                        "_" ||
                    textBeforeSelection[textBeforeSelection.length - 1] ==
                        "~") {
                  starBefore = "^";
                } else if (textBeforeSelection[textBeforeSelection.length - 1]
                    .isNotEmpty) {
                  starBefore = " ^";
                }
              }
              String textUnderline = starBefore + textInSelection + starAfter;
              delegate.textEditingValue = TextEditingValue(
                text: textBeforeSelection + textUnderline + textAfterSelection,
                selection: TextSelection.collapsed(
                    offset: (value.selection.end +
                        starBefore.length +
                        starAfter.length)),
              );
              delegate
                  .bringIntoView(delegate.textEditingValue.selection.extent);
              delegate.hideToolbar();
            }
          : null,
      handleStriketrough: canCopy(delegate)
          ? () {
              final TextEditingValue value = delegate.textEditingValue;
              String textInSelection = value.selection.textInside(value.text);
              String textAfterSelection = value.selection.textAfter(value.text);
              String textBeforeSelection =
                  value.selection.textBefore(value.text);
              String starBefore = "";
              String starAfter = "";
              if (textAfterSelection.isEmpty) {
                starAfter = "~ ";
              } else if (textAfterSelection.isNotEmpty) {
                if (textAfterSelection[0] == " " ||
                    textAfterSelection[0] == "\n" ||
                    textAfterSelection[0] == "*" ||
                    textAfterSelection[0] == "^" ||
                    textAfterSelection[0] == "_" ||
                    textAfterSelection[0] == "~") {
                  starAfter = "~";
                } else if (textAfterSelection[0].isNotEmpty) {
                  starAfter = "~ ";
                }
              }
              if (textBeforeSelection.isEmpty) {
                starBefore = "~";
              } else if (textBeforeSelection.isNotEmpty) {
                if (textBeforeSelection[textBeforeSelection.length - 1] ==
                        " " ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "\n" ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "*" ||
                    textBeforeSelection[
                            textBeforeSelection.length - 1] ==
                        "^" ||
                    textBeforeSelection[textBeforeSelection.length - 1] ==
                        "_" ||
                    textBeforeSelection[textBeforeSelection.length - 1] ==
                        "~") {
                  starBefore = "~";
                } else if (textBeforeSelection[textBeforeSelection.length - 1]
                    .isNotEmpty) {
                  starBefore = " ~";
                }
              }
              String textStrike = starBefore + textInSelection + starAfter;
              delegate.textEditingValue = TextEditingValue(
                text: textBeforeSelection + textStrike + textAfterSelection,
                selection: TextSelection.collapsed(
                    offset: (value.selection.end +
                        starBefore.length +
                        starAfter.length)),
              );
              delegate
                  .bringIntoView(delegate.textEditingValue.selection.extent);
              delegate.hideToolbar();
            }
          : null,
    );
  }

  /// Builder for material-style text selection handles.
  @override
  Widget buildHandle(
      BuildContext context, TextSelectionHandleType type, double textHeight) {
    final Widget handle = SizedBox(
      width: _kHandleSize,
      height: _kHandleSize,
      child: Image.asset(
        'assets/images/41.png',
        package: "palio_lite",
      ),
    );

    // [handle] is a circle, with a rectangle in the top left quadrant of that
    // circle (an onion pointing to 10:30). We rotate [handle] to point
    // straight up or up-right depending on the handle type.
    switch (type) {
      case TextSelectionHandleType.left: // points up-right
        return Transform.rotate(
          angle: math.pi / 4.0,
          child: handle,
        );
      case TextSelectionHandleType.right: // points up-left
        return Transform.rotate(
          angle: -math.pi / 4.0,
          child: handle,
        );
      case TextSelectionHandleType.collapsed: // points up
        return handle;
    }
    return Container();
  }

  /// Gets anchor for material-style text selection handles.
  ///
  /// See [TextSelectionControls.getHandleAnchor].
  @override
  Offset getHandleAnchor(TextSelectionHandleType type, double textLineHeight) {
    switch (type) {
      case TextSelectionHandleType.left:
        return const Offset(_kHandleSize, 0);
      case TextSelectionHandleType.right:
        return Offset.zero;
      default:
        return const Offset(_kHandleSize / 2, -4);
    }
  }

  @override
  bool canSelectAll(TextSelectionDelegate delegate) {
    // Android allows SelectAll when selection is not collapsed, unless
    // everything has already been selected.
    final TextEditingValue value = delegate.textEditingValue;
    return delegate.selectAllEnabled &&
        value.text.isNotEmpty &&
        !(value.selection.start == 0 &&
            value.selection.end == value.text.length);
  }
}

// The label and callback for the available default text selection menu buttons.
class _TextSelectionToolbarItemData {
  const _TextSelectionToolbarItemData({
    required this.label,
    required this.onPressed,
  });

  final String label;
  final VoidCallback onPressed;
}

// The highest level toolbar widget, built directly by buildToolbar.
class _TextSelectionControlsToolbar extends StatefulWidget {
  const _TextSelectionControlsToolbar({
    Key? key,
    required this.clipboardStatus,
    required this.delegate,
    required this.endpoints,
    required this.globalEditableRegion,
    required this.handleCut,
    required this.handleCopy,
    required this.handlePaste,
    required this.handleSelectAll,
    required this.handleBold,
    required this.handleUnderline,
    required this.handleStriketrough,
    required this.handleItalic,
    required this.selectionMidpoint,
    required this.textLineHeight,
  }) : super(key: key);

  final ClipboardStatusNotifier clipboardStatus;
  final TextSelectionDelegate delegate;
  final List<TextSelectionPoint> endpoints;
  final Rect globalEditableRegion;
  final VoidCallback? handleCut;
  final VoidCallback? handleCopy;
  final VoidCallback? handlePaste;
  final VoidCallback? handleSelectAll;
  final VoidCallback? handleBold;
  final VoidCallback? handleUnderline;
  final VoidCallback? handleStriketrough;
  final VoidCallback? handleItalic;

  final Offset selectionMidpoint;
  final double textLineHeight;

  @override
  _TextSelectionControlsToolbarState createState() =>
      _TextSelectionControlsToolbarState();
}

class _TextSelectionControlsToolbarState
    extends State<_TextSelectionControlsToolbar> with TickerProviderStateMixin {
  void _onChangedClipboardStatus() {
    setState(() {
      // Inform the widget that the value of clipboardStatus has changed.
    });
  }

  @override
  void initState() {
    super.initState();
    widget.clipboardStatus.addListener(_onChangedClipboardStatus);
    widget.clipboardStatus.update();
  }

  @override
  void didUpdateWidget(_TextSelectionControlsToolbar oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.clipboardStatus != oldWidget.clipboardStatus) {
      widget.clipboardStatus.addListener(_onChangedClipboardStatus);
      oldWidget.clipboardStatus.removeListener(_onChangedClipboardStatus);
    }
    widget.clipboardStatus.update();
  }

  @override
  void dispose() {
    super.dispose();
    // When used in an Overlay, it can happen that this is disposed after its
    // creator has already disposed _clipboardStatus.
    if (!widget.clipboardStatus.disposed) {
      widget.clipboardStatus.removeListener(_onChangedClipboardStatus);
    }
  }

  @override
  Widget build(BuildContext context) {
    // If there are no buttons to be shown, don't render anything.
    if (widget.handleCut == null &&
        widget.handleCopy == null &&
        widget.handlePaste == null &&
        widget.handleSelectAll == null) {
      return const SizedBox.shrink();
    }
    // If the paste button is desired, don't render anything until the state of
    // the clipboard is known, since it's used to determine if paste is shown.
    if (widget.handlePaste != null &&
        widget.clipboardStatus.value == ClipboardStatus.unknown) {
      return const SizedBox.shrink();
    }

    // Calculate the positioning of the menu. It is placed above the selection
    // if there is enough room, or otherwise below.
    final TextSelectionPoint startTextSelectionPoint = widget.endpoints[0];
    final TextSelectionPoint endTextSelectionPoint =
        widget.endpoints.length > 1 ? widget.endpoints[1] : widget.endpoints[0];
    final Offset anchorAbove = Offset(
        widget.globalEditableRegion.left + widget.selectionMidpoint.dx,
        widget.globalEditableRegion.top +
            startTextSelectionPoint.point.dy -
            widget.textLineHeight -
            _kToolbarContentDistance);
    final Offset anchorBelow = Offset(
      widget.globalEditableRegion.left + widget.selectionMidpoint.dx,
      widget.globalEditableRegion.top +
          endTextSelectionPoint.point.dy +
          _kToolbarContentDistanceBelow,
    );

    // Determine which buttons will appear so that the order and total number is
    // known. A button's position in the menu can slightly affect its
    // appearance.
    assert(debugCheckHasMaterialLocalizations(context));
    final MaterialLocalizations localizations =
        MaterialLocalizations.of(context);
    final List<_TextSelectionToolbarItemData> itemDatas =
        <_TextSelectionToolbarItemData>[
      if (widget.handleCut != null)
        _TextSelectionToolbarItemData(
          label: localizations.cutButtonLabel,
          onPressed: widget.handleCut!,
        ),
      if (widget.handleCopy != null)
        _TextSelectionToolbarItemData(
          label: localizations.copyButtonLabel,
          onPressed: widget.handleCopy!,
        ),
      if (widget.handlePaste != null &&
          widget.clipboardStatus.value == ClipboardStatus.pasteable)
        _TextSelectionToolbarItemData(
          label: localizations.pasteButtonLabel,
          onPressed: widget.handlePaste!,
        ),
      if (widget.handleSelectAll != null)
        _TextSelectionToolbarItemData(
          label: localizations.selectAllButtonLabel,
          onPressed: widget.handleSelectAll!,
        ),
      if (widget.handleBold != null)
        _TextSelectionToolbarItemData(
          label: "Bold",
          onPressed: widget.handleBold!,
        ),
      if (widget.handleItalic != null)
        _TextSelectionToolbarItemData(
          label: "Italic",
          onPressed: widget.handleItalic!,
        ),
      if (widget.handleUnderline != null)
        _TextSelectionToolbarItemData(
          label: "Underline",
          onPressed: widget.handleUnderline!,
        ),
      if (widget.handleStriketrough != null)
        _TextSelectionToolbarItemData(
          label: "Strikethrough",
          onPressed: widget.handleStriketrough!,
        ),
    ];

    // If there is no option available, build an empty widget.
    if (itemDatas.isEmpty) {
      return const SizedBox(width: 0.0, height: 0.0);
    }

    return TextSelectionToolbar(
      anchorAbove: anchorAbove,
      anchorBelow: anchorBelow,
      children: itemDatas
          .asMap()
          .entries
          .map((MapEntry<int, _TextSelectionToolbarItemData> entry) {
        return TextSelectionToolbarTextButton(
          padding: TextSelectionToolbarTextButton.getPadding(
              entry.key, itemDatas.length),
          onPressed: entry.value.onPressed,
          child: entry.value.label == 'like'
              ? const Icon(Icons.favorite)
              : Text(entry.value.label),
        );
      }).toList(),
    );
  }
}
