import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:open_file/open_file.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/app_color.dart';
import 'package:palio_lite/call/audio_call.dart';
import 'package:palio_lite/call/video_call.dart';
import 'package:palio_lite/chat/chat_group_screen.dart';
import 'package:palio_lite/chat/chat_screen.dart';
import 'package:palio_lite/chat/initiate_broadcast.dart';
import 'package:palio_lite/chat/widgets/rtf_text_view.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:path_provider/path_provider.dart';
import 'package:rxdart/subjects.dart';

import '../../request_call_center.dart';
import '../../shared_obj.dart';

class Notifications {
  static final FlutterLocalNotificationsPlugin flutterLocalNotificationsPlugin =
      FlutterLocalNotificationsPlugin();

  static final BehaviorSubject<ReceivedNotification>
      didReceiveLocalNotificationSubject =
      BehaviorSubject<ReceivedNotification>();

  static final BehaviorSubject<String> selectNotificationSubject =
      BehaviorSubject<String>();

  static String? selectedNotificationPayload;
  static final NOTIFICATION_CALL_CENTER_ID = 22;

  static void initNotifications() async {
    final IOSInitializationSettings initializationSettingsIOS =
        IOSInitializationSettings(
            requestAlertPermission: true,
            requestBadgePermission: true,
            requestSoundPermission: true,
            onDidReceiveLocalNotification:
                (int? id, String? title, String? body, String? payload) async {
              didReceiveLocalNotificationSubject.add(ReceivedNotification(
                  id: id!, title: title!, body: body!, payload: payload!));
            });
    final InitializationSettings initializationSettings =
        InitializationSettings(iOS: initializationSettingsIOS);
    await flutterLocalNotificationsPlugin.initialize(initializationSettings,
        onSelectNotification: (String? payload) async {
      selectedNotificationPayload = payload;
      selectNotificationSubject.add(payload!);
    });
  }

  static void _requestPermissions() {
    flutterLocalNotificationsPlugin
        .resolvePlatformSpecificImplementation<
            IOSFlutterLocalNotificationsPlugin>()
        ?.requestPermissions(
          alert: true,
          badge: true,
          sound: true,
        );
  }

  static void showDialogBroadcast(var result, BuildContext context) async {
    API.updateCounter(result["A00"]);
    var resultQuery = await DBHelper().queryCustom(
        "SELECT first_name, last_name FROM BUDDY WHERE f_pin='${result["A00"]}'");
    String name =
        resultQuery[0]["first_name"] + " " + resultQuery[0]["last_name"];
    var myPath = await getApplicationDocumentsDirectory();
    bool download = false;
    showDialog(
        barrierDismissible: false,
        context: context,
        builder: (_) {
          return CupertinoAlertDialog(
            title: Stack(
              children: [
                Column(
                  crossAxisAlignment: CrossAxisAlignment.center,
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    Text(name),
                    Text(result["A07"]),
                    result["A74"] != null && result["A74"].isEmpty
                        ? Container()
                        : Column(
                            mainAxisSize: MainAxisSize.min,
                            mainAxisAlignment: MainAxisAlignment.center,
                            crossAxisAlignment: CrossAxisAlignment.center,
                            children: [
                              Text("-------------------------"),
                              GestureDetector(
                                onTap: () {
                                  if (result["A47"] != null) {
                                    API.onDownloadProgress =
                                        (double progress, String fileName) {
                                      if (fileName == result["A47"]) {
                                        if (progress == 100.0) {
                                          OpenFile.open(myPath.path +
                                              "/" +
                                              result["A47"]);
                                        }
                                      }
                                    };
                                    if (File(myPath.path + "/" + result["A47"])
                                        .existsSync()) {
                                      OpenFile.open(
                                          myPath.path + "/" + result["A47"]);
                                    } else {
                                      if (!download) {
                                        API.download(result["A47"]);
                                        download = true;
                                        Fluttertoast.showToast(
                                            msg: "Downloading file...");
                                      } else {
                                        Fluttertoast.showToast(
                                            msg: "Download is running...");
                                      }
                                    }
                                  } else if (result["BN"] != null) {
                                    API.onDownloadProgress =
                                        (double progress, String fileName) {
                                      if (fileName == result["BN"]) {
                                        if (progress == 100.0) {
                                          OpenFile.open(
                                              myPath.path + "/" + result["BN"]);
                                        }
                                      }
                                    };
                                    if (File(myPath.path + "/" + result["BN"])
                                        .existsSync()) {
                                      OpenFile.open(
                                          myPath.path + "/" + result["BN"]);
                                    } else {
                                      if (!download) {
                                        API.download(result["BN"]);
                                        download = true;
                                        Fluttertoast.showToast(
                                            msg: "Downloading file...");
                                      } else {
                                        Fluttertoast.showToast(
                                            msg: "Download is running...");
                                      }
                                    }
                                  }
                                },
                                child: Container(
                                  height: 300,
                                  width: MediaQuery.of(context).size.width,
                                  decoration: result["A74"] == null
                                      ? BoxDecoration()
                                      : BoxDecoration(
                                          image: DecorationImage(
                                              fit: result["A47"] != null
                                                  ? BoxFit.cover
                                                  : BoxFit.fill,
                                              image: AssetImage(
                                                  "${myPath.path}/${result["A74"]}"))),
                                  child: result["A47"] != null
                                      ? Container(
                                          alignment: Alignment.center,
                                          child: Icon(Icons.play_circle_outline,
                                              color:
                                                  Colors.white.withOpacity(0.5),
                                              size: 50),
                                        )
                                      : result["BN"] != null
                                          ? result["A74"] == null
                                              ? new Icon(
                                                  Icons.insert_drive_file,
                                                  size: 250.0,
                                                  color: AppColor.primaryColor,
                                                )
                                              : Container()
                                          : Container(),
                                ),
                              )
                            ],
                          )
                  ],
                ),
              ],
            ),
            actions: [
              Container(
                width: 50,
                height: 50,
                decoration: BoxDecoration(
                  shape: BoxShape.circle,
                  color: Colors.black.withOpacity(0.1),
                ),
                child: Material(
                  color: Colors.transparent,
                  child: InkWell(
                    onTap: () {
                      Navigator.pop(context);
                    },
                    child: Icon(Icons.clear),
                  ),
                ),
              )
            ],
          );
        });
  }

  static void _configureSelectNotificationSubject(BuildContext context) {
    selectNotificationSubject.stream.listen((String payload) async {
      var data = jsonDecode(payload);
      if (data["type_notification"] == 0) {
        if (data["type_message"] == "3") {
          API.updateCounter(data["data"]["A00"]);
          if (data["data"]["BR"] != null) {
            bool isContactCenter = data["is_call_center"] == "1";
            var queryResult;
            if (isContactCenter) {
              queryResult = await DBHelper().queryCustom(
                  "SELECT first_name, last_name,f_pin, offline_mode, image_id FROM BUDDY where official_account = '1'");
            } else {
              queryResult = await DBHelper().queryCustom(
                  "SELECT first_name, last_name,f_pin, offline_mode, image_id FROM BUDDY WHERE f_pin='${data["data"]["A00"]}'");
            }
            var realName =
                "${queryResult[0]["first_name"]}${queryResult[0]["last_name"].isNotEmpty ? " " + queryResult[0]["last_name"] : ""}";
            print("route current: ${ModalRoute.of(context)!.settings.name}");
            if ((ModalRoute.of(context)!.settings.name != "/contactCenter") ||
                data["level"] == 1) {
              isContactCenter
                  ? Navigator.pushNamed(context, '/contactCenter', arguments: {
                      'fullName': realName,
                      'image': queryResult[0]["image_id"],
                      'identifier': data["data"]["A00"],
                      'refresh': true,
                      'isContactCenter': true,
                      'data': jsonEncode(data["data"]),
                      'contactCenterHistoryId': data["call_center_id"],
                    })
                  : Navigator.push(
                      context,
                      PageTransition(
                          type: PageTransitionType.fade,
                          child: ChatPage(
                            fullName: realName,
                            image: queryResult[0]["image_id"],
                            identifier: data["data"]["A00"],
                            refresh: true,
                            isContactCenter: false,
                            contactCenterHistoryId: data["call_center_id"],
                            data: jsonEncode(data["data"]),
                          )));
            } else if (ModalRoute.of(context)!.settings.name !=
                "/contactCenter") {
              isContactCenter
                  ? Navigator.pushReplacementNamed(context, '/contactCenter',
                      arguments: {
                          'fullName': realName,
                          'image': queryResult[0]["image_id"],
                          'identifier': data["data"]["A00"],
                          'refresh': true,
                          'isContactCenter': true,
                          'data': jsonEncode(data["data"]),
                          'contactCenterHistoryId': data["call_center_id"],
                        })
                  : Navigator.pushReplacement(
                      context,
                      PageTransition(
                          type: PageTransitionType.fade,
                          child: ChatPage(
                            fullName: realName,
                            image: queryResult[0]["image_id"],
                            identifier: data["data"]["A00"],
                            refresh: true,
                            isContactCenter: false,
                            contactCenterHistoryId: data["call_center_id"],
                            data: jsonEncode(data["data"]),
                          )));
            }
            if (isContactCenter) {
              const IOSNotificationDetails iOSPlatformChannelSpecifics =
                  IOSNotificationDetails(
                      presentAlert: false,
                      presentBadge: false,
                      presentSound: false);
              const NotificationDetails platformChannelSpecifics =
                  NotificationDetails(iOS: iOSPlatformChannelSpecifics);
              var title = "Interaction with Call Center In Progress";
              var body = "Click to go to the Call Center Screen";
              await flutterLocalNotificationsPlugin.show(
                  NOTIFICATION_CALL_CENTER_ID,
                  title,
                  body,
                  platformChannelSpecifics,
                  payload: payload);
            }
          } else if (data["data"]["A149"] == "27" ||
              data["data"]["A149"] == "25") {
            Navigator.push(
                context,
                PageTransition(
                    type: PageTransitionType.fade,
                    child: ChatPage(
                      fullName: "Bot",
                      image: "",
                      identifier: data["data"]["A00"],
                      refresh: true,
                    )));
          } else {
            showDialogBroadcast(data["data"], context);
          }
        } else {
          String idGroup = data["data"]["A01"];
          String topicGroup = data["data"]["BA"];
          if (topicGroup != null) {
            API.updateCounter(topicGroup);
          } else {
            API.updateCounter(idGroup);
          }
          if (data["level"] == 1) {
            if (topicGroup == null) {
              topicGroup = "Lounge";
            } else {
              var resultQueryTopic = await DBHelper().queryCustom(
                  "SELECT title FROM DISCUSSION_FORUM WHERE chat_id='$topicGroup'");
              topicGroup = resultQueryTopic["title"];
            }
            var resultQueryBuddy = await DBHelper().queryCustom(
                "SELECT f_name, image_id FROM GROUPZ WHERE group_id='$idGroup'");
            if (resultQueryBuddy.length > 0) {
              Navigator.push(context, CupertinoPageRoute(builder: (context) {
                return ChatGroupPage(resultQueryBuddy[0]["f_name"], topicGroup,
                    resultQueryBuddy[0]["image_id"],
                    identifier:
                        data["data"]["BA"] == null ? "" : data["data"]["BA"],
                    idGroup: idGroup);
              }));
            } else {
              Fluttertoast.showToast(
                  msg: "This Group has been deleted",
                  toastLength: Toast.LENGTH_LONG);
            }
          } else {
            if (topicGroup == null) {
              topicGroup = "Lounge";
            } else {
              var resultQueryTopic = await DBHelper().queryCustom(
                  "SELECT title FROM DISCUSSION_FORUM WHERE chat_id='$topicGroup'");
              topicGroup = resultQueryTopic["title"];
            }
            var resultQueryBuddy = await DBHelper().queryCustom(
                "SELECT f_name, image_id FROM GROUPZ WHERE group_id='$idGroup'");
            if (resultQueryBuddy.length > 0) {
              Navigator.pushReplacement(context,
                  CupertinoPageRoute(builder: (context) {
                return ChatGroupPage(resultQueryBuddy[0]["f_name"], topicGroup,
                    resultQueryBuddy[0]["image_id"],
                    identifier:
                        data["data"]["BA"] == null ? "" : data["data"]["BA"],
                    idGroup: idGroup);
              }));
            } else {
              Fluttertoast.showToast(
                  msg: "This Group has been deleted",
                  toastLength: Toast.LENGTH_LONG);
            }
          }
        }
      } else if (data["type_notification"] == 2) {
        if (data["flag"] == "PUSH_CALL_CENTER") {
          var jo = {'f_pin': data["f_pin"], 'status': data["status"]};
          var jos = jsonEncode(jo);
          API.setCallCenterData(data["f_pin"], jos);
          SharedObj.CALL_CENTER_SESSION_F_PIN = data["f_pin"];
          showDialog(
              context: context,
              barrierDismissible: false,
              builder: (_) {
                return Dialog(
                  backgroundColor: Colors.grey.withOpacity(0.5),
                  shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(10.0),
                  ),
                  child: Container(
                    width: double.infinity,
                    child: RequestCallCenter(
                      name: data["real_name"],
                      image: data["image_id"],
                      deviceId: data["f_pin"],
                      channel:
                          data.containsKey("channel") ? data["channel"] : "0",
                    ),
                  ),
                );
              });

          // Navigator.push(
          //     context, PageTransition(type: PageTransitionType.fade,
          //     child: RequestCallCenter(
          //       name: data["real_name"],
          //       image: data["image_id"],
          //       deviceId: data["f_pin"],
          //       channel: data.containsKey("channel") ? data["channel"] : "0",
          //     )
          // ));
          // else{
          //   Fluttertoast.showToast(msg: "Sorry, this session has expired");
          // }
        } else if (data["flag"] == "ACCEPT_CALL_CENTER") {
          print("route current: ${ModalRoute.of(context)!.settings.name}");
          if (ModalRoute.of(context)!.settings.name != "/contactCenter" &&
              !SharedObj.isOnGoingCallCenter) {
            var jo = {'f_pin': data["f_pin"], 'status': data["status"]};
            var jos = jsonEncode(jo);
            API.setCallCenterData(data["f_pin"], jos);
            var queryResult = await DBHelper().queryCustom(
                "SELECT first_name, last_name,f_pin, offline_mode, image_id FROM BUDDY where official_account = '1'");
            var realName =
                "${queryResult[0]["first_name"]}${queryResult[0]["last_name"].isNotEmpty ? " " + queryResult[0]["last_name"] : ""}";
            print("real name contacr center: $realName");
            if (queryResult.isNotEmpty) {
              Navigator.pushNamed(context, '/contactCenter', arguments: {
                'fullName': realName,
                'image': queryResult[0]["image_id"],
                'identifier': data["f_pin"],
                'refresh': true,
                'isContactCenter': true,
                'data': jsonEncode(data),
                'contactCenterHistoryId': data["complaint_id"],
                'channel': data.containsKey("channel") ? data["channel"] : "0",
              });
            }
            const IOSNotificationDetails iOSPlatformChannelSpecifics =
                IOSNotificationDetails(
                    presentAlert: true,
                    presentBadge: true,
                    presentSound: false);
            const NotificationDetails platformChannelSpecifics =
                NotificationDetails(iOS: iOSPlatformChannelSpecifics);
            var title = "Interaction with Call Center In Progress";
            var body = "Click to go to the Call Center Screen";
            await flutterLocalNotificationsPlugin.show(
                NOTIFICATION_CALL_CENTER_ID,
                title,
                body,
                platformChannelSpecifics,
                payload: payload);
          }
        }
      } else {
        if (data["state"] == 21) {
          Navigator.push(
              context,
              PageTransition(
                  type: PageTransitionType.fade,
                  child: AudioCall(
                    name: "${data["first_name"]} ${data["last_name"]}",
                    image: data["image_id"],
                    isInitiatior: 0,
                    deviceId: data["f_pin"],
                  )));
        } else if (data["state"] == 31) {
          Navigator.push(
              context,
              PageTransition(
                  type: PageTransitionType.fade,
                  child: VideoCall(
                    name: "${data["first_name"]} ${data["last_name"]}",
                    image: data["image_id"],
                    isInitiatior: 0,
                    deviceId: data["f_pin"],
                    cameraId: "1",
                  )));
        }
      }
    });
  }

  static void init(BuildContext context) {
    initNotifications();
    _requestPermissions();
    _configureSelectNotificationSubject(context);
  }

  static void showNotifications(var result, int type, BuildContext context,
      {int level = 1, int? state}) async {
    const IOSNotificationDetails iOSPlatformChannelSpecifics =
        IOSNotificationDetails(
            presentAlert: true, presentBadge: true, presentSound: true);
    const NotificationDetails platformChannelSpecifics =
        NotificationDetails(iOS: iOSPlatformChannelSpecifics);
    if (type == 0) {
      print("result: $result");
      String? text, payload, name;
      if (result["A57"] != null) {
        text = "Sent Image \uD83D\uDCF7";
      } else if (result["A149"].toString() == "11") {
        text = "Sent Sticker \uD83D\uDE42";
      } else if (result["A47"] != null) {
        text = "Sent Video \uD83D\uDCF9";
      } else if (result["BN"] != null) {
        text = "Sent File \uD83D\uDCC4";
      } else if (result["A63"] != null) {
        text = "Sent Audio \uD83C\uDFB5";
      } else if (result["A07"].contains("Share%20location%20")) {
        text = "Sent Location \uD83D\uDCCC";
      } else if (result["A149"] == "27") {
        text = "Send link Live Video Promotion";
      } else if (result["A149"] == "25") {
        text = "Send link Video Conference Room";
      } else if (result["A149"] == "24") {
        text = "Send link Quiz";
      } else {
        text = result["A07"];
        if (text!.contains("@")) {
          var resultQuery = await DBHelper()
              .queryCustom("SELECT f_pin, first_name, last_name FROM BUDDY");
          for (var i = 0; i < resultQuery.length; i++) {
            if (text!.contains("@${resultQuery[i]["f_pin"]}")) {
              if (resultQuery[i]["last_name"].isNotEmpty) {
                text = text.replaceAll("@${resultQuery[i]["f_pin"]}",
                    "@${resultQuery[i]["first_name"]} ${resultQuery[i]["last_name"]}");
              } else {
                text = text.replaceAll("@${resultQuery[i]["f_pin"]}",
                    "@${resultQuery[i]["first_name"]}");
              }
            }
          }
        }
        String? fromRichTextToPlainText(final Widget widget) {
          if (widget is RichText) {
            if (widget.text is TextSpan) {
              final buffer = StringBuffer();
              (widget.text as TextSpan).computeToPlainText(buffer);
              return buffer.toString();
            }
          }
          return null;
        }

        RichText richText = RichText(
            text: TextSpan(children: RTFText.toRTF(text!, isPreview: true)));
        text = fromRichTextToPlainText(richText);
      }

      if (result["A06"] == "3") {
        name = result["BR"];
        if (name == null) {
          var resultQuery = await DBHelper().queryCustom(
              "SELECT first_name, last_name FROM BUDDY WHERE f_pin='${result["A00"]}'");
          if (resultQuery.isNotEmpty) {
            name = resultQuery[0]["first_name"] +
                " " +
                resultQuery[0]["last_name"];
          }
        }
      } else {
        String idGroup = result["A01"];
        String topicGroup = result["BA"];
        if (topicGroup == null) {
          topicGroup = "Lounge";
        } else {
          var resultQueryTopic = await DBHelper().queryCustom(
              "SELECT title FROM DISCUSSION_FORUM WHERE chat_id='$topicGroup'");
          topicGroup = resultQueryTopic["title"];
        }
        var resultQueryBuddy = await DBHelper()
            .queryCustom("SELECT f_name FROM GROUPZ WHERE group_id='$idGroup'");
        name =
            "${result["BR"]} \u2022 ${resultQueryBuddy[0]["f_name"]}($topicGroup)";
      }
      if (result["A38"] != null && result["A38"] == "2") {
        Future.delayed(Duration(seconds: 2), () {
          showDialogBroadcast(result, context);
        });
      } else {
        if (result["A149"] == "12" || result["A149"] == "19") {
          return null;
        }
        payload = jsonEncode({
          "type_notification": 0, //message = 0, call = 1
          "type_message": result["A06"],
          "level": level,
          "data": result,
          "is_call_center": result.containsKey("icc") ? result["icc"] : "0",
          "call_center_id": result.containsKey("icc") ? result["ccid"] : ""
        });
        await flutterLocalNotificationsPlugin
            .show(0, name, text, platformChannelSpecifics, payload: payload);
      }
    } else if (type == 2) {
      var id = 2;
      final isAcceptCallCenter = result["flag"] == "ACCEPT_CALL_CENTER";
      final IOSNotificationDetails iOSPlatformChannelSpecifics =
          IOSNotificationDetails(
              presentAlert: true,
              presentBadge: true,
              presentSound: !isAcceptCallCenter);
      final NotificationDetails platformChannelSpecifics =
          NotificationDetails(iOS: iOSPlatformChannelSpecifics);
      var title = "";
      var body = "";
      result["type_notification"] = 2;
      if (result["flag"] == "PUSH_CALL_CENTER") {
        title = "Call Center";
        body = "User is waiting for your confirmation";
      } else if (result["flag"] == "ACCEPT_CALL_CENTER") {
        title = "Interaction with Call Center In Progress";
        body = "Click to go to the Call Center Screen";
        id = NOTIFICATION_CALL_CENTER_ID;
      }
      await flutterLocalNotificationsPlugin.show(
          id, title, body, platformChannelSpecifics,
          payload: jsonEncode(result));
    } else {
      var payload = {
        "f_pin": result["f_pin"],
        "state": state,
        "first_name": result["first_name"],
        "last_name": result["last_name"],
        "image_id": result["image_id"],
        "type_notification": 1
      };
      final IOSNotificationDetails iOSPlatformChannelSpecifics =
          IOSNotificationDetails(
              presentAlert: true, presentBadge: true, presentSound: false);
      final NotificationDetails platformChannelSpecifics =
          NotificationDetails(iOS: iOSPlatformChannelSpecifics);
      await flutterLocalNotificationsPlugin.show(
          1,
          "${result["first_name"]} ${result["last_name"]}",
          (state == 21) ? 'Incoming VoIP Call' : 'Incoming Video Call',
          platformChannelSpecifics,
          payload: jsonEncode(payload));
    }
  }

  static Future<void> clearNotification(int id) async {
    await flutterLocalNotificationsPlugin.cancel(id);
  }
}

class ReceivedNotification {
  ReceivedNotification({
    required this.id,
    required this.title,
    required this.body,
    required this.payload,
  });

  final int id;
  final String title;
  final String body;
  final String payload;
}
