import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:palio_lite/api.dart';
import 'package:palio_lite/chat/chat_group_screen.dart';
import 'package:palio_lite/chat/chat_screen.dart';
import 'package:palio_lite/chat/tab_3.dart';
import 'package:palio_lite/chat/widgets/custom_expansion_tile.dart';
import 'package:palio_lite/chat/widgets/rtf_text_view.dart';
import 'package:palio_lite/chat/widgets/widget_Image_Download.dart';

class UcList extends StatelessWidget {
  final ListUcList data;
  final VoidCallback refresh;

  UcList({required this.data, required this.refresh});

  @override
  Widget build(BuildContext context) {
    if (data.type == "1") {
      return _ucPersonal(data, context, refresh);
    } else {
      return _ucGroup(data, refresh);
    }
  }
}

Widget _ucPersonal(
    ListUcList data, BuildContext context, VoidCallback refresh) {
  return Container(
    child: ListTile(
      contentPadding:
          EdgeInsets.only(left: 8.0, bottom: 8.0, top: 8.0, right: 16.0),
      leading: CircleAvatar(
        radius: 28.0,
        backgroundColor: Colors.transparent,
        child: Container(
          decoration: BoxDecoration(
              shape: BoxShape.circle,
              color: Colors.grey,
              image: DecorationImage(
                  fit: BoxFit.cover,
                  image: data.fPicture == ""
                      ? data.fName == "Bot"
                          ? AssetImage("assets/images/palio_ball.png",
                              package: "palio_lite")
                          : AssetImage("assets/images/default_pp.png",
                              package: "palio_lite")
                      : data.fPicture == "palio_logo_round.png"
                          ? AssetImage("assets/images/palio.png",
                              package: "palio_lite")
                          : LocalImage(data.fPicture) as ImageProvider)),
        ),
      ),
      title: Row(
        mainAxisAlignment: MainAxisAlignment.start,
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          data.isVerified == "1"
              ? Image.asset("assets/images/ic_verified_flag.png",
                  width: 15, height: 15, package: "palio_lite")
              : data.isOfficial == "1"
                  ? Image.asset("assets/images/ic_official_flag.png",
                      width: 15, height: 15, package: "palio_lite")
                  : Container(),
          Padding(
            padding: EdgeInsets.only(
              left:
                  data.isVerified == "1" || data.isOfficial == "1" ? 5.0 : 0.0,
            ),
            child: Text(
              data.fName,
            ),
          )
        ],
      ),
      subtitle: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          data.lPin == "-999"
              ? Text(
                  "Your reminders/Your reminder assistant",
                  style: TextStyle(
                    fontSize: 12,
                    color: Colors.black54,
                  ),
                )
              : Container(),
          Row(
            mainAxisAlignment: MainAxisAlignment.start,
            children: <Widget>[
              data.fPin == data.myPin &&
                      data.credential != "1" &&
                      !data.message.contains("🚫") &&
                      data.contentType != "12" &&
                      data.contentType != "19"
                  ? data.statusRead == "8"
                      ? Container(
                          padding: EdgeInsets.only(right: 5),
                          width: 18.0,
                          height: 18.0,
                          child: Image.asset(
                              "assets/images/message_status_ack.png"),
                        )
                      : Padding(
                          padding: EdgeInsets.only(right: 5),
                          child: Icon(
                            data.statusRead == "1" || data.statusRead == "2"
                                ? Icons.done
                                : Icons.done_all,
                            size: 12,
                            color: data.statusRead == "4"
                                ? Colors.blue
                                : Colors.grey,
                          ),
                        )
                  : Container(),
              data.contentType != "0" &&
                      data.contentType != "11" &&
                      data.contentType != "24" &&
                      data.contentType != "25" &&
                      data.contentType != "27"
                  ? Padding(
                      padding: EdgeInsets.only(right: 5),
                      child: Icon(
                        data.contentType == "12"
                            ? Icons.call_end
                            : data.contentType == "19"
                                ? Icons.phone_missed
                                : data.contentType == "-1"
                                    ? CupertinoIcons.volume_up
                                    : data.contentType == "1"
                                        ? CupertinoIcons.photo_camera_solid
                                        : data.contentType == "2"
                                            ? CupertinoIcons.video_camera_solid
                                            : data.contentType == "3"
                                                ? CupertinoIcons.location_solid
                                                : CupertinoIcons
                                                    .collections_solid,
                        size: 20,
                        color: data.contentType == "12"
                            ? Colors.green
                            : data.contentType == "19"
                                ? Colors.red
                                : data.credential == "-1"
                                    ? Colors.red
                                    : Colors.grey,
                      ),
                    )
                  : Container(),
              Expanded(
                  child: data.contentType == "12" && data.fPin == data.myPin
                      ? Text(
                          data.message.contains("VoIP")
                              ? "Outgoing Voip Call"
                              : "Outgoing Video Call",
                          style: TextStyle(color: Colors.black54, fontSize: 12))
                      : data.contentType == "27"
                          ? Text("Live Video Promotion -, Description...",
                              style: TextStyle(
                                  color: Colors.black54, fontSize: 12))
                          : data.contentType == "25"
                              ? Text("Video Conference Room -, Description...",
                                  style: TextStyle(
                                      color: Colors.black54, fontSize: 12))
                              : data.contentType == "11"
                                  ? data.fPin == data.myPin
                                      ? Text("You attached sticker",
                                          style: TextStyle(
                                              color: Colors.black54,
                                              fontSize: 12))
                                      : Text("Attachment sticker",
                                          style: TextStyle(
                                              color: Colors.black54,
                                              fontSize: 12))
                                  : data.credential == "1" &&
                                          data.countCredential == 0
                                      ? RTFText(
                                          "🚫 _This message already expired..._",
                                          textColor: Colors.black54,
                                          fontSize: 12)
                                      : data.credential == "1"
                                          ? Text("Confidential Message",
                                              style: TextStyle(
                                                  color: Colors.black54,
                                                  fontSize: 12))
                                          : data.contentType == "-1"
                                              ? Text("Attachment audio",
                                                  style: TextStyle(
                                                      color: Colors.black54,
                                                      fontSize: 12))
                                              : data.contentType == "1" &&
                                                      data.message
                                                          .trim()
                                                          .isEmpty
                                                  ? data.fPin == data.myPin
                                                      ? Text(
                                                          "You attached picture",
                                                          style: TextStyle(
                                                              color: Colors
                                                                  .black54,
                                                              fontSize: 12))
                                                      : Text(
                                                          "Attachment picture",
                                                          style: TextStyle(
                                                              color: Colors
                                                                  .black54,
                                                              fontSize: 12))
                                                  : data.contentType == "2" &&
                                                          data.message
                                                              .trim()
                                                              .isEmpty
                                                      ? data.fPin == data.myPin
                                                          ? Text("You attached video", style: TextStyle(color: Colors.black54, fontSize: 12))
                                                          : Text("Attachment video", style: TextStyle(color: Colors.black54, fontSize: 12))
                                                      : data.contentType == "3"
                                                          ? data.fPin == data.myPin
                                                              ? Text("You attached location", style: TextStyle(color: Colors.black54, fontSize: 12))
                                                              : Text("Attachment location", style: TextStyle(color: Colors.black54))
                                                          : data.contentType == "4" && data.message.trim().isEmpty
                                                              ? data.fPin == data.myPin
                                                                  ? Text("You attached document", style: TextStyle(color: Colors.black54, fontSize: 12))
                                                                  : Text("Attachment document", style: TextStyle(color: Colors.black54, fontSize: 12))
                                                              : RTFText(
                                                                  "${data.message}",
                                                                  overflow:
                                                                      TextOverflow
                                                                          .ellipsis,
                                                                  fontSize: 12,
                                                                  textColor: Colors
                                                                      .black54,
                                                                  maxLines: 1,
                                                                ))
            ],
          ),
        ],
      ),
      trailing: IntrinsicWidth(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          children: <Widget>[
            Text(
              "${data.time}",
              style: TextStyle(fontSize: 10),
            ),
            data.counter != "0"
                ? Container(
                    alignment: Alignment.center,
                    padding: EdgeInsets.all(5.0),
                    decoration: BoxDecoration(
                        color: Colors.red, shape: BoxShape.circle),
                    child: Text(
                      "${data.counter}",
                      style: TextStyle(color: Colors.white, fontSize: 12),
                    ),
                  )
                : Container(height: 0, width: 0)
          ],
        ),
      ),
      onTap: () async {
        API.updateCounter(data.lPin);
        await Navigator.pushReplacement(
            context,
            CupertinoPageRoute(
                builder: (_) => ChatPage(
                      fullName: data.fName,
                      image: data.fPicture,
                      identifier: data.lPin,
                      refresh: data.counter != "0" ? true : false,
                    )));
        refresh();
      },
    ),
  );
}

Widget _ucGroup(ListUcList data, VoidCallback refresh) {
  var topic = data.topicList;
  int totalCnt = 0;
  for (var i = 0; i < topic.length; i++) {
    totalCnt += int.parse(topic[i].counter);
  }
  return Container(
    child: CustomExpansionTile(
      initiallyExpanded: totalCnt > 0 ? true : false,
      leading: CircleAvatar(
        radius: 28.0,
        backgroundColor: Colors.transparent,
        child: Container(
          decoration: BoxDecoration(
              shape: BoxShape.circle,
              color: Colors.grey,
              image: DecorationImage(
                  fit: BoxFit.cover,
                  image: data.fPicture == ""
                      ? AssetImage("assets/images/groupicon.png",
                          package: "palio_lite")
                      : data.fPicture == "palio_logo_round.png"
                          ? AssetImage("assets/images/palio.png",
                              package: "palio_lite")
                          : LocalImage(data.fPicture) as ImageProvider)),
        ),
      ),
      title: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[
          Row(
            children: <Widget>[
              data.isVerified == "1"
                  ? Image.asset("assets/images/ic_verified_flag.png",
                      width: 15, height: 15, package: "palio_lite")
                  : data.isOfficial == "1"
                      ? Image.asset("assets/images/ic_official_flag.png",
                          width: 15, height: 15, package: "palio_lite")
                      : Container(),
              Padding(
                padding: EdgeInsets.only(
                  left: data.isVerified == "1" || data.isOfficial == "1"
                      ? 8.0
                      : 0.0,
                ),
                child: Text(
                  data.fName,
                ),
              )
            ],
          ),
          totalCnt != 0
              ? Container(
                  alignment: Alignment.center,
                  padding: EdgeInsets.all(8.0),
                  decoration:
                      BoxDecoration(color: Colors.red, shape: BoxShape.circle),
                  child: Text(
                    "$totalCnt",
                    style: TextStyle(color: Colors.white, fontSize: 12),
                  ),
                )
              : Container(height: 0, width: 0)
        ],
      ),
      children: <Widget>[
        Container(
          margin: EdgeInsets.only(left: 5.0),
          child: ListView.builder(
            shrinkWrap: true,
            physics: NeverScrollableScrollPhysics(),
            itemCount: topic.length,
            itemBuilder: (context, index) {
              return ListTile(
                  onTap: () async {
                    if (topic[index].lPin != "" &&
                        topic[index].counter != "0") {
                      API.updateCounter(topic[index].lPin);
                    } else if (topic[index].counter != "0") {
                      API.updateCounter(topic[index].idGroup);
                    }
                    await Navigator.pushReplacement(
                        context,
                        CupertinoPageRoute(
                            builder: (_) => ChatGroupPage(
                                  data.fName,
                                  topic[index].fName,
                                  data.fPicture,
                                  identifier: topic[index].lPin,
                                  idGroup: topic[index].idGroup,
                                  refresh: topic[index].counter != "0"
                                      ? true
                                      : false,
                                )));
                    refresh();
                  },
                  title: Text(
                    "${topic[index].fName}",
                    style: TextStyle(fontSize: 15, color: Colors.black),
                  ),
                  subtitle: Row(
                    mainAxisAlignment: MainAxisAlignment.start,
                    children: <Widget>[
                      topic[index].fPin == topic[index].myPin &&
                              !topic[index].message.contains("🚫")
                          ? topic[index].statusRead == "8"
                              ? Container(
                                  padding: EdgeInsets.only(right: 5),
                                  width: 18.0,
                                  height: 18.0,
                                  child: Image.asset(
                                      "assets/images/message_status_ack.png"),
                                )
                              : Padding(
                                  padding: EdgeInsets.only(right: 5),
                                  child: Icon(
                                    topic[index].statusRead == "1" ||
                                            topic[index].statusRead == "2"
                                        ? Icons.done
                                        : Icons.done_all,
                                    size: 12,
                                    color: topic[index].statusRead == "4"
                                        ? Colors.blue
                                        : Colors.grey,
                                  ),
                                )
                          : Container(),
                      topic[index].contentType != "0" &&
                              data.contentType != "11" &&
                              data.contentType != "30" &&
                              data.contentType != "24"
                          ? Padding(
                              padding: EdgeInsets.only(right: 5),
                              child: Icon(
                                topic[index].contentType == "-1"
                                    ? CupertinoIcons.volume_up
                                    : topic[index].contentType == "1"
                                        ? CupertinoIcons.photo_camera_solid
                                        : topic[index].contentType == "2"
                                            ? CupertinoIcons.video_camera_solid
                                            : topic[index].contentType == "3"
                                                ? CupertinoIcons.location_solid
                                                : CupertinoIcons
                                                    .collections_solid,
                                size: 20,
                                color: Colors.grey,
                              ),
                            )
                          : Container(),
                      Expanded(
                          child: topic[index].contentType == "30"
                              ? Text("Bot: Post",
                                  style: TextStyle(
                                      color: Colors.black54, fontSize: 12))
                              : topic[index].contentType == "11"
                                  ? topic[index].fPin == topic[index].myPin
                                      ? Text("You attached sticker",
                                          style: TextStyle(
                                              color: Colors.black54,
                                              fontSize: 12))
                                      : Text("Attachment sticker",
                                          style: TextStyle(
                                              color: Colors.black54,
                                              fontSize: 12))
                                  : topic[index].contentType == "-1"
                                      ? Text("Attachment audio",
                                          style: TextStyle(
                                              color: Colors.black54,
                                              fontSize: 12))
                                      : topic[index].contentType == "1" &&
                                              topic[index]
                                                  .message
                                                  .trim()
                                                  .isEmpty
                                          ? topic[index].fPin ==
                                                  topic[index].myPin
                                              ? Text("You attached picture",
                                                  style: TextStyle(
                                                      color: Colors.black54,
                                                      fontSize: 12))
                                              : Text("Attachment picture",
                                                  style: TextStyle(
                                                      color: Colors.black54,
                                                      fontSize: 12))
                                          : topic[index].contentType == "2" &&
                                                  topic[index]
                                                      .message
                                                      .trim()
                                                      .isEmpty
                                              ? topic[index].fPin ==
                                                      topic[index].myPin
                                                  ? Text("You attached video",
                                                      style: TextStyle(
                                                          color: Colors.black54,
                                                          fontSize: 12))
                                                  : Text("Attachment video",
                                                      style: TextStyle(
                                                          color: Colors.black54,
                                                          fontSize: 12))
                                              : topic[index].contentType == "3"
                                                  ? topic[index].fPin ==
                                                          topic[index].myPin
                                                      ? Text("You attached location",
                                                          style: TextStyle(
                                                              color: Colors
                                                                  .black54,
                                                              fontSize: 12))
                                                      : Text("Attachment location",
                                                          style: TextStyle(
                                                              color: Colors
                                                                  .black54,
                                                              fontSize: 12))
                                                  : topic[index].contentType == "4" &&
                                                          topic[index]
                                                              .message
                                                              .trim()
                                                              .isEmpty
                                                      ? topic[index].fPin ==
                                                              topic[index].myPin
                                                          ? Text("You attached document",
                                                              style: TextStyle(
                                                                  color: Colors
                                                                      .black54,
                                                                  fontSize: 12))
                                                          : Text("Attachment document",
                                                              style: TextStyle(
                                                                  color: Colors.black54,
                                                                  fontSize: 12))
                                                      : RTFText("${topic[index].message}", overflow: TextOverflow.ellipsis, fontSize: 12, mentions: topic[index].listMembers, textColor: Colors.black54, maxLines: 1))
                    ],
                  ),
                  leading: Container(
                    height: 45,
                    width: 45,
                  ),
                  trailing: IntrinsicWidth(
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: <Widget>[
                        Text(
                          "${topic[index].time}",
                          style: TextStyle(fontSize: 10),
                        ),
                        topic[index].counter != "0"
                            ? Container(
                                padding: EdgeInsets.all(5.0),
                                alignment: Alignment.center,
                                decoration: BoxDecoration(
                                    color: Colors.red, shape: BoxShape.circle),
                                child: Text(
                                  "${topic[index].counter}",
                                  style: TextStyle(
                                      color: Colors.white, fontSize: 12),
                                ),
                              )
                            : Container(height: 0, width: 0)
                      ],
                    ),
                  ));
            },
          ),
        )
      ],
//    trailing: ,
    ),
  );
}
