import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/chat/widgets/video_full_screen.dart';
import 'package:video_player/video_player.dart';

class VideoPlayerHeader extends StatefulWidget {
  final VideoPlayerController controller;
  final bool inTimeline;
  final ValueChanged<bool> isPlaying;
  final openFrom;

  const VideoPlayerHeader(
      {Key? key,
      required this.controller,
      required this.inTimeline,
      required this.isPlaying,
      this.openFrom})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => _VideoPlayerHeaderState();
}

class _VideoPlayerHeaderState extends State<VideoPlayerHeader> {
  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.only(
          bottom:
              widget.openFrom == null || widget.openFrom != "list" ? 0.0 : 0.0),
      child: Stack(
        children: <Widget>[
          SingleChildScrollView(
            child: Column(
              children: <Widget>[
                Container(
                  child: AspectRatio(
                    aspectRatio: widget.controller.value.aspectRatio,
                    child: Stack(
                      alignment: Alignment.bottomCenter,
                      children: <Widget>[
                        VideoPlayer(widget.controller),
                        ClosedCaption(
                            text: widget.controller.value.caption.text),
                        PlayPauseOverlay(controller: widget.controller),
                        _Fullscreen(controller: widget.controller),
                        VideoProgressIndicator(widget.controller,
                            allowScrubbing: true),
                      ],
                    ),
                  ),
                ),
              ],
            ),
          ),
        ],
      ),
    );
  }
}

class _Fullscreen extends StatelessWidget {
  final VideoPlayerController controller;

  _Fullscreen({Key? key, required this.controller}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
        alignment: Alignment.bottomRight,
        margin: EdgeInsets.only(bottom: 10, right: 10),
        child: InkWell(
            onTap: () {
              Navigator.push(
                  context,
                  PageTransition(
                      type: PageTransitionType.fade,
                      child: VideoFullScreen(
                        controller: controller,
                      )));
            },
            child: Icon(
              CupertinoIcons.fullscreen,
              color: Colors.white,
            )));
  }
}

class PlayPauseOverlay extends StatefulWidget {
  final VideoPlayerController controller;

  const PlayPauseOverlay({Key? key, required this.controller})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => PlayPauseOverlayState();
}

class PlayPauseOverlayState extends State<PlayPauseOverlay> {
  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        AnimatedSwitcher(
          duration: Duration(milliseconds: 50),
          reverseDuration: Duration(milliseconds: 200),
          child: widget.controller.value.isPlaying
              ? Container()
              : Container(
                  color: Colors.black54,
                  child: Center(
                    child: Icon(
                      Icons.play_arrow,
                      color: Colors.white,
                      size: 60.0,
                    ),
                  ),
                ),
        ),
        GestureDetector(
          onTap: () {
            setState(() {
              if (widget.controller.value.isPlaying) {
                widget.controller.pause();
              } else {
                widget.controller.play();
              }
            });
          },
        ),
      ],
    );
  }
}
