import 'dart:async';
import 'dart:io';
import 'dart:typed_data';
import 'dart:ui' as ui show Codec;
import 'dart:ui' show Size, Locale, TextDirection, hashValues;

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:flutter/src/painting/image_provider.dart';
import 'package:flutter/src/painting/image_stream.dart';
import 'package:palio_lite/api.dart';
import 'package:path_provider/path_provider.dart';

class LocalImage extends ImageProvider<LocalImage> {
  /// Creates an object that decodes a [File] as an image.
  ///
  /// The arguments must not be null.
  LocalImage(this.image, {this.scale = 1.0}) : assert(scale != null);

  /// The file to decode into an image.
  String? image;

  /// The scale to place in the [ImageInfo] object of the image.
  final double? scale;

  @override
  Future<LocalImage> obtainKey(ImageConfiguration configuration) {
    return SynchronousFuture<LocalImage>(this);
  }

  @override
  ImageStreamCompleter load(LocalImage key, DecoderCallback decode) {
    return MultiFrameImageStreamCompleter(
      codec: _loadAsync(key, decode),
      scale: key.scale!,
//      informationCollector: () sync* {
//        yield ErrorDescription('Path: ${file?.path}');
//      },
    );
  }

  // Future<ui.Codec> loadImageAfterDownload(File file, DecoderCallback decode) async {
  //   final Uint8List bytes = file.readAsBytesSync();
  //   if (bytes.lengthInBytes == 0) return null;
  //   return await decode(bytes);
  // }

  Future<ui.Codec> _loadAsync(LocalImage key, DecoderCallback decode) async {
    assert(key == this);
    Directory directory = await getApplicationDocumentsDirectory();
    if (key.image != null && key.image!.isNotEmpty) {
      File file = File("${directory.path}/${key.image}");
      if (file.existsSync()) {
        final Uint8List bytes = file.readAsBytesSync();
        if (bytes.lengthInBytes == 0) {
          PaintingBinding.instance!.imageCache!.evict(key);
          throw StateError('$file is empty and cannot be loaded as an image.');
        }
        return await decode(bytes);
      } else {
        API.download(image!);
      }
    }
    final ByteData data = await rootBundle
        .load('packages/palio_lite/assets/images/default_pp.png');
    key.image = DateTime.now().millisecondsSinceEpoch.toString();
    return await decode(data.buffer.asUint8List());
  }

  @override
  bool operator ==(dynamic other) {
    if (other.runtimeType != runtimeType) return false;
    final LocalImage typedOther = other;
    return image == typedOther.image && scale == typedOther.scale;
  }

  @override
  int get hashCode => hashValues(image, scale);

  @override
  String toString() => '$runtimeType("$image", scale: $scale)';
}
