import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';

class TimeForm extends StatefulWidget {
  final Function(String) time;
  final int timeValue;

  TimeForm({required this.time, required this.timeValue});

  @override
  _TimeFormState createState() => _TimeFormState();
}

class _TimeFormState extends State<TimeForm> {
  String time = TimeOfDay.now().toString().substring(10, 15);
  Color color = Colors.white;
  int timeVal = 0;

  @override
  void initState() {
    super.initState();
    timeVal = widget.timeValue;
    if (timeVal > 0) {
      time =
          TimeOfDay.fromDateTime(DateTime.fromMillisecondsSinceEpoch(timeVal))
              .toString()
              .substring(10, 15);
    }
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        if (timeVal != 0) {
          showTimePicker(
                  context: context,
                  initialTime: TimeOfDay.fromDateTime(
                      DateTime.fromMillisecondsSinceEpoch(timeVal)))
              .then((value) {
            if (value != null) {
              widget.time(value.toString().substring(10, 15));
              setState(() {
                time = value.toString().substring(10, 15);
              });
            }
          });
        } else {
          showTimePicker(context: context, initialTime: TimeOfDay.now())
              .then((value) {
            if (value != null) {
              widget.time(value.toString().substring(10, 15));
              setState(() {
                time = value.toString().substring(10, 15);
              });
            }
          });
        }
      },
      onTapDown: (tap) {
        setState(() {
          color = Colors.grey;
        });
      },
      onTapUp: (tap) {
        setState(() {
          color = Colors.white;
        });
      },
      onTapCancel: () {
        setState(() {
          color = Colors.white;
        });
      },
      child: Container(
        padding: EdgeInsets.only(left: 10.0, right: 10.0),
        height: 40,
        decoration: BoxDecoration(
            color: color,
            border: Border.all(color: Colors.black),
            borderRadius: BorderRadius.circular(3.0)),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            Icon(
              Icons.access_time,
              color: Colors.grey[600],
              size: 14,
            ),
            SizedBox(width: 10.0),
            Text(time)
          ],
        ),
      ),
    );
  }
}
