import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/call/audio_call.dart';
import 'package:palio_lite/call/new_call.dart';
import 'package:palio_lite/call/video_call.dart';
import 'package:palio_lite/chat/chat_screen.dart';
import 'package:palio_lite/chat/new_chat.dart';
import 'package:palio_lite/chat/settings_screen.dart';
import 'package:palio_lite/chat/widgets/notifications.dart';
import 'package:palio_lite/conference/add_live_streaming.dart';
import 'package:palio_lite/db_helper.dart';

import 'package:palio_lite/api.dart';
import 'package:palio_lite/request_call_center_history.dart';
import 'package:palio_lite/shared_obj.dart';
import 'package:shared_preferences/shared_preferences.dart';

class PalioButton extends StatefulWidget {
  final double xpos;
  final double ypos;
  final String apiKey;

  static Future<bool> palioDispose() async {
    SharedObj.disposeCallCenter();
    return true;
  }

  static palioRoute(settings) {
    if (settings.name == '/contactCenter') {
      final args = settings.arguments as Map<String, dynamic>;
      return PageTransition(
          child: ChatPage(
            fullName: args["fullName"],
            image: args["image"],
            identifier: args["identifier"],
            refresh: args["refresh"],
            isContactCenter: args["isContactCenter"],
            data: args.containsKey('data') ? args["data"] : "",
            channel: args.containsKey('channel') ? args["channel"] : "",
            contactCenterHistoryId: args.containsKey('contactCenterHistoryId')
                ? args["contactCenterHistoryId"]
                : "",
          ),
          type: PageTransitionType.fade,
          settings: settings);
    }
    return null;
  }

  PalioButton({required this.xpos, required this.ypos, required this.apiKey});

  @override
  PalioButtonState createState() => PalioButtonState();
}

class PalioButtonState extends State<PalioButton> {
  double xPosition = 0;
  double yPosition = 0;
  double? rightPosition;
  double? bottomPosition;
  int state = 2;
  bool showButton = false;

  Future<bool> setFpin(String fpin) async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.setString('hasLogin', fpin);
  }

  @override
  void initState() {
    API.init();
    Notifications.init(this.context);
    incomingCall(this.context);
    receiveMessage(this.context);
    if (widget.apiKey != null) {
      initPalio();
    }
    xPosition = widget.xpos;
    yPosition = widget.ypos;
    super.initState();
  }

  @override
  void dispose() {
    // disposeCallCenter();
    super.dispose();
  }

  void initPalio() async {
    String fpin = await API.connect(widget.apiKey);
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    String? idMe = prefs.getString("hasLogin");
    API.postRegistration();
    if (idMe == null) {
      setFpin(fpin);
      setState(() {
        showButton = true;
      });
    } else if (idMe != null) {
      setState(() {
        showButton = true;
      });
    }
  }

  void receiveMessage(BuildContext context) {
    API.onReceiveMessage = (dynamic result) async {
      print("result receive message: $result");
      if (result.containsKey('flag')) {
        final SharedPreferences prefs = await SharedPreferences.getInstance();
        if (result["flag"] == "PUSH_CALL_CENTER") {
          if (SharedObj.timerRequest == null) {
            SharedObj.initTimer(result["f_pin"], result["channel"]);
            Notifications.showNotifications(result, 2, context);
          }
          //   var jo = {
          //     'f_pin' : result["f_pin"],
          //     'status' : result["status"]
          //   };
          //   var jos = jsonEncode(jo);
          //   API.setCallCenterData(result["f_pin"], jos);
          //   SharedObj.CALL_CENTER_SESSION_F_PIN = result["f_pin"];
          //   Navigator.push(context, PageTransition(type: PageTransitionType.fade,
          //       child: RequestCallCenter(
          //         name: result["real_name"],
          //         image: result["image_id"],
          //         deviceId: result["f_pin"],
          //         channel: result["channel"],
          //       )
          //   ));
        } else if (result["flag"] == "ACCEPT_CALL_CENTER") {
          SharedObj.CALL_CENTER_SESSION_COMPLAINT_ID = result["complaint_id"];
          Notifications.showNotifications(result, 2, context);
          // var jo = {
          //   'f_pin' : result["f_pin"],
          //   'status' : result["status"]
          // };
          // var jos = jsonEncode(jo);
          // API.setCallCenterData(result["f_pin"], jos);
          // var resultQuery = await DBHelper().queryCustom(
          //     "SELECT * from BUDDY WHERE f_pin = '${result["f_pin"]}'");
          // if (resultQuery.isNotEmpty) {
          //   Navigator.push(
          //       context,
          //       PageTransition(
          //         type: PageTransitionType.fade,
          //         child: ChatPage(
          //           result["real_name"],
          //           resultQuery[0]["image_id"],
          //           identifier: result["f_pin"],
          //           refresh: true,
          //           isContactCenter: true,
          //           data: result,
          //           channel: result["channel"],
          //         ),
          //       ));
          // }
        } else if (result["flag"] == "END_CALL_CENTER") {
          SharedObj.deleteCacheCallCenter();
        }
        return;
      } else {
        Notifications.showNotifications(result, 0, context);
      }
    };
  }

  void incomingCall(BuildContext context) {
    // CallNotifications.init(context);
    API.incomingCall = (int state, String message) {
      String idMe = "";
      var queryMe;
      print("message : " + message);
      String deviceId = message.substring(0, message.indexOf(","));
      print("DEVICE ID : " + deviceId);
      SharedPreferences.getInstance().then((prefs) {
        idMe = prefs.getString("hasLogin") ?? "";
      });
      DBHelper()
          .queryCustom(
              "SELECT first_name, last_name,f_pin, offline_mode, image_id FROM BUDDY")
          .then((value) {
        for (var i = 0; i < value.length; i++) {
          if (value[i]["f_pin"] == idMe) {
            queryMe = value[i];
            break;
          }
        }
      });
      DBHelper()
          .queryCustom(
              "SELECT first_name, last_name,f_pin, offline_mode, image_id FROM BUDDY WHERE f_pin = '$deviceId'")
          .then((result) async {
        print("Result : " + result.toString());
        if (queryMe["offline_mode"].toString() != "1") {
          if (state == 21) {
            print("fpin : " + result[0]["f_pin"]);
            await SharedObj.startRingtone();
            Notifications.showNotifications(result[0], 1, context,
                state: state);
            await Navigator.push(
                context,
                PageTransition(
                    type: PageTransitionType.fade,
                    child: AudioCall(
                      name:
                          "${result[0]["first_name"]} ${result[0]["last_name"]}",
                      image: result[0]["image_id"],
                      isInitiatior: 0,
                      deviceId: deviceId,
                    )));
            incomingCall(context);
          } else if (state == 31) {
            print("fpin : " + result[0]["f_pin"]);
            SharedObj.startRingtone();
            Notifications.showNotifications(result[0], 1, context,
                state: state);
            await Navigator.push(
                context,
                PageTransition(
                    type: PageTransitionType.fade,
                    child: VideoCall(
                      name:
                          "${result[0]["first_name"]} ${result[0]["last_name"]}",
                      isInitiatior: 0,
                      deviceId: deviceId,
                      cameraId: "1",
                      image: result[0]["image_id"],
                    )));
            incomingCall(context);
          }
        } else {
          API.terminateCall(deviceId);
          SharedObj.stopRingtone();
        }
      });
    };
  }

  @override
  Widget build(BuildContext context) {
    return showButton
        ? Positioned(
            left: rightPosition == null ? xPosition : null,
            top: bottomPosition == null ? yPosition : null,
            right: rightPosition,
            bottom: bottomPosition,
            child: GestureDetector(
              onPanUpdate: (tapInfo) {
                setState(() {
                  rightPosition = null;
                  bottomPosition = null;
                  xPosition += tapInfo.delta.dx;
                  yPosition += tapInfo.delta.dy;
                  // print(xPosition);
                  // print(yPosition);
                  if (state == 2) {
                    if (xPosition < 0 || yPosition < 0) {
                      if (xPosition.isNegative && yPosition.isNegative) {
                        xPosition = 0.0;
                        yPosition = 0.0;
                      } else if (xPosition >
                              MediaQuery.of(context).size.width - 60 &&
                          yPosition.isNegative) {
                        xPosition = MediaQuery.of(context).size.width - 60;
                        rightPosition = 0.0;
                        yPosition = 0.0;
                      } else if (xPosition.isNegative) {
                        xPosition = 0;
                      } else {
                        yPosition = 0;
                      }
                    } else if (xPosition >
                            MediaQuery.of(context).size.width - 60 ||
                        yPosition > MediaQuery.of(context).size.height - 142) {
                      if (xPosition > MediaQuery.of(context).size.width - 60 &&
                          yPosition >
                              MediaQuery.of(context).size.height - 142) {
                        xPosition = MediaQuery.of(context).size.width - 60;
                        yPosition = MediaQuery.of(context).size.height - 142;
                      } else if (xPosition >
                          MediaQuery.of(context).size.width - 60) {
                        xPosition = MediaQuery.of(context).size.width - 60;
                      } else {
                        yPosition = MediaQuery.of(context).size.height - 142;
                      }
                    }
                  } else {
                    if (xPosition < 0 || yPosition < 0) {
                      if (xPosition.isNegative && yPosition.isNegative) {
                        xPosition = 0.0;
                        yPosition = 0.0;
                      } else if (xPosition >
                              MediaQuery.of(context).size.width - 65 &&
                          yPosition.isNegative) {
                        xPosition = MediaQuery.of(context).size.width - 65;
                        rightPosition = 0.0;
                        yPosition = 0.0;
                      } else if (xPosition.isNegative) {
                        xPosition = 0;
                      } else {
                        yPosition = 0;
                      }
                    } else if (xPosition >
                            MediaQuery.of(context).size.width - 65 ||
                        yPosition > MediaQuery.of(context).size.height - 362) {
                      if (xPosition > MediaQuery.of(context).size.width - 65 &&
                          yPosition >
                              MediaQuery.of(context).size.height - 362) {
                        xPosition = MediaQuery.of(context).size.width - 65;
                        yPosition = MediaQuery.of(context).size.height - 362;
                      } else if (xPosition >
                          MediaQuery.of(context).size.width - 65) {
                        xPosition = MediaQuery.of(context).size.width - 65;
                      } else {
                        yPosition = MediaQuery.of(context).size.height - 362;
                      }
                    }
                  }
                });
              },
              child: Container(
                child: state == 1
                    ? Row(
                        mainAxisSize: MainAxisSize.min,
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          // showSeminarLS
                          //     ? Column(
                          //         crossAxisAlignment: CrossAxisAlignment.start,
                          //         mainAxisAlignment: MainAxisAlignment.start,
                          //         children: [
                          //           GestureDetector(
                          //             onTap: () {
                          //               setState(() {
                          //                 showSeminarLS = !showSeminarLS;
                          //               });
                          //               Navigator.push(
                          //                   context,
                          //                   PageTransition(
                          //                       type: PageTransitionType.fade,
                          //                       child: AddSeminar()));
                          //             },
                          //             child: Container(
                          //               margin: EdgeInsets.only(
                          //                   top: 5.0, right: 2.0),
                          //               alignment: Alignment.center,
                          //               width: 140,
                          //               height: 30,
                          //               decoration: BoxDecoration(
                          //                   borderRadius:
                          //                       BorderRadius.circular(8.0),
                          //                   border: Border.all(
                          //                       color: Colors.white,
                          //                       width: 1.0),
                          //                   color:
                          //                       Colors.black.withOpacity(0.8)),
                          //               child: Text("Seminar Session",
                          //                   style:
                          //                       TextStyle(color: Colors.white)),
                          //             ),
                          //           ),
                          //           SizedBox(height: 20.0),
                          //           GestureDetector(
                          //             onTap: () {
                          //               Navigator.push(
                          //                   context,
                          //                   PageTransition(
                          //                       type: PageTransitionType.fade,
                          //                       child: AddLiveStreaming()));
                          //             },
                          //             child: Container(
                          //               margin: EdgeInsets.only(right: 2.0),
                          //               alignment: Alignment.center,
                          //               width: 140,
                          //               height: 30,
                          //               decoration: BoxDecoration(
                          //                   borderRadius:
                          //                       BorderRadius.circular(8.0),
                          //                   border: Border.all(
                          //                       color: Colors.white,
                          //                       width: 1.0),
                          //                   color:
                          //                       Colors.black.withOpacity(0.8)),
                          //               child: Text("Video Promotion Session",
                          //                   style:
                          //                       TextStyle(color: Colors.white)),
                          //             ),
                          //           )
                          //         ],
                          //       )
                          //     :
                          // Container(),
                          Column(
                            children: [
                              Container(
                                width: 65,
                                height: 220,
                                decoration: BoxDecoration(
                                    color: Colors.grey[300],
                                    border: Border.all(
                                        color: Colors.white, width: 1.0),
                                    borderRadius: BorderRadius.circular(8.0)),
                                child: Column(
                                  crossAxisAlignment: CrossAxisAlignment.center,
                                  mainAxisAlignment: MainAxisAlignment.center,
                                  children: [
                                    GestureDetector(
                                      onTap: () async {
                                        final SharedPreferences prefs =
                                            await SharedPreferences
                                                .getInstance();
                                        String idMe =
                                            prefs.getString("hasLogin") ?? "";
                                        API.pullBatchBuddyList(idMe);
                                        var resultQueryMe = await DBHelper()
                                            .queryCustom(
                                                "SELECT * FROM BUDDY where f_pin='$idMe'");
                                        if (resultQueryMe[0]["user_type"] !=
                                            '24') {
                                          var queryResult = await DBHelper()
                                              .queryCustom(
                                                  "SELECT first_name, last_name,f_pin, offline_mode, image_id FROM BUDDY where official_account = '1'");
                                          if (queryResult.isNotEmpty) {
                                            var realName =
                                                "${queryResult[0]["first_name"]}${queryResult[0]["last_name"].isNotEmpty ? " " + queryResult[0]["last_name"] : ""}";
                                            // Navigator.push(
                                            //     context,
                                            //     PageTransition(
                                            //       type: PageTransitionType.fade,
                                            //       child: ChatPage(
                                            //         "$realName",
                                            //         queryResult[0]["image_id"],
                                            //         identifier: queryResult[0]
                                            //         ["f_pin"],
                                            //         refresh: true,
                                            //         isContactCenter: true,
                                            //       ),
                                            //     ));
                                            Navigator.pushNamed(
                                                context, '/contactCenter',
                                                arguments: {
                                                  'fullName': realName,
                                                  'image': queryResult[0]
                                                      ["image_id"],
                                                  'identifier': queryResult[0]
                                                      ["f_pin"],
                                                  'refresh': true,
                                                  'isContactCenter': true,
                                                });
                                          }
                                          return;
                                        }
                                        showDialog(
                                            context: context,
                                            builder: (BuildContext context) =>
                                                SimpleDialog(
                                                  children: <Widget>[
                                                    ListTile(
                                                      title: Text(
                                                          "Contact Call Center"),
                                                      onTap: () async {
                                                        var queryResult =
                                                            await DBHelper()
                                                                .queryCustom(
                                                                    "SELECT first_name, last_name,f_pin, offline_mode, image_id FROM BUDDY where official_account = '1'");
                                                        if (queryResult
                                                            .isNotEmpty) {
                                                          var real_name = queryResult[
                                                                      0][
                                                                  "first_name"] +
                                                              (queryResult[0][
                                                                          "last_name"]
                                                                      .isEmpty
                                                                  ? ""
                                                                  : " ${queryResult[0]['last_name']}");
                                                          Navigator.pop(
                                                              context);
                                                          // Navigator.push(
                                                          //     context,
                                                          //     PageTransition(
                                                          //       type: PageTransitionType.fade,
                                                          //       child: ChatPage(
                                                          //         real_name,
                                                          //         queryResult[0]["image_id"],
                                                          //         identifier: queryResult[0]
                                                          //         ["f_pin"],
                                                          //         refresh: true,
                                                          //         isContactCenter: false,
                                                          //       ),
                                                          //     ));
                                                          Navigator.pushNamed(
                                                              context,
                                                              '/contactCenter',
                                                              arguments: {
                                                                'fullName':
                                                                    real_name,
                                                                'image':
                                                                    queryResult[
                                                                            0][
                                                                        "image_id"],
                                                                'identifier':
                                                                    queryResult[
                                                                            0][
                                                                        "f_pin"],
                                                                'refresh': true,
                                                                'isContactCenter':
                                                                    false,
                                                              });
                                                        }
                                                        return;
                                                      },
                                                    ),
                                                    ListTile(
                                                      title: Text(
                                                          "Contact Center History"),
                                                      onTap: () {
                                                        Navigator.pop(context);
                                                        Navigator.push(
                                                            context,
                                                            PageTransition(
                                                              type:
                                                                  PageTransitionType
                                                                      .fade,
                                                              child:
                                                                  CallCenterHistoryPage(),
                                                            ));
                                                      },
                                                    )
                                                  ],
                                                ));
                                      },
                                      child: Image.asset(
                                          "assets/images/button_cc_1.png",
                                          package: 'palio_lite',
                                          width: 45,
                                          height: 45),
                                    ),
                                    SizedBox(height: 5.0),
                                    GestureDetector(
                                      onTap: () async {
                                        await Navigator.push(
                                            context,
                                            PageTransition(
                                                type: PageTransitionType.fade,
                                                child: NewChat()));
                                        receiveMessage(context);
                                      },
                                      child: Image.asset(
                                          "assets/images/button_chat_1.png",
                                          package: 'palio_lite',
                                          width: 45,
                                          height: 45),
                                    ),
                                    SizedBox(height: 5.0),
                                    GestureDetector(
                                      onTap: () {
                                        Navigator.push(
                                            context,
                                            PageTransition(
                                                type: PageTransitionType.fade,
                                                child: NewCall()));
                                      },
                                      child: Image.asset(
                                          "assets/images/button_call_1.png",
                                          package: 'palio_lite',
                                          width: 45,
                                          height: 45),
                                    ),
                                    SizedBox(height: 5.0),
                                    GestureDetector(
                                      onTap: () {
                                        Navigator.push(
                                            context,
                                            PageTransition(
                                                type: PageTransitionType.fade,
                                                child: AddLiveStreaming()));
                                        // setState(() {
                                        //   showSeminarLS = !showSeminarLS;
                                        // });
                                      },
                                      child: Image.asset(
                                          "assets/images/button_stream_1.png",
                                          package: 'palio_lite',
                                          width: 45,
                                          height: 45),
                                    )
                                  ],
                                ),
                              ),
                              GestureDetector(
                                onTap: () {
                                  setState(() {
                                    state = 2;
                                    // rightPosition = null;
                                    // xPosition = 60;
                                    // bottomPosition = null;
                                    // yPosition = MediaQuery.of(context).size.height / 3;
                                  });
                                },
                                onLongPress: () {
                                  Navigator.push(
                                      context,
                                      PageTransition(
                                          type: PageTransitionType.fade,
                                          child: SettingsView()));
                                },
                                child: Image.asset(
                                    "assets/images/palio_button.png",
                                    package: 'palio_lite',
                                    width: 60,
                                    height: 60),
                              )
                            ],
                          ),
                        ],
                      )
                    : GestureDetector(
                        onTap: () {
                          setState(() {
                            state = 1;
                            rightPosition = null;
                            xPosition = 300;
                            bottomPosition = null;
                            // yPosition = MediaQuery.of(context).size.height / 3;
                            yPosition =
                                MediaQuery.of(context).size.height - 500;
                          });
                        },
                        onLongPress: () {
                          Navigator.push(
                              context,
                              PageTransition(
                                  type: PageTransitionType.fade,
                                  child: SettingsView()));
                        },
                        child: Image.asset("assets/images/palio_button.png",
                            package: 'palio_lite', width: 60, height: 60),
                      ),
              ),
            ),
          )
        : Container();
  }
}
