import 'dart:async';
import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/palio_button.dart';
import 'package:palio_lite/shared_obj.dart';

import 'api.dart';
import 'app_color.dart';
import 'chat/chat_screen.dart';
import 'chat/widgets/widget_Image_Download.dart';

class RequestCallCenter extends StatefulWidget {
  String name;
  final String image;
  final String deviceId;
  final String channel;

  RequestCallCenter({required this.name, required this.image, required this.deviceId, required this.channel});

  _RequestCallCenter createState() => _RequestCallCenter();
}

class _RequestCallCenter extends State<RequestCallCenter> {
  String title = "";

  @override
  void initState() {
    title = "Contact Center";
    super.initState();
    // TODO: start timer
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisAlignment: MainAxisAlignment.start,
      mainAxisSize: MainAxisSize.min,
      children: [
        Container(
          margin: EdgeInsets.all(20.0),
          child: Text("Call Center",
              style: TextStyle(
                  color: Colors.grey[100],
                  fontWeight: FontWeight.bold,
                  fontSize: 18)),
        ),
        Container(
          margin: EdgeInsets.fromLTRB(20.0, 0.0, 20.0, 20.0),
          padding: EdgeInsets.all(5.0),
          decoration: BoxDecoration(
              color: Colors.white.withOpacity(0.5),
              borderRadius: BorderRadius.circular(5.0)),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              _profile(),
              SizedBox(
                width: 10.0,
              ),
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisAlignment: MainAxisAlignment.start,
                children: [
                  Text("Name", style: TextStyle(color: Colors.indigo)),
                  Text(widget.name,
                      style: TextStyle(
                          color: Colors.indigo, fontWeight: FontWeight.bold)),
                  SizedBox(
                    height: 10.0,
                  ),
                  Text("Request Type", style: TextStyle(color: Colors.indigo)),
                  Text(
                      widget.channel == "0"
                          ? "Chat"
                          : widget.channel == "1"
                              ? "Audio Call"
                              : widget.channel == "2"
                                  ? "Video Call"
                                  : "Email",
                      style: TextStyle(
                          color: Colors.indigo, fontWeight: FontWeight.bold)),
                  SizedBox(
                    height: 10.0,
                  ),
                  Text("Identity Number",
                      style: TextStyle(color: Colors.indigo)),
                  Text(widget.deviceId,
                      style: TextStyle(
                          color: Colors.indigo, fontWeight: FontWeight.bold)),
                ],
              )
            ],
          ),
        ),
        bottom()
      ],
    );
  }

  Widget _profile() {
    return Container(
        height: 140,
        width: 120,
        decoration: BoxDecoration(
            color: Colors.white.withOpacity(0.5),
            borderRadius: BorderRadius.circular(5.0),
            image: DecorationImage(
                fit: BoxFit.cover, image: LocalImage(widget.image))));
  }

  Widget bottom() {
    return Container(
      alignment: Alignment.bottomCenter,
      padding: EdgeInsets.symmetric(vertical: 18),
      // child: _toolbar(),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          _toolbar(),
          // Align(
          //   alignment: Alignment.bottomRight,
          //   child: _poweredBy(),
          // ),
        ],
      ),
    );
  }

  Widget _toolbar() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
      children: <Widget>[
        Container(
          width: 150,
          height: 60,
          color: Colors.white.withOpacity(0.5),
          child: OutlinedButton(
            child: Text(
              "PASS TO OTHER REPRESENTATIVE",
              textAlign: TextAlign.center,
              style: TextStyle(fontSize: 13, color: AppColor.primaryColor),
            ),
            onPressed: () {
              SharedObj.stopTimer();
              API.timeOutRequestCallCenter(widget.deviceId, widget.channel);
              Navigator.pop(context);
            },
          ),
        ),
        Container(
          width: 140,
          height: 60,
          child: OutlinedButton(
            style: ButtonStyle(
              backgroundColor: MaterialStateProperty.resolveWith<Color>(
                  (states) => AppColor.primaryColor),
            ),
            child: Text(
              "I'LL HANDLE THE CUSTOMER",
              textAlign: TextAlign.center,
              style: TextStyle(fontSize: 13, color: AppColor.secondaryColor),
            ),
            onPressed: () async {
              if (SharedObj.timerRequest == null ||
                  (SharedObj.CALL_CENTER_SESSION_F_PIN != widget.deviceId)) {
                Fluttertoast.showToast(msg: "Sorry, this session has expired");
                PalioButtonState statePalio = PalioButtonState();
                statePalio.receiveMessage(context);
                statePalio.incomingCall(context);
                Navigator.pop(context);
                return;
              }
              SharedObj.stopTimer();
              var complaintId = await API.acceptRequestCallCenter(
                  widget.deviceId, widget.channel);
              if (complaintId != null) {
                var data = {"complaint_id": complaintId};
                var chan = widget.channel;
                // assume sudah add friend
                SharedObj.CALL_CENTER_SESSION_F_PIN = widget.deviceId;
                await API.setLiveCenterSession(widget.deviceId, "2");
                print("Real Name request: ${widget.name}");
                print("ComplaintID request: ${complaintId}");
                print("Channel request: $chan");
                SharedObj.setCacheCallCenter(widget.deviceId, complaintId);
                // Navigator.pushReplacement(
                //     context,
                //     PageTransition(
                //       type: PageTransitionType.fade,
                //       child: ChatPage(
                //         fullName: widget.name,
                //         image: widget.image,
                //         identifier: widget.deviceId,
                //         refresh: true,
                //         isContactCenter: true,
                //         data: jsonEncode(data),
                //         channel: chan,
                //         contactCenterHistoryId: complaintId,
                //       ),
                //     ));
                Navigator.pushReplacementNamed(
                    context,
                    '/contactCenter',
                      arguments: {
                        'fullName': widget.name,
                        'image': widget.image,
                        'identifier': widget.deviceId,
                        'refresh': true,
                        'isContactCenter': true,
                        'data': jsonEncode(data),
                        'channel': chan,
                        'contactCenterHistoryId': complaintId,
                      });
              }
            },
          ),
        ),
      ],
    );
  }
}
