import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:page_transition/page_transition.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'app_color.dart';
import 'chat/chat_screen.dart';
import 'chat/widgets/widget_Image_Download.dart';

class ChangeState with ChangeNotifier {
  List<Map<dynamic, dynamic>> _dataSubAccount = [];

  List<Map<dynamic, dynamic>> get getDataSubAccount => _dataSubAccount;

  set dataSubAccount(Map<dynamic, dynamic> value) {
    _dataSubAccount.add(value);
    notifyListeners();
  }

  set dataSubAccounts(List<Map<dynamic, dynamic>> value) {
    _dataSubAccount.addAll(value);
    notifyListeners();
  }

  set clearDataSubAccount(bool value) {
    _dataSubAccount.clear();
    notifyListeners();
  }
}

class CallCenterHistoryPage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
        create: (_) => ChangeState(),
        child: Consumer<ChangeState>(
            builder: (context, changeState, _) =>
                CallCenterHistoryPages(changeState)));
  }
}

class CallCenterHistoryPages extends StatefulWidget {
  final ChangeState changeState;

  CallCenterHistoryPages(this.changeState);

  @override
  _CallCenterHistoryPagesState createState() => _CallCenterHistoryPagesState();
}

class _CallCenterHistoryPagesState extends State<CallCenterHistoryPages> {
  @override
  void initState() {
    getData();
    super.initState();
  }

  void getData() async {
    var callHistory = await DBHelper().queryCustom("SELECT * FROM CALL_CENTER_HISTORY where f_pin != '' ORDER BY time DESC");
    print(callHistory);
    for (var i = 0; i < callHistory.length; i++) {
      var ch = callHistory[i];
      var queryPerson = await DBHelper().queryCustom("SELECT * FROM BUDDY WHERE f_pin = '${ch["f_pin"]}'");
      String realName = "${queryPerson[0]["first_name"]} ${queryPerson[0]["last_name"]}";
      widget.changeState.dataSubAccount = {
        "thumb_id": queryPerson[0]["image_id"],
        "real_name": realName,
        "complaint_id" : ch["complaint_id"],
        "f_pin" : ch["f_pin"]
      };
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        automaticallyImplyLeading: true,
        title: Text("Contact Center History"),
        backgroundColor: AppColor.primaryColor,
        iconTheme: IconThemeData(color: Colors.white),
      ),
      body: Container(
        child: ListView.builder(
          itemCount: widget.changeState.getDataSubAccount.length,
          itemBuilder: (builder, index) {
            return Container(
              child: Column(
                children: [
                  Container(
                    height: 30,
                    alignment: Alignment.center,
                    decoration: BoxDecoration(color: AppColor.secondaryColor),
                    child: Text(
                      "Call Center",
                    ),
                  ),
                  ListTile(
                    leading: Container(
                      width: 45,
                      height: 45,
                      decoration: BoxDecoration(
                          shape: BoxShape.circle,
                          image: DecorationImage(
                              fit: BoxFit.cover,
                              // image: Image.asset(
                              //   "assets/images/ic_person_boy.png",
                              // ).image
                              image: LocalImage(widget.changeState
                                  .getDataSubAccount[index]["thumb_id"]))),
                    ),
                    title: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Container(
                          height: 20,
                          child: Text(
                            widget.changeState.getDataSubAccount[index]
                            ["real_name"],
                            textAlign: TextAlign.start,
                          ),
                        ),
                        Container(
                          height: 20,
                          child: Text(
                              widget.changeState.getDataSubAccount[index]
                              ["complaint_id"],
                              textAlign: TextAlign.start),
                        )
                      ],
                    ),
                    trailing: Container(
                      width: 48,
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.spaceAround,
                        children: <Widget>[
                          IconButton(
                            icon: Icon(Icons.message,
                                color: AppColor.primaryColor),
                            onPressed: () {
                              Navigator.push(
                                  context,
                                  PageTransition(
                                      type: PageTransitionType.fade,
                                      child: ChatPage(
                                        fullName: widget.changeState.getDataSubAccount[index]['real_name'],
                                        image: widget.changeState
                                            .getDataSubAccount[index]
                                        ["thumb_id"],
                                        identifier: widget.changeState
                                            .getDataSubAccount[index]["f_pin"],
                                        refresh: true,
                                        isContactCenter: true,
                                        isContactCenterHistory: true,
                                        contactCenterHistoryId: widget.changeState.getDataSubAccount[index]["complaint_id"],
                                        data: jsonEncode(widget.changeState.getDataSubAccount[index]),
                                      )));
                            },
                          ),
                        ],
                      ),
                    ),
                  )
                ],
              ),
            );
          },
        ),
      ),
    );
  }
}