import 'dart:async';

import 'package:flutter/scheduler.dart';
import 'package:palio_lite/chat/widgets/notifications.dart';
import 'package:palio_lite/db_helper.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:assets_audio_player/assets_audio_player.dart';

import 'api.dart';

class SharedObj {
  static String CALL_CENTER_SESSION_F_PIN = "";
  static String CALL_CENTER_SESSION_COMPLAINT_ID = "";
  static bool isOnGoingCallCenter = false;
  static Timer? timerRequest;
  static Timer? ringtone;
  static String tempChannel = "0";
  static bool ringtoneLoop = false;
  static AssetsAudioPlayer audioPlayer = AssetsAudioPlayer.newPlayer();

  static Future<void> startRingtone() async {
    try {
      audioPlayer.setLoopMode(LoopMode.single);
      audioPlayer.open(
          Playlist(
            audios: [Audio('assets/call_in.mp3', package: 'palio_lite')]
          ),
          respectSilentMode: false,
          showNotification: false,
          loopMode: LoopMode.single,
          audioFocusStrategy: AudioFocusStrategy.request(resumeAfterInterruption: true, resumeOthersPlayersAfterDone: false)
      );
      audioPlayer.playlistPlayAtIndex(0);
    }
    catch(e,s){
      print(e.toString());
      print(s.toString());
    }
    ringtoneLoop = true;
    ringtone = Timer.periodic(Duration(seconds: 3), (timer) {
        if(!ringtoneLoop){
          timer.cancel();
        }
    });
  }

  static Future<void> stopRingtone() async {
    if(audioPlayer != null){
      audioPlayer.stop();
    }
    ringtoneLoop = false;
  }

  static void clearNotificationCall() async{
    await Notifications.flutterLocalNotificationsPlugin.cancel(1);
  }

  static void initTimer(String f_pin, String channel){
    if(timerRequest == null) {
      CALL_CENTER_SESSION_F_PIN = f_pin;
      tempChannel = channel;
      timerRequest = Timer(Duration(seconds: 30), (){
        timerRequest?.cancel();
        timerRequest = null;
        CALL_CENTER_SESSION_F_PIN = "";
        tempChannel = "0";
        API.timeOutRequestCallCenter(f_pin, channel);
      });
    }
  }
  static void stopTimer(){
    if(timerRequest != null) {
      timerRequest?.cancel();
      timerRequest = null;
    }
    CALL_CENTER_SESSION_F_PIN = "";
    tempChannel = "0";

  }

  static void setCacheCallCenter(String f_pin, String complaint_id){
    isOnGoingCallCenter = true;
    CALL_CENTER_SESSION_F_PIN = f_pin;
    CALL_CENTER_SESSION_COMPLAINT_ID = complaint_id;
  }

  static void deleteCacheCallCenter(){
    isOnGoingCallCenter = false;
    CALL_CENTER_SESSION_F_PIN = "";
    CALL_CENTER_SESSION_COMPLAINT_ID = "";
    API.setCCSessionWelcome("");
  }

  static void disposeCallCenter(){
    if(SharedObj.CALL_CENTER_SESSION_F_PIN.isNotEmpty){


      API.endRequestCallCenter(SharedObj.CALL_CENTER_SESSION_F_PIN, SharedObj.CALL_CENTER_SESSION_COMPLAINT_ID);
      // Future.delayed(const Duration(seconds: 3), () async {
      //   SharedPreferences prefs = await SharedPreferences.getInstance();
      //   String idMe = prefs.getString("hasLogin");
      //   var queryMe = await DBHelper()
      //       .queryCustom(
      //       "SELECT * FROM BUDDY WHERE f_pin = '$idMe'");
      //   if(queryMe[0]["user_type"] == "24"){
      //     API.timeOutRequestCallCenter(SharedObj.CALL_CENTER_SESSION_F_PIN, SharedObj.CALL_CENTER_SESSION_COMPLAINT_ID);
      //   }
      // });
      API.setLiveCenterSession(SharedObj.CALL_CENTER_SESSION_F_PIN, "0");
      SharedObj.deleteCacheCallCenter();
    }
  }
}