import Flutter
import UIKit
import nuSDKService
import AVFoundation
import Photos
import PushKit
import PalioLite

public class SwiftPalioLitePlugin: NSObject, FlutterPlugin, FlutterStreamHandler, LoginDelegate, MessageDelegate, GroupDelegate, DownloadDelegate, CallDelegate, LiveStreamingDelegate, PersonInfoDelegate, ScreenSharingDelegate, CommentDelegate,UploadDelegate, WhiteboardReceiver,TimelineDelegate {
    
    public static var instance: SwiftPalioLitePlugin?
    
    public static var dispatch: DispatchGroup?
    
    var registrar: FlutterPluginRegistrar?
    
    var methodChannel: FlutterMethodChannel?
    
    var eventChannel: FlutterEventChannel?
    
    var eventSink: FlutterEventSink?
    
    var cameraView: CameraView?
    
    var remoteViews = [RemoteView]()
    
    var remoteView: RemoteView?
    
    var remoteChannel = [String:String]()
    
    var zoomView: ZoomView?
    
    var boardview: BoardView?
    
    public func onReceive(message: TMessage) {
        postEvent(name: "receiveMessage", map: message.mBodies)
    }
    
    public func onReceive(message: [AnyHashable: Any?]){
        postEvent(name: "receiveMessage", map: message)
    }
    
    public func onMessage(message: TMessage) {
        postEvent(name: "statusMessage", map: message.mBodies)
    }
    
    public func onTyping(message: TMessage) {
        postEvent(name: "typingStatus", map: message.mBodies)
    }
    
    public func openWhiteboard(opposite: String) {
        var map = [AnyHashable: Any?]()
        map[0] = opposite
        postEvent(name: "whiteboardReceiver", map: map)
    }
    
    public func removeWhiteboard() {
        var map = [AnyHashable: Any?]()
        postEvent(name: "whiteboardTerminate", map: map)
    }
    
    
    public func onUploadProgress(fileName: String, progress: Double) {
        var map = [AnyHashable: Any?]()
        map["name"] = fileName
        map["progress"] = progress
        postEvent(name: "uploadProgress", map: map)
    }
    
    public func onUpload(name: String, progress: Double) {
        var map = [AnyHashable: Any?]()
        map["name"] = name
        map["progress"] = progress
        postEvent(name: "upload", map: map)
    }
    
    public func onProgress(code: String, progress: Int) {
        var map = [AnyHashable: Any?]()
        map["code"] = code
        map["progress"] = progress
        postEvent(name: "loginProgress", map: map)
    }
    
    public func onProcess(message: String, status: String) {
        var map = [AnyHashable: Any?]()
        map["message"] = message
        map["status"] = status
        postEvent(name: "loginProcess", map: map)
    }
    
    public func onGroup(code: String, f_pin: String, groupId: String) {
        var map = [AnyHashable: Any?]()
        map["code"] = code
        map["f_pin"] = f_pin
        map["groupId"] = groupId
        postEvent(name: "group", map: map)
    }
    
    public func onTopic(code: String, f_pin: String, topicId: String) {
        var map = [AnyHashable: Any?]()
        map["code"] = code
        map["f_pin"] = f_pin
        map["topicId"] = topicId
        postEvent(name: "topic", map: map)
    }
    
    public func onMember(code: String, f_pin: String, groupId: String, member: String) {
        var map = [AnyHashable: Any?]()
        map["code"] = code
        map["f_pin"] = f_pin
        map["groupId"] = groupId
        map["member"] = member
        postEvent(name: "member", map: map)
    }
    
    public func onDownloadProgress(fileName: String, progress: Double) {
        var map = [AnyHashable: Any?]()
        map["fileName"] = fileName
        map["progress"] = progress
        postEvent(name: "downloadProgress", map: map)
    }
    
    public func onIncomingCall(state: Int, message: String) {
        var map = [AnyHashable: Any?]()
        map["state"] = state
        map["message"] = message
        postEvent(name: "incomingCall", map: map)
    }
    
    public func onStatusCall(state: Int, message: String) {
        var map = [AnyHashable: Any?]()
        let arrayMessage = message.split(separator: ",")
        print("message \(message)")
        let me = UserDefaults.standard.string(forKey: "me")!
        if(state == 35 && arrayMessage[3] == me){
            DispatchQueue.main.async {
                guard let zoom = self.zoomView?.zoomView else {
                    print("zoom view null")
                    return
                }
                print("zoom transform b \(zoom.transform.b)")
                print("zoom transform a \(zoom.transform.a)")
                let rotation = atan2(zoom.transform.b, zoom.transform.a)
                let angle = rotation * (180/CGFloat.pi)
                print("rotation \(rotation)")
                print("angle \(angle)")
                print("is rotate \(angle>=90.0)")
                if (angle >= 90.0 && angle <= 180.0){
                    print("is rotate")
                    zoom.transform   = CGAffineTransform.init(scaleX: -1, y: 1).rotated(by: (CGFloat.pi * 3)/2)
                }
            }
            //            let zoom = (zoomView?.zoomView)!
        }
        else if (state == 34){
            let channel = arrayMessage[arrayMessage.count - 1]
            if(remoteChannel[String(channel)] != "2"){
                DispatchQueue.main.async {
                    guard let zoom = self.zoomView?.zoomView else {
                        print("zoom view null")
                        return
                    }
                    if(arrayMessage[arrayMessage.count - 2] == "0"){ // back camera
                        zoom.transform = CGAffineTransform.init(scaleX: 1, y: 1).rotated(by: (CGFloat.pi)/2)
                    }
                    else { // front camera
                        zoom.transform = CGAffineTransform.init(scaleX: -1, y: 1).rotated(by: (CGFloat.pi * 3)/2)
                    }
                }
            }
        }
        else if (state == 32){
            let channel = arrayMessage[3]
            remoteChannel[String(channel)] = String(arrayMessage[5])
            if(arrayMessage[5] != "2"){
                DispatchQueue.main.async {
                    guard let zoom = self.zoomView?.zoomView else {
                        print("zoom view null")
                        return
                    }
                    zoom.transform   = CGAffineTransform.init(scaleX: 1, y: 1).rotated(by: (CGFloat.pi * 3)/2)
                }
            }
        } else if (state == 38) {
            DispatchQueue.main.async {
                let channel = arrayMessage[1]
                if(Int(arrayMessage[2])! == 2 && !self.remoteViews.isEmpty) {
                    self.remoteViews[0].view().isHidden = true
                    self.remoteViews[1].view().isHidden = true
                } else if(!self.remoteViews.isEmpty){
                    self.remoteViews[Int(channel)!].view().isHidden = true
                }
            }
        }
        map["state"] = state
        map["message"] = message
        postEvent(name: "statusCall", map: map)
    }
    
    public func onStartLS(state: Int, message: String) {
        var map = [AnyHashable: Any?]()
        map["state"] = state
        map["message"] = message
        postEvent(name: "startLS", map: map)
    }
    
    public func onJoinLS(state: Int, message: String) {
        print("message ls nusdk \(message)")
        if (state == 22){
            let arrayMessage = message.split(separator: ",")
            print("message ls offhook \(arrayMessage)")
            if(arrayMessage[3] != "2"){
                DispatchQueue.main.async {
                    var remote : RemoteView?
                    if (self.remoteViews.count > 0){
                        remote = self.remoteViews[0]
                    }
                    guard let views = remote?.remoteView else {
                        print("remote not ready")
                        return
                    }
                    if(arrayMessage[2] == "0") {
                        views.transform   = CGAffineTransform.init(scaleX: 1, y: 1).rotated(by: (CGFloat.pi * 5)/2)
                    } else {
                        views.transform   = CGAffineTransform.init(scaleX: 1, y: 1).rotated(by: (CGFloat.pi * 3)/2)
                    }
                }
            }
        } else if(state == 23){
            let arrayMessage = message.split(separator: ",")
            print("message ls rotate \(arrayMessage)")
            if(arrayMessage[2] == "0"){
                print("rotate view ls kamera belakang")
                DispatchQueue.main.async {
                    var remote : RemoteView?
                    if (self.remoteViews.count > 0){
                        remote = self.remoteViews[0]
                    }
                    guard let views = remote?.remoteView else {
                        print("remote not ready")
                        return
                    }
                    views.transform   = CGAffineTransform.init(scaleX: 1, y: 1).rotated(by: (CGFloat.pi * 5)/2)
                }
            }else{
                print("rotate view ls kamera depan")
                DispatchQueue.main.async {
                    var remote : RemoteView?
                    if (self.remoteViews.count > 0){
                        remote = self.remoteViews[0]
                    }
                    guard let views = remote?.remoteView else {
                        print("remote not ready")
                        return
                    }
                    views.transform   = CGAffineTransform.init(scaleX: 1, y: 1).rotated(by: (CGFloat.pi * 3)/2)
                }
            }
        }
        var map = [AnyHashable: Any?]()
        map["state"] = state
        map["message"] = message
        postEvent(name: "joinLS", map: map)
    }
    
    func onInitiateVideoCall(destination: String,state: Int, message: String) {
        var map = [AnyHashable: Any?]()
        map["destination"] = destination
        map["state"] = state
        map["message"] = message
        postEvent(name: "initiateVideoCallback", map: map)
    }
    
    func onVideoCallReceiverTerminate(originator: String,state: Int, message: String) {
        var map = [AnyHashable: Any?]()
        map["originator"] = originator
        map["state"] = state
        map["message"] = message
        postEvent(name: "videoCallReceiverTerminate", map: map)
    }
    
    func onAcceptVideoCall(originator: String,state: Int, message: String) {
        var map = [AnyHashable: Any?]()
        map["originator"] = originator
        map["state"] = state
        map["message"] = message
        postEvent(name: "acceptVideoCallback", map: map)
    }
    
    public func onUpdatePersonInfo(state: Int, message: String) {
        var map = [AnyHashable: Any?]()
        map["state"] = state
        map["message"] = message
        print ("Native plugin \(message)")
        postEvent(name: "personInfoCallback", map: map)
    }
    
    public func onReceiveComment(message: TMessage) {
        postEvent(name: "receiveComment", map: message.mBodies)
    }
    
    public func onDeleteComment(message: TMessage) {
        postEvent(name: "deleteComment", map: message.mBodies)
    }
    
    public func onPostUpdate(status: String, message: String) {
        var map = [AnyHashable: Any?]()
        map["status"] = status
        map["message"] = message
        postEvent(name: "postTimeline", map: map)
    }
    
    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        eventSink = events
        return nil
    }
    
    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        eventSink = nil
        return nil
    }
    
    public func postEvent(name: String, map: [AnyHashable: Any?]){
        var m = map
        m["event"] = name
        DispatchQueue.main.async {
            print("postEvent -> \(m) eventSink ? ", self.eventSink)
            if let event = self.eventSink {
                event(m)
            }
        }
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        if(instance == nil){
            instance = SwiftPalioLitePlugin()
            instance?.registrar = registrar
            instance?.methodChannel = FlutterMethodChannel(name: "palio_lite", binaryMessenger: registrar.messenger())
            instance?.eventChannel = FlutterEventChannel(name: "palio_lite_event", binaryMessenger: registrar.messenger())
            instance?.eventChannel?.setStreamHandler(instance!)
            instance?.registrar?.register(CameraFactory(plugin: instance!), withId: "CameraView")
            instance?.registrar?.register(RemoteFactory(plugin: instance!), withId: "RemoteView")
            instance?.registrar?.register(ZoomFactory(plugin: instance!), withId: "ZoomView")
            instance?.registrar?.register(BoardFactory(plugin: instance!), withId: "BoardView")
            
            // set delegate
            Palio.setMessageDelegate(delegate: instance!)
            Palio.setGroupDelegate(delegate: instance!)
            Palio.setCallDelegate(delegate: instance!)
            Palio.setLSDelegate(delegate: instance!)
            Palio.setScreenSharingDelegate(delegate: instance!)
            Palio.setPersonInfoDelegate(delegate: instance!)
            Palio.setCommentDelegate(delegate: instance!)
            Palio.setUploadDelegate(delegate: instance!)
            Palio.setTimelineDelegate(delegate: instance!)
            registrar.addMethodCallDelegate(instance!, channel: instance!.methodChannel!)
        }
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if(call.method == "getPlatformVersion"){
            result("iOS " + UIDevice.current.systemVersion)
        } else if call.method == "getPin" {
            self.handleGetPin(call: call, result: result)
        } else if call.method == "connect" {
            self.handleConnect(call: call, result: result)
        } else if call.method == "signUp" {
            self.handleSignUp(call: call, result: result)
        } else if call.method == "signIn" {
            self.handleSignIn(call: call, result: result)
        } else if call.method == "signUpOTP" {
            self.handleSignUpOTP(call: call, result: result)
        } else if call.method == "postRegistration" {
            self.handlePostRegistration(call: call, result: result)
        } else if call.method == "sendMessage" {
            self.handleSendMessage(call: call, result: result)
        } else if call.method == "pullBatchBuddy" {
            self.handleGetBatchBuddies(call: call, result: result)
        } else if call.method == "sendReadMessageStatus" {
            self.handleSendReadMessageStatus(call: call, result: result)
        } else if call.method == "deleteMessage" {
            self.handleDeleteMessage(call: call, result: result)
        } else if call.method == "createGroup" {
            self.handleCreateGroup(call: call, result: result)
        } else if call.method == "updateGroup" {
            self.handleUpdateGroup(call: call, result: result)
        } else if call.method == "addGroupMember" {
            self.handleAddGroupMember(call: call, result: result)
        } else if call.method == "changePosition" {
            self.handleChangeMemberPosition(call: call, result: result)
        } else if call.method == "exitGroup" {
            self.handleExitGroup(call: call, result: result)
        } else if call.method == "addChat" {
            self.handleAddChat(call: call, result: result)
        } else if call.method == "updateChat" {
            self.handleUpdateChat(call: call, result: result)
        } else if call.method == "deleteChat" {
            self.handleDeleteChat(call: call, result: result)
        } else if call.method == "download" {
            self.handleDownload(call: call, result: result)
        } else if call.method == "initiateAudioCall" {
            self.handleInitiateAudioCall(call: call, result: result)
        } else if call.method == "receiveAudioCall" {
            self.handleReceiveAudioCall(call: call, result: result)
        } else if call.method == "addAudioCall" {
            self.handleAddAudioCall(call: call, result: result)
        } else if call.method == "addVideoCall" {
            self.handleAddVideoCall(call: call, result: result)
        } else if call.method == "terminateCall" {
            self.handleTerminateCall(call: call, result: result)
        } else if call.method == "terminateConfCall" {
            self.handleTerminateConfCall(call: call, result: result)
        } else if call.method == "initiateVideoCall" {
            self.handleInitiateVideoCall(call: call, result: result)
        } else if call.method == "receiveVideoCall" {
            self.handleReceiveVideoCall(call: call, result: result)
        } else if call.method == "addFriend" {
            self.handleAddFriend(call: call, result: result)
        } else if call.method == "removeFriend"{
            self.handleRemoveFriend(call: call, result: result)
        } else if call.method == "retrieveTimeline" {
            self.handleRetrieveTimeline(call: call, result: result)
        } else if call.method == "retrievePostProfile" {
            self.handleRetrievePostProfile(call: call, result: result)
        } else if call.method == "retrieveChannel" {
            self.handleRetrieveChannel(call: call, result: result)
        } else if call.method == "retrieveNearby" {
            self.handleRetrieveNearby(call: call, result: result)
        } else if call.method == "searchPerson" {
            self.handleSearchPerson(call: call, result: result)
        } else if call.method == "trackPerson" {
            self.handleTrackPerson(call: call, result: result)
        } else if call.method == "statusCovid" {
            self.handleStatusCovid(call: call, result: result)
        } else if call.method == "retrieveNearbyDoctor" {
            self.handleRetrieveNearbyDoctor(call: call, result: result)
        } else if call.method == "suggestionPerson" {
            self.handleSuggestionPerson(call: call, result: result)
        } else if call.method == "searchFriend" {
            self.handleSearchFriend(call: call, result: result)
        } else if call.method == "pullMainContent" {
            self.handlePullMainContent(call: call, result: result)
        } else if call.method == "pullStoryList" {
            self.handlePullStoryList(call: call, result: result)
        } else if call.method == "createStory" {
            self.handleCreateStory(call: call, result: result)
        } else if call.method == "updateStory" {
            self.handleUpdateStory(call: call, result: result)
        } else if call.method == "pullStoryListPersonal" {
            self.handlePullStoryListPersonal(call: call, result: result)
        } else if call.method == "updateCounter" {
            self.handleUpdateCounter(call: call, result: result)
        } else if call.method == "submitKuisioner" {
            self.handleSubmitKuisioner(call: call, result: result)
        } else if call.method == "submitKuisionerAdditional" {
            self.handleSubmitKuisionerAdditional(call: call, result: result)
        } else if call.method == "startLS" {
            self.handleStartLiveStreaming(call: call, result: result)
        } else if call.method == "joinLS" {
            self.handleJoinLiveStreaming(call: call, result: result)
        } else if call.method == "leftLS" {
            self.handleLeftLiveStreaming(call: call, result: result)
        } else if call.method == "endLS" {
            self.handleEndLiveStreaming(call: call, result: result)
        } else if call.method == "sendLSBroadcast" {
            self.handleSendLSBroadcast(call: call, result: result)
        } else if call.method == "submitPost" {
            self.handleSubmitPost(call: call, result: result)
        } else if call.method == "submitPostInvitation" {
            self.handleSubmitPostInvitation(call: call, result: result)
        } else if call.method == "submitPostGroup" {
            self.handleSubmitPostGroup(call: call, result: result)
        } else if call.method == "submitPostEdu" {
            self.handleSubmitPostEdu(call: call, result: result)
        } else if call.method == "submitPostEduInvitation" {
            self.handleSubmitPostEduInvitation(call: call, result: result)
        } else if call.method == "submitPostEduGroup" {
            self.handleSubmitPostEduGroup(call: call, result: result)
        } else if call.method == "joinLiveVideo" {
            self.joinLiveVideo(call: call, result: result)
        } else if call.method == "leftLiveVideo" {
            self.leftLiveVideo(call: call, result: result)
        } else if call.method == "removeLiveVideo" {
            self.removeLiveVideo(call: call, result: result)
        } else if call.method == "sendLSChat" {
            self.handleSendLSChat(call: call, result: result)
        } else if call.method == "sendLSEmotion" {
            self.handleSendLSEmotion(call: call, result: result)
        } else if call.method == "updateLSTitle" {
            self.handleUpdateLSTitle(call: call, result: result)
        } else if call.method == "changeUserName" {
            self.handleChangePersonInfoName(call: call, result: result)
        } else if call.method == "changeUserPwd" {
            self.handleChangePersonInfoPassword(call: call, result: result)
        } else if call.method == "changeUserStatus" {
            self.handleChangePersonStatus(call: call, result: result)
        } else if call.method == "changeUserImage"{
            self.handleChangePersonImage(call: call, result: result)
        } else if call.method == "setAcceptCall" {
            self.handleSetAcceptCall(call: call, result: result)
        } else if call.method == "setPrivate" {
            self.handleSetPrivate(call: call, result: result)
        } else if call.method == "setSpeaker" {
            self.handleSetSpeakerOn(call: call, result: result)
        } else if call.method == "getLSData" {
            self.handleGetLSData(call: call, result: result)
        } else if call.method == "initiateVCRoom" {
            self.handleInitiateVCRoom(call: call, result: result)
        } else if call.method == "joinVCRoom" {
            self.handleJoinVCRoom(call: call, result: result)
        } else if call.method == "initiateACRoom" {
            self.handleInitiateACRoom(call: call, result: result)
        } else if call.method == "joinACRoom" {
            self.handleJoinACRoom(call: call, result: result)
        } else if call.method == "createLS" {
            self.handleCreateLS(call: call, result: result)
        } else if call.method == "requestLiveVideo" {
            self.handleRequestLiveVideo(call: call, result: result)
        } else if call.method == "createCRoom" {
            self.handleCreateVCRoom(call: call, result: result)
        } else if call.method == "startCRoom" {
            self.handleStartVCR(call: call, result: result)
        } else if call.method == "joinCRoom" {
            self.handleJoinVCR(call: call, result: result)
        } else if call.method == "endCRoom" {
            self.handleEndVCR(call: call, result: result)
        } else if call.method == "startScreenSharing" {
            self.handleStartScreenSharing(call: call, result: result)
        } else if call.method == "joinScreenSharing" {
            self.handleJoinScreenSharing(call: call, result: result)
        } else if call.method == "endScreenSharing" {
            self.handleEndScreenSharing(call: call, result: result)
        } else if call.method == "initScreenSharing" {
            self.handleInitScreenSharing(call: call, result: result)
        } else if call.method == "leaveScreenSharing" {
            self.handleLeaveScreenSharing(call: call, result: result)
        } else if call.method == "sendTyping" {
            self.handleSendTyping(call: call, result: result)
        } else if call.method == "getBuddyInfo" {
            self.handleGetBuddyInfo(call: call, result: result)
        } else if call.method == "sendToBot" {
            self.handleSendInvitationMyselfBot(call: call, result: result)
        } else if call.method == "sendAckMessage" {
            self.handleSendAckMessage(call: call, result: result)
        } else if call.method == "sendReport" {
            self.handleSendReport(call: call, result: result)
        } else if call.method == "getComment" {
            self.handleGetComment(call: call, result: result)
        } else if call.method == "sendComment" {
            self.handleSendComment(call: call, result: result)
        } else if call.method == "sendFollow" {
            self.handleSendFollow(call: call, result: result)
        } else if call.method == "sendUnFollow" {
            self.handleSendUnFollow(call: call, result: result)
        } else if call.method == "deleteComment" {
            self.handleDeleteComment(call: call, result: result)
        } else if call.method == "receiveUpdateComment" {
            self.handleReceiveUpdateComment(call: call, result: result)
        } else if call.method == "sendReaction" {
            self.handleSendReaction(call: call, result: result)
        } else if call.method == "clearChat" {
            self.handleClearChat(call: call, result: result)
        } else if call.method == "addLink" {
            self.handleAddLink(call: call, result: result)
        } else if call.method == "addMessageFavorite" {
            self.handleAddMessageFavorite(call: call, result: result)
        } else if call.method == "deleteMessageFavorite" {
            self.handleDeleteMessageFavorite(call: call, result: result)
        } else if call.method == "blockUser" {
            self.handleBlockUser(call: call, result: result)
        } else if call.method == "unblockUser" {
            self.handleUnblockUser(call: call, result: result)
        } else if call.method == "getOpenGroups" {
            self.handleGetOpenGroups(call: call, result: result)
        } else if call.method == "uploadImage" {
            self.handleUploadImage(call: call, result: result)
        } else if call.method == "sendOTPSignup" {
            self.handleSendOTPSignup(call: call, result: result)
        } else if call.method == "verifyOTP" {
            self.handleVerifyOTP(call: call, result: result)
        } else if call.method == "signinOTP" {
            self.handleSigninOTP(call: call, result: result)
        } else if call.method == "changeConnection" {
            self.handleChangeUserConnection(call: call, result: result)
        } else if call.method == "sendOTPChangeMSISDN" {
            self.handleSendOTPChangeMSISDN(call: call, result: result)
        } else if call.method == "changePersonMSISDN" {
            self.handleChangePersonMSISDN(call: call, result: result)
        } else if call.method == "initWhiteboard" {
            self.handleInitWhiteboard(call: call, result: result)
        } else if call.method == "joinWhiteboard" {
            self.handleJoinWhiteboard(call: call, result: result)
        } else if call.method == "terminateWhiteboard" {
            self.handleTerminateWhiteboard(call: call, result: result)
        } else if call.method == "clearWhiteboard" {
            self.handleClearWhiteboard(call: call, result: result)
        } else if call.method == "setWBPenSize" {
            self.handleWBPenSize(call: call, result: result)
        } else if call.method == "setWBPenColor" {
            self.handleWBPenColor(call: call, result: result)
        } else if call.method == "turnAudioCCall" {
            self.handleTurnCRoomAudio(call: call, result: result)
        } else if call.method == "saveToGallery" {
            self.handleSaveToGallery(call: call, result: result)
        } else if call.method == "setVibrate" {
            self.handleSetVibrate(call: call, result: result)
        } else if call.method == "vibrate" {
            self.handleVibrate(call: call, result: result)
        } else if call.method == "stopVibrate" {
            self.handleVibrate(call: call, result: result)
        } else if call.method == "createSubGroup" {
            self.handleCreateSubGroup(call: call, result: result)
        } else if call.method == "sendLoginEmail" {
            self.handleSendLoginEmail(call: call, result: result)
        } else if call.method == "sendVerificationEmail" {
            self.handleSendVerificationEmail(call: call, result: result)
        } else if call.method == "getUpdateUser" {
            self.handleGetUpdateUser(call: call, result: result)
        } else if call.method == "submitSchool" {
            self.handleSubmitSchool(call: call, result: result)
        } else if call.method == "approvalRequestTeacher" {
            self.handleGetRequestApprovalTeacher(call: call, result: result)
        } else if call.method == "requestStudent" {
            self.handleRequestStudent(call: call, result: result)
        } else if call.method == "getListClass" {
            self.handleGetListClassName(call: call, result: result)
        } else if call.method == "getListSchool" {
            self.handleGetListSchool(call: call, result: result)
        } else if call.method == "getListSchoolByName" {
            self.handleGetListSchoolByName(call: call, result: result)
        } else if call.method == "deletePost" {
            self.handleDeletePostTimeline(call: call, result: result)
        } else if call.method == "isInitiatorJoin" {
            self.handleIsInitiatorJoin(call: call, result: result)
        } else if call.method == "startLPInvited" {
            self.handleStartLPInvited(call: call, result: result)
        } else if call.method == "getRequestStudent" {
            self.handleGetRequestStudent(call: call, result: result)
        } else if call.method == "approveRequestStudent" {
            self.handleApproveRequestStudent(call: call, result: result)
        } else if call.method == "getRequestTeacher" {
            self.handleGetRequestTeacher(call: call, result: result)
        } else if call.method == "approveRequestTeacher" {
            self.handleApproveRequestTeacher(call: call, result: result)
        } else if call.method == "getListSubAccount" {
            self.handleGetListSubAccount(call: call, result: result)
        } else if call.method == "quizDetail" {
            self.handleQuizDetail(call: call, result: result)
        } else if call.method == "quizAnswer" {
            self.handleQuizAnswer(call: call, result: result)
        } else if call.method == "quizScoring" {
            self.handleQuizScoring(call: call, result: result)
        } else if call.method == "requestCallCenter" {
            self.handleRequestCallCenter(call: call, result: result)
        } else if call.method == "requestEmailCallCenter" {
            self.handleRequestEmailCallCenter(call: call, result: result)
        } else if call.method == "acceptRequestCallCenter" {
            self.handleAcceptRequestCallCenter(call: call, result: result)
        } else if call.method == "endRequestCallCenter" {
            self.handleEndRequestCallCenter(call: call, result: result)
        } else if call.method == "getFeatureAccess" {
            self.handleGetFeatureAccess(call: call, result: result)
        } else if call.method == "broadcastMessage" {
            self.handleBroadcastMessage(call: call, result: result)
        } else if call.method == "timeOutRequestCallCenter" {
            self.handleTimeOutRequestCallCenter(call: call, result: result)
        } else if call.method == "managementContactCenter" {
            self.handleManagementContactCenter(call: call, result: result)
        } else if call.method == "signInCreator" {
            self.handleSignInCreator(call: call, result: result)
        } else if call.method == "signInAdmin" {
            self.handleSignInAdmin(call: call, result: result)
        } else if call.method == "signInInternal" {
            self.handleSignInInternal(call: call, result: result)
        } else if call.method == "changePasswordAdmin" {
            self.handleChangePasswordAdmin(call: call, result: result)
        } else if call.method == "changePasswordInternal" {
            self.handleChangePasswordInternal(call: call, result: result)
        } else if call.method == "queuingCallCenter" {
            self.handleQueuingCallCenter(call: call, result: result)
        } else if call.method == "statusContactCenter" {
            self.handleStatusContactCenter(call: call, result: result)
        } else if call.method == "setLiveCenterSession" {
            self.handleSetLiveCenterSession(call: call, result: result)
        } else if call.method == "setCallCenterDateStart" {
            self.handleSetCallCenterDateStart(call: call, result: result)
        } else if call.method == "setCallCenterData" {
            self.handleSetCallCenterData(call: call, result: result)
        } else if call.method == "setCCSessionWelcome" {
            self.handleSetCCSessionWelcome(call: call, result: result)
        }
    }
    
    var whiteboard: Whiteboard? = nil
    
    private func handleGetPin(call: FlutterMethodCall, result: @escaping FlutterResult){
        var id = ""
        if let me = UserDefaults.standard.string(forKey: "me") {
            id = me
        } else {
            let uuid = UIDevice.current.identifierForVendor?.uuidString ?? "UNK-DEVICE"
            id = uuid
        }
        result(id)
    }
    
    private func handleConnect(call: FlutterMethodCall, result: @escaping FlutterResult){

        let data = call.arguments as! [AnyHashable: Any?]
        let apiKey = data["api_key"] as! String
        
        result(Palio.connect(apiKey:apiKey))
    }
    
    private func handleRequestCallCenter(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_channel = data["p_channel"] as! Int
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getRequestCallCenter(p_channel: p_channel)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: ""))
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleRequestEmailCallCenter(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_channel = data["p_channel"] as! Int
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getRequestEmailCallCenter(p_channel: p_channel)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.EMAIL, default_value: ""))
                }
                return
            }
        }
    }
    
    private func handleAcceptRequestCallCenter(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let channel = data["channel"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.acceptRequestCallCenter(channel: channel, l_pin: l_pin)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: ""))
                }
                return
            }
            result(nil)
        }
    }
    
    private func handleEndRequestCallCenter(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let complaint_id = data["complaint_id"] as! String
            _ = Palio.write(message: CoreMessage_TMessageBank.endCallCenter(complaint_id: complaint_id, l_pin: l_pin))
            result("done")
        }
    }
    
    private func handleGetFeatureAccess(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let key = data["key"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getFeatureAccess(key: key)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: ""))
                }
                return
            }
        }
    }
    
    private func handleBroadcastMessage(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let title = data["title"] as! String
            let broadcast_flag = data["broadcast_flag"] as! String
            let message = data["message"] as! String
            let starting_date = data["starting_date"] as! Int64
            let ending_date = data["ending_date"] as! Int64
            let destination = data["destination"] as! String
            let dataBroadcast = data["data"] as! String
            let category_flag = data["category_flag"] as! String
            let notification_type = data["notification_type"] as! String
            let link = data["link"] as! String
            let thumb_id = data["thumb_id"] as! String
            let file_id = data["file_id"] as! String
            
            if !thumb_id.isEmpty {
                Network().upload(name: String(thumb_id)) { (result1, progress) in
                    if result1 {
                        if progress == 100 {
                            Network().upload(name: String(file_id)) { (result2, progress) in
                                if result2 {
                                    if progress == 100 {
                                        if let response = Palio.writeSync(message: CoreMessage_TMessageBank.broadcastMessage(title: title, broadcast_flag: broadcast_flag, message: message, starting_date: starting_date, ending_date: ending_date, destination: destination, data: dataBroadcast, category_flag: category_flag, notification_type: notification_type, link: link, thumb_id: thumb_id, file_id: file_id)) {
                                            result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
                                            return
                                        }
                                    }
                                } else {
                                    result(nil)
                                }
                            }
                        }
                    } else {
                        result(nil)
                    }
                }
            } else if !file_id.isEmpty {
                Network().upload(name: String(file_id)) { (result2, progress) in
                    if result2 {
                        if progress == 100 {
                            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.broadcastMessage(title: title, broadcast_flag: broadcast_flag, message: message, starting_date: starting_date, ending_date: ending_date, destination: destination, data: dataBroadcast, category_flag: category_flag, notification_type: notification_type, link: link, thumb_id: "", file_id: file_id)) {
                                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
                                return
                            }
                        }
                    }
                    result(nil)
                }
            } else {
                if let response = Palio.writeSync(message: CoreMessage_TMessageBank.broadcastMessage(title: title, broadcast_flag: broadcast_flag, message: message, starting_date: starting_date, ending_date: ending_date, destination: destination, data: dataBroadcast, category_flag: category_flag, notification_type: notification_type, link: link, thumb_id: "", file_id: "")) {
                    result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
                    return
                }
            }
        }
    }
    
    private func handleTimeOutRequestCallCenter(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let channel = data["channel"] as! String
            _ = Palio.write(message: CoreMessage_TMessageBank.timeOutRequestCallCenter(channel: channel, l_pin: l_pin))
        }
    }
    
    private func handleManagementContactCenter(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let user_type = data["user_type"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getManagementContactCenter(user_type: user_type, l_pin: l_pin)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
            }
        }
    }
    
    private func handleSignInCreator(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_name = data["p_name"] as! String
            let p_password = data["p_password"] as! String
            let md5Hex = Utils.getMD5(string: p_password).map { String(format: "%02hhx", $0) }.joined()
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getSignInApiCreator(p_name: p_name, p_password: md5Hex)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
            }
        }
    }
    
    private func handleSignInAdmin(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_name = data["p_name"] as! String
            let p_password = data["p_password"] as! String
            let md5Hex = Utils.getMD5(string: p_password).map { String(format: "%02hhx", $0) }.joined()
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getSignInApiAdmin(p_name: p_name, p_password: md5Hex)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
            }
        }
    }
    
    private func handleSignInInternal(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_name = data["p_name"] as! String
            let p_password = data["p_password"] as! String
            let md5Hex = Utils.getMD5(string: p_password).map { String(format: "%02hhx", $0) }.joined()
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getSignInApiInternal(p_name: p_name, p_password: md5Hex)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
            }
        }
    }
    
    private func handleChangePasswordAdmin(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_f_pin = data["p_f_pin"] as! String
            let pwd_en = data["pwd_en"] as! String
            let pwd_old = data["pwd_old"] as! String
            let md5HexEn = Utils.getMD5(string: pwd_en).map { String(format: "%02hhx", $0) }.joined()
            let md5HexOld = Utils.getMD5(string: pwd_old).map { String(format: "%02hhx", $0) }.joined()
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getChangePasswordAdmin(p_f_pin: p_f_pin, pwd_en: md5HexEn, pwd_old: md5HexOld)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
            }
        }
    }
    
    private func handleChangePasswordInternal(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_f_pin = data["p_f_pin"] as! String
            let pwd_en = data["pwd_en"] as! String
            let pwd_old = data["pwd_old"] as! String
            let md5HexEn = Utils.getMD5(string: pwd_en).map { String(format: "%02hhx", $0) }.joined()
            let md5HexOld = Utils.getMD5(string: pwd_old).map { String(format: "%02hhx", $0) }.joined()
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getChangePasswordInternal(p_f_pin: p_f_pin, pwd_en: md5HexEn, pwd_old: md5HexOld)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
            }
        }
    }
    
    private func handleQueuingCallCenter(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_channel = data["p_channel"] as! Int
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getQueuingCallCenter(p_channel: p_channel)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
                return
            }
            result(nil)
        }
    }
    
    private func handleStatusContactCenter(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_complaint_id = data["p_complaint_id"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getStatusContactCenter(p_complaint_id: p_complaint_id)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.STATUS, default_value: ""))
                }
                return
            }
        }
    }
    
    private func handleInitWhiteboard(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let me = UserDefaults.standard.string(forKey: "me")!
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            self.whiteboard = Whiteboard(destination: l_pin, canvas: (self.boardview?.boardView)!)
            Palio.setWhiteboard(whiteboard: self.whiteboard!)
            Palio.getWhiteboard()?.sendInit()
            _ = Palio.writeDraw(data: "WB/0/\(me)/\(l_pin)")
        }
    }
    
    private func handleCreateSubGroup(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let group_name = data["group_name"] as! String
            let id = CoreMessage_TMessageUtil.getTID()
            let parent = data["parent"] as! String
            let level = data["level"] as! String
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getCreateSubGroup(group_id: id, group_name: group_name, parent_id: parent, level: level)) {
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "GROUPZ", cvalues: [
                            "group_id" : id,
                            "f_name" : group_name,
                            "created_date" : Date().currentTimeMillis(),
                            "created_by" : UserDefaults.standard.string(forKey: "me")!,
                            "level" : level,
                            "parent" : parent
                        ], replace: true)
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                    result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleJoinWhiteboard(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            self.whiteboard = Whiteboard(destination: l_pin, canvas: (self.boardview?.boardView)!)
            Palio.setWhiteboard(whiteboard: self.whiteboard!)
            Palio.getWhiteboard()?.sendJoin()
            _ = Palio.writeDraw(data: "WB/22/\(l_pin)")
        }
    }
    
    private func handleClearWhiteboard(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            self.whiteboard?.clearWhiteboard()
            _ = Palio.writeDraw(data: "WB/3\(NSDate().timeIntervalSince1970 * 1000)")
        }
    }
    
    private func handleTerminateWhiteboard(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            Palio.getWhiteboard()?.sendTerminate()
            _ = Palio.writeDraw(data: "WB/88")
        }
    }
    
    private func handleWBPenSize(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let size = data["size"] as! Double
            self.whiteboard?.setPenSize(size: size)
        }
    }
    
    private func handleWBPenColor(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let color = data["color"] as! Int
            self.whiteboard?.setPenColor(color: color)
        }
    }
    
    private func handleGetOpenGroups(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_account = data["p_account"] as! String
            let offset = data["offset"] as! String
            let search = data["search"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getOpenGroups(p_account: p_account, offset: offset, search: search)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: "[]"))
                }
                return
            }
        }
    }
    
    private func handleBlockUser(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let block = data["block"] as! String
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getBlock(l_pin: l_pin)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    Database.shared.database?.inTransaction({ (fmdb, rollback) in
                        _ = Database.shared.updateRecord(fmdb: fmdb, table: "BUDDY", cvalues: [
                            "ex_block" : block
                        ], _where: "f_pin = '\(l_pin)'")
                    })
                }
                result(errCode)
                return
            }
        }
    }
    
    private func handleUnblockUser(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let block = data["block"] as! String
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getUnBlock(l_pin: l_pin)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    Database.shared.database?.inTransaction({ (fmdb, rollback) in
                        _ = Database.shared.updateRecord(fmdb: fmdb, table: "BUDDY", cvalues: [
                            "ex_block" : block
                        ], _where: "f_pin = '\(l_pin)'")
                    })
                }
                result(errCode)
                return
            }
        }
    }
    
    private func handleDeleteMessageFavorite(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let message_id = data["message_id"] as! String
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE_FAVORITE", _where: "message_id='\(message_id)'")
            })
            result(nil)
        }
    }
    
    private func handleAddMessageFavorite(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let message_id = data["message_id"] as! String
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                do {
                    _ = try Database.shared.insertRecord(fmdb: fmdb, table: "MESSAGE_FAVORITE", cvalues: [
                        "message_id" : message_id,
                    ], replace: true)
                } catch {
                    rollback.pointee = true
                    print(error)
                }
            })
            result(nil)
        }
    }
    
    private func handleAddLink(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let id_link = data["id_link"] as! String
            let link = data["link"] as! String
            let data_link = data["data_link"] as! String
            let retry = data["retry"] as! Int
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                do {
                    _ = try Database.shared.insertRecord(fmdb: fmdb, table: "LINK_PREVIEW", cvalues: [
                        "id" : id_link,
                        "link" : link,
                        "data_link" : data_link,
                        "retry" : retry
                    ], replace: true)
                } catch {
                    rollback.pointee = true
                    print(error)
                }
            })
            result(nil)
        }
    }
    
    private func handleClearChat(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            let l_pin = data["l_pin"] as! String
            let chat_id = data["chat_id"] as! String
            let id_summary = data["id_summary"] as! String
            let scope = data["scope"] as! String
            if (scope == "3") {
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE", _where: "(f_pin='\(l_pin)' or l_pin='\(l_pin)') and message_scope_id='\(scope)'")
                    _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE_SUMMARY", _where: "l_pin='\(id_summary)'")
                })
            } else {
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    if (chat_id.isEmpty) {
                        _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE", _where: "chat_id = '' AND l_pin='\(l_pin)'")
                    } else {
                        _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE", _where: "chat_id = '\(chat_id)'")
                    }
                    _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE_SUMMARY", _where: "l_pin='\(id_summary)'")
                })
            }
            result(nil)
        }
    }
    
    
    private func handleSendReaction(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let post_id = data["post_id"] as! String
            let flag_reaction = data["flag_reaction"] as! String // 0: unlike, 1: like, -1: dislike
            let last_update = data["last_update"] as! String
            let beforeFlagChanged = data["beforeFlagChanged"] as! String // 0: unlike, 1: like, -1: dislike
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getReaction(post_id: post_id, flag_reaction: flag_reaction, last_update: last_update, beforeFlagChanged: beforeFlagChanged, lac: "", cid: "", mcc: "", mnc: "", pci: "")) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    Database.shared.database?.inTransaction({ (fmdb, rollback) in
                        var total_like = "+ 1"
                        if flag_reaction == "0" {
                            total_like = "- 1"
                        }
                        _ = Database.shared.rawQuery(fmdb: fmdb, queries: "update POST set total_like = total_like \(total_like), like_flag = \(flag_reaction) where post_id = '\(post_id)'")
                    })
                }
                result(errCode)
                return
            }
            result(nil)
        }
    }
    
    private func handleReceiveUpdateComment(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let post_id = data["post_id"] as! String
            let status = data["status"] as! String // 1: open, 2: close editor
            _ = Palio.write(message: CoreMessage_TMessageBank.getUpadateComment(post_id: post_id, status: status))
            result(nil)
        }
    }
    
    private func handleDeleteComment(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let comment_id = data["comment_id"] as! String
            let post_id = data["post_id"] as! String
            let ref_comment_id = data["ref_comment_id"] as! String
            _ = Palio.write(message: CoreMessage_TMessageBank.deleteComment(comment_id: comment_id, post_id: post_id, ref_comment_id: ref_comment_id))
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                _ = Database.shared.rawQuery(fmdb: fmdb, queries: "update POST set total_comment = total_comment - 1 where post_id = '\(post_id)'")
            })
            result(nil)
        }
    }
    
    private func handleSendUnFollow(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let post_id = data["post_id"] as! String
            let l_pin = data["l_pin"] as! String
            _ = Palio.write(message: CoreMessage_TMessageBank.getUnFollow(post_id: post_id, l_pin: l_pin))
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                do {
                    _ = try Database.shared.deleteRecord(fmdb: fmdb, table: "FOLLOW", _where: "f_pin = '\(l_pin)'")
                } catch {
                    rollback.pointee = true
                    print(error)
                }
            })
            if let delegate = Palio.getTimelineDelegate() {
                delegate.onPostUpdate(status: "Unfollow", message: "Success")
            }
            result(nil)
        }
    }
    
    private func handleSendFollow(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let post_id = data["post_id"] as! String
            let l_pin = data["l_pin"] as! String
            let followDate = data["followDate"] as! String
            _ = Palio.write(message: CoreMessage_TMessageBank.getFollow(post_id: post_id, l_pin: l_pin, followDate: followDate))
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                do {
                    _ = try Database.shared.insertRecord(fmdb: fmdb, table: "FOLLOW", cvalues: [
                        "f_pin" : l_pin
                    ], replace: true)
                } catch {
                    rollback.pointee = true
                    print(error)
                }
            })
            if let delegate = Palio.getTimelineDelegate() {
                delegate.onPostUpdate(status: "Follow", message: "Success")
            }
            
            result(nil)
        }
    }
    
    private func handleSendComment(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let image = data["image"] as! String
            let name = data["name"] as! String
            let comment_id = data["comment_id"] as! String
            let post_id = data["post_id"] as! String
            let ref_comment_id = data["ref_comment_id"] as! String
            let comment = data["comment"] as! String
            let commentDate = data["commentDate"] as! String
            _ = Palio.write(message: CoreMessage_TMessageBank.sendComment(image: image, name: name, comment_id: comment_id, post_id: post_id, ref_comment_id: ref_comment_id, comment: comment, commentDate: commentDate, lac: "", cid: "", mcc: "", mnc: "", pci: ""))
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                _ = Database.shared.rawQuery(fmdb: fmdb, queries: "update POST set total_comment = total_comment + 1 where post_id = '\(post_id)'")
            })
            if let delegate = Palio.getTimelineDelegate() {
                delegate.onPostUpdate(status: "Comment", message: "Success")
            }
            result(nil)
        }
    }
    
    private func handleGetComment(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let post_id = data["post_id"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getComment(post_id: post_id)) {
                let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                result(data)
                return
            }
            result(nil)
        }
    }
    
    private func handleSendReport(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let post_id = data["post_id"] as! String
            let report_date = data["report_date"] as! String
            let reason = data["reason"] as! String
            _ = Palio.write(message: CoreMessage_TMessageBank.getReport(post_id: post_id, report_date: report_date, reason: reason))
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                _ = Database.shared.updateRecord(fmdb: fmdb, table: "POST", cvalues: [
                    "report_flag" : "1"
                ], _where: "post_id = '\(post_id)'")
            })
            result(nil)
        }
    }
    
    private func handleSendAckMessage(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let f_pin = data["f_pin"] as! String
        let l_pin = data["l_pin"] as! String
        let scope = data["scope"] as! String
        let server_date = data["server_date"] as! String
        let message_id = data["message_id"] as! String
        let longitude = data["longitude"] as! String
        let latitude = data["latitude"] as! String
        let description = data["description"] as! String
        let tmessage = CoreMessage_TMessageBank.getAckLocationMessage(f_pin: f_pin, message_id: message_id, l_pin: l_pin, server_date: server_date, message_scope_id: scope, longitude: longitude, latitude: latitude, description: description)
        Database.shared.database?.inTransaction({ (fmdb, rollback) in
            do {
                _ = Database.shared.updateRecord(fmdb: fmdb, table: "MESSAGE", cvalues: [
                    "status" : "8"
                ], _where: "message_id = '\(message_id)'")
            } catch {
                rollback.pointee = true
                print(error)
            }
        })
        _ = Palio.write(message: tmessage)
        result(nil)
    }
    
    private func handleGetBuddyInfo(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let l_pin = data["l_pin"] as! String
        Database.shared.database?.inTransaction({ (fmdb, rollback) in
            if let cursor = Database.shared.getRecords(fmdb: fmdb, query: "select last_update from BUDDY where f_pin = '\(l_pin)'"), cursor.next() {
                if let lastUpdate = cursor.string(forColumnIndex: 0), let l = Int(lastUpdate) {
                    _ = Palio.write(message: CoreMessage_TMessageBank.getBuddyInfo(l_pin: l_pin, last_update: l))
                }
                cursor.close()
            }
        })
        result(nil)
    }
    
    private func handleSendTyping(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let l_pin = data["l_pin"] as! String
        let scope = data["scope"] as! String
        let status = data["status"] as! String
        let tmessage = CoreMessage_TMessageBank.getUpdateTypingStatus(p_opposite: l_pin, p_scope: scope, p_status: status)
        _ = Palio.write(message: tmessage)
        result(nil)
    }
    
    private func handleDeleteMessage(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let l_pin = data["l_pin"] as! String
        let message_id = data["message_id"] as! String
        let scope = data["scope"] as! String
        let type = data["type"] as! String // 1 = delete for me, 2 = delete for everyone
        let chat = data["chat"] as! String
        let tmessage = CoreMessage_TMessageBank.deleteMessage(l_pin: l_pin, messageId: message_id, scope: scope, type: type, chat: chat)
        Palio.addQueueMessage(message: tmessage)
        result(nil)
    }
    
    private func handleSubmitPost(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            guard let me = UserDefaults.standard.string(forKey: "me") else {
                result(nil)
                return
            }
            let data = call.arguments as! [AnyHashable: Any?]
            let id = CoreMessage_TMessageUtil.getTID()
            let author_name = data["author_name"] as! String
            let author_thumb = data["author_thumb"] as! String
            let title = data["title"] as! String
            let description = data["description"] as! String
            let link = data["link"] as! String
            let type = data["type"] as! String
            let created_date = String(Date().currentTimeMillis())
            let audition_date = created_date
            let thumb_id = data["thumb_id"] as! String
            let privacy = data["privacy"] as! String
            let file_id = data["file_id"] as! String
            let category = data["category"] as! String
            let file_type = data["file_type"] as! String
            let pricing = data["pricing"] as! String
            let pricing_money = data["pricing_money"] as! String
            if !thumb_id.isEmpty {
                let thumbs = thumb_id.split(separator: "|")
                let group = DispatchGroup()
                for thumb in thumbs {
                    group.enter()
                    Network().upload(name: String(thumb)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.thumb.wait")
                    group.wait()
                }
            }
            if !file_id.isEmpty {
                let files = file_id.split(separator: "|")
                let group = DispatchGroup()
                for file in files {
                    group.enter()
                    Network().upload(name: String(file)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.file.wait")
                    group.wait()
                }
            }
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getUploadTimeline(post_id: id, title: title, description: description, link: link, type: type, created_date: created_date, audition_date: audition_date, thumb_id: thumb_id, privacy: privacy, file_id: file_id, video_duration: "0", category: category, file_type: file_type, ads_type: "1")) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "POST", cvalues: [
                            "post_id" : id,
                            "author_f_pin" : me,
                            "author_name":author_name,
                            "author_thumbnail":author_thumb,
                            "title" : title,
                            "description" : description,
                            "link" : link,
                            "type" : type,
                            "created_date" : created_date,
                            "audition_date" : audition_date,
                            "thumb_id" : thumb_id,
                            "privacy" : privacy,
                            "file_id" : file_id,
                            "file_type" : file_type,
                            "target":"0",
                            "pricing":pricing,
                            "pricing_money":pricing_money
                        ], replace: true)
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleSubmitPostInvitation(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            guard let me = UserDefaults.standard.string(forKey: "me") else {
                result(nil)
                return
            }
            let data = call.arguments as! [AnyHashable: Any?]
            let id = CoreMessage_TMessageUtil.getTID()
            let author_name = data["author_name"] as! String
            let author_thumb = data["author_thumb"] as! String
            let title = data["title"] as! String
            let description = data["description"] as! String
            let link = data["link"] as! String
            let type = data["type"] as! String
            let created_date = String(Date().currentTimeMillis())
            let audition_date = created_date
            let thumb_id = data["thumb_id"] as! String
            let privacy = data["privacy"] as! String
            let file_id = data["file_id"] as! String
            let category = data["category"] as! String
            let file_type = data["file_type"] as! String
            let target = data["target"] as! String
            let members = data["members"] as! String
            let pricing = data["pricing"] as! String
            let pricing_money = data["pricing_money"] as! String
            if !thumb_id.isEmpty {
                let thumbs = thumb_id.split(separator: "|")
                let group = DispatchGroup()
                for thumb in thumbs {
                    group.enter()
                    Network().upload(name: String(thumb)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.thumb.wait")
                    group.wait()
                }
            }
            if !file_id.isEmpty {
                let files = file_id.split(separator: "|")
                let group = DispatchGroup()
                for file in files {
                    group.enter()
                    Network().upload(name: String(file)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.file.wait")
                    group.wait()
                }
            }
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getUploadTimelineInvitation(post_id: id, title: title, description: description, link: link, type: type, created_date: created_date, audition_date: audition_date, thumb_id: thumb_id, privacy: privacy, file_id: file_id, video_duration: "0", category: category, file_type: file_type, ads_type: "1",target: target, members:members)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "POST", cvalues: [
                            "post_id" : id,
                            "author_f_pin" : me,
                            "author_name":author_name,
                            "author_thumbnail":author_thumb,
                            "title" : title,
                            "description" : description,
                            "link" : link,
                            "type" : type,
                            "created_date" : created_date,
                            "audition_date" : audition_date,
                            "thumb_id" : thumb_id,
                            "privacy" : privacy,
                            "file_id" : file_id,
                            "file_type" : file_type,
                            "target":target,
                            "pricing":pricing,
                            "pricing_money":pricing_money
                        ], replace: true)
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleSubmitPostGroup(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            guard let me = UserDefaults.standard.string(forKey: "me") else {
                result(nil)
                return
            }
            let data = call.arguments as! [AnyHashable: Any?]
            let id = CoreMessage_TMessageUtil.getTID()
            let author_name = data["author_name"] as! String
            let author_thumb = data["author_thumb"] as! String
            let title = data["title"] as! String
            let description = data["description"] as! String
            let link = data["link"] as! String
            let type = data["type"] as! String
            let created_date = String(Date().currentTimeMillis())
            let audition_date = created_date
            let thumb_id = data["thumb_id"] as! String
            let privacy = data["privacy"] as! String
            let file_id = data["file_id"] as! String
            let category = data["category"] as! String
            let file_type = data["file_type"] as! String
            let target = data["target"] as! String
            let groups = data["groups"] as! String
            let topics = data["topics"] as! String
            let pricing = data["pricing"] as! String
            let pricing_money = data["pricing_money"] as! String
            if !thumb_id.isEmpty {
                let thumbs = thumb_id.split(separator: "|")
                let group = DispatchGroup()
                for thumb in thumbs {
                    group.enter()
                    Network().upload(name: String(thumb)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.thumb.wait")
                    group.wait()
                }
            }
            if !file_id.isEmpty {
                let files = file_id.split(separator: "|")
                let group = DispatchGroup()
                for file in files {
                    group.enter()
                    Network().upload(name: String(file)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.file.wait")
                    group.wait()
                }
            }
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getUploadTimelineGroup(post_id: id, title: title, description: description, link: link, type: type, created_date: created_date, audition_date: audition_date, thumb_id: thumb_id, privacy: privacy, file_id: file_id, video_duration: "0", category: category, file_type: file_type, ads_type: "1",target: target, groups: groups,topics: topics)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "POST", cvalues: [
                            "post_id" : id,
                            "author_f_pin" : me,
                            "author_name":author_name,
                            "author_thumbnail":author_thumb,
                            "title" : title,
                            "description" : description,
                            "link" : link,
                            "type" : type,
                            "created_date" : created_date,
                            "audition_date" : audition_date,
                            "thumb_id" : thumb_id,
                            "privacy" : privacy,
                            "file_id" : file_id,
                            "file_type" : file_type,
                            "target":target,
                            "pricing":pricing,
                            "pricing_money":pricing_money
                        ], replace: true)
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleSubmitPostEdu(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            guard let me = UserDefaults.standard.string(forKey: "me") else {
                result(nil)
                return
            }
            let data = call.arguments as! [AnyHashable: Any?]
            let id = CoreMessage_TMessageUtil.getTID()
            let author_name = data["author_name"] as! String
            let author_thumb = data["author_thumb"] as! String
            let title = data["title"] as! String
            let description = data["description"] as! String
            let link = data["link"] as! String
            let type = data["type"] as! String
            let created_date = String(Date().currentTimeMillis())
            let audition_date = created_date
            let thumb_id = data["thumb_id"] as! String
            let privacy = data["privacy"] as! String
            let file_id = data["file_id"] as! String
            let category = data["category"] as! String
            let file_type = data["file_type"] as! String
            let level_edu = data["level_edu"] as! String
            let materi_edu = data["materi_edu"] as! String
            let finaltest_edu = data["finaltest_edu"] as! String
            let pricing = data["pricing"] as! String
            let pricing_money = data["pricing_money"] as! String
            let question = data["question"] as! String
            if !thumb_id.isEmpty {
                let thumbs = thumb_id.split(separator: "|")
                let group = DispatchGroup()
                for thumb in thumbs {
                    group.enter()
                    Network().upload(name: String(thumb)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.thumb.wait")
                    group.wait()
                }
            }
            if !file_id.isEmpty {
                let files = file_id.split(separator: "|")
                let group = DispatchGroup()
                for file in files {
                    group.enter()
                    Network().upload(name: String(file)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.file.wait")
                    group.wait()
                }
            }
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getUploadTimelineEdu(post_id: id, title: title, description: description, link: link, type: type, created_date: created_date, audition_date: audition_date, thumb_id: thumb_id, privacy: privacy, file_id: file_id, video_duration: "0", category: category, file_type: file_type, ads_type: "1", level_edu: level_edu, materi_edu: materi_edu, finaltest_edu: finaltest_edu, target: "0", pricing: pricing, pricing_money: pricing_money, question: question)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "POST", cvalues: [
                            "post_id" : id,
                            "author_f_pin" : me,
                            "author_name":author_name,
                            "author_thumbnail":author_thumb,
                            "title" : title,
                            "description" : description,
                            "link" : link,
                            "type" : type,
                            "created_date" : created_date,
                            "audition_date" : audition_date,
                            "thumb_id" : thumb_id,
                            "privacy" : privacy,
                            "file_id" : file_id,
                            "file_type" : file_type,
                            "target":"0",
                            "level_edu":level_edu,
                            "materi_edu":materi_edu,
                            "finaltest_edu":finaltest_edu,
                            "pricing":pricing,
                            "pricing_money":pricing_money
                        ], replace: true)
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleSubmitPostEduInvitation(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            guard let me = UserDefaults.standard.string(forKey: "me") else {
                result(nil)
                return
            }
            let data = call.arguments as! [AnyHashable: Any?]
            let id = CoreMessage_TMessageUtil.getTID()
            let author_name = data["author_name"] as! String
            let author_thumb = data["author_thumb"] as! String
            let title = data["title"] as! String
            let description = data["description"] as! String
            let link = data["link"] as! String
            let type = data["type"] as! String
            let created_date = String(Date().currentTimeMillis())
            let audition_date = created_date
            let thumb_id = data["thumb_id"] as! String
            let privacy = data["privacy"] as! String
            let file_id = data["file_id"] as! String
            let category = data["category"] as! String
            let file_type = data["file_type"] as! String
            let target = data["target"] as! String
            let members = data["members"] as! String
            let level_edu = data["level_edu"] as! String
            let materi_edu = data["materi_edu"] as! String
            let finaltest_edu = data["finaltest_edu"] as! String
            let pricing = data["pricing"] as! String
            let pricing_money = data["pricing_money"] as! String
            let question = data["question"] as! String
            if !thumb_id.isEmpty {
                let thumbs = thumb_id.split(separator: "|")
                let group = DispatchGroup()
                for thumb in thumbs {
                    group.enter()
                    Network().upload(name: String(thumb)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.thumb.wait")
                    group.wait()
                }
            }
            if !file_id.isEmpty {
                let files = file_id.split(separator: "|")
                let group = DispatchGroup()
                for file in files {
                    group.enter()
                    Network().upload(name: String(file)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.file.wait")
                    group.wait()
                }
            }
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getUploadTimelineEduInvitation(post_id: id, title: title, description: description, link: link, type: type, created_date: created_date, audition_date: audition_date, thumb_id: thumb_id, privacy: privacy, file_id: file_id, video_duration: "0", category: category, file_type: file_type, ads_type: "1", level_edu: level_edu, materi_edu: materi_edu, finaltest_edu: finaltest_edu, target: target, pricing: pricing, pricing_money: pricing_money, members: members, question: question)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "POST", cvalues: [
                            "post_id" : id,
                            "author_f_pin" : me,
                            "author_name":author_name,
                            "author_thumbnail":author_thumb,
                            "title" : title,
                            "description" : description,
                            "link" : link,
                            "type" : type,
                            "created_date" : created_date,
                            "audition_date" : audition_date,
                            "thumb_id" : thumb_id,
                            "privacy" : privacy,
                            "file_id" : file_id,
                            "file_type" : file_type,
                            "target":target,
                            "level_edu":level_edu,
                            "materi_edu":materi_edu,
                            "finaltest_edu":finaltest_edu,
                            "pricing":pricing,
                            "pricing_money":pricing_money
                        ], replace: true)
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleSubmitPostEduGroup(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            guard let me = UserDefaults.standard.string(forKey: "me") else {
                result(nil)
                return
            }
            let data = call.arguments as! [AnyHashable: Any?]
            let id = CoreMessage_TMessageUtil.getTID()
            let author_name = data["author_name"] as! String
            let author_thumb = data["author_thumb"] as! String
            let title = data["title"] as! String
            let description = data["description"] as! String
            let link = data["link"] as! String
            let type = data["type"] as! String
            let created_date = String(Date().currentTimeMillis())
            let audition_date = created_date
            let thumb_id = data["thumb_id"] as! String
            let privacy = data["privacy"] as! String
            let file_id = data["file_id"] as! String
            let category = data["category"] as! String
            let file_type = data["file_type"] as! String
            let target = data["target"] as! String
            let groups = data["groups"] as! String
            let topics = data["topics"] as! String
            let level_edu = data["level_edu"] as! String
            let materi_edu = data["materi_edu"] as! String
            let finaltest_edu = data["finaltest_edu"] as! String
            let pricing = data["pricing"] as! String
            let pricing_money = data["pricing_money"] as! String
            let question = data["question"] as! String
            if !thumb_id.isEmpty {
                let thumbs = thumb_id.split(separator: "|")
                let group = DispatchGroup()
                for thumb in thumbs {
                    group.enter()
                    Network().upload(name: String(thumb)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.thumb.wait")
                    group.wait()
                }
            }
            if !file_id.isEmpty {
                let files = file_id.split(separator: "|")
                let group = DispatchGroup()
                for file in files {
                    group.enter()
                    Network().upload(name: String(file)) { (result, progress) in
                        print("result: \(result), progress: \(progress)")
                        if result {
                            if progress == 100 {
                                group.leave()
                            }
                        } else {
                            group.leave()
                        }
                    }
                    print("handleSubmitPost.file.wait")
                    group.wait()
                }
            }
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getUploadTimelineEduClass(post_id: id, title: title, description: description, link: link, type: type, created_date: created_date, audition_date: audition_date, thumb_id: thumb_id, privacy: privacy, file_id: file_id, video_duration: "0", category: category, file_type: file_type, ads_type: "1", level_edu: level_edu, materi_edu: materi_edu, finaltest_edu: finaltest_edu, target: target, pricing: pricing, pricing_money: pricing_money, groups: groups, topics: topics, question: question)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "POST", cvalues: [
                            "post_id" : id,
                            "author_f_pin" : me,
                            "author_name":author_name,
                            "author_thumbnail":author_thumb,
                            "title" : title,
                            "description" : description,
                            "link" : link,
                            "type" : type,
                            "created_date" : created_date,
                            "audition_date" : audition_date,
                            "thumb_id" : thumb_id,
                            "privacy" : privacy,
                            "file_id" : file_id,
                            "file_type" : file_type,
                            "target":target,
                            "level_edu":level_edu,
                            "materi_edu":materi_edu,
                            "finaltest_edu":finaltest_edu,
                            "pricing":pricing,
                            "pricing_money":pricing_money
                        ], replace: true)
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleSubmitKuisionerAdditional(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let d = data["data"] as! String
            let survey_id = data["survey_id"] as! String
            _ = Palio.write(message: CoreMessage_TMessageBank.submitKuisionerAdditional(data: d, survey_id: survey_id))
            result(nil)
        }
    }
    
    private func handleSubmitKuisioner(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let d = data["data"] as! String
            let latitude = data["latitude"] as! String
            let longitude = data["longitude"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.submitKuisioner(data: d, latitude: latitude, longitude: longitude)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    result(response.mBodies)
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleUpdateCounter(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                _ = Database.shared.updateRecord(fmdb: fmdb, table: "MESSAGE_SUMMARY", cvalues: [
                    "counter" : "0"
                ], _where: "l_pin = '\(l_pin)'")
                result(nil)
            })
        }
    }
    
    private func handleSearchFriend(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let search_keyword = data["search"] as! String
            let limit = data["limit"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getSearchFriend(search_keyword: search_keyword, limit: limit)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                    result(data)
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleSuggestionPerson(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_last_seq = data["p_last_seq"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getPersonSuggestion(p_last_seq: p_last_seq)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                    result(data)
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleRetrieveNearbyDoctor(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let latitude = data["latitude"] as! String
            let longitude = data["longitude"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getDoctorNearby(latitude: latitude, longitude: longitude)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                    result(data)
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleStatusCovid(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getSituasiCovid()) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                    result(data)
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleTrackPerson(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.trackPerson(pin: f_pin)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                    result(data)
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleSearchPerson(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let name = data["name"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.searchPerson(name: name)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                    result(data)
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleRetrieveNearby(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let latitude = data["latitude"] as! String
            let longitude = data["longitude"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.requestNearbyPerson(latitude: latitude, longitude: longitude)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                    result(data)
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleRetrieveChannel(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let offset = data["offset"] as! String
            let filter_account = data["filter_account"] as! String
            let filter_category = data["filter_category"] as! String
            let search = data["search"] as! String
            let shr = data["shr"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.pullChannelList(offset: offset, filter_account: filter_account, filter_category: filter_category, search: search, shr: shr)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                    result(data)
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleRetrieveTimeline(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let score = data["score"] as! String
            let last_last_update = data["last_last_update"] as! String
            let filter_account = data["filter_account"] as! String
            let filter_category = data["filter_category"] as! String
            let search = data["search"] as! String
            let shr = data["shr"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.retrievePostTimeline(score: score, last_last_update: last_last_update, filter_account: filter_account, filter_category: filter_category, search: search, type: "1", shr: shr)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                    result         (data)
                    if data != "[]" {
                        self.insertPost(data: data)
                    }
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleRetrievePostProfile(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            let merchant_id = data["merchant_id"] as! String
            let last_created_date = data["last_created_date"] as! String
            let type = data["type"] as! String
            let storyId = data["storyId"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.retrievePostProfile(f_pin: f_pin, merchant_id: merchant_id, last_created_date: last_created_date, type: type, storyId: storyId)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    let data = response.getBody(key: CoreMessage_TMessageKey.DATA)
                    result(data)
                    if data != "[]" {
                        self.insertPost(data: data)
                    }
                    return
                }
            }
            result(nil)
        }
    }
    
    private func insertPost(data: String) {
        guard let jsonArray = try! JSONSerialization.jsonObject(with: data.data(using: String.Encoding.utf8)!, options: JSONSerialization.ReadingOptions()) as? [AnyObject] else {
            return
        }
        Database.shared.database?.inTransaction({ (fmdb, rollback) in
            do {
                var counter = 0
                for json in jsonArray {
                    _ = try Database.shared.insertRecord(fmdb: fmdb, table: "POST", cvalues: [
                        "post_id" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.POST_ID),
                        "author_f_pin" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.F_PIN),
                        "author_name" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.NAME),
                        "author_thumbnail" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.THUMB_ID),
                        "type" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.TYPE_POST),
                        "created_date" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.CREATED_DATE),
                        "title" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.TITLE),
                        "description" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.DESCRIPTION),
                        "privacy" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.PRIVACY_FLAG),
                        "audition_date" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.AUDITION_DATE),
                        "total_comment" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.COMMENTS),
                        "total_like" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.LIKES),
                        "total_dislike" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.DISLIKES),
                        "last_update" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.LAST_UPDATE),
                        "file_type" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.FILE_TYPE),
                        "thumb_id" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.FILE_THUMB),
                        "file_id" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.FILE_ID),
                        "video_duration" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.DURATION),
                        "category_id" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.CATEGORY_ID),
                        "like_flag" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.FLAG_REACTION),
                        "last_edit" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.LAST_EDIT),
                        "post_id_participate" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.POST_ID_PARTICIPATE),
                        "participate_date" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.PARTICIPATE_ID),
                        "participate_size" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.PARTICIPATE_SIZE),
                        "total_view" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.N_VIEWS),
                        "total_followers" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.FOLLOWER_SIZE),
                        "score" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.SCORE),
                        "link" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.LINK),
                        "category_flag" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.CATEGORY_FLAG),
                        "official_account" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.OFFICIAL_ACCOUNT),
                        "target": CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.TARGET_CONTENT,def:"0"),
                        "roc_date" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.ROC_DATE,def:"0"),
                        "roc_size" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.ROC_SIZE,def:"0"),
                        "level_edu" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.LEVEL_EDU,def:"0"),
                        "materi_edu" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.MATERI_EDU,def:"0"),
                        "finaltest_edu" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.FINALTEST_EDU,def:"0"),
                        "file_summarization" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.FILE_SUMMARIZATION,def:""),
                        "pricing" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.PRICING,def:"0"),
                        "pricing_money" : CoreMessage_TMessageUtil.getString(json: json, key: CoreMessage_TMessageKey.PRICING_MONEY,def:"")
                        
                    ], replace: true)
                    counter = counter + 1
                }
                UserDefaults.standard.set(counter, forKey: "post_sum")
            } catch {
                rollback.pointee = true
                print(error)
            }
        })
    }
    
    private func handleAddFriend(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getAddFriendQRCode(fpin: f_pin)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                return
            }
            result(nil)
        }
    }
    
    private func handleRemoveFriend(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            var retval : String? = nil
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.removeFriend(lpin: l_pin)) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    Database.shared.database?.inTransaction({ (fmdb, rollback) in
                        do {
                            _ = Database.shared.deleteRecord(fmdb: fmdb, table: "BUDDY", _where:  "f_pin = '\(l_pin)'" )
                            _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE", _where: "chat_id = '' AND l_pin='\(l_pin)'")
                            _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE_SUMMARY", _where: "l_pin='\(l_pin)'")
                            _ = Database.shared.deleteRecord(fmdb: fmdb, table: "POST", _where: "author_f_pin='\(l_pin)'")
                            retval = l_pin
                        } catch {
                            rollback.pointee = true
                            print(error)
                        }
                    })
                }
            }
            result(retval)
        }
    }
    
    private func handleReceiveVideoCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        print(device_id)
        let camera_id = data["camera_id"] as! String
        do {
            //            try AVAudioSession.sharedInstance().setCategory(.playAndRecord)
            //            try AVAudioSession.sharedInstance().setMode(.voiceChat)
            //            try AVAudioSession.sharedInstance().setActive(true)
            Palio.startAudio(nMode: 2, bSpeakerOn: false)
            let screenSize: CGRect = UIScreen.main.bounds
            let screenWidth = screenSize.width
            let screenHeight = screenSize.height
            let viewWidth = screenWidth/3
            let viewHeight = 4/3*viewWidth
            var views = [UIImageView]()
            for i in remoteViews {
                if let view = i.remoteView {
                    view.transform = CGAffineTransform.init(scaleX: -1, y: 1).rotated(by: (CGFloat.pi * 5)/2)
                    view.translatesAutoresizingMaskIntoConstraints=false
                    view.heightAnchor.constraint(equalToConstant: viewHeight).isActive = true
                    view.widthAnchor.constraint(equalToConstant: viewWidth).isActive = true
                    view.contentMode = .scaleAspectFill
                    view.layer.cornerRadius = 5
                    view.clipsToBounds = true
                    //                    view.isHidden = true
                    views.append(view)
                    print("remote view added")
                }
            }
            guard let camera = cameraView?.cameraView else {
                print("camera view null")
                result(nil)
                return
            }
            //            let camera = (cameraView?.cameraView)!
            camera.translatesAutoresizingMaskIntoConstraints = false
            camera.heightAnchor.constraint(equalToConstant: viewHeight).isActive = true
            camera.widthAnchor.constraint(equalToConstant: viewWidth).isActive = true
            camera.contentMode = .scaleAspectFill
            camera.layer.cornerRadius = 5
            camera.clipsToBounds = true
            //            camera.isHidden = true
            print("camera view added")
            guard let zoom = zoomView?.zoomView else {
                print("zoom view null")
                result(nil)
                return
            }
            //            let zoom = (zoomView?.zoomView)!
            zoom.transform = CGAffineTransform(rotationAngle: CGFloat.pi / 2 )
            zoom.translatesAutoresizingMaskIntoConstraints = false
            zoom.heightAnchor.constraint(equalToConstant: screenHeight).isActive = true
            zoom.widthAnchor.constraint(equalToConstant: screenWidth).isActive = true
            zoom.contentMode = .scaleAspectFit
            zoom.layer.cornerRadius = 5
            zoom.clipsToBounds = true
            print("zoom view added")
            nuSDKService.receiveCCall(sParty: device_id, nCamIdx: Int(camera_id), nResIdx: 2, nVQuality: 2, ivRemoteView: views, ivLocalView: camera,ivRemoteZ: zoom)
        }
        catch {
            print(error)
        }
        result(nil)
    }
    
    private func handleInitiateVideoCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        print(device_id)
        let camera_id = data["camera_id"] as! String
        do {
            //            try AVAudioSession.sharedInstance().setCategory(.playAndRecord)
            //            try AVAudioSession.sharedInstance().setMode(.voiceChat)
            //            try AVAudioSession.sharedInstance().setActive(true)
            Palio.startAudio(nMode: 2, bSpeakerOn: false)
            let screenSize: CGRect = UIScreen.main.bounds
            let screenWidth = screenSize.width
            let screenHeight = screenSize.height
            let viewWidth = screenWidth/3
            let viewHeight = 4/3*viewWidth
            var views = [UIImageView]()
            for i in remoteViews {
                if let view = i.remoteView {
                    view.transform = CGAffineTransform.init(scaleX: -1, y: 1).rotated(by: (CGFloat.pi * 5)/2)
                    view.translatesAutoresizingMaskIntoConstraints=false
                    view.heightAnchor.constraint(equalToConstant: viewHeight).isActive = true
                    view.widthAnchor.constraint(equalToConstant: viewWidth).isActive = true
                    view.contentMode = .scaleAspectFill
                    view.layer.cornerRadius = 5
                    view.clipsToBounds = true
                    //                    view.isHidden = true
                    views.append(view)
                    print("remote view added")
                }
            }
            guard let camera = cameraView?.cameraView else {
                print("camera view null")
                result(nil)
                return
            }
            //        let camera = (cameraView?.cameraView)!
            camera.translatesAutoresizingMaskIntoConstraints = false
            camera.heightAnchor.constraint(equalToConstant: viewHeight).isActive = true
            camera.widthAnchor.constraint(equalToConstant: viewWidth).isActive = true
            camera.contentMode = .scaleAspectFill
            camera.layer.cornerRadius = 5
            camera.clipsToBounds = true
            //            camera.isHidden = true
            print("camera view added")
            guard let zoom = zoomView?.zoomView else {
                print("zoom view null")
                result(nil)
                return
            }
            //        let zoom = (zoomView?.zoomView)!
            zoom.transform = CGAffineTransform.init(scaleX: -1, y: 1).rotated(by: CGFloat.pi / 2)
            zoom.translatesAutoresizingMaskIntoConstraints = false
            zoom.heightAnchor.constraint(equalToConstant: screenHeight).isActive = true
            zoom.widthAnchor.constraint(equalToConstant: screenWidth).isActive = true
            zoom.contentMode = .scaleAspectFit
            zoom.layer.cornerRadius = 5
            zoom.clipsToBounds = true
            print("zoom view added")
            nuSDKService.initiateCCall(sParty: device_id, nCamIdx: Int(camera_id), nResIdx: 2, nVQuality: 2, ivRemoteView: views, ivLocalView: camera,ivRemoteZ: zoom)
        }
        catch {
            print(error)
        }
        
        result(nil)
    }
    
    private func handleAddVideoCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        print(device_id)
        let camera_id = data["camera_id"] as! String
        print(camera_id)
        let screenSize: CGRect = UIScreen.main.bounds
        let screenWidth = screenSize.width
        let screenHeight = screenSize.height
        let viewWidth = screenWidth/3
        let viewHeight = 4/3*viewWidth
        var views = [UIImageView]()
        for i in remoteViews {
            if let view = i.remoteView {
                view.translatesAutoresizingMaskIntoConstraints=false
                view.heightAnchor.constraint(equalToConstant: viewHeight).isActive = true
                view.widthAnchor.constraint(equalToConstant: viewWidth).isActive = true
                view.contentMode = .scaleAspectFill
                view.layer.cornerRadius = 5
                view.clipsToBounds = true
                //                    view.isHidden = true
                views.append(view)
                print("remote view added")
            }
        }
        guard let camera = cameraView?.cameraView else {
            print("camera view null")
            result(nil)
            return
        }
        guard let zoom = zoomView?.zoomView else {
            print("zoom view null")
            result(nil)
            return
        }
        nuSDKService.initiateCCall(sParty: device_id, nCamIdx: Int(camera_id), nResIdx: 2, nVQuality: 2, ivRemoteView: views,  ivLocalView: camera, ivRemoteZ: zoom)
        
    }
    
    private func handleTerminateCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        print("end call")
        print(device_id)
        nuSDKService.terminateCall(sParty: nil)
        print("success end call")
        do {
            print("end audio session")
            try AVAudioSession.sharedInstance().setCategory(.ambient)
            try AVAudioSession.sharedInstance().setMode(.default)
            try AVAudioSession.sharedInstance().overrideOutputAudioPort(.none)
            print("end audio session success")
        } catch {
            print("end audio session fail")
            print(error)
        }
        
        print("remove image view")
        cameraView = nil
        zoomView = nil
        remoteViews.removeAll()
        print("remove image view success")
        result(nil)
    }
    
    private func handleTerminateConfCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        print("end call")
        print(device_id)
        nuSDKService.terminateCall(sParty: device_id)
        print("success end call")
        do {
            print("end audio session")
            try AVAudioSession.sharedInstance().setCategory(.ambient)
            try AVAudioSession.sharedInstance().setMode(.default)
            try AVAudioSession.sharedInstance().overrideOutputAudioPort(.none)
            print("end audio session success")
        } catch {
            print("end audio session fail")
            print(error)
        }
        
        print("remove image view")
        cameraView = nil
        zoomView = nil
        remoteViews.removeAll()
        print("remove image view success")
        result(nil)
    }
    
    private func handleReceiveAudioCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        print(device_id)
        do {
            //            try AVAudioSession.sharedInstance().setCategory(.playAndRecord)
            try AVAudioSession.sharedInstance().setCategory(.playAndRecord, options: .duckOthers)
            try AVAudioSession.sharedInstance().setMode(.voiceChat)
            try AVAudioSession.sharedInstance().setActive(true)
            //            App.startAudio(nMode: 1, bSpeakerOn: false)
        } catch {
            print(error)
        }
        nuSDKService.receiveCCall(sParty: device_id)
        result(nil)
    }
    
    private func handleInitiateAudioCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        print(device_id)
        do {
            //            try AVAudioSession.sharedInstance().setCategory(.playAndRecord)
            try AVAudioSession.sharedInstance().setCategory(.playAndRecord, options: .duckOthers)
            try AVAudioSession.sharedInstance().setMode(.voiceChat)
            try AVAudioSession.sharedInstance().setActive(true)
            //            App.startAudio(nMode: 1, bSpeakerOn: false)
        } catch {
            print(error)
        }
        nuSDKService.initiateCCall(sParty: device_id)
        result(nil)
    }
    
    private func handleAddAudioCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        nuSDKService.initiateCCall(sParty: device_id)
        result(nil)
    }
    
    private func handleDownload(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let file = data["file"] as! String
            guard !file.isEmpty else {
                result(nil)
                return
            }
            Download().start(forKey: file, delegate: self)
            result(nil)
        }
    }
    
    private func handleSaveToGallery(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let save = data["save"] as! Bool
            UserDefaults.standard.set(save, forKey: "saveToGallery")
            result(save)
        }
        
    }
    
    
    private func handleSetVibrate(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let vibrate = data["vibrate"] as! Bool
            UserDefaults.standard.set(vibrate, forKey: "vibrate")
            result(vibrate)
        }
        
    }
    
    private func handleVibrate(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            AudioServicesPlaySystemSoundWithCompletion(kSystemSoundID_Vibrate) { }
            result(nil)
        }
        
    }
    
    private func handleStopVibrate(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            AudioServicesDisposeSystemSoundID(kSystemSoundID_Vibrate)
            result(nil)
        }
        
    }
    
    private func handleDeleteChat(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let chat_id = data["chat_id"] as! String
            let group_id = data["group_id"] as! String
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getDeleteChat(chat_id: chat_id, f_pin: group_id)) {
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    _ = Database.shared.deleteRecord(fmdb: fmdb, table: "DISCUSSION_FORUM", _where: "chat_id = '\(chat_id)'")
                    result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleUpdateChat(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let chat_id = data["chat_id"] as! String
            let title = data["title"] as! String
            let group_id = data["group_id"] as! String
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getUpdateChat(p_chat_id: chat_id, p_f_pin: group_id, p_title: title, p_anonym: "", p_image: "")) {
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    _ = Database.shared.updateRecord(fmdb: fmdb, table: "DISCUSSION_FORUM", cvalues: [
                        "title" : title
                    ], _where: "chat_id = '\(chat_id)'")
                    result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleAddChat(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let chat_id = Date().currentTimeMillis().toHex()
            let title = data["title"] as! String
            let group_id = data["group_id"] as! String
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getCreateChat(chat_id: chat_id, title: title, group_id: group_id)) {
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "DISCUSSION_FORUM", cvalues: [
                            "group_id" : group_id,
                            "title" : title,
                            "chat_id" : chat_id,
                            "scope_id" : "4"
                        ], replace: true)
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                    result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleExitGroup(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            DispatchQueue.global().async {
                let data = call.arguments as! [AnyHashable: Any?]
                let member_pin = data["member_pin"] as! String
                let group_id = data["group_id"] as! String
                if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getExitGroup(p_group_id: group_id, p_pin: member_pin)) {
                    Database.shared.database?.inTransaction({ (fmdb, rollbac) in
                        if let me = UserDefaults.standard.string(forKey: "me"), member_pin == me {
                            _ = Database.shared.deleteRecord(fmdb: fmdb, table: "GROUPZ", _where: "group_id = '\(group_id)'")
                            _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE", _where: "l_pin='\(group_id)'")
                            _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE_SUMMARY", _where: "l_pin='\(group_id)'")
                            if let cursor = Database.shared.getRecords(fmdb: fmdb, query: "select chat_id from DISCUSSION_FORUM where group_id='\(group_id)'") {
                                while cursor.next() {
                                    if let chat_id = cursor.string(forColumnIndex: 0) {
                                        _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE_SUMMARY", _where: "l_pin='\(chat_id)'")
                                    }
                                }
                                cursor.close()
                            }
                            if let cursorSub = Database.shared.getRecords(fmdb: fmdb, query: "select group_id from GROUPZ where parent='\(group_id)'") {
                                while cursorSub.next() {
                                    if let subGroup = cursorSub.string(forColumnIndex: 0) {
                                        _ = Database.shared.deleteRecord(fmdb: fmdb, table: "GROUPZ", _where: "group_id = '\(subGroup)'")
                                        _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE", _where: "l_pin='\(subGroup)'")
                                        _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE_SUMMARY", _where: "l_pin='\(subGroup)'")
                                        if let cursorSubGroupTopic = Database.shared.getRecords(fmdb: fmdb, query: "select chat_id from DISCUSSION_FORUM where group_id='\(subGroup)'") {
                                            while cursorSubGroupTopic.next() {
                                                if let chat_id_sub = cursorSubGroupTopic.string(forColumnIndex: 0) {
                                                    _ = Database.shared.deleteRecord(fmdb: fmdb, table: "MESSAGE_SUMMARY", _where: "l_pin='\(chat_id_sub)'")
                                                }
                                            }
                                            cursorSubGroupTopic.close()
                                        }
                                    }
                                }
                                cursorSub.close()
                            }
                        }
                        result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                    })
                    return
                }
                result(nil)
            }
        }
    }
    
    private func handleChangeMemberPosition(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let group_id = data["group_id"] as! String
            let member_pin = data["member_pin"] as! String
            let position = data["position"] as! String
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getChangeGroupMemberPosition(p_group_id: group_id, p_pin: member_pin, p_position: position)) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                return
            }
            result(nil)
        }
    }
    
    private func handleAddGroupMember(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let group_id = data["group_id"] as! String
            let member_pin = data["member_pin"] as! String
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getAddGroupMember(p_group_id: group_id, p_member_pin: member_pin)) {
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "GROUPZ_MEMBER", cvalues: [
                            "group_id" : group_id,
                            "f_pin" : member_pin,
                            "created_date" : Date().currentTimeMillis()
                        ], replace: true)
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                    result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                })
                return
            }
            result(nil)
        }
    }
    
    private func updateGroup(group_id: String, group_name: String, open: String, thumb_id: String, quote: String) -> TMessage? {
        if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getChangeGroupInfo(p_group_id: group_id, p_name: group_name, p_open: open, p_thumb_id: thumb_id, p_quote: quote)) {
            var cvalues = [String: String]()
            if !group_name.isEmpty {
                cvalues["f_name"] = group_name
            }
            if !open.isEmpty {
                cvalues["is_open"] = open
            }
            if !thumb_id.isEmpty {
                cvalues["image_id"] = thumb_id
            }
            if !quote.isEmpty {
                cvalues["quote"] = quote
            }
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                _ = Database.shared.updateRecord(fmdb: fmdb, table: "GROUPZ", cvalues: cvalues, _where: "group_id = '\(group_id)'")
            })
            return response
        }
        return nil
    }
    
    private func handleUpdateGroup(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let group_id = data["group_id"] as! String
            let group_name = data["group_name"] as! String
            let open = data["open"] as! String
            let thumb_id = data["thumb_id"] as! String
            let quote = data["quote"] as! String
            
            if !thumb_id.isEmpty {
                Network().upload(name: thumb_id) { (upload, progress) in
                    if upload, progress == 100 {
                        if let response = self.updateGroup(group_id: group_id, group_name: group_name, open: open, thumb_id: thumb_id, quote: quote) {
                            result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                        }
                    }
                }
            } else {
                if let response = self.updateGroup(group_id: group_id, group_name: group_name, open: open, thumb_id: thumb_id, quote: quote) {
                    result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                }
            }
        }
    }
    
    private func handleCreateGroup(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let group_name = data["group_name"] as! String
            let id = CoreMessage_TMessageUtil.getTID()
            if let response = Palio.writeAndWait(message: CoreMessage_TMessageBank.getCreateGroup(p_group_id: id, p_group_name: group_name)) {
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "GROUPZ", cvalues: [
                            "group_id" : id,
                            "f_name" : group_name,
                            "created_date" : Date().currentTimeMillis(),
                            "created_by" : UserDefaults.standard.string(forKey: "me")!,
                            "level" : "-1"
                        ], replace: true)
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                    result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99"))
                })
                return
            }
            result(nil)
        }
    }
    
    private func handleSendMessage(call: FlutterMethodCall, result: FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let l_pin = data["l_pin"] as! String
        let message_scope_id = data["message_scope_id"] as! String
        let status = data["status"] as! String
        let message_text = data["message_text"] as! String
        let credential = data["credential"] as! String
        let attachment_flag = data["attachment_flag"] as! String
        let ex_blog_id = data["ex_blog_id"] as! String
        let message_large_text = data["message_large_text"] as! String
        let ex_format = data["ex_format"] as! String
        let image_id = data["image_id"] as! String
        let audio_id = data["audio_id"] as! String
        let video_id = data["video_id"] as! String
        let file_id = data["file_id"] as! String
        let thumb_id = data["thumb_id"] as! String
        let reff_id = data["reff_id"] as! String
        let read_receipts = data["read_receipts"] as! String
        let chat_id = data["chat_id"] as! String
        let is_call_center = data["is_call_center"] as! String
        let call_center_id = data["call_center_id"] as! String
        let message = CoreMessage_TMessageBank.sendMessage(l_pin: l_pin, message_scope_id: message_scope_id, status: status, message_text: message_text, credential: credential, attachment_flag: attachment_flag, ex_blog_id: ex_blog_id, message_large_text: message_large_text, ex_format: ex_format, image_id: image_id, audio_id: audio_id, video_id: video_id, file_id: file_id, thumb_id: thumb_id, reff_id: reff_id, read_receipts: read_receipts, chat_id: chat_id, is_call_center: is_call_center, call_center_id: call_center_id)
        Palio.addQueueMessage(message: message)
        result(String(message.mBodies[CoreMessage_TMessageKey.MESSAGE_ID]!))
    }
    
    private func handlePostRegistration(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            if let me = UserDefaults.standard.string(forKey: "me") {
                Palio.setLoginDelegate(delegate: self)
                let message = CoreMessage_TMessageBank.getPostRegistration(p_pin: me)
                _ = Palio.write(message: message)
            }
            result(nil)
        }
    }
    
    private func handlePostRegistrationUserId(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            if let me = UserDefaults.standard.string(forKey: "me") {
                Palio.setLoginDelegate(delegate: self)
                let message = CoreMessage_TMessageBank.getPostRegistration(p_pin: me)
                _ = Palio.write(message: message)
            }
            result(nil)
        }
    }
    
    private func changeUser(f_pin :String){
        Palio.changeUser(f_pin: f_pin)
    }
    
    private func handleSignIn(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            var f_pin = ""
            let data = call.arguments as! [AnyHashable: Any?]
            let name = data["name"] as! String
            let password = data["password"] as! String
            let message = CoreMessage_TMessageBank.getSignIn(p_name: name, p_password: password)
            if let response = Palio.writeSync(message: message) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "")
                if errCode == "00" {
                    f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN, default_value: "")
                    UserDefaults.standard.set(f_pin, forKey: "me")
                    let token = UserDefaults.standard.string(forKey: "token")!
                    let pushkit = UserDefaults.standard.string(forKey: "pushkit")!
                    _ = Palio.write(message: CoreMessage_TMessageBank.getApnToken(token: token, callToken: "", fPin: f_pin))
                    _ = Palio.write(message: CoreMessage_TMessageBank.getApnToken(token: "", callToken: pushkit, fPin: f_pin))
                    self.changeUser(f_pin: f_pin)
                }
                result(f_pin)
                return
            }
            result(f_pin)
        }
    }
    
    private func handleSignUp(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            var f_pin = ""
            let data = call.arguments as! [AnyHashable: Any?]
            let name = data["name"] as! String
            let msisdn = data["msisdn"] as! String
            let password = data["password"] as! String
            let thumb_id = data["thumb_id"] as? String ?? ""
            let message = CoreMessage_TMessageBank.getSendSignup(p_pin: "", p_name: name, p_last_name: "", p_msisdn: msisdn, p_card_type: "0", p_card_id: "", p_email: "", p_thumb_id: thumb_id, flag: "0", imei: "", imsi: "",password: password)
            if let response = Palio.writeSync(message: message) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "")
                if errCode == "00" {
                    f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN, default_value: "")
                    UserDefaults.standard.set(f_pin, forKey: "me")
                    Database.shared.database?.inTransaction({ (fmdb, rollback) in
                        do {
                            _ = try Database.shared.insertRecord(fmdb: fmdb, table: "BUDDY", cvalues: [
                                "f_pin" : message.getBody(key: CoreMessage_TMessageKey.F_PIN),
                                "upline_pin" : message.getBody(key: CoreMessage_TMessageKey.UPLINE_PIN),
                                "first_name" : message.getBody(key: CoreMessage_TMessageKey.FIRST_NAME),
                                "last_name" : message.getBody(key: CoreMessage_TMessageKey.LAST_NAME),
                                "image_id" : message.getBody(key: CoreMessage_TMessageKey.THUMB_ID),
                                "user_id" : message.getBody(key: CoreMessage_TMessageKey.USER_ID),
                                "quote" : message.getBody(key: CoreMessage_TMessageKey.QUOTE),
                                "connected" : "0",
                                "last_update" : message.getBody(key: CoreMessage_TMessageKey.LAST_UPDATE),
                                "latitude" : message.getBody(key: CoreMessage_TMessageKey.LATITUDE),
                                "longitude" : message.getBody(key: CoreMessage_TMessageKey.LONGITUDE),
                                "altitude" : message.getBody(key: CoreMessage_TMessageKey.ALTITUDE),
                                "cell" : message.getBody(key: CoreMessage_TMessageKey.CELL),
                                "last_loc_update" : message.getBody(key: CoreMessage_TMessageKey.LAST_LOC_UPDATE),
                                "type" : "1",
                                "empty_2" : message.getBody(key: CoreMessage_TMessageKey.PLACE_NAME),
                                "timezone" : message.getBody(key: CoreMessage_TMessageKey.TIMEZONE),
                                "privacy_flag" : message.getBody(key: CoreMessage_TMessageKey.PRIVACY_FLAG),
                                "msisdn" : message.getBody(key: CoreMessage_TMessageKey.MSISDN),
                                "email" : message.getBody(key: CoreMessage_TMessageKey.EMAIL),
                                "created_date" : message.getBody(key: CoreMessage_TMessageKey.CREATED_DATE),
                                "offline_mode" : message.getBody(key: CoreMessage_TMessageKey.OFFLINE_MODE),
                                "network_type" : message.getBody(key: CoreMessage_TMessageKey.NETWORK_TYPE),
                                "ex_block" : "0",
                                "ex_follow" : "0",
                                "ex_follower" : message.getBody(key: CoreMessage_TMessageKey.TOTAL_FOLLOWERS),
                                "ex_offmp" : message.getBody(key: CoreMessage_TMessageKey.OFFMP),
                                "ex_status" : "0",
                                "shop_code" : message.getBody(key: CoreMessage_TMessageKey.SHOP_CODE),
                                "shop_name" : message.getBody(key: CoreMessage_TMessageKey.SHOP_NAME),
                                "extension" : message.getBody(key: CoreMessage_TMessageKey.EXTENSION),
                                "auto_quote" : message.getBody(key: CoreMessage_TMessageKey.AUTO_QUOTE),
                                "auto_quote_type" : message.getBody(key: CoreMessage_TMessageKey.AUTO_QUOTE_TYPE),
                                "be_info" : message.getBody(key: CoreMessage_TMessageKey.BE_INFO),
                                "org_id" : message.getBody(key: CoreMessage_TMessageKey.ORG_ID),
                                "org_name" : message.getBody(key: CoreMessage_TMessageKey.ORG_NAME),
                                "org_thumb" : message.getBody(key: CoreMessage_TMessageKey.ORG_THUMB),
                                "gender" : message.getBody(key: CoreMessage_TMessageKey.GENDER),
                                "birthdate" : message.getBody(key: CoreMessage_TMessageKey.BIRTHDATE),
                                "type_ads" : message.getBody(key: CoreMessage_TMessageKey.TYPE_ADS),
                                "type_lp" : message.getBody(key: CoreMessage_TMessageKey.TYPE_LP),
                                "type_post" : message.getBody(key: CoreMessage_TMessageKey.TYPE_POST),
                                "address" : message.getBody(key: CoreMessage_TMessageKey.ADDRESS),
                                "bidang_industri" : message.getBody(key: CoreMessage_TMessageKey.BIDANG_INDUSTRI),
                                "visi" : message.getBody(key: CoreMessage_TMessageKey.VISI),
                                "misi" : message.getBody(key: CoreMessage_TMessageKey.MISI),
                                "company_lat" : message.getBody(key: CoreMessage_TMessageKey.COMPANY_LAT),
                                "company_lng" : message.getBody(key: CoreMessage_TMessageKey.COMPANY_LNG),
                                "web" : message.getBody(key: CoreMessage_TMessageKey.COMPANY_WEB),
                                "certificate_image" : message.getBody(key: CoreMessage_TMessageKey.COMPANY_CERTIFICATE),
                                "card_id" : message.getBody(key: CoreMessage_TMessageKey.CARD_ID),
                                "user_type" : message.getBody(key: CoreMessage_TMessageKey.USER_TYPE),
                                "real_name" : message.getBody(key: CoreMessage_TMessageKey.REAL_NAME),
                                "official_account" : message.getBody(key: CoreMessage_TMessageKey.OFFICIAL_ACCOUNT),
                                "is_sub_account" : message.getBody(key: CoreMessage_TMessageKey.IS_SUB_ACCOUNT),
                                "last_sign" : message.getBody(key: CoreMessage_TMessageKey.LAST_SIGN),
                                "android_id" : message.getBody(key: CoreMessage_TMessageKey.ANDROID_ID),
                            ], replace: true)
                        } catch {
                            rollback.pointee = true
                            print(error)
                        }
                    })
                    self.changeUser(f_pin: f_pin)
                }
                result(f_pin)
                return
            }
            result(f_pin)
        }
    }
    
    private func handleSignUpOTP(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            var f_pin = ""
            let data = call.arguments as! [AnyHashable: Any?]
            let name = data["name"] as! String
            let msisdn = data["msisdn"] as! String
            let password = data["password"] as! String
            let thumb_id = data["thumb_id"] as! String
            let message = CoreMessage_TMessageBank.getSendSignupOTP(p_pin: "", p_name: name, p_last_name: "", p_msisdn: msisdn, p_card_type: "0", p_card_id: "", p_email: "", p_thumb_id: thumb_id, flag: "0", imei: "", imsi: "",password: password)
            if let response = Palio.writeSync(message: message) {
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "")
                if errCode == "00" {
                    f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN, default_value: "")
                    UserDefaults.standard.set(f_pin, forKey: "me")
                    Database.shared.database?.inTransaction({ (fmdb, rollback) in
                        do {
                            _ = try Database.shared.insertRecord(fmdb: fmdb, table: "BUDDY", cvalues: [
                                "f_pin" : message.getBody(key: CoreMessage_TMessageKey.F_PIN),
                                "upline_pin" : message.getBody(key: CoreMessage_TMessageKey.UPLINE_PIN),
                                "first_name" : message.getBody(key: CoreMessage_TMessageKey.FIRST_NAME),
                                "last_name" : message.getBody(key: CoreMessage_TMessageKey.LAST_NAME),
                                "image_id" : message.getBody(key: CoreMessage_TMessageKey.THUMB_ID),
                                "user_id" : message.getBody(key: CoreMessage_TMessageKey.USER_ID),
                                "quote" : message.getBody(key: CoreMessage_TMessageKey.QUOTE),
                                "connected" : "0",
                                "last_update" : message.getBody(key: CoreMessage_TMessageKey.LAST_UPDATE),
                                "latitude" : message.getBody(key: CoreMessage_TMessageKey.LATITUDE),
                                "longitude" : message.getBody(key: CoreMessage_TMessageKey.LONGITUDE),
                                "altitude" : message.getBody(key: CoreMessage_TMessageKey.ALTITUDE),
                                "cell" : message.getBody(key: CoreMessage_TMessageKey.CELL),
                                "last_loc_update" : message.getBody(key: CoreMessage_TMessageKey.LAST_LOC_UPDATE),
                                "type" : "1",
                                "empty_2" : message.getBody(key: CoreMessage_TMessageKey.PLACE_NAME),
                                "timezone" : message.getBody(key: CoreMessage_TMessageKey.TIMEZONE),
                                "privacy_flag" : message.getBody(key: CoreMessage_TMessageKey.PRIVACY_FLAG),
                                "msisdn" : message.getBody(key: CoreMessage_TMessageKey.MSISDN),
                                "email" : message.getBody(key: CoreMessage_TMessageKey.EMAIL),
                                "created_date" : message.getBody(key: CoreMessage_TMessageKey.CREATED_DATE),
                                "offline_mode" : message.getBody(key: CoreMessage_TMessageKey.OFFLINE_MODE),
                                "network_type" : message.getBody(key: CoreMessage_TMessageKey.NETWORK_TYPE),
                                "ex_block" : "0",
                                "ex_follow" : "0",
                                "ex_follower" : message.getBody(key: CoreMessage_TMessageKey.TOTAL_FOLLOWERS),
                                "ex_offmp" : message.getBody(key: CoreMessage_TMessageKey.OFFMP),
                                "ex_status" : "0",
                                "shop_code" : message.getBody(key: CoreMessage_TMessageKey.SHOP_CODE),
                                "shop_name" : message.getBody(key: CoreMessage_TMessageKey.SHOP_NAME),
                                "extension" : message.getBody(key: CoreMessage_TMessageKey.EXTENSION),
                                "auto_quote" : message.getBody(key: CoreMessage_TMessageKey.AUTO_QUOTE),
                                "auto_quote_type" : message.getBody(key: CoreMessage_TMessageKey.AUTO_QUOTE_TYPE),
                                "be_info" : message.getBody(key: CoreMessage_TMessageKey.BE_INFO),
                                "org_id" : message.getBody(key: CoreMessage_TMessageKey.ORG_ID),
                                "org_name" : message.getBody(key: CoreMessage_TMessageKey.ORG_NAME),
                                "org_thumb" : message.getBody(key: CoreMessage_TMessageKey.ORG_THUMB),
                                "gender" : message.getBody(key: CoreMessage_TMessageKey.GENDER),
                                "birthdate" : message.getBody(key: CoreMessage_TMessageKey.BIRTHDATE),
                                "type_ads" : message.getBody(key: CoreMessage_TMessageKey.TYPE_ADS),
                                "type_lp" : message.getBody(key: CoreMessage_TMessageKey.TYPE_LP),
                                "type_post" : message.getBody(key: CoreMessage_TMessageKey.TYPE_POST),
                                "address" : message.getBody(key: CoreMessage_TMessageKey.ADDRESS),
                                "bidang_industri" : message.getBody(key: CoreMessage_TMessageKey.BIDANG_INDUSTRI),
                                "visi" : message.getBody(key: CoreMessage_TMessageKey.VISI),
                                "misi" : message.getBody(key: CoreMessage_TMessageKey.MISI),
                                "company_lat" : message.getBody(key: CoreMessage_TMessageKey.COMPANY_LAT),
                                "company_lng" : message.getBody(key: CoreMessage_TMessageKey.COMPANY_LNG),
                                "web" : message.getBody(key: CoreMessage_TMessageKey.COMPANY_WEB),
                                "certificate_image" : message.getBody(key: CoreMessage_TMessageKey.COMPANY_CERTIFICATE),
                                "card_id" : message.getBody(key: CoreMessage_TMessageKey.CARD_ID),
                                "user_type" : message.getBody(key: CoreMessage_TMessageKey.USER_TYPE),
                                "real_name" : message.getBody(key: CoreMessage_TMessageKey.REAL_NAME),
                                "official_account" : message.getBody(key: CoreMessage_TMessageKey.OFFICIAL_ACCOUNT),
                                "is_sub_account" : message.getBody(key: CoreMessage_TMessageKey.IS_SUB_ACCOUNT),
                                "last_sign" : message.getBody(key: CoreMessage_TMessageKey.LAST_SIGN),
                                "android_id" : message.getBody(key: CoreMessage_TMessageKey.ANDROID_ID),
                            ], replace: true)
                        } catch {
                            rollback.pointee = true
                            print(error)
                        }
                    })
                    self.changeUser(f_pin: f_pin)
                }
                result(f_pin)
                return
            }
            result(f_pin)
        }
    }
    
    private func handlePullMainContent(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            var hasil: String = ""
            let message = CoreMessage_TMessageBank.pullMainContent()
            if let response = Palio.writeSync(message: message) {
                let rdata = response.getBody(key: CoreMessage_TMessageKey.DATA,default_value:"")
                let data = rdata.components(separatedBy: "|")
                if data.count == 2 {
                    UserDefaults.standard.set(rdata, forKey:"main_content")
                    hasil = rdata
                }
                result(hasil)
                return
            }
            result(hasil)
        }
    }
    
    private func handlePullStoryList(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let offset = data["offset"] as! String
            let filter = data["filter"] as! String
            let message = CoreMessage_TMessageBank.pullStoryList(offset:offset,filter:filter)
            if let response = Palio.writeSync(message: message) {
                result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: ""))
                return
            }
            result(nil)
        }
    }
    
    private func handlePullStoryListPersonal(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let offset = data["offset"] as! String
            let filter = data["filter"] as! String
            let l_pin = data["l_pin"] as! String
            let message = CoreMessage_TMessageBank.pullStoryListPersonal(offset: offset, filter: filter, l_pin: l_pin)
            if let response = Palio.writeSync(message: message) {
                result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: ""))
                return
            }
            result(nil)
        }
    }
    
    private func handleCreateStory(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let post_id = data["post_id"] as! String
            let title = data["title"] as! String
            let message = CoreMessage_TMessageBank.postCreateStory(post_id: post_id, title: title)
            if let _ = Palio.writeSync(message: message) {
                if let delegate = Palio.getTimelineDelegate() {
                    delegate.onPostUpdate(status: "UploadStory", message: "Success")
                }
                result("Sukses")
            } else {
                result(nil)
            }
        }
    }
    
    private func handleUpdateStory(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let story_id = data["story_id"] as! String
            let post_id = data["post_id"] as! String
            let title = data["title"] as! String
            let message = CoreMessage_TMessageBank.postUpdateStory(story_id: story_id, post_id: post_id, title: title)
            if let _ = Palio.writeSync(message: message) {
                result("Sukses")
            } else {
                result(nil)
            }
        }
    }
    
    private func handleStartLiveStreaming(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let title = data["title"] as! String
        let camera_id = data["camera_id"] as! Int
        do {
            try AVAudioSession.sharedInstance().setCategory(.playAndRecord)
            try AVAudioSession.sharedInstance().setMode(.videoChat)
            try AVAudioSession.sharedInstance().setActive(true)
            guard let camera = cameraView?.cameraView else {
                print("camera not ready")
                result(nil)
                return
            }
            nuSDKService.initiateBC(sTitle: title, nCamIdx:camera_id , nResIdx: 2, nVQuality: 2, ivLocalView: camera)
            if let delegate = Palio.getLSDelegate() {
                delegate.onStartLS(state: 0, message: title)
            }
        } catch {
            print(error)
        }
        result(nil)
    }
    
    private func handleJoinLiveStreaming(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        do {
            try AVAudioSession.sharedInstance().setCategory(.playAndRecord)
            try AVAudioSession.sharedInstance().setMode(.videoChat)
            try AVAudioSession.sharedInstance().setActive(true)
            
            let screenSize: CGRect = UIScreen.main.bounds
            let screenWidth = screenSize.width
            let screenHeight = screenSize.height
            let viewWidth = screenWidth/3
            let viewHeight = 4/3*viewWidth
            var remote : RemoteView?
            if (remoteViews.count > 0){
                remote = remoteViews[0]
            }
            guard let views = remote?.remoteView else {
                print("remote not ready")
                result(nil)
                return
            }
            views.transform = CGAffineTransform.init(scaleX: -1, y: 1).rotated(by: (CGFloat.pi * 5)/2)
            //            views.transform = CGAffineTransform.identity.rotated(by: (CGFloat.pi * 5)/2)
            views.translatesAutoresizingMaskIntoConstraints = false
            views.heightAnchor.constraint(equalToConstant: screenHeight).isActive = true
            //            views.widthAnchor.constraint(equalToConstant: screenWidth).isActive = true
            views.contentMode = .scaleAspectFill
            //            views.layer.cornerRadius = 5
            views.clipsToBounds = true
            nuSDKService.joinBC(sBroadcasterID: device_id, ivRemoteView: views)
            if let delegate = Palio.getLSDelegate() {
                delegate.onStartLS(state: 22, message: device_id)
            }
        } catch {
            print(error)
        }
        result(nil)
    }
    
    private func handleLeftLiveStreaming(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        nuSDKService.terminateBC(sBroadcasterID: device_id)
        remoteViews.removeAll()
        result(nil)
    }
    
    private func handleEndLiveStreaming(call: FlutterMethodCall, result: @escaping FlutterResult) {
        nuSDKService.terminateBC(sBroadcasterID: nil)
        cameraView = nil
        result(nil)
    }
    
    private func handleSendLSBroadcast(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let title = data["title"] as! String
            let type = data["type"] as! String
            let typeValue = data["typeValue"] as! String
            let category = data["category"] as! String
            let message = CoreMessage_TMessageBank.sendLSBroadcast(title: title, type: type, typeValue: typeValue, category: category)
            if let response = Palio.writeSync(message: message) {
                result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: ""))
                return
            }
            result(nil)
        }
    }
    
    private func joinLiveVideo(call: FlutterMethodCall,result: @escaping FlutterResult){
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let broadcast_id = data["broadcast_id"] as! String
            let request_id = data["request_id"] as! String
            let message = CoreMessage_TMessageBank.joinLiveVideo(broadcast_id: broadcast_id, request_id: request_id)
            if let response = Palio.writeSync(message: message) {
                print("response")
                print(message)
                let broadcaster = response.getBody(key: CoreMessage_TMessageKey.L_PIN,default_value: "")
                let f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN,default_value: "")
                let thumb_id = response.getBody(key: CoreMessage_TMessageKey.THUMB_ID,default_value: "")
                let name = response.getBody(key: CoreMessage_TMessageKey.NAME,default_value: "")
                let quantity = response.getBody(key: CoreMessage_TMessageKey.QUANTITY,default_value: "")
                let data = broadcaster+","+f_pin+","+thumb_id+","+name+","+quantity
                result(data)
                return
            }
            result(nil)
        }
    }
    private func removeLiveVideo(call: FlutterMethodCall,result: @escaping FlutterResult){
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let broadcast_id = data["broadcast_id"] as! String
            let request_id = data["request_id"] as! String
            let message = CoreMessage_TMessageBank.removeLiveVideo(broadcast_id: broadcast_id, request_id: request_id)
            if let _ = Palio.writeSync(message: message) {
                print("response")
                print(message)
                result(nil)
                return
            }
            result(nil)
        }
    }
    private func leftLiveVideo(call: FlutterMethodCall,result: @escaping FlutterResult){
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let broadcast_id = data["broadcast_id"] as! String
            let request_id = data["request_id"] as! String
            let message = CoreMessage_TMessageBank.leftLiveVideo(broadcast_id: broadcast_id, request_id: request_id)
            if let response = Palio.writeSync(message: message) {
                print("response")
                print(message)
                let broadcaster = response.getBody(key: CoreMessage_TMessageKey.L_PIN,default_value: "")
                let f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN,default_value: "")
                let thumb_id = response.getBody(key: CoreMessage_TMessageKey.THUMB_ID,default_value: "")
                let name = response.getBody(key: CoreMessage_TMessageKey.NAME,default_value: "")
                let quantity = response.getBody(key: CoreMessage_TMessageKey.QUANTITY,default_value: "")
                let data = broadcaster+","+f_pin+","+thumb_id+","+name+","+quantity
                result(data)
                return
            }
            result(nil)
        }
    }
    private func handleSendLSChat(call: FlutterMethodCall,result: @escaping FlutterResult){
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let message_text = data["message_text"] as! String
            let message = CoreMessage_TMessageBank.getSendLSChat(l_pin: l_pin, message_text: message_text)
            if let response = Palio.writeSync(message: message) {
                print("response")
                print(message)
                let l_pin = response.getBody(key: CoreMessage_TMessageKey.L_PIN,default_value: "")
                let f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN,default_value: "")
                let thumb_id = response.getBody(key: CoreMessage_TMessageKey.THUMB_ID,default_value: "")
                let name = response.getBody(key: CoreMessage_TMessageKey.NAME,default_value: "")
                let message = response.getBody(key: CoreMessage_TMessageKey.MESSAGE_TEXT,default_value: "")
                let data = l_pin+","+f_pin+","+thumb_id+","+name+","+message
                result(data)
                return
            }
            result(nil)
        }
    }
    private func handleSendLSEmotion(call: FlutterMethodCall,result: @escaping FlutterResult){
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let emotion = data["emotion"] as! String
            let message = CoreMessage_TMessageBank.getSendLSEmotion(l_pin: l_pin, emotion_type: emotion)
            if let response = Palio.writeSync(message: message) {
                print("response")
                print(message)
                let l_pin = response.getBody(key: CoreMessage_TMessageKey.L_PIN,default_value: "")
                let likes = response.getBody(key: CoreMessage_TMessageKey.LIKES,default_value: "")
                let quantity = response.getBody(key: CoreMessage_TMessageKey.QUANTITY,default_value: "")
                let data = l_pin+","+likes+","+quantity
                result(data)
                return
            }
            result(nil)
        }
    }
    private func handleUpdateLSTitle(call: FlutterMethodCall,result: @escaping FlutterResult){
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let title = data["title"] as! String
            let message = CoreMessage_TMessageBank.getUpdateLSTitle(title: title)
            if let response = Palio.writeSync(message: message) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
                return
            }
            result(nil)
        }
    }
    
    private func handleChangePersonInfoName(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let fname = data["firstname"] as! String
            let lname = data["lastname"] as! String
            let message = CoreMessage_TMessageBank.getChangePersonInfoName(firstname:fname,lastname:lname)
            if let response = Palio.writeAndWait(message: message, timeout: 5000) {
                result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: ""))
            } else {
                result(nil)
            }
        }
    }
    private func handleChangePersonInfoPassword(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let password = data["password"] as! String
            let oldpassword = data["oldpassword"] as! String
            let message = CoreMessage_TMessageBank.getChangePersonInfoPassword(password:password,oldpassword:oldpassword)
            if let _ = Palio.writeSync(message: message) {
                result("Sukses")
            } else {
                result(nil)
            }
        }
    }
    private func handleChangePersonStatus(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let status = data["status"] as! String
            let message = CoreMessage_TMessageBank.getChangePersonInfoQuote(quote: status)
            if let _ = Palio.writeSync(message: message) {
                result("Sukses")
            } else {
                result(nil)
            }
        }
    }
    
    private func handleChangePersonImage(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let thumb_id = data["thumb_id"] as! String
            if !thumb_id.isEmpty {
                let group = DispatchGroup()
                group.enter()
                Network().upload(name: String(thumb_id)) { (result, progress) in
                    print("result: \(result), progress: \(progress)")
                    if result {
                        if progress == 100 {
                            group.leave()
                        }
                    } else {
                        group.leave()
                    }
                }
                print("changeImage.thumb.wait")
                group.wait()
            }
            let message = CoreMessage_TMessageBank.getChangePersonImage(thumb_id: thumb_id)
            if let _ = Palio.writeSync(message: message) {
                result("Sukses")
            } else {
                result(nil)
            }
        }
    }
    
    private func handleSetAcceptCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let me = UserDefaults.standard.string(forKey: "me")!
            let accept_call = data["accept_call"] as! Bool
            let offline_mode = !accept_call
            let offline = offline_mode ? "1" : "0"
            let message = CoreMessage_TMessageBank.getChangePersonOfflineMode(offline: offline_mode)
            if let _ = Palio.writeSync(message: message) {
                Database.shared.database?.inTransaction({ (fmdb, rollback) in
                    do {
                        _ = Database.shared.updateRecord(fmdb: fmdb, table: "BUDDY", cvalues: [
                            "offline_mode" : offline
                        ], _where: "f_pin = '\(me)'")
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                })
                result("Sukses")
            } else {
                result(nil)
            }
        }
    }
    
    private func handleSetPrivate(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let _private = data["private"] as! Bool
            let message = CoreMessage_TMessageBank.getChangePersonPrivacy(privacy: _private)
            if let _ = Palio.writeSync(message: message) {
                result("Sukses")
            } else {
                result(nil)
            }
        }
    }
    
    private func handleSetSpeakerOn(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let speaker = data["speaker"] as! Bool
        Palio.turnSpeakerOn(bSpeakerOn: speaker)
        result(nil)
    }
    
    private func handleGetLSData(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let message = CoreMessage_TMessageBank.getLSData(l_pin: l_pin)
            if let response = Palio.writeSync(message: message) {
                print("response")
                print(message)
                let l_pin = response.getBody(key: CoreMessage_TMessageKey.L_PIN,default_value: "")
                let likes = response.getBody(key: CoreMessage_TMessageKey.LIKES,default_value: "")
                let quantity = response.getBody(key: CoreMessage_TMessageKey.QUANTITY,default_value: "")
                let data = l_pin+","+likes+","+quantity
                result(data)
                return
            }
            result(nil)
        }
    }
    
    private func handleCreateLS(call: FlutterMethodCall, result: @escaping FlutterResult) {
        print("create ls")
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let title = data["title"] as! String
            let type = data["type"] as! String
            let typeValue = data["typeValue"] as! String
            let category = data["category"] as! String
            let blog_id = data["blog_id"] as! String
            let datas = data["data"] as! String
            let message = CoreMessage_TMessageBank.createLS(title: title, type: type, typeValue: typeValue, category: category, blog_id: blog_id, data: datas)
            print("sent to server create ls")
            if let response = Palio.writeSync(message: message) {
                print("response create ls")
                print(response)
                result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: ""))
                return
            }
            result(nil)
        }
    }
    
    private func handleSendReadMessageStatus(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            let chat_id = data["chat_id"] as! String
            let message_scope_id = data["message_scope_id"] as! String
            let message_id = data["message_id"] as! String
            let message = CoreMessage_TMessageBank.getUpdateRead(p_chat_id: chat_id, p_f_pin: f_pin, p_scope_id: message_scope_id, qty: 1)
            let fPin = message.getBody(key: CoreMessage_TMessageKey.F_PIN)
            let scope = message.getBody(key: CoreMessage_TMessageKey.SCOPE_ID)
            message.mBodies[CoreMessage_TMessageKey.SERVER_DATE] = String(Date().currentTimeMillis())
            if (fPin.elementsEqual("-999") || scope.elementsEqual("16") || scope.elementsEqual("15")){
                return
            }
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                do {
                    _ = Database.shared.updateRecord(fmdb: fmdb, table: "MESSAGE", cvalues: [
                        "status" : "4"
                    ], _where: "message_id = '\(message_id)'")
                } catch {
                    rollback.pointee = true
                    print(error)
                }
            })
            message.mStatus = CoreMessage_TMessageUtil.getTID()
            message.mBodies[CoreMessage_TMessageKey.L_PIN] = f_pin
            message.mBodies[CoreMessage_TMessageKey.MESSAGE_ID] = "-2,\(message_id)"
            _ = Palio.write(message: message)
        }
    }
    
    private func handleInitiateVCRoom(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        let camera_id = data["camera_id"] as! Int
        do {
            try AVAudioSession.sharedInstance().setCategory(.playAndRecord)
            try AVAudioSession.sharedInstance().setMode(.videoChat)
            try AVAudioSession.sharedInstance().setActive(true)
            let screenSize: CGRect = UIScreen.main.bounds
            let screenWidth = screenSize.width
            let screenHeight = screenSize.height
            let viewWidth = screenWidth/3
            let viewHeight = 4/3*viewWidth
            var views = [UIImageView]()
            for i in remoteViews {
                if let view = i.remoteView {
                    view.transform = CGAffineTransform.init(scaleX: 1, y: 1).rotated(by: (CGFloat.pi * 5)/2)
                    view.translatesAutoresizingMaskIntoConstraints=false
                    view.heightAnchor.constraint(equalToConstant: viewHeight).isActive = true
                    view.widthAnchor.constraint(equalToConstant: viewWidth).isActive = true
                    view.contentMode = .scaleAspectFill
                    view.layer.cornerRadius = 5
                    view.clipsToBounds = true
                    //                    view.isHidden = true
                    views.append(view)
                    print("remote view added")
                }
            }
            guard let camera = cameraView?.cameraView else {
                print("camera view null")
                result(nil)
                return
            }
            //        let camera = (cameraView?.cameraView)!
            camera.translatesAutoresizingMaskIntoConstraints = false
            camera.heightAnchor.constraint(equalToConstant: viewHeight).isActive = true
            camera.widthAnchor.constraint(equalToConstant: viewWidth).isActive = true
            camera.contentMode = .scaleAspectFill
            camera.layer.cornerRadius = 5
            camera.clipsToBounds = true
            //            camera.isHidden = true
            print("camera view added")
            guard let zoom = zoomView?.zoomView else {
                print("zoom view null")
                result(nil)
                return
            }
            //        let zoom = (zoomView?.zoomView)!
            zoom.transform = CGAffineTransform(rotationAngle: CGFloat.pi / 2 )
            zoom.translatesAutoresizingMaskIntoConstraints = false
            zoom.heightAnchor.constraint(equalToConstant: screenHeight).isActive = true
            zoom.widthAnchor.constraint(equalToConstant: screenWidth).isActive = true
            zoom.contentMode = .scaleAspectFit
            zoom.layer.cornerRadius = 5
            zoom.clipsToBounds = true
            print("zoom view added")
            
            nuSDKService.initiateCR(sConfRoom: device_id, nCamIdx: camera_id, nResIdx: 2, nVQuality: 2, ivRemoteView: views, ivLocalView: camera, ivRemoteZ: zoom)
        }
        catch {
            print(error)
        }
        
        result(nil)
    }
    private func handleJoinVCRoom(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        let camera_id = data["camera_id"] as! Int
        do {
            try AVAudioSession.sharedInstance().setCategory(.playAndRecord)
            try AVAudioSession.sharedInstance().setMode(.videoChat)
            try AVAudioSession.sharedInstance().setActive(true)
            let screenSize: CGRect = UIScreen.main.bounds
            let screenWidth = screenSize.width
            let screenHeight = screenSize.height
            let viewWidth = screenWidth/3
            let viewHeight = 4/3*viewWidth
            var views = [UIImageView]()
            for i in remoteViews {
                if let view = i.remoteView {
                    view.transform = CGAffineTransform.init(scaleX: 1, y: 1).rotated(by: (CGFloat.pi * 5)/2)
                    view.translatesAutoresizingMaskIntoConstraints=false
                    view.heightAnchor.constraint(equalToConstant: viewHeight).isActive = true
                    view.widthAnchor.constraint(equalToConstant: viewWidth).isActive = true
                    view.contentMode = .scaleAspectFill
                    view.layer.cornerRadius = 5
                    view.clipsToBounds = true
                    //                    view.isHidden = true
                    views.append(view)
                    print("remote view added")
                }
            }
            guard let camera = cameraView?.cameraView else {
                print("camera view null")
                result(nil)
                return
            }
            //        let camera = (cameraView?.cameraView)!
            camera.translatesAutoresizingMaskIntoConstraints = false
            camera.heightAnchor.constraint(equalToConstant: viewHeight).isActive = true
            camera.widthAnchor.constraint(equalToConstant: viewWidth).isActive = true
            camera.contentMode = .scaleAspectFill
            camera.layer.cornerRadius = 5
            camera.clipsToBounds = true
            //            camera.isHidden = true
            print("camera view added")
            guard let zoom = zoomView?.zoomView else {
                print("zoom view null")
                result(nil)
                return
            }
            //        let zoom = (zoomView?.zoomView)!
            zoom.transform = CGAffineTransform(rotationAngle: CGFloat.pi / 2 )
            zoom.translatesAutoresizingMaskIntoConstraints = false
            zoom.heightAnchor.constraint(equalToConstant: screenHeight).isActive = true
            zoom.widthAnchor.constraint(equalToConstant: screenWidth).isActive = true
            zoom.contentMode = .scaleAspectFit
            zoom.layer.cornerRadius = 5
            zoom.clipsToBounds = true
            print("zoom view added")
            nuSDKService.joinCR(sConfRoom: device_id, nCamIdx: camera_id, nResIdx: 2, nVQuality: 2, ivRemoteView: views, ivLocalView: camera, ivRemoteZ: zoom)
        }
        catch {
            print(error)
        }
        
        result(nil)
    }
    
    private func handleJoinACRoom(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        do {
            //            try AVAudioSession.sharedInstance().setCategory(.playAndRecord)
            try AVAudioSession.sharedInstance().setCategory(.playAndRecord, options: .duckOthers)
            try AVAudioSession.sharedInstance().setMode(.voiceChat)
            try AVAudioSession.sharedInstance().setActive(true)
        } catch {
            print(error)
        }
        nuSDKService.initiateCR(sConfRoom: device_id)
        result(nil)
    }
    
    private func handleInitiateACRoom(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        do {
            //            try AVAudioSession.sharedInstance().setCategory(.playAndRecord)
            try AVAudioSession.sharedInstance().setCategory(.playAndRecord, options: .duckOthers)
            try AVAudioSession.sharedInstance().setMode(.voiceChat)
            try AVAudioSession.sharedInstance().setActive(true)
        } catch {
            print(error)
        }
        nuSDKService.joinCR(sConfRoom: device_id)
        result(nil)
    }
    
    private func handleTurnCRoomAudio(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        let speaker = data["speaker"] as! Bool
        nuSDKService.turnCCallAudio(sParty: device_id, bOnOff: speaker)
        result(nil)
    }
    
    private func handleRequestLiveVideo(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            let message = CoreMessage_TMessageBank.getRequestLiveVideo(f_pin:f_pin)
            if let response = Palio.writeSync(message: message) {
                result("Sent")
                return
            }
            result(nil)
        }
    }
    private func handleCreateVCRoom(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let blog_id = data["blog_id"] as! String
            let datas = data["data"] as! String
            let message = CoreMessage_TMessageBank.createVCallConference(blog_id: blog_id, data: datas)
            if let response = Palio.writeSync(message: message) {
                result("Create VCRoom")
                return
            }
            result(nil)
        }
    }
    
    private func handleStartVCR(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let blog_id = data["blog_id"] as! String
            let time = data["time"] as! String
            let message = CoreMessage_TMessageBank.startVCallConference(blog_id: blog_id, time: time)
            if let response = Palio.write(message: message) {
                result("Start VCRoom")
                return
            }
            result(nil)
        }
    }
    
    private func handleJoinVCR(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let blog_id = data["blog_id"] as! String
            let message = CoreMessage_TMessageBank.joinVCallConference(blog_id: blog_id)
            if let response = Palio.write(message: message) {
                result("Join VCRoom")
                return
            }
            result(nil)
        }
    }
    
    private func handleEndVCR(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let blog_id = data["blog_id"] as! String
            let message = CoreMessage_TMessageBank.endVCallConference(blog_id: blog_id)
            if let response = Palio.write(message: message) {
                result("End VCRoom")
                return
            }
            result(nil)
        }
    }
    
    private func handleStartScreenSharing(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data =  call.arguments as! [AnyHashable: Any?]
            let device_id = data["device_id"] as! String
            print(device_id)
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.startScreenSharing(device_id: device_id, title: device_id)){
                let errCode = response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99")
                if errCode == "00" {
                    result("Start Screen Sharing")
                    return
                }
            }
            result(nil)
        }
    }
    
    private func handleInitScreenSharing(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data =  call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        print("init ss")
        print(device_id)
        sleep(2)
        nuSDKService.initiateSS(sTitle: device_id, nQuality: 0)
        print("init ss success")
        result(nil)
    }
    
    private func handleJoinScreenSharing(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let data = call.arguments as! [AnyHashable: Any?]
        let device_id = data["device_id"] as! String
        let screenSize: CGRect = UIScreen.main.bounds
        let screenWidth = screenSize.width
        let screenHeight = screenSize.height
        
        guard let zoom = zoomView?.zoomView else {
            print("zoom view null")
            result(nil)
            return
        }
        zoom.transform = CGAffineTransform.identity.rotated(by: (CGFloat.pi * 3)/2)
        zoom.translatesAutoresizingMaskIntoConstraints = false
        zoom.heightAnchor.constraint(equalToConstant: screenHeight).isActive = true
        zoom.widthAnchor.constraint(equalToConstant: screenWidth).isActive = true
        zoom.contentMode = .scaleAspectFit
        zoom.layer.cornerRadius = 5
        zoom.clipsToBounds = true
        print("zoom view added")
        nuSDKService.joinSS(sBroadcasterID: device_id, ivRemoteView: zoom)
        result(nil)
    }
    
    private func handleEndScreenSharing(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data =  call.arguments as! [AnyHashable: Any?]
            let device_id = data["device_id"] as! String
            print(device_id)
            if let _ = Palio.writeSync(message: CoreMessage_TMessageBank.terminateScreenSharing(device_id: device_id)){
                result("Success Send Terminate SS")
            }
            print("remove image view end ss")
            nuSDKService.terminateSS()
            self.cameraView = nil
            self.zoomView = nil
            print("remove image view end ss success")
            result(nil)
        }
    }
    
    private func handleLeaveScreenSharing(call: FlutterMethodCall, result: @escaping FlutterResult) {
        print("remove image view leave ss")
        nuSDKService.terminateSS()
        self.cameraView = nil
        self.zoomView = nil
        print("remove image view leave ss success")
        result(nil)
    }
    
    public func onStartScreenSharing(state: Int, message: String) {
        var map = [AnyHashable: Any?]()
        map["state"] = state
        map["message"] = message
        postEvent(name: "startSS", map: map)
    }
    
    public func onJoinScreenSharing(state: Int, message: String) {
        var map = [AnyHashable: Any?]()
        map["state"] = state
        map["message"] = message
        postEvent(name: "joinSS", map: map)
    }
    private func handleSendInvitationMyselfBot(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let blog_id = data["blog_id"] as! String
            let text = data["text"] as! String
            let type = data["type"] as! String
            Palio.saveMessageBot(textMessage: text, blog_id: blog_id, attachment_type: type)
            result(nil)
        }
    }
    private func handleUploadImage(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let thumb_id = data["thumb_id"] as! String
            if !thumb_id.isEmpty {
                let group = DispatchGroup()
                group.enter()
                Network().upload(name: String(thumb_id)) { (results, progress) in
                    print("result: \(results), progress: \(progress)")
                    if results {
                        if progress == 100 {
                            if let delegate = Palio.getUploadDelegate() {
                                delegate.onUploadProgress(fileName: thumb_id, progress: progress)
                            }
                            result(results)
                            group.leave()
                        }else{
                            if let delegate = Palio.getUploadDelegate() {
                                delegate.onUploadProgress(fileName: thumb_id, progress: progress)
                            }
                        }
                    } else {
                        if let delegate = Palio.getUploadDelegate() {
                            delegate.onUploadProgress(fileName: thumb_id, progress: progress)
                        }
                        result(results)
                        group.leave()
                    }
                }
                print("changeImage.thumb.wait")
                group.wait()
            }
        }
    }
    
    private func handleSendOTPSignup(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let msisdn = data["msisdn"] as! String
            let demo = data["demo"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.sendOTPMSISDN(p_pin: "",msisdn: msisdn)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    if demo == "1"{
                        let reg_status = response.getBody(key: CoreMessage_TMessageKey.REG_STATUS, default_value: "0")
                        if reg_status == "1" {
                            let f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN, default_value: "")
                            let f_name = response.getBody(key: CoreMessage_TMessageKey.FIRST_NAME, default_value: "")
                            let l_name = response.getBody(key: CoreMessage_TMessageKey.LAST_NAME, default_value: "")
                            let thumb_id = response.getBody(key: CoreMessage_TMessageKey.THUMB_ID, default_value: "")
                            let data_result = f_pin+"|"+f_name+"|"+l_name+"|"+thumb_id
                            UserDefaults.standard.set(f_pin, forKey: "me")
                            if let delegate = Palio.getLoginDelegate() {
                                delegate.onProcess(message: data_result, status: reg_status)
                            }
                            result(data_result)
                        } else {
                            if let delegate = Palio.getLoginDelegate() {
                                delegate.onProcess(message: "Signup", status: reg_status)
                            }
                            result(reg_status)
                        }
                    }else{
                        if let delegate = Palio.getLoginDelegate() {
                            delegate.onProcess(message: "Success", status: "1")
                        }
                        result("Success")
                    }
                }else{
                    if let delegate = Palio.getLoginDelegate() {
                        delegate.onProcess(message: "Failed", status: "0")
                    }
                    result("Failed")
                }
                return
            }
            
        }
    }
    
    private func handleVerifyOTP(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let msisdn = data["msisdn"] as! String
            let otp = data["otp"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.verifyOTP(p_pin: "",msisdn: msisdn, otp: otp)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "00") == "00") {
                    let reg_status = response.getBody(key: CoreMessage_TMessageKey.REG_STATUS, default_value: "0")
                    if reg_status == "1" {
                        let f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN, default_value: "")
                        let f_name = response.getBody(key: CoreMessage_TMessageKey.FIRST_NAME, default_value: "")
                        let l_name = response.getBody(key: CoreMessage_TMessageKey.LAST_NAME, default_value: "")
                        let thumb_id = response.getBody(key: CoreMessage_TMessageKey.THUMB_ID, default_value: "")
                        let data_result = f_pin+"|"+f_name+"|"+l_name+"|"+thumb_id
                        UserDefaults.standard.set(f_pin, forKey: "me")
                        if let delegate = Palio.getLoginDelegate() {
                            delegate.onProcess(message: data_result, status: reg_status)
                        }
                        result(data_result)
                    } else {
                        if let delegate = Palio.getLoginDelegate() {
                            delegate.onProcess(message: "Signup", status: reg_status)
                        }
                        result(reg_status)
                    }
                } else {
                    if let delegate = Palio.getLoginDelegate() {
                        delegate.onProcess(message: "Wrong OTP", status: response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "00"))
                    }
                    result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "00"))
                }
                return
            }
            
        }
    }
    
    private func handleSigninOTP(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let pin = data["f_pin"] as! String
            let f_name = data["f_name"] as! String
            let l_name = data["l_name"] as! String
            let thumb_id = data["thumb_id"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.signInOTP(p_pin: pin,f_name:f_name,l_name:l_name,thumb_id:thumb_id)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "00") == "00") {
                    let f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN, default_value: "00")
                    UserDefaults.standard.set(f_pin, forKey: "me")
                    self.changeUser(f_pin: f_pin)
                    if let delegate = Palio.getLoginDelegate() {
                        delegate.onProcess(message: f_pin, status: response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "00"))
                    }
                    result(f_pin)
                } else {
                    if let delegate = Palio.getLoginDelegate() {
                        delegate.onProcess(message: "Failed", status: response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "00"))
                    }
                    result(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "00"))
                }
                return
            }
            
        }
    }
    private func handleChangeUserConnection(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            self.changeUser(f_pin: f_pin)
            result(nil)
        }
    }
    
    private func handleSendOTPChangeMSISDN(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let msisdn = data["msisdn"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.sendOTPChangeMSISDN(msisdn: msisdn)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    if let delegate = Palio.getLoginDelegate() {
                        delegate.onProcess(message: "Success", status: "1")
                    }
                    result("Success")
                }else if(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "1f"){
                    if let delegate = Palio.getLoginDelegate() {
                        delegate.onProcess(message: "MSISDN Already Registered", status: "2")
                    }
                    result("MSISDN Already Registered")
                }else if(response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "10"){
                    if let delegate = Palio.getLoginDelegate() {
                        delegate.onProcess(message: "Failed", status: "0")
                    }
                    result("Failed")
                }
                return
            }
            
        }
    }
    
    private func handleChangePersonMSISDN(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let msisdn = data["msisdn"] as! String
            let message = CoreMessage_TMessageBank.getChangePersonMSISDN(msisdn: msisdn)
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result("Success")
                }else{
                    result("Failed")
                }
            } else {
                result(nil)
            }
        }
    }
    
    private func handleGetBatchBuddies(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let p_f_pin = data["p_f_pin"] as! String
            let last_update = data["last_update"] as! Int
            let message = CoreMessage_TMessageBank.getBatchBuddiesInfos(p_f_pin: p_f_pin, last_update: last_update)
            Palio.write(message: message)
            result(nil)
        }
    }
    
    private func handleSendLoginEmail(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let email = data["email"] as! String
            let message = CoreMessage_TMessageBank.getSendLoginEmail(email: email)
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result("Success")
                }else{
                    result("Failed")
                }
            } else {
                result(nil)
            }
        }
    }
    
    private func handleSendVerificationEmail(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let email = data["email"] as! String
            let token = data["token"] as! String
            let message = CoreMessage_TMessageBank.getVerificationEmail(email: email, token: token)
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result("Success")
                }else{
                    result("Failed")
                }
            } else {
                result(nil)
            }
        }
    }
    
    private func handleGetListSchool(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let message = CoreMessage_TMessageBank.getListSchool()
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: "[]"))
                }else{
                    result("[]")
                }
            } else {
                result("[]")
            }
        }
    }
    
    private func handleGetListSchoolByName(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let keyword = data["keyword"] as! String
            let message = CoreMessage_TMessageBank.getListSchool(keyword: keyword)
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: "[]"))
                }else{
                    result("[]")
                }
            } else {
                result("[]")
            }
        }
    }
    
    private func handleGetListClassName(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let school_id = data["school_id"] as! String
            let level = data["level"] as! String
            let major = data["major"] as! String
            let classType = data["class_type"] as! String
            let message = CoreMessage_TMessageBank.getListClassName(school_id: school_id, level: level, major: major, class_type: classType)
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: "[]"))
                }else{
                    result("[]")
                }
            } else {
                result("[]")
            }
        }
    }
    
    private func handleGetRequestApprovalTeacher(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let msisdn = data["msisdn"] as! String
            let f_pin = data["f_pin"] as! String
            let l_pin = data["l_pin"] as! String
            let school_id = data["school_id"] as! String
            let level = data["level"] as! String
            let stage = data["stage"] as! String
            let major = data["major"] as! String
            let type = data["type"] as! String
            let className = data["class_name"] as! String
            let email = data["email"] as! String
            let name = data["name"] as! String
            let message = CoreMessage_TMessageBank.requestApprovalTeacher(f_pin: f_pin, l_pin: l_pin, school_id: school_id, level: level, stage: stage, major: major, type: type, className: className, email: email, name: name, msisdn: msisdn)
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result("Success")
                }else{
                    result("Failed")
                }
            } else {
                result("Failed")
            }
        }
    }
    private func handleRequestStudent(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let msisdn = data["msisdn"] as! String
            let f_pin = data["f_pin"] as! String
            let l_pin = data["l_pin"] as! String
            let school_id = data["school_id"] as! String
            let group_id = data["group_id"] as! String
            let email = data["email"] as! String
            let name = data["name"] as! String
            let message = CoreMessage_TMessageBank.requestStudent(f_pin:f_pin,school_id: school_id, group_id: group_id, email: msisdn, name: name, l_pin: l_pin,msisdn:msisdn)
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result("Success")
                }else{
                    result("Failed")
                }
            } else {
                result("Failed")
            }
        }
    }
    private func handleGetUpdateUser(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let msisdn = data["msisdn"] as! String
            let role = data["role"] as! String
            let image = data["image"] as! String
            let password = data["password"] as! String
            let email = data["email"] as! String
            let datas = data["data"] as! String
            let name = data["name"] as! String
            let message = CoreMessage_TMessageBank.getUpdateUser(msisdn: msisdn,email: email, name: name, image: image, role: role, password: password, data: datas)
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.L_PIN, default_value: ""))
                }else{
                    result("Failed")
                }
            } else {
                result("Failed")
            }
        }
    }
    
    private func handleSubmitSchool(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            let school_id = data["school_id"] as! String
            let school_name = data["school_name"] as! String
            let level = data["level"] as! String
            let stage = data["stage"] as! String
            let class_ = data["class"] as! String
            let major = data["major"] as! String
            let name = data["name"] as! String
            let message = CoreMessage_TMessageBank.submitSchool(f_pin: f_pin, schoolId: school_id, schoolName: school_name, level: level, eduStage: stage, p_class_type: class_, major: major, p_class_name: name)
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result(response.getBody(key: CoreMessage_TMessageKey.GROUP_ID, default_value: ""))
                }else{
                    result("Failed")
                }
            } else {
                result("Failed")
            }
        }
    }
    
    private func handleDeletePostTimeline(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let post_id = data["post_id"] as! String
            let ec_date = data["ec_date"] as! String
            let last_update = data["last_update"] as! String
            let message = CoreMessage_TMessageBank.deletePost(post_id: post_id, last_update: last_update, ec_date: ec_date)
            let _ = Palio.write(message: message)
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                _ = Database.shared.deleteRecord(fmdb: fmdb, table: "POST", _where: "post_id='\(post_id)'")
            })
            if let delegate = Palio.getTimelineDelegate() {
                delegate.onPostUpdate(status: "Delete", message: "Success")
            }
            result("Success")
        }
    }
    
    private func handleIsInitiatorJoin(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let broadcaster_id = data["broadcast_id"] as! String
            let category_id = data["category_id"] as! String
            let blog_id = data["blog_id"] as! String
            let message = CoreMessage_TMessageBank.getIsInitiatorJoin(p_broadcaster: broadcaster_id, p_category: category_id, blog_id: blog_id)
            if let response = Palio.writeSync(message: message) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    let title = response.getBody(key: CoreMessage_TMessageKey.TITLE, default_value: "Live Video Promotion")
                    result("0~\(title)")
                }else if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "10"){
                    result("-1")
                }else{
                    result("-2")
                }
            } else {
                result("-2")
            }
        }
    }
    
    private func handleStartLPInvited(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let title = data["title"] as! String
            let type = data["type"] as! String
            let typeValue = data["typeValue"] as! String
            let category_id = data["category_id"] as! String
            let blog_id = data["blog_id"] as! String
            let message = CoreMessage_TMessageBank.getStartLPInvited(title: title, type: type, typeValue: typeValue, category: category_id, blog_id: blog_id)
            let _ = Palio.write(message: message)
            result("Success")
        }
    }
    
    private func handleGetRequestStudent(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let formId = data["form_id"] as! String
            print("formId: \(formId)")
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getRequestStudent(form_id: formId)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    let f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN)
                    let l_pin = response.getBody(key: CoreMessage_TMessageKey.L_PIN)
                    let ktp = response.getBody(key: CoreMessage_TMessageKey.CARD_ID)
                    let name = response.getBody(key: CoreMessage_TMessageKey.SCHOOL_STUDENT_NAME)
                    let email = response.getBody(key: CoreMessage_TMessageKey.SCHOOL_STUDENT_EMAIL)
                    let status = response.getBody(key: CoreMessage_TMessageKey.STATUS, default_value: "0")
                    let thumb = response.getBody(key: CoreMessage_TMessageKey.THUMB_ID, default_value: "")
                    let school_name = response.getBody(key: CoreMessage_TMessageKey.SCHOOL_DESC, default_value: "")
                    let class_name = response.getBody(key: CoreMessage_TMessageKey.GROUP_NAME, default_value: "")
                    let parent_name = response.getBody(key: CoreMessage_TMessageKey.SCHOOL_STUDENT_PARENT_NAME, default_value: "")
                    let _result = "\(f_pin)|\(l_pin)|\(ktp)|\(name)|\(email)|\(status)|\(thumb)|\(school_name)|\(class_name)|\(parent_name)"
                    result(_result)
                }else{
                    result("Failed")
                }
            }
        }
    }
    
    private func handleApproveRequestStudent(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let formId = data["form_id"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getApproveRequestStudent(form_id: formId, l_pin: l_pin)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result("Success")
                }else{
                    result("Failed")
                }
            }
        }
    }
    
    private func handleGetRequestTeacher(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let formId = data["form_id"] as! String
            print("formId: \(formId)")
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getRequestTeacher(form_id: formId)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    let f_pin = response.getBody(key: CoreMessage_TMessageKey.F_PIN)
                    let ktp = response.getBody(key: CoreMessage_TMessageKey.CARD_ID)
                    let name = response.getBody(key: CoreMessage_TMessageKey.SCHOOL_STUDENT_NAME)
                    let email = response.getBody(key: CoreMessage_TMessageKey.SCHOOL_STUDENT_EMAIL)
                    let status = response.getBody(key: CoreMessage_TMessageKey.STATUS, default_value: "0")
                    let thumb = response.getBody(key: CoreMessage_TMessageKey.THUMB_ID, default_value: "")
                    let school_name = response.getBody(key: CoreMessage_TMessageKey.SCHOOL_DESC, default_value: "")
                    let stage_name = response.getBody(key: CoreMessage_TMessageKey.LEVEL, default_value: "")
                    let major_name = response.getBody(key: CoreMessage_TMessageKey.MAJOR_EDU, default_value: "")
                    let class_type_name = response.getBody(key: CoreMessage_TMessageKey.TYPE, default_value: "")
                    let class_name = response.getBody(key: CoreMessage_TMessageKey.GROUP_NAME, default_value: "")
                    let _result = "\(f_pin)|\(ktp)|\(name)|\(email)|\(status)|\(thumb)|\(school_name)|\(stage_name)|\(major_name)|\(class_type_name)|\(class_name)"
                    result(_result)
                }else{
                    result("Failed")
                }
            }
        }
    }
    
    private func handleApproveRequestTeacher(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let formId = data["form_id"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getApproveRequestTeacher(form_id: formId, l_pin: l_pin)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result("Success")
                }else{
                    result("Failed")
                }
            }
        }
    }
    
    private func handleGetListSubAccount(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getListSubAccount()) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    let _result = response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: "[]")
                    result(_result)
                }else{
                    result("Failed")
                }
            }
        }
    }
    
    private func handleQuizDetail(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let l_pin = data["l_pin"] as! String
            let post_id = data["post_id"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getQuizDetail(post_id: post_id, lpin: l_pin)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    let question = response.getBody(key: CoreMessage_TMessageKey.DATA, default_value: "[]")
                    let answer = response.getBody(key: CoreMessage_TMessageKey.TEXT_PLAIN, default_value: "")
                    let created_date = response.getBody(key: CoreMessage_TMessageKey.CREATED_DATE, default_value: "")
                    let status = response.getBody(key: CoreMessage_TMessageKey.STATUS, default_value: "RO")
                    let payment = response.getBody(key: CoreMessage_TMessageKey.PAYMENT_ID, default_value: "1")
                    let score = response.getBody(key: CoreMessage_TMessageKey.SCORE, default_value: "")
                    let poster = response.getBody(key: CoreMessage_TMessageKey.F_PIN, default_value: "")
                    let _result = "\(question)|\(answer)|\(created_date)|\(status)|\(payment)|\(score)|\(poster)"
                    result(_result)
                }else{
                    result("Failed")
                }
            }
        }
    }
    
    private func handleQuizAnswer(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let datas = data["data"] as! String
            let post_id = data["post_id"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getQuizAnswer(post_id: post_id, data: datas)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result("Success")
                }else{
                    result("Failed")
                }
            }
        }
    }
    
    private func handleQuizScoring(call: FlutterMethodCall, result: @escaping FlutterResult) {
        DispatchQueue.global().async {
            let data = call.arguments as! [AnyHashable: Any?]
            let score = data["score"] as! String
            let post_id = data["post_id"] as! String
            let l_pin = data["l_pin"] as! String
            if let response = Palio.writeSync(message: CoreMessage_TMessageBank.getQuizScoring(score: score, post_id: post_id, lpin: l_pin)) {
                if (response.getBody(key: CoreMessage_TMessageKey.ERRCOD, default_value: "99") == "00") {
                    result("Success")
                }else{
                    result("Failed")
                }
            }
        }
    }
    
    private func handleSetLiveCenterSession(call: FlutterMethodCall, result: @escaping FlutterResult){
        DispatchQueue.global().async {
            var ret : String? = nil
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            let status = data["status"] as! String
            let complaint_id = data["complaint_id"] as! String
            ret = self.setLiveCenterSession(f_pin: f_pin, status: status, complaint_id: complaint_id) ? "done" : nil
            result(ret)
        }
    }
    
    public func setLiveCenterSession(f_pin: String, status: String, complaint_id: String) -> Bool{
        var ret = false;
        if(status == "2"){
//            self.setCallCenterDateStart(f_pin: f_pin, date: date)
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                let date = "\(Date().currentTimeMillis())"
                do {
                    _ = try Database.shared.insertRecord(fmdb: fmdb, table: "PREFS", cvalues: [
                        "key" : "CC_SC:\(f_pin)",
                        "value" : date,
                    ], replace: true)
                    _ = try Database.shared.insertRecord(fmdb: fmdb, table: "PREFS", cvalues: [
                        "key" : "CC:\(f_pin)",
                        "value" : status,
                    ], replace: true)
                    ret = true
                } catch {
                    rollback.pointee = true
                    print(error)
                }
            })
        }
        else if(status == "0"){
            Database.shared.database?.inTransaction({ (fmdb, rollback) in
                let date = "\(Date().currentTimeMillis())"
                var dataCC = ""
                var displayName = ""
                var timeStart = ""
                if let cursor = Database.shared.getRecords(fmdb: fmdb, query: "select value from PREFS where key = 'CC_DT:\(f_pin)'"), cursor.next() {
                    if let lastUpdate = cursor.string(forColumnIndex: 0) {
                        dataCC = lastUpdate;
                    }
                    cursor.close()
                    if let cursor = Database.shared.getRecords(fmdb: fmdb, query: "select first_name, last_name from BUDDY where f_pin = '\(f_pin)'"), cursor.next() {
                        let firstName = cursor.string(forColumnIndex: 0)
                        let lastName = cursor.string(forColumnIndex: 1)
                        displayName = "\(firstName) \(lastName)"
                        // todo: insert to history db
                        cursor.close()
                    }
                    if let cursor = Database.shared.getRecords(fmdb: fmdb, query: "select value from PREFS where key = 'CC_SC:\(f_pin)'"), cursor.next() {
                        if let lastUpdate = cursor.string(forColumnIndex: 0) {
                            timeStart = lastUpdate;
                        }
                        cursor.close()
                    }
                    do {
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "CALL_CENTER_HISTORY", cvalues: [
                            "type" : "1",
                            "title" : displayName,
                            "time" : timeStart,
                            "f_pin" : f_pin,
                            "data" : dataCC,
                            "time_end" : date,
                            "complaint_id" : complaint_id.isEmpty ? "C\(date)" : complaint_id
                        ], replace: true)
                        _ = try Database.shared.insertRecord(fmdb: fmdb, table: "PREFS", cvalues: [
                            "key" : "CC:\(f_pin)",
                            "value" : status,
                        ], replace: true)
                        ret = true
                    } catch {
                        rollback.pointee = true
                        print(error)
                    }
                }
            })
        }
        return ret
    }
    
    private func handleSetCallCenterDateStart(call: FlutterMethodCall, result: @escaping FlutterResult){
        DispatchQueue.global().async {
            var ret : String? = nil
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            let date = data["date"] as! String
            _ = self.setCallCenterDateStart(f_pin: f_pin, date: date)
            result("done")
        }
    }
    
    public func setCallCenterDateStart(f_pin: String, date: String) -> Bool{
        var ret = false;
        Database.shared.database?.inTransaction({ (fmdb, rollback) in
            do {
                _ = try Database.shared.insertRecord(fmdb: fmdb, table: "PREFS", cvalues: [
                    "key" : "CC_SC:\(f_pin)",
                    "value" : date,
                ], replace: true)
                ret = true
            } catch {
                rollback.pointee = true
                print(error)
            }
        })
        return ret
    }
    
    private func handleSetCallCenterData(call: FlutterMethodCall, result: @escaping FlutterResult){
        DispatchQueue.global().async {
            var ret : String? = nil
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            let dataCC = data["data"] as? String ?? ""
            _ = self.setCallCenterData(f_pin: f_pin, data: dataCC)
            result("done")
        }
    }
    
    public func setCallCenterData(f_pin: String, data: String) -> Bool{
        var ret = false;
        Database.shared.database?.inTransaction({ (fmdb, rollback) in
            do {
                _ = try Database.shared.insertRecord(fmdb: fmdb, table: "PREFS", cvalues: [
                    "key" : "CC_DT:\(f_pin)",
                    "value" : data,
                ], replace: true)
                ret = true
            } catch {
                rollback.pointee = true
                print(error)
            }
        })
        return ret
    }
    
    private func handleSetCCSessionWelcome(call: FlutterMethodCall, result: @escaping FlutterResult){
        DispatchQueue.global().async {
            var ret : String? = nil
            let data = call.arguments as! [AnyHashable: Any?]
            let f_pin = data["f_pin"] as! String
            _ = self.setCCSessionWelcome(f_pin: f_pin) ? "done" : nil
            result("done")
        }
    }
    
    public func setCCSessionWelcome(f_pin: String) -> Bool{
        var ret = false;
        Database.shared.database?.inTransaction({ (fmdb, rollback) in
            do {
                _ = try Database.shared.insertRecord(fmdb: fmdb, table: "PREFS", cvalues: [
                    "key" : "CC_SS",
                    "value" : f_pin,
                ], replace: true)
                ret = true
            } catch {
                rollback.pointee = true
                print(error)
            }
        })
        return ret
    }
}
